#coding: utf-8
from PyQt4.QtCore import SIGNAL, \
                         QDateTime, \
                         QVariant, \
                         Qt, \
                         QDate
from PyQt4.QtGui import QDialog, \
                        QTableWidgetItem, \
                        QMessageBox, \
                        QAbstractItemView, \
                        QFont, \
                        QItemSelectionModel, \
                        QIcon

from console_edenwall.ev_audit_dialog import AuditEventDialog
from console_edenwall.ui.audit_ui import Ui_DialogAlerts
from console_edenwall.option_audit_dialog import OptionDialog
from nucentral.common import tr
from compatibility import Compatibility


def toTimeStamp(time):
    secs = time.second()
    secs += 60*time.minute()
    secs += 3600*time.hour()
    return secs

class AuditDialog(QDialog):
    CLIENT_VERSION = "1.0"

    COLUMNS = (
        ("priority", tr("Priority")),
        ("timestamp", tr("Time")),
        ("message", tr("Description")),
        ("uniqueid", tr("Id")),
    )

    TABLE_ACK = 0
    TABLE_TIME = 1

    def __init__(self, parent):
        QDialog.__init__(self, parent)
        self.client = parent.client
        self.window = parent

        # ***** Dialog Alerts ******
        self.ui = Ui_DialogAlerts()
        self.ui.setupUi(self)

        # Table non acknowledged
        self.table = self.ui.alerts_table
        self.table.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.table.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.table.verticalHeader().hide()
        self.table.horizontalHeader().setHighlightSections(False)
        self.table.horizontalHeader().setStretchLastSection(True)

        # Table on custom time
        self.table_time = self.ui.alerts_table_time
        self.table_time.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.table_time.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.table_time.verticalHeader().hide()
        self.table_time.horizontalHeader().setHighlightSections(False)
        self.table_time.horizontalHeader().setStretchLastSection(True)

        self.calendar = self.ui.dateEdit
        self.calendar.setDate(QDate.currentDate())
        self.pb_refresh = self.ui.pb_refresh
        self.pb_refresh.setIcon(QIcon(":/icons/refresh.png"))

        self.connect(self.ui.read_button, SIGNAL("clicked()"), self.readAlert)
        self.connect(self.ui.pb_close, SIGNAL("clicked()"), self.close)
        self.connect(self.ui.pb_ack_alert, SIGNAL("clicked()"), self.acknowledge)
        self.connect(self.table, SIGNAL("cellClicked(int, int)"), self._rowClicked)
        self.connect(self.table, SIGNAL("cellDoubleClicked(int, int)"), self._rowDoubleClicked)
        self.connect(self.table_time, SIGNAL("cellClicked(int, int)"), self._rowClicked)
        self.connect(self.table_time, SIGNAL("cellDoubleClicked(int, int)"), self._rowDoubleClicked)
        self.connect(self.ui.pb_option, SIGNAL("clicked()"), self._loadOptionDialog)

        self.connect(self.ui.tabWidget, SIGNAL("currentChanged(int)"), self._loadCurrentPage)
        self.connect(self.pb_refresh, SIGNAL("clicked()"), self._refresh)
        self.connect(self.calendar, SIGNAL("dateChanged (QDate)"), self._dateChanged)

        # ******* Dialog Option ******
        self.ui_option = OptionDialog(self)
        self.connect(self.ui_option.pb_ok, SIGNAL("clicked()"), self._applyOption)

        self.table_type = self.TABLE_ACK
        self.client.async().call('audit', 'getAlerts', float(self.CLIENT_VERSION), callback=self.fill, errback=self._errorHandler, cache=True)

        self.events = []
        self.alerts = {}

        # NOT used for now
        self.compatibility = Compatibility(self)

    def error(self, message):
        QMessageBox.warning(self,
                            "Version error",
                            message
                            )

    def _dateChanged(self, date):
        # Tab alert_time
        index = 1
        self._loadCurrentPage(index)

    def _refresh(self):
        index = 0
        if self.table_type == self.TABLE_TIME:
            index = 1
        self._loadCurrentPage(index)

    def _loadCurrentPage(self, page_index):

        if page_index == 0:
            self.table_type = self.TABLE_ACK
            self.client.async().call('audit', 'getAlerts',
                                     float(self.CLIENT_VERSION),
                                     callback=self.fill, errback=self._errorHandler, cache=True)
        elif page_index == 1:
            self.table_type = self.TABLE_TIME
            date = self.ui.dateEdit.date()

            day = date.day()
            month = date.month()
            year = date.year()

            self.client.async().call('audit', 'getAlertsByDate',
                                               float(self.CLIENT_VERSION),
                                               day, month, year,
                                               callback=self.fill, errback=self._errorHandler, cache=True)

    def _applyOption(self):
        threshold_source = self.ui_option.sb_source_threshold.value()
        threshold_category = self.ui_option.sb_category_threshold.value()
        max_stored_event = self.ui_option.sb_max_events.value()

        # Combination
        combination_threshold = self.ui_option.sb_threshold_comb.value()
        combination_time = toTimeStamp(self.ui_option.te_time.time())

        notification = 0
        if self.ui_option.cb_notice.isChecked():
            notification = 1

        self.ui_option.close()

        parameters = {'source_threshold' : threshold_source,
                      'category_threshold' : threshold_category,
                      'max_stored_events' : max_stored_event,
                      'additional_action_enabled' : notification,

                      'combination_time' : combination_time,
                      'combination_threshold' : combination_threshold,

                      'client_version' : float(self.CLIENT_VERSION),
                      }
        self.client.call('audit', 'configure', parameters)


    def _loadOptionDialog(self):
        parameters = self.client.call('audit', 'parameters', float(self.CLIENT_VERSION))

        if parameters is not None:
            self.ui_option.parameters = parameters
            self.ui_option.show()

    def _rowClicked(self, row, column):
        new_column = 0
        table = self._currentTable()
        table.setCurrentCell(row, new_column, QItemSelectionModel.Select)

    def _rowDoubleClicked(self, row, column):
        self.readAlert()

    def _errorHandler(self, err):
        QMessageBox.critical(self, tr("Alerts"), unicode(err))

    def acknowledge(self):
        unbold = True
        alerts_id, alerts, ack_allowed = self.selectedAlerts(unbold)
        if ack_allowed:

            for alert in alerts:
                alert['acknowledged'] = True

            message = ''
            self.client.call('audit', 'ackAlerts', float(self.CLIENT_VERSION), alerts_id, message)
        else:
            message = "Some alerts are already acknowledged"
            QMessageBox.critical(self, tr("Acknowledged"), unicode(message))

    def _currentTable(self):
        if self.table_type == self.TABLE_ACK:
            return self.table
        elif self.table_type == self.TABLE_TIME:
            return self.table_time

    def selectedAlerts(self, unbold=False):

        table = self._currentTable()

        indexes = table.selectedIndexes()
        column_id = 0
        alerts = []
        alerts_id = []
        indexed = []
        ack_allowed = True

        for index in indexes:
            if unbold:
                row = index.row()
                column = index.column()
                item = table.item(row, column)
                font = QFont()
                font.setBold(False)
                item.setFont(font)
                table.setItem(row, column, item)

            if index.row() in indexed:
                continue
            indexed.append(index.row())
            item = table.item(index.row(), column_id)

            alert_id = item.data(Qt.UserRole).toInt()[0]
            if alert_id is None:
                return []

            alert = self.alerts[alert_id]
            if alert['acknowledged']:
                ack_allowed = False
            alerts.append(alert)
            alerts_id.append(alert_id)

        return [alerts_id, alerts, ack_allowed]

    def readAlert(self):
        alerts = self.selectedAlerts()[1]
        if alerts:
            events_dialog = AuditEventDialog(alerts, self)
            events_dialog.exec_()

    def fill(self, alerts):
        self.alerts.clear()

        table = self._currentTable()

        table.setRowCount(len(alerts))
        for row, alert in enumerate(alerts):
            for column, attr_label in enumerate(self.COLUMNS):
                attr = attr_label[0]
                if attr == "timestamp":
                    value = QDateTime.fromTime_t(alert[attr]).toString()
                else:
                    value = alert[attr]
                    value = unicode(value)

                item = QTableWidgetItem(value)
                item.setData(Qt.UserRole, QVariant(alert['uniqueid']))
                self.alerts[alert['uniqueid']] = alert

                if 'acknowledged' in alert and not alert['acknowledged']:
                    font = QFont()
                    font.setBold(True)
                    item.setFont(font)
                table.setItem(row, column, item)

        if table.rowCount() > 0:
            table.resizeColumnsToContents()
            table.horizontalHeader().setStretchLastSection(True)

        if not self.isVisible():
            self.exec_()
        return True

