from PyQt4.QtGui import QDialog, QTableWidgetItem, QAbstractItemView, QTextEdit, QFontMetrics, QFont,QLabel
from PyQt4.QtCore import SIGNAL, QDateTime, Qt

from nucentral.common import tr

from console_edenwall.ui.ev_audit_ui import Ui_DialogEvents

class AuditEventDialog(QDialog):
    COLUMNS = (
#        ("priority", tr("Priority")),
        ("timestamp", tr("Time")),
        ("user", tr("User")),
        ("message", tr("Description")),
        ("ip_info", tr("IP Information")),
    )

    def __init__(self, alerts, parent = None):
        QDialog.__init__(self, parent)
        self.ui = Ui_DialogEvents()
        self.ui.setupUi(self)

        self.table = self.ui.events_table
        self.table.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.table.setEditTriggers(QAbstractItemView.NoEditTriggers)

        self.hheader = self.table.horizontalHeader()
        self.vheader = self.table.verticalHeader()
        self.hheader.setStretchLastSection(True)

        self.connect(self.ui.pb_close, SIGNAL('clicked()'), self.close)
        self.alerts = alerts
        self.fill()

    def fill(self):
        if len(self.alerts) == 0:
            return

        alert = self.alerts[0]
        category = ''
        if 'category' in alert and len(alert['category']) > 0:
            category = alert['category']
        else:
            if 'events' in alert and isinstance(alert['events'], list) and len(alert['events']) > 0:
                event = alert['events'][0]
                category = event['category']

        self.ui.label_title.setText(tr("category: ") + category)
        self.table.verticalHeader().hide()
        self.table.horizontalHeader().setHighlightSections(False)
        events = alert['events']

        self.table.setRowCount(len(events))
        for row, event in enumerate(events):
            for column, attr_label in enumerate(self.COLUMNS):
                attr = attr_label[0]
                value = event[attr]

                if isinstance(value, list) and len(value) == 1:
                    value = value[0]

                if attr == 'timestamp':
                    if isinstance(value, int) and value > 1000000:
                        value = QDateTime.fromTime_t(value).toString()
                else:
                    value = unicode(value)

                text = QTextEdit(self)
                self.table.setCellWidget(row, column, text)
                text.setText(value)
                text.setTextInteractionFlags(Qt.TextSelectableByMouse | Qt.TextSelectableByKeyboard)
                text.adjustSize()

                fm = QFontMetrics(QFont())
                self.table.resizeColumnToContents(column)

                width = fm.size(Qt.TextExpandTabs, value).width()
                height = 100

                self.table.setColumnWidth(column, width)
                if attr == 'message':
                    self.table.setRowHeight(row, height)
                if attr == 'user':
                    width = fm.width(value*2)
                    self.table.setColumnWidth(column, width)
