#coding: utf-8

"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from PyQt4.QtGui import (QSplitter, QSizePolicy, QGroupBox,
    QVBoxLayout, QHBoxLayout, QLabel, QPixmap,
    QFrame)
from PyQt4.QtCore import Qt, SIGNAL, QSize
import locale
from sys import exit

from nucentral.client import NuCentralError, SOFTWARE_MAIN_NAME
from nucentral.common import tr
from nucentral.common.multisite import MULTISITE_SLAVE, MULTISITE_MASTER, NO_MULTISITE
from nucentral.qt.application import create_nucentral_application, load_resource
from nucentral.qt.central_window import CentralWindow, EMBEDDED, STANDALONE
from nulog.client.qt.frag_splitter import FragSplitter
from nulog.client.qt.user_settings.fragment import Fragment
from nulog.client.qt.user_settings import UserSettings
from nucentral.common import EDENWALL

if EDENWALL:
    from nuconf.common.ha_statuses import ENOHA


MULTISITE_LABELS = {
    NO_MULTISITE: tr('Firewall'),
    MULTISITE_SLAVE: tr('Firewall'),
    MULTISITE_MASTER: tr('Master'),
}

supervisor_fields = {
    "purge_system_log": tr("System logs disk usage:"),
    "purge_sql_log": tr("Database disk usage estimate:"),
    "purge_edenlog": tr("Configuration service logs<br/>&nbsp;&nbsp;"
                        "&nbsp;&nbsp;disk usage estimate:"),
}

def italic(string):
    return "<i>%s</i>" % string

def strong(string):
    return "<strong>%s</strong>" % string

def SelectableLabel(text):
    label = QLabel(text)
    label.setTextInteractionFlags(Qt.TextSelectableByKeyboard |
                                  Qt.TextSelectableByMouse)
    return label

def _format(ID):
    locale.resetlocale()
    try:
        return SelectableLabel(locale.format('%d', int(ID), 1))
    except:
        return SelectableLabel(ID)

class HomePageWindow(CentralWindow):
    APPLICATION_NAME = SOFTWARE_MAIN_NAME
    RESOURCE = 'nulog3.rcc'
    LOCALE = ':/nulog/locale/nulog'
    ROLES = set(['multisite_read', 'nulog_read', 'nuconf_read', 'nuauth_command_read'])
    ICON = ':/icons/info.png'

    FRAGMENTS = [{'type':     'LoadStream',
                  'title':    tr('Load average'),
                  'view':     'line',
                  'args':     {'interval': 5}
                 },
                 {'type':  'TrafficStream',
                  'title': tr('Traffic'),
                  'view':  'line',
                  'args':  {'interval': 15}
                 },
                 {'type':  'ConUserTable',
                  'title': tr('Connected users'),
                  'view':  'table',
                  'args':  {'interval': 5}
                 },
                 {'type':  'LastPacketsStream',
                  'title' : tr('Total number of packets'),
                  'view':  'line',
                  'args':  {'interval': 5}
                 }
                ]

    def __init__(self, application, client, standalone=EMBEDDED, parent=None, eas_window=None):
        CentralWindow.__init__(self, client, parent, eas_window)
        self.setupCentralWindow(application, standalone)
        self.setCentralWindowTitle(tr("Summary"), client)
        self.frag_widgets = []
        load_resource('system.rcc')
        self.user_settings = UserSettings(self, client)

        wdg = QSplitter(Qt.Horizontal, self)
        self.setCentralWidget(wdg)

        groupbox = QGroupBox(self)
        if EDENWALL:
            groupbox.setTitle(self.tr("Device Information"))
        else:
            groupbox.setTitle(self.tr("NuFirewall Server"))
        groupbox.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        vbox = QVBoxLayout(groupbox)

        ## Device informations
        try:
            self.createWidgets(client, vbox)
        except NuCentralError, e:
            device_information_loaded = False
            if e.type != 'AclError':
                message = unicode(self.tr('<b>Error</b>: %s')) % unicode(e)
                vbox.addWidget(QLabel(message))
            else:
                # Make sure the groupbox will be deleted
                groupbox.setParent(None)
        else:
            wdg.addWidget(groupbox)
            device_information_loaded = True

        ## Display nulog fragments
        self.vertical_splitter = FragSplitter(self.client, self)
        wdg.addWidget(self.vertical_splitter)
        self.preLoad()

        if not device_information_loaded and not self.frag_widgets:
            # if data are not displayed: hide homepage application
            raise Exception

    def createWidgets(self, client, vbox):
        values = client.call('system_info', 'systemInfo')

        edenwall_image = self.getImage(values)
        label = QLabel()
        label.setPixmap(QPixmap(edenwall_image))
        vbox.addWidget(label)

        if EDENWALL:
            frame = self.addPicto(client)
            vbox.addWidget(frame)

        label = self.createLabel(client, values)
        vbox.addWidget(label)
        vbox.addStretch()

    def getImage(self, values):
        if not EDENWALL:
            return ':/images/nufirewall_logo.png'

        fw_model = values.get('model')
        if 'E120' == fw_model:
            return ':/images/ew4-e120'
        elif 'E230' == fw_model:
            return ':/images/ew4-e230'
        elif 'E260' == fw_model:
            return ':/images/ew4-e230'
        elif 'E3200' == fw_model:
            return ':/images/ew4-e3200'
        elif 'E3200' == fw_model:
            return ':/images/ew4-e3200'
        elif 'E4300' == fw_model:
            return ':/images/ew4-e4300'
        elif 'E4500' == fw_model:
            return ':/images/ew4-e4300'
        else:
            return ':/images/ew4-unknown'

    # Add an HA and or a multisite icon
    def addPicto(self, client):
        components = client.call('CORE', 'getComponentList')
        emf = 'multisite_master' in components

        ha = False
        if 'ha' in components:
            try:
                if client.call('ha', 'getHAMode') != ENOHA:
                    ha = True
            except NuCentralError:
                pass

        frame = QFrame()
        picto_layout = QHBoxLayout(frame)
        frame.setLayout(picto_layout)

        if emf:
            picto_img = QPixmap(':/icons/picto_emf')
            picto_img = picto_img.scaled(QSize(32, 32))
            picto = QLabel()
            picto.setPixmap(picto_img)
            #picto.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
            picto_layout.addWidget(picto)

        if ha:
            picto_img = QPixmap(':/icons/picto_ha')
            picto_img = picto_img.scaled(QSize(32, 32))
            picto = QLabel()
            picto.setPixmap(picto_img)
            #picto.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
            picto_layout.addWidget(picto)
        picto_layout.addStretch()
        return frame

    def getInfoText(self, values):
        if not EDENWALL:
            version = values.get('sw_version', 'unknown')
            return '%s %s<br/>' % (tr("Software version"), strong(version))

        info_text = ''
        for item, value in (
            (self.tr("Device model:"), 'model'),
            (self.tr("Property of:"), 'client'),
            (self.tr("Hardware version:"), 'hw_version'),
            (self.tr("Software version:"), 'sw_version'),
            (self.tr("Serial number:"), 'serial'),
        ):
            info_text += '%s %s<br/>' % (item, strong(values.get(value, '')))

        multisite = MULTISITE_LABELS[self.multisite_type]
        info_text += '%s %s<br/>' % (self.tr("Type:"), strong(multisite))

        if self.multisite_type == MULTISITE_MASTER:
            num = len(client.call('multisite_master', 'listFirewalls'))
            info_text += '%s %s<br>' % (self.tr('Firewalls:'), strong(str(num)))
        return info_text

    def createLabel(self, client, values):
        items = [(self.tr("Host:"), client.host)]
        try:
            hostname = client.call('hostname', 'getShortHostname')
        except Exception:
            # ignore hostname errors
            pass
        else:
            items.append((self.tr("Hostname:"), hostname))

        if EDENWALL:
            groups = ', '.join(self.getGroups())
            items.extend((
                (self.tr("Logged in as:"), client.login),
                (self.tr("Groups:"), groups),
            ))

        # Supervisor states.
        try:
            supervisor_states = client.call("supervisor", "getStates")
            self._add_supervisor_states(items, supervisor_states)
        except Exception:
            pass

        info_text = self.getInfoText(values)
        for item, value in items:
            info_text += '%s %s<br/>' % (item, strong(value))

        label = QLabel(info_text)
        label.setTextInteractionFlags(Qt.TextSelectableByKeyboard |
                                      Qt.TextSelectableByMouse)
        return label

    def _colorized_criticity(self, state):
        color = ""
        if state["level"] == 0:
            color = "green"
        elif state["level"] >= 3:  # Last alert.
            color = "red"
        if not color:  # The first alerts are in black because orange is hard
                       # to read.
            return "%s%s" % (state["criticity"], tr("%"))
        return "<span style='color:%s'>%s%s</span>" % (
            color, state["criticity"], tr("%"))

    def _add_supervisor_states(self, items, supervisor_states):
        items.append(("", ""))  # Add an empty line.
        for state in supervisor_states:
            try:
                field_name = supervisor_fields[state["name"]]
                items.append((field_name, self._colorized_criticity(state)))
            except KeyError:
                pass

    def isReadOnly(self):
        """
        nothing to modify in summary page
        """
        return False

    def preLoad(self):
        if not self.client.call('CORE', 'hasComponent', 'nuauth_command'):
            d = {'type':  'MemoryStream',
                 'title': self.tr('Memory usage'),
                 'view':  'line',
                 'args':  {'interval': 5}}
            self.FRAGMENTS[2] = d

        for args in self.FRAGMENTS:
            self.addFragment(args)

    def quit(self):
        for frag in self.frag_widgets:
            frag.destructor()
        CentralWindow.quit(self)

    def addFragment(self, args):
        frag = Fragment(args['type'])
        frag.load(args)
        fragwidget = self.vertical_splitter.addFragment(frag, {}, menu_pos=False)
        self.connect(fragwidget, SIGNAL('removeFragment()'), lambda: self.vertical_splitter.removeFragment(fragwidget))
        self.frag_widgets.append(fragwidget)
        fragwidget.updateData()
        if hasattr(frag, 'error'):
            self.vertical_splitter.removeFragment(fragwidget)
            self.frag_widgets.remove(fragwidget)

def _get_window_class():
    return HomePageWindow


if __name__ == "__main__":
    app, client = create_nucentral_application(
        name="home",
        resource="",
        locale="")
    window = HomePageWindow(app, client, standalone=STANDALONE)
    window.show()
    exit(app.exec_())
