from __future__ import with_statement # This isn't required in Python 2.6

from PyQt4.QtCore import SIGNAL, QTimer, QTime
from PyQt4.QtGui import (QMessageBox, QDockWidget, QFileDialog, QTextCursor,
    QColor, QWidget, QDialog, QVBoxLayout, QIcon, QPixmap)

DEFAULT_REFRESH = 10
DEFAULT_MIN_LEVEL = 30

from nucentral.client import NuCentralError

from console_edenwall.ui.log_viewer_ui import Ui_LogViewer

log_color_map = {
  20: "darkgreen",
  30: "darkblue",
  40: "darkred",
  50: "red"
}

def my_sort(a,b):
    return cmp(a[0],b[0])

class LogViewerInstance(QWidget, Ui_LogViewer):

    def __init__(self, app, client, parent=None):
        self.filter = None
        self.min_level = DEFAULT_MIN_LEVEL
        self.last_id_displayed = 0
        self.app = app
        self.client = client
        QWidget.__init__(self,parent)
        self.setupUi(self)
        self.client.call("logger","getComponentVersion")
        self.setup()
        self.connectSlots()
        self.refresh()
        self.timerRefresh.start(DEFAULT_REFRESH * 1000)
        self.show()

    def connectSlots(self):
        self.connect(self.pushButtonRefresh, SIGNAL("clicked()"), self.refresh)
        self.connect(self.pushButtonExport, SIGNAL("clicked()"), self.export)
        self.connect(self.editRefreshDelay, SIGNAL("returnPressed()"), self.changeTimerRefresh)
        self.connect(self.editFilter, SIGNAL("returnPressed()"), self.changeFilter)
        self.connect(self.editMinLevel, SIGNAL("returnPressed()"), self.changeMinLevel)
        self.connect(self.timerRefresh, SIGNAL("timeout()"), self.refresh)

    def setup(self):
        self.editRefreshDelay.setText("%s" % DEFAULT_REFRESH)
        self.editMinLevel.setText("%s" % DEFAULT_MIN_LEVEL)
        #self.textEdit.setLineWrapMode( QTextEdit.NoWrap )
        self.timerRefresh = QTimer(self)

    def changeFilter(self):
        self.filter = unicode(self.editFilter.text())
        self.last_id_displayed = 0
        self.textEdit.clear()
        self.refresh()

    def changeMinLevel(self):
        try:
            t = self.editMinLevel.text()
            if len(t)>0:
                self.min_level = int(t)
            else:
                self.min_level = DEFAULT_MIN_LEVEL
            self.last_id_displayed = 0
            self.textEdit.clear()
            self.refresh()
        except ValueError, e:
            QMessageBox.warning(self, self.tr("EAS"),
                                self.tr("Value is invalid"))

    def changeTimerRefresh(self):
        try:
            self.timerRefresh.stop()
            delay = int(self.editRefreshDelay.text())
            if (delay > 0):
                self.timerRefresh.start(delay * 1000)
        except ValueError, e:
            QMessageBox.warning(self, self.tr("EAS"),
                                self.tr("Value is invalid"))

    def export(self):
        fileName = QFileDialog.getSaveFileName(self, self.tr("Save File"),
                    "", self.tr("CSV file (*.csv);;Text file (*.txt)"));
        if not fileName or len(fileName)==0:
            return
        fileName = unicode(fileName)

        if fileName.endswith(".csv"):
            return self._export_to_csv(fileName)
        elif fileName.endswith(".txt"):
            return self._export_to_txt(fileName)
        else:
            QMessageBox.warning(self, self.tr("EAS"),
                                self.tr("You must add a known extension to the file name."))

    def refresh(self):
        self.logs = sorted(self._get_logs(),my_sort)
        for log_line in self.logs:
            if (log_line[0] > self.last_id_displayed):
                self.append_line(log_line)
                self.last_id_displayed = log_line[0]
        self.setWindowTitle( unicode(self.tr("Logs: last refreshed at %s")) % QTime.currentTime().toString() )

    def stopRefresh(self):
        self.pushButtonRefresh.setDisabled(True)
        if self.timerRefresh is not None and self.timerRefresh.isActive():
            self.timerRefresh.stop()

    def append_line(self, log_line):
        color = None
        if log_line[2] in log_color_map:
            color = QColor(log_color_map[log_line[2]])
        self.textEdit.moveCursor(QTextCursor.End, QTextCursor.MoveAnchor)
        saved_color = self.textEdit.textColor();
        self.textEdit.setTextColor( QColor("black") )
        self.textEdit.font().setFixedPitch(True)
        self.textEdit.append( "%s\t%s\t" % (log_line[1].ljust(26),log_line[2]) )
        self.textEdit.font().setFixedPitch(False)
        if (color is not None):
            self.textEdit.setTextColor( color )
            self.textEdit.insertPlainText( "%s" % log_line[3] )
        else:
            self.textEdit.insertPlainText( "%s" % log_line[3] )
        self.textEdit.setTextColor( saved_color )

    def _get_logs(self):
        if self.filter:
            filter = self.filter
        else:
            filter = ""
        try:
            logs = self.client.call("logger","get_log",self.min_level,100,filter)
        except NuCentralError, e:
            # error - disabling timer
            self.timerRefresh.stop()
            raise
        return logs

    def _export_to_csv(self, fileName):
        with open(fileName,"w+") as f:
            for log_line in self.logs:
                f.write("%s,%d,%s\n" % (log_line[1],log_line[2], log_line[3]))

    def _export_to_txt(self, fileName):
        with open(fileName,"w+") as f:
            for log_line in self.logs:
                f.write("%s %d %s\n" % (log_line[1],log_line[2], log_line[3]))

class DockLogViewer(QDockWidget):

    def __init__(self, app, client, parent=None):
        QDockWidget.__init__(self, parent)
        l = LogViewerInstance(app, client, self)
        self.setWidget(l)

class DialogLogViewer(QDialog):

    def __init__(self, app, client, parent=None):
        QDialog.__init__(self, parent)
        self.viewer = LogViewerInstance(app, client, self)
        self.v_layout = QVBoxLayout(self)
        self.v_layout.addWidget(self.viewer)
        icon = QIcon()
        icon.addPixmap(QPixmap(":/icons/EW-01.png"), QIcon.Normal, QIcon.Off)
        self.setWindowIcon(icon)
        self.resize(800,600)
