from sys import exit

from PyQt4.QtCore import SIGNAL
from PyQt4.QtGui import QMessageBox, QAction

from main_window import MainWindow as MonositeMainWindow
from main_window import APPLICATIONS, INDEX_TO_APP, ApplicationInfo
#from nucentral.common.multisite import MULTISITE_MASTER, MULTISITE_SLAVE
from nucentral.common import tr
from nucentral.qt.central_window import FLOATING, EMBEDDED

MULTISITE = 'ew4_multisite'

NUCONF_APP = ['nuconf.client.system', 'nuconf.client.services',]
APP_IN_MENU = [ 'nufaceqt', 'nulog.client.qt', ] + NUCONF_APP

app = ApplicationInfo(MULTISITE, 'multisite.rcc', ':/locale/multisite')
app.index = len(APPLICATIONS)
APPLICATIONS[MULTISITE] = app
INDEX_TO_APP[app.index] = app

def windowCloseEvent(self, event):
    event.ignore()
    self.hide()

class MainWindow(MonositeMainWindow):
    def __init__(self, app, client, parent=None):
        MonositeMainWindow.__init__(self, app, client, parent)
        self.EAS_MESSAGES['show_app'] = self.show_app

    def buildMenu(self):
        menu = self.menuBar().addMenu(tr('EMF appliance'))

        for app in APP_IN_MENU:
            def make_lambda(func, *args):
                return lambda: func(*args)

            action = QAction(APPLICATIONS[app].label, menu)
            self.connect(action, SIGNAL('triggered()'), make_lambda(self.show_app, None, app))
            menu.addAction(action)

    def init_app(self, app, standalone = FLOATING):
        if app.name in self.apps.keys():
            return
        self.logger.warning("Load application %s" % app.label)

        try:
            window = self.search_app(app,standalone)
        except SystemExit:
            window = None
        except Exception, err:
            self.writeError(err, tr("Application %s initialization error") % app.label)
            window = None

        if window:
            # Overwrite the application
            if app != MULTISITE:
                func_type = type(window.closeEvent)
                window.closeEvent = func_type(windowCloseEvent, window, window.__class__)

            self.connect(window, SIGNAL('EAS_Message'), lambda *args, **kwargs: self.gotMessage(MULTISITE, *args, **kwargs))
        self.apps[app.name] = window

    def init_apps(self):
        self.main_app = MULTISITE
        if self.client.call('acl', 'getMinimalMode'):
            self.main_app = 'nuconf.client.system'
            self.load_nuconf(EMBEDDED)
        else:
            self.buildMenu()
            self.init_app(APPLICATIONS[self.main_app], EMBEDDED)

        if self.apps[self.main_app] is None:
            QMessageBox.critical(
                self,
                tr("Critical error"),
                tr("Connection refused. The queried server does not have EdenWall Multi-Firewall (EMF) capabilities.")
                )
            exit(0)
        self.configure_app(self.apps[self.main_app], APPLICATIONS[self.main_app])
        self.setCentralWidget(self.apps[self.main_app])

    def load_nuconf(self, standalone):
        for app_name in NUCONF_APP:
            self.init_app(APPLICATIONS[app_name])

        for app_name in NUCONF_APP:
            app = self.apps[app_name]
            if hasattr(app, 'initialized'):
                func = app.initialized
                if callable(func):
                    func()

        for app_name in NUCONF_APP:
            self.load_app(app_name)

    def show_app(self, from_app, app_name):
        if not app_name in self.apps:
            if app_name in NUCONF_APP:
                self.load_nuconf(FLOATING)
            else:
                self.init_app(APPLICATIONS[app_name])
                self.load_app(app_name)

        try:
            self.apps[app_name].show()
        except Exception, e:
            self.writeError(unicode(e))

