#define _GNU_SOURCE
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>

/* Options layout (cycling -- Esc key goes back to home screen):
 *
 * ^ Number of connected users
 * ^ Load average
 * > EdenWall (home)
 * v Identifier
 * v HA status
 * v Repair menu
 *   - (Restore factory (disabled))
 *   - Open admin access
 *   - Send diag to support
 *   - Open VPN support
 *   - Exit menu
 * v Reboot
 * v Halt
 *
 * */

#define WAIT_MAX 60
#define WAIT_DEFAULT_S "10"
#define WAIT_DEFAULT 10

#define NUAUTH_PORT 4129

/* init_display is the home screen only if there is no error to display.
 * Errors to display are stored in a file handled by init scripts. */
#define ERRORS_FILENAME "/errors_to_display.txt"
#define IP_IFACE_COMMAND "/usr/share/nucentral/scripts/ip_iface.sh 2>/dev/null"
#define GET_ID_COMMAND "/usr/share/nucentral/scripts/getID"
#define MODEL_FILENAME "/var/lib/nucentral/license/model"
#define SEND_DIAG_COMMAND "/usr/share/nucentral/scripts/send_diag_to_support 2>/dev/null"
#define HA_STATUS_COMMAND "/usr/bin/cl_status rscstatus 2>/dev/null"
char init_display_line1[17] = "    EdenWall";
char init_display_line2[17] = "";
/* These 2 lines should be overwritten by open_access(): */
char access_opened_display_line1[17] = "access opened but";
char access_opened_display_line2[17] = " IP not found!";

char identifier_line2[17] = "";

#define IDENTIFIER_LINE1 "Identifier"
#define LOAD_DISPLAY_LINE1 "Load is"

#define NB_DISPLAY_LINE1 "# current users"

#define REBOOT_DISPLAY_LINE1 "Reboot?"
#define REBOOT_DISPLAY_LINE2 " Press Enter"

#define REBOOT_RUNNING_DISPLAY_LINE1 "Rebooting in " WAIT_DEFAULT_S "s"
#define REBOOT_RUNNING_DISPLAY_LINE2 " ESC to abort"

#define REBOOT_CANCELLED_DISPLAY_LINE1 "Reboot"
#define REBOOT_CANCELLED_DISPLAY_LINE2 " Aborted"

#define REBOOT_REBOOTING_DISPLAY_LINE1 "Rebooting..."
#define REBOOT_REBOOTING_DISPLAY_LINE2 ""

#define REPAIR_MENU_DISPLAY_LINE1 "Go to"
#define REPAIR_MENU_DISPLAY_LINE2 " Repair Menu"

#define REPAIR_BACK_DISPLAY_LINE1 "Go back to"
#define REPAIR_BACK_DISPLAY_LINE2 " Main Menu"

#define RESTORE_FACTORY_DISPLAY_LINE1 "Restore HD from"
#define RESTORE_FACTORY_DISPLAY_LINE2 "factory default?"

#define RESTORE_CONFIRM_DISPLAY_LINE1 "Really erase the"
#define RESTORE_CONFIRM_DISPLAY_LINE2 " hard disk?"

#define RESTORE_RUNNING_DISPLAY_LINE1 "Restoring in " WAIT_DEFAULT_S "s"
#define RESTORE_RUNNING_DISPLAY_LINE2 " ESC to abort"

#define RESTORE_CANCELLED_DISPLAY_LINE1 "Restore"
#define RESTORE_CANCELLED_DISPLAY_LINE2 " Aborted"

#define RESTORE_RESTORING_DISPLAY_LINE1 "Rebooting and"
#define RESTORE_RESTORING_DISPLAY_LINE2 " restoring..."

#define OPEN_ACCESS_DISPLAY_LINE1 "Open access on"
#define OPEN_ACCESS_DISPLAY_LINE2 " all ports?"

#define SEND_DIAG_DISPLAY_LINE1 "Send diagnostic"
#define SEND_DIAG_DISPLAY_LINE2 " to support"

#define DIAG_SENT_DISPLAY_LINE1 "Diagnostic sent"
#define DIAG_SENT_DISPLAY_LINE2 " to support"

#define VPN_DISPLAY_LINE1 "Start"
#define VPN_DISPLAY_LINE2 " VPN support"

#define VPN_STARTING_DISPLAY_LINE1 "Starting VPN."
#define VPN_STARTING_DISPLAY_LINE2 " Call support"

#define HALT_DISPLAY_LINE1 "Halt?"
#define HALT_DISPLAY_LINE2 " Press Enter"

#define HALT_RUNNING_DISPLAY_LINE1 "Halting in " WAIT_DEFAULT_S "s"
#define HALT_RUNNING_DISPLAY_LINE2 REBOOT_RUNNING_DISPLAY_LINE2

#define HALT_CANCELLED_DISPLAY_LINE1 "Halt"
#define HALT_CANCELLED_DISPLAY_LINE2 REBOOT_CANCELLED_DISPLAY_LINE2

#define HALT_HALTING_DISPLAY_LINE1 "Halting..."
#define HALT_HALTING_DISPLAY_LINE2 REBOOT_REBOOTING_DISPLAY_LINE2

#define DISPLAY_HA_STATUS_LINE1 "HA status is"

#define EMPTYMESSAGE "                                        "

typedef enum {
	ASK_LOAD,
	ASK_NB,
	ASK_INIT,
	ASK_ID,
	ASK_HA_STATUS,
	REPAIR_MENU,
	REPAIR_ASK_FACTORY,
	REPAIR_ASK_OPEN,
	REPAIR_ASK_DIAG,
	REPAIR_ASK_VPN,
	REPAIR_ASK_BACK,
	ASK_REBOOT,
	ASK_HALT
} action_t;

typedef enum {
	INIT_LEVEL=0x1,
	CONFIRM_LEVEL,  /* For restore options. */
	ENTER_LEVEL,
	CANCEL_LEVEL,
	OUT_LEVEL  /* Used when applying shutdown. */
} action_level_t;

typedef enum {
#ifdef LEGACY_HARDWARE
	BUTTON_UP=0x4D,
	BUTTON_DOWN=0x47,
	BUTTON_ENTER=0x4B,
	BUTTON_ESCAPE=0x4E
#else
	BUTTON_UP=0xFFFFFFBE,
	BUTTON_DOWN=0xFFFFFFBD,
	BUTTON_ENTER=0xFFFFFFBB,
	BUTTON_ESCAPE=0xFFFFFFB7
#endif
} button_t;

typedef struct State {
	action_t action;
	action_level_t level;
} state_t;

typedef struct conn {
#ifdef  UDP_AUTH_ENABLE
	unsigned char proto;
#endif
	unsigned long lcl;
	unsigned int lclp;
	unsigned long rmt;
	unsigned int rmtp;
	unsigned long uid;
	unsigned long ino;
	unsigned int retransmit;
#ifdef  UDP_AUTH_ENABLE
	time_t createtime;
#endif

	struct conn *next;
} conn_t;


static int fd;

state_t state;  /* Global for signal handling. */
int seconds_to_wait = WAIT_DEFAULT;

void open_access() {
	FILE *iipipe;
	char iface[8] = "?", ip[16] = "";

	system("iptables -I INPUT 5 -p tcp --dport 8443 -j ACCEPT");
	/* Bypass NAT rules on local networks (useful if the admin has blocked
	 * their own access to the firewall with an unwise DNAT rule): */
	system("iptables -t nat -I PREROUTING -j ACCEPT");
	/* Reset interface to media negotiation */
	system("for IFACE in $(cat /etc/edenwall/hardware/network_devices  | cut -f 1); do ethtool -s $IFACE autoneg on; done");
	iipipe = popen(IP_IFACE_COMMAND, "r");
	if (iipipe != NULL) {
		/* Scan iface last, since iface is less unlikely than IP not to
		 * fit the given number of characters and not fitting is less
		 * dangerous at the end of the scanning:  in other words, this
		 * reduces the risk of fscanf scanning the next characters of the
		 * "word" which is too long instead of the next "word". */
		fscanf(iipipe, "%15s %6s", ip, iface);
		pclose(iipipe);
	}
	if (ip[0] != '\0') {
		snprintf(access_opened_display_line1, 17, "access on %s:", iface);
		snprintf(access_opened_display_line2, 17, " %s", ip);
	}
}

void start_vpn_support() {
	system("/sbin/iptables -I FORWARD -i support -j DROP");
	system("/sbin/iptables -I INPUT -i support -p udp --dport 8080 -j ACCEPT");
	system("/sbin/iptables -I INPUT -i support -p tcp --dport 8443 -j ACCEPT");
	system("/sbin/iptables -I INPUT -i support -p tcp --dport 22 -j ACCEPT");
	system("/usr/sbin/openvpn --writepid /var/run/vpn-support.pid --daemon vpn-support --status /var/run/vpn-support.status 10 --cd /etc/vpn-support --config /etc/vpn-support/support.conf");
}

void send_diag_to_support() {
	system(SEND_DIAG_COMMAND);
}

void SetEnvironment () {
	system("stty ispeed 2400 < /dev/ttyS1");
	system("stty raw < /dev/ttyS1");
}

int Cmd = 254; /* EZIO Command */

int cls = 1; /* Clear screen */
void Cls () {
	write(fd,&Cmd,1);
	write(fd,&cls,1);
}

int init = 0x28;
void Init () {
	write(fd,&Cmd,1);
	write(fd,&init,1);
}

int stopsend = 0x37;
void StopSend () {
	write(fd,&Cmd,1);
	write(fd,&init,1);
}

int home = 2 ; /* Home cursor */
void Home () {
	write(fd,&Cmd,1);
	write(fd,&home,1);
}

int readkey = 6         ; /* Read key */
void ReadKey () {
	write(fd,&Cmd,1);
	write(fd,&readkey,1);
}

int blank = 8 ; /* Blank display */
void Blank () {
	write(fd,&Cmd,1);
	write(fd,&blank,1);
}

int hide = 12 ; /* Hide cursor & display blanked characters */
void Hide () {
	write(fd,&Cmd,1);
	write(fd,&hide,1);
}

int turn = 13 ; /* Turn On (blinking block cursor) */
void TurnOn () {
	write(fd,&Cmd,1);
	write(fd,&turn,1);
}

int show = 14 ; /* Show underline cursor */
void Show () {
	write(fd,&Cmd,1);
	write(fd,&show,1);
}

int movel = 16 ; /* Move cursor 1 character left */
void MoveL () {
	write(fd,&Cmd,1);
	write(fd,&movel,1);
}

int mover = 20 ; /* Move cursor 1 character right */
void MoveR () {
	write(fd,&Cmd,1);
	write(fd,&mover,1);
}

int scl = 24;        /* Scroll cursor 1 character left */
void ScrollL(){
	write(fd,&Cmd,1);
	write(fd,&scl,1);
}

int scr = 28;      /* Scroll cursor 1 character right */
void ScrollR(){
	write(fd,&Cmd,1);
	write(fd,&scr,1);
}

int setdis = 64;/* Command */
void SetDis(){
	write(fd,&Cmd,1);
	write(fd,&setdis,1);
}

void ShowMessage (char *str1 , char *str2) {
	int a,b;
	a = strlen(str1);
	b = 40 - a;
	write(fd,str1,a);
	write(fd, EMPTYMESSAGE ,b);
	write(fd,str2,strlen(str2));
}

void sig_handler(int sig) {
	switch (state.action) {
		case ASK_LOAD:
		case ASK_NB:
		case ASK_ID:
			state.action = ASK_INIT;
			state.level = INIT_LEVEL;
			return;
		default:
			break;
	}
	switch (state.level) {
		case ENTER_LEVEL:
			switch (state.action) {
				case ASK_REBOOT:
				case ASK_HALT:
				case REPAIR_ASK_FACTORY:
					state.level = OUT_LEVEL;
					break;
				default:
					break;
			}
			break;
		case INIT_LEVEL:  /* Go back to home menu after a timeout: */
		case CONFIRM_LEVEL:
			state.action = ASK_INIT;
			state.level = INIT_LEVEL;
			break;
		default:
			break;
	}
}

char* get_load_average(){
	FILE* procfile;
	char *line=NULL;
	size_t n = 0;
	float l1, l15;
	char* out=calloc(17,sizeof(char));
	procfile=fopen("/proc/loadavg","r");
	getline(&line,&n,procfile);
	sscanf(line,"%f %*f %f", &l1, &l15);
	snprintf(out,17,"1m:%.2f 15m:%.2f", l1, l15);
	out[16]=0;
	fclose(procfile);
	free(line);
	return out;
}

char* get_nb_users() {
	/* Function adapted from nufw/src/clients/lib/tcptable.c
	   Return the number of connected users. */
	conn_t c;
	static FILE *fp = NULL;
	char buf[1024];
	char* out = calloc(17, sizeof(char));
	int nb = 0;
	unsigned int st;  /* st for status ; will be checked if == 1
			     (ESTABLISHED). */

	fp = fopen ("/proc/net/tcp", "r");
	if (fp == NULL) {
		out[0] = '?';
		out[1] = '\0';
		return out;
	}

	if (fgets(buf, sizeof(buf), fp) == NULL) {
		out[0] = '?';
		out[1] = '\0';
		return out;
	}

	while (fgets (buf, sizeof(buf), fp) != NULL) {
		if (sscanf(buf, "%*d: %lx:%x %lx:%x %x %*x:%*x %*x:%*x %x %lu %*d %lu",
					&c.lcl, &c.lclp, &c.rmt, &c.rmtp, &st, &c.retransmit, &c.uid, &c.ino) != 8)
			continue;

		if (c.ino == 0)
			continue;

		if (st == 1 && c.lclp == NUAUTH_PORT) nb++;
	}
	fclose(fp);
	snprintf(out, 17, "%d", nb);
	out[16] = 0;
	return out;
}

char * get_ha_status() {
	FILE *pipe;
	char status[16] = "";
	char* out = calloc(17, sizeof(char));

	pipe = popen(HA_STATUS_COMMAND, "r");
	if (pipe != NULL) {
		fscanf(pipe, "%15s", status);
		pclose(pipe);
	}
	if (status[0] != '\0') snprintf(out, 17, "%s", status);
	else snprintf(out, 17, "(No HA)");
	out[16] = '\0';
	return out;
}

void get_and_set_id() {
	FILE *pipe;
	char id[16] = "";

	pipe = popen(GET_ID_COMMAND, "r");
	if (pipe != NULL) {
		fscanf(pipe, "%15s", id);
		pclose(pipe);
	}
	snprintf(identifier_line2, 17, "%s", id);
}

int switch_state(state_t* state,int button)
{
	switch(state->action){
		case REPAIR_MENU:
			switch(button) {
				case BUTTON_UP:
					state->action = ASK_HA_STATUS;
					state->level = INIT_LEVEL;
					break;
				case BUTTON_DOWN:
					state->action = ASK_REBOOT;
					state->level = INIT_LEVEL;
					break;
				case BUTTON_ENTER:
					state->action = REPAIR_ASK_OPEN;
					state->level = INIT_LEVEL;
					break;
				case BUTTON_ESCAPE:
					state->action = ASK_INIT;
					state->level = INIT_LEVEL;
					break;
			}
			break;
		case ASK_REBOOT:
			if (state->level == OUT_LEVEL) break;
			switch(button) {   
				case BUTTON_UP :
					if (state->level != ENTER_LEVEL) {
						state->level=INIT_LEVEL;
						state->action=REPAIR_MENU;
					}
					break;
				case BUTTON_DOWN :
					if (state->level != ENTER_LEVEL) {
						state->level=INIT_LEVEL;
						state->action=ASK_HALT;
					}
					break;
				case BUTTON_ENTER :  
					state->level=ENTER_LEVEL;
					break;         
				case BUTTON_ESCAPE :
					if (state->level == ENTER_LEVEL)
						state->level=CANCEL_LEVEL;
					else
						state->action=ASK_INIT;
					break;     
			}
			break;
		case ASK_HALT:
			if (state->level == OUT_LEVEL) break;
			switch(button) {   
				case BUTTON_UP :
					if (state->level != ENTER_LEVEL) {
						state->action=ASK_REBOOT;
						state->level=INIT_LEVEL;
					}
					break;
				case BUTTON_DOWN :
					if (state->level != ENTER_LEVEL) {
						state->action=ASK_NB;
						state->level=INIT_LEVEL;
					}
					break;
				case BUTTON_ENTER :  
					state->level=ENTER_LEVEL;
					break;         
				case BUTTON_ESCAPE :
					if (state->level == ENTER_LEVEL)
						state->level=CANCEL_LEVEL;
					else
						state->action=ASK_INIT;
					break;     
			}

			break;
		case ASK_NB:
			switch(button) {
				case BUTTON_UP :
					state->level=INIT_LEVEL;
					state->action=ASK_HALT;
					break;
				case BUTTON_DOWN :
					state->level=INIT_LEVEL;
					state->action=ASK_LOAD;
					break;
				case BUTTON_ENTER :
					break;
				case BUTTON_ESCAPE :
					state->action=ASK_INIT;
					break;
			}

			break;
		case ASK_LOAD:
			switch(button) {   
				case BUTTON_UP :
					state->level=INIT_LEVEL;
					state->action=ASK_NB;
					break;       
				case BUTTON_DOWN :
					state->level=INIT_LEVEL;
					state->action=ASK_INIT;
					break;
				case BUTTON_ENTER :  
					break;         
				case BUTTON_ESCAPE :
					state->action=ASK_INIT;
					break;     
			}

			break;
		case ASK_INIT:
			switch(button) {   
				case BUTTON_UP :
					state->action = ASK_LOAD;
					state->level = INIT_LEVEL;
					break;
				case BUTTON_DOWN :
					state->action = ASK_ID;
					state->level = INIT_LEVEL;
					break;
				case BUTTON_ENTER :  
					break;         
				case BUTTON_ESCAPE :
					break;     
			}
			break;
		case ASK_ID:
			switch(button) {
				case BUTTON_UP :
					state->action = ASK_INIT;
					state->level = INIT_LEVEL;
					break;
				case BUTTON_DOWN :
					state->action = ASK_HA_STATUS;
					state->level = INIT_LEVEL;
					break;
				case BUTTON_ENTER :
					break;
				case BUTTON_ESCAPE :
					state->action=ASK_INIT;
					break;
			}
			break;
		case ASK_HA_STATUS:
			switch(button) {
				case BUTTON_UP :
					state->action = ASK_ID;
					state->level = INIT_LEVEL;
					break;
				case BUTTON_DOWN :
					state->action = REPAIR_MENU;
					state->level = INIT_LEVEL;
					break;
				case BUTTON_ENTER :
					break;
				case BUTTON_ESCAPE :
					state->action=ASK_INIT;
					break;
			}
			break;
		case REPAIR_ASK_FACTORY:
			switch(button) {
				case BUTTON_UP:
					if (state->level != ENTER_LEVEL) {
						state->action = REPAIR_ASK_BACK;
						state->level = INIT_LEVEL;
					}
					break;
				case BUTTON_DOWN:
					if (state->level != ENTER_LEVEL) {
						state->action = REPAIR_ASK_OPEN;
						state->level = INIT_LEVEL;
					}
					break;
				case BUTTON_ENTER:
					if (state->level == INIT_LEVEL)
						state->level = CONFIRM_LEVEL;
					else if (state->level == CONFIRM_LEVEL)
						state->level = ENTER_LEVEL;
					break;
				case BUTTON_ESCAPE:
					if (state->level == ENTER_LEVEL)
						state->level = CANCEL_LEVEL;
					else
						state->action = REPAIR_MENU;
					state->level = INIT_LEVEL;
					break;
			}
			break;
		case REPAIR_ASK_OPEN:
			switch(button) {
				case BUTTON_UP:
					if (state->level != ENTER_LEVEL) {
						state->action = REPAIR_ASK_BACK;
						state->level = INIT_LEVEL;
					}
					break;
				case BUTTON_DOWN:
					if (state->level != ENTER_LEVEL) {
						state->action = REPAIR_ASK_DIAG;
						state->level = INIT_LEVEL;
					}
					break;
				case BUTTON_ENTER:
					state->level = ENTER_LEVEL;
					break;
				case BUTTON_ESCAPE:
					if (state->level == ENTER_LEVEL)
						state->level = CANCEL_LEVEL;
					else
						state->action = REPAIR_MENU;
					state->level = INIT_LEVEL;
					break;
			}
			break;
		case REPAIR_ASK_DIAG:
			switch(button) {
				case BUTTON_UP:
					if (state->level != ENTER_LEVEL) {
						state->action = REPAIR_ASK_OPEN;
						state->level = INIT_LEVEL;
					}
					break;
				case BUTTON_DOWN:
					if (state->level != ENTER_LEVEL) {
						state->action = REPAIR_ASK_VPN;
						state->level = INIT_LEVEL;
					}
					break;
				case BUTTON_ENTER:
					state->level = ENTER_LEVEL;
					break;
				case BUTTON_ESCAPE:
					if (state->level == ENTER_LEVEL)
						state->level = CANCEL_LEVEL;
					else
						state->action = REPAIR_MENU;
					state->level = INIT_LEVEL;
					break;
			}
			break;
		case REPAIR_ASK_VPN:
			switch(button) {
				case BUTTON_UP:
					if (state->level != ENTER_LEVEL) {
						state->action = REPAIR_ASK_DIAG;
						state->level = INIT_LEVEL;
					}
					break;
				case BUTTON_DOWN:
					if (state->level != ENTER_LEVEL) {
						state->action = REPAIR_ASK_BACK;
						state->level = INIT_LEVEL;
					}
					break;
				case BUTTON_ENTER:
					state->level = ENTER_LEVEL;
					break;
				case BUTTON_ESCAPE:
					if (state->level == ENTER_LEVEL)
						state->level = CANCEL_LEVEL;
					else
						state->action = REPAIR_MENU;
					state->level = INIT_LEVEL;
					break;
			}
			break;
		case REPAIR_ASK_BACK:
			switch(button) {
				case BUTTON_UP:
					state->action = REPAIR_ASK_VPN;
					state->level = INIT_LEVEL;
					break;
				case BUTTON_DOWN:
					state->action = REPAIR_ASK_OPEN;
					state->level = INIT_LEVEL;
					break;
				case BUTTON_ENTER:
				case BUTTON_ESCAPE:
					state->action = REPAIR_MENU;
					state->level = INIT_LEVEL;
					break;
			}
			break;
	}
	return 1;
}

int display_state(state_t * state)
{
	switch(state->action){
		case ASK_LOAD:
			{
				char* load=get_load_average();
				Cls();
				ShowMessage (LOAD_DISPLAY_LINE1 ,load);
				free(load);
				state->level = INIT_LEVEL;  /* To force next redisplay. */
			}
			break;
		case ASK_NB:
			{
				char* text_nb=get_nb_users();
				Cls();
				ShowMessage (NB_DISPLAY_LINE1, text_nb);
				free(text_nb);
				state->level = INIT_LEVEL;  /* To force next redisplay. */
			}
			break;
		case ASK_HA_STATUS:
			{
				char* status=get_ha_status();
				Cls();
				ShowMessage (DISPLAY_HA_STATUS_LINE1, status);
				free(status);
				state->level = INIT_LEVEL;  /* To force next redisplay. */
			}
			break;

		case REPAIR_MENU:
			Cls();
			ShowMessage(REPAIR_MENU_DISPLAY_LINE1,
					REPAIR_MENU_DISPLAY_LINE2);
			break;
		case REPAIR_ASK_FACTORY:
			Cls();
			switch (state->level) {
				case INIT_LEVEL:
					ShowMessage(RESTORE_FACTORY_DISPLAY_LINE1,
							RESTORE_FACTORY_DISPLAY_LINE2);
					break;
				case CONFIRM_LEVEL:
					ShowMessage(RESTORE_CONFIRM_DISPLAY_LINE1,
							RESTORE_CONFIRM_DISPLAY_LINE2);
					break;
				case ENTER_LEVEL:
					ShowMessage(RESTORE_RUNNING_DISPLAY_LINE1,
							RESTORE_RUNNING_DISPLAY_LINE2);
					break;
				case OUT_LEVEL:
					ShowMessage(RESTORE_RESTORING_DISPLAY_LINE1,
							RESTORE_RESTORING_DISPLAY_LINE2);
					break;
				case CANCEL_LEVEL:
					break;
				default:
					break;
			}
			break;
		case REPAIR_ASK_OPEN:
			Cls();
			switch (state->level) {
				case INIT_LEVEL:
					ShowMessage(OPEN_ACCESS_DISPLAY_LINE1,
							OPEN_ACCESS_DISPLAY_LINE2);
					break;
				case ENTER_LEVEL:
					ShowMessage(access_opened_display_line1,
							access_opened_display_line2);
					break;
				default:
					break;
			}
			break;
		case REPAIR_ASK_DIAG:
			Cls();
			switch (state->level) {
				case INIT_LEVEL:
					ShowMessage(SEND_DIAG_DISPLAY_LINE1,
					            SEND_DIAG_DISPLAY_LINE2);
					break;
				case ENTER_LEVEL:
					ShowMessage(DIAG_SENT_DISPLAY_LINE1,
					            DIAG_SENT_DISPLAY_LINE2);
					break;
				default:
					break;
			}
			break;
		case REPAIR_ASK_VPN:
			Cls();
			switch (state->level) {
				case INIT_LEVEL:
					ShowMessage(VPN_DISPLAY_LINE1,
					            VPN_DISPLAY_LINE2);
					break;
				case ENTER_LEVEL:
					ShowMessage(VPN_STARTING_DISPLAY_LINE1,
					            VPN_STARTING_DISPLAY_LINE2);
					break;
				default:
					break;
			}
			break;
		case REPAIR_ASK_BACK:
			Cls();
			switch (state->level) {
				case INIT_LEVEL:
					ShowMessage(REPAIR_BACK_DISPLAY_LINE1,
							REPAIR_BACK_DISPLAY_LINE2);
					break;
				default:
					break;
			}
			break;
		case ASK_REBOOT:
			Cls();
			switch (state->level){
				case INIT_LEVEL:
					ShowMessage (REBOOT_DISPLAY_LINE1 ,REBOOT_DISPLAY_LINE2);
					break;
				case ENTER_LEVEL:
					ShowMessage (REBOOT_RUNNING_DISPLAY_LINE1 ,REBOOT_RUNNING_DISPLAY_LINE2);
					break;
				case OUT_LEVEL:
					ShowMessage(REBOOT_REBOOTING_DISPLAY_LINE1, REBOOT_REBOOTING_DISPLAY_LINE2);
					break;
				default:
					break; 
			}
			break;
		case ASK_HALT:
			Cls();
			switch (state->level){
				case INIT_LEVEL:
					ShowMessage (HALT_DISPLAY_LINE1 ,HALT_DISPLAY_LINE2);
					break;
				case ENTER_LEVEL:
					ShowMessage (HALT_RUNNING_DISPLAY_LINE1 ,HALT_RUNNING_DISPLAY_LINE2);
					break;
				case OUT_LEVEL:
					ShowMessage(HALT_HALTING_DISPLAY_LINE1, HALT_HALTING_DISPLAY_LINE2);
					break;
				default:
					break; 
			}
			break;
		case ASK_ID:
			Cls();
			ShowMessage(IDENTIFIER_LINE1, identifier_line2);
			state->level = INIT_LEVEL;  /* To force next redisplay. */
			break;
		case ASK_INIT:
		default:
			Cls();
			ShowMessage(init_display_line1, init_display_line2);
	}
	return 1;
}

int execute_state(state_t* state)
{
	switch (state->action) {
		case REPAIR_ASK_FACTORY:
			switch (state->level){
				case CANCEL_LEVEL:
					alarm(0);  /* Cancels the signal. */
					Cls();
					ShowMessage(RESTORE_CANCELLED_DISPLAY_LINE1,
							RESTORE_CANCELLED_DISPLAY_LINE2);
					state->action = REPAIR_ASK_FACTORY;
					state->level = INIT_LEVEL;
					sleep(1);
					break;
				case OUT_LEVEL:
					Cls();
					ShowMessage(RESTORE_RESTORING_DISPLAY_LINE1,
							RESTORE_RESTORING_DISPLAY_LINE2);
					system("/usr/share/edenwall-backup/scripts/ask_factory.sh");
					sleep(1);
					system("shutdown -r now");
					break;
				default:
					alarm(seconds_to_wait);
					break;
			}
			break;
		case REPAIR_ASK_OPEN:
			if (state->level == ENTER_LEVEL) {
				alarm(0);
				open_access();
			}
			else alarm(seconds_to_wait);
			break;
		case REPAIR_ASK_DIAG:
			if (state->level == ENTER_LEVEL) {
				alarm(0);
				send_diag_to_support();
			}
			else alarm(seconds_to_wait);
			break;
		case REPAIR_ASK_VPN:
			if (state->level == ENTER_LEVEL) {
				alarm(0);
				start_vpn_support();
			}
			else alarm(seconds_to_wait);
			break;
		case ASK_REBOOT:
			switch (state->level){
				case CANCEL_LEVEL:
					alarm(0);  /* Cancels the signal. */
					Cls(); 
					ShowMessage (REBOOT_CANCELLED_DISPLAY_LINE1 ,REBOOT_CANCELLED_DISPLAY_LINE2);
					state->action = ASK_INIT;
					state->level = INIT_LEVEL;
					sleep(1);
					break;
				case OUT_LEVEL:
					Cls();
					ShowMessage(REBOOT_REBOOTING_DISPLAY_LINE1,
							REBOOT_REBOOTING_DISPLAY_LINE2);
					sleep(1);
					system("shutdown -r now");
					break;
				default:
					alarm(seconds_to_wait);
					break;
			}
			break;
		case ASK_HALT:
			switch (state->level){
				case CANCEL_LEVEL:
					alarm(0);  /* Cancels the signal. */
					Cls(); 
					ShowMessage (HALT_CANCELLED_DISPLAY_LINE1 ,HALT_CANCELLED_DISPLAY_LINE2);
					state->action = ASK_INIT;
					state->level = INIT_LEVEL;
					sleep(1);
					break;
				case OUT_LEVEL:
					Cls();
					ShowMessage(HALT_HALTING_DISPLAY_LINE1,
							HALT_HALTING_DISPLAY_LINE2);
					sleep(1);
					system("shutdown -h now");
					break;
				default:
					alarm(seconds_to_wait);
					break;
			}
			break;
		case ASK_ID:
			alarm(60);
			break;
		case ASK_HA_STATUS:
		case ASK_LOAD:
		case ASK_NB:
			alarm(WAIT_DEFAULT);
			break;
		default:
			alarm(0);
			break;
	}

	return 1;
}

#ifdef DEBUG
void print_hex(int res, const char *s) {
	FILE *log_file;
	int i;

	log_file = fopen("/var/tmp/ezio_keys.log", "a");
	if (log_file == NULL)
		return;
	fprintf(log_file, "(%d:", res);
	for (i = 0; s[i]; i++)
		fprintf(log_file, " %x", s[i]);
	fprintf(log_file, ")\n");
	fclose(log_file);
}
#endif

int serial_daemon()
{
	state_t p_state;
	state.action=ASK_INIT;
	state.level=INIT_LEVEL;

	Cls(); /* Clear screen */
	display_state(&state);
	signal(SIGALRM, sig_handler);
	while (1) {
		int res;
		char buf[255];
		p_state.action=state.action;
		p_state.level=state.level;
		SetDis();
		ReadKey(); /* sub-routine to send "read key" command */
		res = read(fd,buf,255); /* read response from EZIO */
#ifdef DEBUG
		print_hex(res, buf);
#endif
		switch_state(&state,buf[1]);
		if ( (p_state.action != state.action) || ((p_state.level != state.level) && (p_state.action == state.action)) ){
			execute_state(&state);
			display_state(&state);
		}
		usleep(1000);
	}
}

int main (int argc, char* argv[]) 
{
	FILE *errors_f = NULL, *model_f = NULL;
	int daemon=0;
	char title[17]="                ";
	char message[17]="                ";
	int ch, ret;
	unsigned long int error_mask = 0, tmp;

	SetEnvironment();       /* Set RAW mode */
	fd = open("/dev/ttyS1" ,O_RDWR);/** Open Serial port (COM2) */
	Init(); /* Initialize EZIO twice */
	Init();
	while ((ch = getopt (argc, argv, "Dt:m:w:")) != -1) {
		switch (ch) {
			case 't':
				strncpy(title,optarg,16);
				title[16]=0;
				break;
			case 'm':
				strncpy(message,optarg,16);
				message[16]=0;
				break;
			case 'w':
				/* Warning: The LCD will display WAIT_DEFAULT and not
				 * seconds_to_wait. */
				ret = sscanf(optarg, "%d", &seconds_to_wait);
				if (ret != 1 || seconds_to_wait > WAIT_MAX ||
						seconds_to_wait < 0)
					seconds_to_wait = WAIT_DEFAULT;
				break;
			case 'D':
				daemon = 1;
				break;
			default:
				printf("edenserial -D -t title -m message -w seconds\n");
		}
	}
	if (daemon){
		/* Fetch errors to display if there are any. */
		errors_f = fopen(ERRORS_FILENAME, "r");
		if (errors_f != NULL) {
			while ((fscanf(errors_f, "%lu", &tmp) == 1)) {
				error_mask |= tmp;  /* Tolerant to duplicates. */
			}
			fclose(errors_f);
			/* To be sure, ignore the file if there were no numbers in it. */
			if (error_mask != 0) {
				/* An 8-digit decimal number allows for 29 errors. */
				snprintf(init_display_line1, 16, "Error #%lu", error_mask);
				strcpy(init_display_line2, " See manual");
			}
		}
		unlink(ERRORS_FILENAME);

		/* Model: */
		model_f = fopen(MODEL_FILENAME, "r");
		if (model_f != NULL) {
		    fscanf(model_f, "%16s", init_display_line2);
		}

		/* Identifier: */
		get_and_set_id();

		serial_daemon();
	} else {
		Cls();
		ShowMessage (title,message);
	}
	return 1;
}

