from edenwall_snmp import SNMPTreeNode
from edenwall_snmp.log import logData

import string
import time

class NuauthNode(SNMPTreeNode):

    def get_command_result(self, oid, type, command):
        global client
        if has_nuauth and client:
            ret = client.execute(command)
            if ret.ok:
                logData("command returned %s\n" % ret.content)
                pass
            else:
                logData("Erreur while executing command %s\n", command);
            return [oid, type, ret.content]
        return None

class NuauthVersionNode(NuauthNode):

    def get_value(self, oid):
        return self.get_command_result(oid, "string", "version")

class NuauthUptimeNode(NuauthNode):

    def get_value(self, oid):
        return self.get_command_result(oid, "string", "uptime")

class NuauthUsersCountNode(NuauthNode):

    def get_value(self, oid):
        return self.get_command_result(oid, "integer", "user count")

    def getnextoid(self, oid, oid_list, current_oid=".1"):
        """ Override getnextoid to return first element of User Table (and
            not oid of the table itself."""
        my_list = current_oid.split(".")
        # convert last arg to integer and add 1
        my_list[-1] = str( int(my_list[-1]) + 1 )
        # then pack list to string
        current_oid = string.join(my_list, '.')
        ret = "%s.%d.%d.%d" % (current_oid,1,1,1)
        return ret

class NuauthUserTableNode(NuauthNode):

    TABLE_ENTRY_SIZE = 2
    ttl = 10

    def __init__(self, name, index):
        self.last_req = 0
        NuauthNode.__init__(self, name, index)

    def refresh_cache(self, oid):
        now = int(time.time())
        if (now - self.last_req) > self.ttl:
            #logData("refreshing cache for users\n")
            ret = self.get_command_result(oid, "integer", "users")
            self.users = ret[2]
            self.last_req = int(time.time())
        #logData("users: %s\n" % repr(self.users))

    def row_count(self):
        return len(self.users)

    def get(self, oid, oid_list):
        #logData("GET oid_list: %s\n" % oid_list)
        self.refresh_cache(oid)
        if oid_list[0] != self.index:
            # not for me !
            return None
        # are we a leaf ?
        if len(oid_list) == 1:
            return None
        # are we receiving a GET on the table itself ?
        if len(oid_list) <= 3:
            return None
        if len(oid_list) >= 5:
            return None
        if oid_list[1] == 0:
            return None
        user_val = int(oid_list[2])
        user_index = int(oid_list[3])
        ret = None
        type = None
        if user_index > self.row_count():
            return None
        if user_val == 1:
            #logData("user index: %d: %s\n" % (user_index,self.users[user_index-1].socket))
            ret = self.users[user_index-1].socket
            type = "integer"
        elif user_val == 2:
            #logData("user: %s\n" % self.users[user_index-1])
            #logData("user name: %s\n" % self.users[user_index-1].name)
            ret = self.users[user_index-1].name
            type = "string"
        #logData("GET oid_list: %s\n" % oid_list)
        return [oid, type, ret]

    def getnextoid(self, oid, oid_list, current_oid=".1"):
        #logData("GETNEXT oid_list: %s\n\tcurrent_oid %s\n" % (oid_list,current_oid))
        self.refresh_cache(oid)
        # getnext on the table ? return the first entry
        if len(oid_list) <= 1:
            #logData( "getnext returning %s.%d.%d.%d\n" % (current_oid,1,1,1) )
            return "%s.%d.%d.%d" % (current_oid,1,1,1)
        if len(oid_list) == 2:
            return None
        # getnext on a table entry: return the first element
        if len(oid_list) == 3:
            if oid_list[2] < self.TABLE_ENTRY_SIZE:
                return "%s.1.%d.%d" % (current_oid,oid_list[2],1)
            return None
        # getnext on a table entry element: try next element, or next entry first element
        if len(oid_list) == 4:
            if oid_list[3] < self.row_count():
                return "%s.1.%d.%d" % (current_oid,oid_list[2],oid_list[3]+1)
            if oid_list[2] < self.TABLE_ENTRY_SIZE:
                return "%s.1.%d.%d" % (current_oid,oid_list[2]+1,1)
        return None

    def get_value(self, oid):
        # split oid: .1.4.6.[index].[value]
        oidList = oid.split(".")
        logData("oid: %s\n" % oid)
        logData("   index : %s\n" % oidList[3])
        logData("   value : %s\n" % oidList[4])
        return self.users

# can I haz nuauth plz ?
has_nuauth = False
client = None
try:
    from sys import path
    path.append("/home/pollux/prefix-trunk/lib/python2.5/site-packages")
    from nuauth_command.cmdline import CommandLineClient
    from nuauth_command import NuauthError
    has_nuauth = True
except Exception, e:
    logData("Could not import nuauth: %s\n" % e)

def connect_nuauth(path="/var/run/nuauth/nuauth-command.socket"):
    global client
    try:
        client = CommandLineClient(path)
        client.connect()
    except NuauthError, err:
        logData("nuauth: %s\n" % err)
        client = None

def populate_snmp_tree(ewTree):
    global client, has_nuauth
    nufw = SNMPTreeNode("nufw",4)
    ewTree.addChild(nufw)
    nufw.addChild( NuauthVersionNode("version",1) )
    nufw.addChild( NuauthUptimeNode("uptime",2) )
    nufw.addChild( NuauthUsersCountNode("users_count",5) )
    nufw.addChild( NuauthUserTableNode("user_table",6) )

    if has_nuauth:
        connect_nuauth("/home/pollux/prefix-trunk/var/run/nuauth/nuauth-command.socket")

