from __future__ import with_statement

from PyQt4.QtGui import QMainWindow, QFileDialog, QTreeWidgetItem, QFont
from PyQt4.QtCore import SIGNAL

import copy
import tarfile
import os

from ui.ewd_mainwindow_ui import Ui_MainWindow

def extract(filename, dest="."):
    """
    Extracts a tarball into the current directory, and returns the directory
    name where the files were extracted.
    """
    print "Extracting %s..." % filename
    tar = tarfile.open(filename, "r:gz")
    dirname = tar.getnames()[0].rstrip("/")

    if hasattr(tar, "extractall"):
        tar.extractall(dest)
    else:
        # Simulate extractall

        directories = []

        for tarinfo in tar:
            if tarinfo.isdir():
                directories.append(tarinfo)
                tarinfo = copy.copy(tarinfo)
                tarinfo.mode = 0700

            tar.extract(tarinfo, dest)

        directories.sort(lambda a, b: cmp(a.name, b.name))
        directories.reverse()

        # Set correct owner, mtime and filemode on directories.
        for tarinfo in directories:
            dirpath = tarinfo.name

            try:
                tar.chown(tarinfo, dirpath)
                tar.utime(tarinfo, dirpath)
                tar.chmod(tarinfo, dirpath)
            except tarfile.ExtractError, e:
                print e

    tar.close()

    return os.path.join(dest,dirname)

class EWDiagFile():

    def __init__(self, dirname):
        self.dirname = dirname
        self.sections = [ ]
        self.sections_value = { }

        # list sections
        self.open_diag_1()
        self.open_diag_2()
        self.open_diag_last()

    def open_diag_1(self):
        d = os.path.join(self.dirname,"diag_1")
        l = os.listdir(d)
        subsections = { }
        for name in l:
            with open(os.path.join(d,name), "rb") as f:
                contents = f.read()
                subsections[name] = contents
        self.sections.append( "diag_1" )
        self.sections_value["diag_1"] = subsections

    def open_diag_2(self):
        d = os.path.join(self.dirname,"diag_2")
        l = os.listdir(d)
        subsections = { }
        for name in l:
            with open(os.path.join(d,name), "rb") as f:
                contents = f.read()
                subsections[name] = contents
        self.sections.append( "diag_2" )
        self.sections_value["diag_2"] = subsections

    def open_diag_last(self):
        encrypted_archive_name = os.path.join(self.dirname, "diag_last.tar.gz.gpg")
        if not os.path.exists(encrypted_archive_name):
            gpg_name = encrypted_archive_name + ".gpg"
            if not os.path.exists(gpg_name):
                print "Skip missing diag_last.tar.gz"
                return
            os.system("gpg -d %s/diag_last.tar.gz.gpg > %s/diag_last.tar.gz" % (self.dirname, self.dirname))
        f = extract("%s/diag_last.tar.gz" % self.dirname, self.dirname)
        d = f
        l = os.listdir(d)
        subsections = { }
        for name in l:
            with open( os.path.join(d,name), "rb") as f:
                contents = f.read()
                subsections[name] = contents
        self.sections.append( "diag_last" )
        self.sections_value["diag_last"] = subsections


class EW4DiagMainWindow(QMainWindow, Ui_MainWindow):

    def __init__(self, app, parent=None):
        QMainWindow.__init__(self,parent)
        self.filename = None
        self.parsedData = None

        self.setupUi(self)
        AndaleMono = QFont("Andale Mono",10)
        self.textBrowser.setCurrentFont(AndaleMono)

        self.connectSlots()

    def connectSlots(self):
        self.connect(self.action_Open, SIGNAL("triggered()"), self.onOpen)
        self.connect(self.treeWidget, SIGNAL("itemClicked (QTreeWidgetItem *,int)"), self.treeItemClicked)

    def onOpen(self):
        filename = QFileDialog.getOpenFileName(self, self.tr("Open File"),
                        "",
                        self.tr("Diagnostic files (*gz)"))
        if not filename:
            return

        self.openDiagnosticFile(filename)


    def openDiagnosticFile(self, filename):
        if self.filename:
            self.closeDiagnosticFile()

        directory = extract(str(filename))
        self.filename = filename
        self.extracted_filename = directory
        self.diag = EWDiagFile(directory)

        self.refreshDisplay()

    def closeDiagnosticFile(self):
        self.filename = None
        self.extracted_filename = None
        self.parsedData = None
        self.keys = None

    def refreshDisplay(self):
        #if not self.parsedData or not self.keys:
        #    return

        self.treeWidget.clear()
        self.textBrowser.clear()

        for section_name in sorted(self.diag.sections, reverse=True):
            w = QTreeWidgetItem(None, (section_name,))
            self.treeWidget.insertTopLevelItem(0, w)
            # commands + contents
            for subsection_name in sorted(self.diag.sections_value[section_name]):
                QTreeWidgetItem(w, (subsection_name,))

    def treeItemClicked(self, it, i):
        k = it.text(i)

        parent = it.parent()
        if not parent:
            return

        section = parent.text(i)
        contents = self.diag.sections_value[str(section)][str(k)]
        self.textBrowser.clear()
        self.textBrowser.setPlainText(contents)

