#!/usr/bin/python

"""
A GUI client to do requests to an NNDaemon

Copyright 2010 EdenWall Technologies

Written by Feth AREZKI <farezki@edenwall.com>
"""

from __future__ import with_statement

from datetime import datetime
from PyQt4.QtCore import QObject, QSettings, QThread, SIGNAL
from PyQt4.QtGui import QApplication, QDialog

from nnd.qt.client_ui import Ui_Clientui
from nnd.client import SOCKET_FILE, Client as BaseClient
from optparse import OptionParser
from sys import exit


def htmlcolor(color, text):
    return '<span style="color: %s;"> %s</span>' % (color, text)


class ClientThread(QThread):

    def __init__(self, target, parent=None):
        QThread.__init__(self, parent)
        self._target = target

    def run(self):
        self._target()


class Client(Ui_Clientui, QDialog, BaseClient):

    def __init__(self, parent=None):
        QDialog.__init__(self, parent)
        self.settings = QSettings("EdenWall", "NND Client", self)
        self._setupui()
        self.options = self.parseOptions()

        #if safe, we go to blocking mode.
        BaseClient.__init__(self, blocking=self.options.safe)

        self._setupsignals()
        #Not overriding connect because it is a reserved slot in Qt
        BaseClient.connect(self, self.options.socket)

    def parseOptions(self):
        parser = OptionParser(usage="%prog [options]")
        parser.add_option("--safe",
            help="Safe mode (avoid a crash)",
            action="store_true")
        parser.add_option("--socket",
            help="Socket filename (default: %s)" % SOCKET_FILE,
            type="str", default=SOCKET_FILE)
        options, arguments = parser.parse_args()
        if arguments:
            parser.print_help()
            exit(1)
        return options

    def _setuplogger(self, fnname, color):
        def logfn(message):
            self._addmsg(message, color)
        logfn.__name__ = fnname
        logfn.color = color
        setattr(self, fnname, logfn)

    def _setuploggers(self, logger=None):
        loggers = {
            "query": "blue",
            "info": "grey",
            "success": "green",
            "error": "red",
            "resulterror": "brown",
            }
        for word, color in loggers.iteritems():
            fnname = "%slog"% word
            self._setuplogger(fnname, color)

    def _mkthread(self, target):
        #Keeping reference to avoid thread being gc'd during exec
        self._thread = ClientThread(target)
        return self._thread

    def _addmsg(self, message, color):
        if message.startswith('200 '):
            count = 1 + message[4:].count(',')
            html = '200<br>' + message[4:].replace(',', '<br>')
            if 1 < count:
                html += '<br>(%s items)' % count
        else:
            html = message
        html = htmlcolor(color, html)
        date = datetime.now()
        html = html.replace('\n', '<br>')
        html = "%s&gt; %s\n" % (date, html)
        self.results.append(html)

    def _setupui(self):
        Ui_Clientui.setupUi(self, self)

    def _setupsignals(self):
        QObject.connect(self.domainsbutton, SIGNAL('clicked()'), self.domains)
        QObject.connect(self.groupsbutton, SIGNAL('clicked()'), self.searchgroups)
        QObject.connect(self.uidbutton, SIGNAL('clicked()'), self.searchuid)
        QObject.connect(self.pwbutton, SIGNAL('clicked()'), self.auth)
        QObject.connect(self.unbindbutton, SIGNAL('clicked()'), self.unbind)

    def searchuid(self):
        uid = self.uid.currentText()
        BaseClient.searchuid(self, uid)

    def auth(self):
        username = unicode(self.username.currentText())
        password = unicode(self.password.currentText())
        BaseClient.auth2(self, username, password)

    def closeEvent(self, event):
        event.accept()
        self.quit()


def main():
    import sys
    app = QApplication(sys.argv)
    dialog = Client()
    dialog.show()
    app.exec_()


if __name__ == '__main__':
    main()

