"""
Copyright 2010 EdenWall Technologies

Written by Pierre Chifflier <chifflier@edenwall.com>
Modified by Pierre-Louis Bonicoli <bonicoli@edenwall.com>
"""

from ConfigParser import SafeConfigParser, NoSectionError, NoOptionError

NOT_SET = object()

class NNDConfigParser(SafeConfigParser):
    def get(self, section, option, default=NOT_SET):
        try:
            return SafeConfigParser.get(self, section, option)
        except NoOptionError:
            if default is not NOT_SET:
                return default
            else:
                raise

    def getboolean(self, section, option, default=NOT_SET):
        try:
            return SafeConfigParser.getboolean(self, section, option)
        except NoOptionError:
            if default is not NOT_SET:
                return default
            else:
                raise

    def getint(self, section, option, default=NOT_SET):
        try:
            return SafeConfigParser.getint(self, section, option)
        except NoOptionError:
            if default is not NOT_SET:
                return default
            else:
                raise
        except ValueError:
            if SafeConfigParser.get(self, section, option) == "":
                #user intentionally did not specify option
                return default

    def rget(self, sections, option, default):
        for section in sections:
            try:
                return SafeConfigParser.get(self, section, option)
            except NoOptionError:
                continue
        return default

    def rgetboolean(self, sections, option, default):
        for section in sections:
            try:
                return SafeConfigParser.getboolean(self, section, option)
            except NoOptionError:
                continue
        return default

    def rgetint(self, sections, option, default):
        """
        Recursively looks up for value.
        Sections must be provided in most specific to global order
        """
        for section in sections:
            try:
                return SafeConfigParser.getint(self, section, option)
            except NoOptionError:
                continue
        return default


CONFIG_FILENAME = "/etc/nnd/nnd.conf"

_runningconfig = CONFIG_FILENAME

def setconfigfilename(filename):
    global _runningconfig
    _runningconfig = filename

def getconfigfilename():
    return _runningconfig

def load_config(filename=None):
    global _runningconfig
    if filename is None:
        filename = _runningconfig

    conf = NNDConfigParser()
    conf.read([filename, _runningconfig])

    try:
        version = conf.getint('global', 'version')
    except NoSectionError:
        raise IncompatibleVersion("Could not read a 'global' section in your configuration")

    if version != 1:
        raise IncompatibleVersion('Min. supported : %s, read : %s' % (1, version))

    return conf

class IncompatibleVersion(Exception):
    pass

