import ldap
from time import time

from .error import NNDError
from .ldap_server import BaseLDAPServer, ALLGROUPS
from .formatter import decode_item
from .server import updatetimes

class EDirectoryServer(BaseLDAPServer):
    def __init__(self, *args, **kwargs):
        BaseLDAPServer.__init__(self, *args, **kwargs)

    def _groups(self, calltime, timeleft):
        base_dn = self.domain.group_base_dn
        ldapfilter = self.domain.group_filter
        attrs = (self.domain.group_attr_name,)

        if self.use_pagination:
            func = self.timedpagedreq
        else:
            func = self.timedsyncreq

        result, timeleft, now = func(
            base_dn,
            ldap.SCOPE_SUBTREE,
            ldapfilter,
            attrs,
            calltime,
            timeleft
            )

        for item in result:
            yield self._get_group(item)

    def _usergroups(self, uid, calltime, timeleft):
        ldapfilter = self.domain.user_filter % uid
        self.debug("Search user %s groups: filter=%s" % (uid, ldapfilter))

        results, timeleft, now = self.timedsyncreq(
            self.domain.user_base_dn,
            ldap.SCOPE_SUBTREE,
            ldapfilter,
            [self.domain.group_member_attr],
            calltime,
            timeleft
            )
        if not results:
            return
        for groups in results[0][1].itervalues():
            for name in groups:
                name = decode_item(name)
                yield (name, self.domain.realm)

    def auth_user_dn(self, username, timeleft):
        ldapfilter = self.domain.user_filter % username
        self.debug("Authenticate user %s: filter=%s" % (username, ldapfilter))
        result, timeleft, now = self.timedsyncreq(
            self.domain.user_base_dn,
            ldap.SCOPE_SUBTREE,
            ldapfilter,
            (),
            time(),
            timeleft
            )

        if len(result) != 1:
            raise NNDError(
                "Invalid number (%s) of results for the user %s"
                % (len(result), username))

        return result[0][0], timeleft

