import ldap
from time import time

from .error import NNDError
from .formatter import decode_item, format_item
from .server import Server, ALLGROUPS

class BaseLDAPServer(Server):
    """LDAP implementation"""
    def __init__(self, *args, **kwargs):
        Server.__init__(self, *args, **kwargs)
        self._searchgroups_attrs = (self.domain.group_attr_name,)
        self._uidgroups_filter = "(&%s(%s=%%s))" % (
            self.domain.group_filter,
            self.domain.group_member_attr
        )

    def _uidfilter(self, uid):
        if uid == ALLGROUPS:
            return self.domain.group_filter

        return self._uidgroups_filter % uid

    def auth_user_dn(self, username, timeleft):
        ldapfilter = self.domain.user_filter % username
        result, timeleft, now = self.timedsyncreq(
            self.domain.user_base_dn,
            ldap.SCOPE_SUBTREE,
            ldapfilter,
            (),
            time(),
            timeleft
            )

        if len(result) != 1:
            raise NNDError(
                "Invalid number (%s) of results for the user %s"
                % (len(result), username))
        return result[0][0], timeleft

    def _get_group(self, entry):
        if self.domain.group_attr_name == "dn":
            name = entry[0]
        else:
            name = entry[1][self.domain.group_attr_name][0]
        name = decode_item(name)
        return (name, self.domain.realm)


class LDAPServer(BaseLDAPServer):
    def _search_groups(self, calltime, uid, timeleft):
        ldapfilter = self._uidfilter(uid)
        self.debug("ldapfilter: %s" % ldapfilter)

        results, timeleft, now = self.timedsyncreq(
            self.domain.group_base_dn,
            ldap.SCOPE_SUBTREE,
            ldapfilter,
            self._searchgroups_attrs,
            calltime,
            timeleft
            )

        for obj in results:
            yield self._get_group(obj)

    def _groups(self, calltime, timeleft):
        return self._search_groups(calltime, ALLGROUPS, timeleft)

    def _usergroups(self, uid, calltime, timeleft):
        self.debug("Searching for uid: %s" % uid)
        return self._search_groups(calltime, uid, timeleft)

