from sys import stdout
from logging import (
    getLogger, StreamHandler, getLevelName, Formatter,
    DEBUG, INFO, WARNING, ERROR, CRITICAL, FATAL
    )
from logging.handlers import RotatingFileHandler

methodsnames = 'debug info warning error critical fatal'.split()

def setmeupaslogger(instance, logger):
    for attrname in methodsnames:
        setattr(instance, attrname, getattr(logger, attrname))

class Filewrapper(object):
    def __init__(self, logger, level=DEBUG, prefix=""):
        self.prefix = prefix
        level = getLevelName(level)
        self.logmethod = getattr(logger, level.lower())

    def write(self, text):
        self.logmethod("%s %s" % (self.prefix, text.strip()))


_LOG_LEVELS = {
    "DEBUG": DEBUG,
    "INFO": INFO,
    "WARNING": WARNING,
    "ERROR": ERROR,
    "CRITICAL": CRITICAL,
    "FATAL": FATAL,
}


_LOGFORMATTER = Formatter("%(asctime)s %(levelname)-8s %(message)s")
MAX_LOG_FILESIZE = 5 * 1024 * 1024
MAX_LOG_FILES = 10   # including .log, so last file prefix is .log.9



def setuplog(log_domain):
    logger = getLogger(log_domain)
    stdouthandler = StreamHandler(stdout)
    logger.addHandler(stdouthandler)
    stdouthandler.setFormatter(_LOGFORMATTER)
    return logger, stdouthandler

def configlog(
        logger,
        stdouthandler,
        daemonize,
        log_file=None,
        log_level=None,
        log_to_file=False,
        log_to_stdout=True,
        ldap_level=0,
        ):

    logger.ldap_level = ldap_level

    handler = stdouthandler
    if daemonize:
        log_to_file = True
        log_to_stdout = False

    if log_to_file:
        try:
            #XXX: I can't seem to get MAX_LOG_FILESIZE to work :-(
#            handler = RotatingFileHandler(
#                log_file,
#                'a',
#                maxBytes=MAX_LOG_FILESIZE,
#                backupCount=(MAX_LOG_FILES - 1),
#                encoding='utf8'
#            )
            handler = RotatingFileHandler(
                log_file,
                backupCount=(MAX_LOG_FILES - 1),
                encoding='utf8'
            )
        except IOError, err:
            logger.fatal("FATAL! Unable to log! %s" % err)
            return False

        if not log_to_stdout:
            stdouthandler.setLevel(FATAL)
        handler.setFormatter(_LOGFORMATTER)
        logger.addHandler(handler)

    _configloglevel(logger, handler, log_level)
    return True


def _configloglevel(logger, handler, levelspec):
    log_level = _LOG_LEVELS.get(levelspec.upper())

    if not log_level:
        log_level = DEBUG
        logger.critical(
            "Log level ('%s') in conf file is invalid! "
            "Logging with level: DEBUG" % levelspec
            )
    handler.setLevel(log_level)
    logger.setLevel(log_level)

    logger.info("Log level: %s" % levelspec)

