from nnd.client.client import ServerError
from nnd.client import Client
from shutil import copy
from py import test
import logging
import random
import subprocess
from py import test
from time import sleep
from os import symlink, chmod
from os.path import exists, islink
from ConfigParser import SafeConfigParser, NoSectionError, NoOptionError

NNDCONF = "/etc/nnd/nnd.conf"


class NullHandler(logging.Handler):
    def emit(self, record):
       pass


class TemplateNND():
    nnd_client = None
    blocking = False
    servers = None
    force_block = False

    # abstract attributes
    conf_filename = None # str
    user = None # str
    domain = None # str
    password = None # str
    user_groups = None # list of unicode
    groups = None # list of unicode, if not set, use user_groups

    @classmethod
    def setup_class(cls):
        conf = SafeConfigParser()
        conf.read(cls.conf_filename)
        cls.socket = conf.get("global", "socket")

        # Save the running NND conf file
        copy(NNDCONF, "./nnd.conf.backup")

        # Copy our test NND conf file
        copy(cls.conf_filename, NNDCONF)
        chmod(NNDCONF, 644)

        # Declare client (not connecting yet)
        cls.nnd_client = Client()

        # Reload NND
        cls.reloadNND()

    @classmethod
    def reloadNND(cls, timeout=60):
        if not islink("/etc/runit/init.d/nufw-nss-daemon"):
            symlink("/usr/bin/sv", "/etc/runit/init.d/nufw-nss-daemon")
        if not islink("/etc/service/nufw-nss-daemon"):
            symlink("/etc/sv/nufw-nss-daemon", "/etc/service/nufw-nss-daemon")

        # /etc/init.d/nnd reload
        pid = cls.pidofNND()

        handle = subprocess.Popen(
            "/etc/init.d/nufw-nss-daemon stop",
            shell=True
        )
        # wait for nnd to restart !!
        handle = subprocess.Popen(
            "/etc/init.d/nufw-nss-daemon start",
            shell=True
        )

        restarted = False
        for iterations in xrange(timeout):
            if pid != cls.pidofNND():
                restarted = True
                timeout -= iterations
                break
            sleep(1)

        connected = False
        if restarted:
            for iterations in xrange(timeout):
                if cls.nnd_client.connect(cls.socket):
                    connected = True
                    break
                sleep(1)

        assert connected, "We waited %d secs but couldn't connect." % timeout

    @classmethod
    def pidofNND(cls):
        cmd = "ps aux|grep [n]ufw-nss-daemon|grep nuauth|awk '{print $2}'"
        handle = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
        pid = handle.stdout.read()
        return pid

    def testHello(self):
        self.activateIptablesRule()
        # Send our hellos and check NND is responding correctly
        ## In: "hello 2\n"
        results = self.nnd_client.connect(self.socket)

        ## Out : "200 2\n"
        error = "The answer from the server to our hello is incorrect. %s"
        assert "200 2" in results, error % results

    def testUsergroups(self):
        self.activateIptablesRule()
        # Check that the groups we get from NND are the ones expected
        print "The blocked server was :", self.findMatchingIptablesRule()
        group_list = self.nnd_client.searchuid(self.user)

        # Quickly redo the same request to test the cache
        cached = self.nnd_client.searchuid(self.user)
        assert group_list == cached, "%r != %r" % (group_list, cached)

        error = "The answer from the server to our search of groups for %r " + \
                "is incorrect :\n %r\n We were expecting :\n %r"
        for group in self.user_groups:
            assert group in group_list, error % (self.user, group_list,
                                                 group)

    def testGroups(self):
        self.activateIptablesRule()
        # Check that the groups we get from NND are the ones expected
        print "The blocked server was :", self.findMatchingIptablesRule()
        group_list = self.nnd_client.searchgroups()

        # Quickly redo the same request to test the cache
        cached = self.nnd_client.searchgroups()
        assert group_list == cached, "%r != %r" % (group_list, cached)

        expected_groups = self.groups
        if not expected_groups:
            expected_groups = self.user_groups

        error = "The answer from the server to our search all groups " + \
                "is incorrect :\n %r\n We were expecting :\n %r"
        for group in expected_groups:
            assert group in group_list, error % (group_list, group)

    def testAuth(self):
        if self.password:
            self.activateIptablesRule()
            # Try to authenticate our test user with NND
            print "The blocked server was :", self.findMatchingIptablesRule()
            try:
                results = self.nnd_client.auth(self.user, self.domain,
                                               self.password)
                failed = False
            except ServerError, err:
                failed = True

            error = "The authentication failed but it shouldn't have."
            assert not failed, error

            expected = "%s@%s" % (self.user, self.domain)
            error = "The answer from the server to our authentication of %s" + \
                    " using the password %s is incorrect :\n%s\nWe were " + \
                    "expecting :\n%s"
            assert expected in results, error % (self.user+"@"+self.domain,
                                                 self.password, results,
                                                 expected)
        else:
            test.skip("No password, no test.")

    def testFailAuth(self):
        if self.password:
            self.activateIptablesRule()
            # Try to authenticate our test user with the wrong password
            try:
                results = self.nnd_client.auth(self.user, self.domain,
                                               self.password+"coin")
                failed = False
            except ServerError, err:
                failed = True

            if self.blocking:
                print "The blocked server was:", self.findMatchingIptablesRule()
            error = "The authentication of our %s using the password %s " + \
                    "didn't fail as it should have."
            assert failed, error % (self.user+"@"+self.domain,
                                    self.password+"coin")
        else:
            test.skip("No password, no test.")

    def activateIptablesRule(self):
        if self.blocking:
            self.deactivateIptablesRule()
            if self.force_block:
                to_reject = self.force_block
            else:
                to_reject = random.choice(self.servers)
            cmd = "/sbin/iptables -I OUTPUT -d %s -j REJECT"
            handle = subprocess.Popen(
                cmd % to_reject,
                shell=True
            )

    def deactivateIptablesRule(self):
        to_allow = self.findMatchingIptablesRule()
        while to_allow:
            cmd = "/sbin/iptables -D OUTPUT -d %s -j REJECT"
            handle = subprocess.Popen(
                cmd % to_allow,
                shell=True
            )
            to_allow = self.findMatchingIptablesRule()

    def findMatchingIptablesRule(self):
        if self.blocking:
            cmd = "iptables -nvL OUTPUT"
            handle = subprocess.Popen(
                cmd,
                shell=True,
                stdout=subprocess.PIPE
            )
            results = handle.stdout.read()
            for server in self.servers:
                if server in results:
                    return server
            return None

    @classmethod
    def teardown_class(cls):
        # Copy back the previous running NND conf file
        copy("./nnd.conf.backup", NNDCONF)

        if cls.blocking:
            for server in cls.servers:
                cmd = "/sbin/iptables -D OUTPUT -d %s -j REJECT"
                handle = subprocess.Popen(
                    cmd % server,
                    shell=True
                )
