# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from PyQt4.QtGui import QMessageBox
from PyQt4.QtCore import SIGNAL

from nucentral.common import tr
from nucentral_admin.acls import AclTab

class AclMultisiteTab(AclTab):
    def __init__(self, client, window):
        self.hosts = []
        AclTab.__init__(self, client, window)
        self.client = client

        # Hide the host list as its only used on multisite permissions
        self.ui.hostLabel.show()
        self.ui.hostList.show()

        self.connect(self.ui.hostList, SIGNAL('currentIndexChanged(int)'), self.hostChanged)

        self.resetHosts()
        self.resetAcls()

    def resetHosts(self):
        current = self.getCurrentHost()

        self.disconnect(self.ui.hostList, SIGNAL('currentIndexChanged(int)'), self.hostChanged)
        self.hosts = self.client.call('multisite_master', "listHostsGroups")

        # Set the host combo
        self.ui.hostList.clear()
        for host in self.hosts.iterkeys():
            self.ui.hostList.addItem(host)

        if current and current in self.hosts and \
        self.ui.hostList.findText(current) != -1:
            # set back the previous entry
            index = self.ui.hostList.findText(current)
            self.ui.hostList.setCurrentIndex(index)
        else:
            self.resetAcls()

        self.connect(self.ui.hostList, SIGNAL('currentIndexChanged(int)'), self.hostChanged)

        ## Disable everything when there is no host:
        state = (len(self.hosts) != 0)
        self.ui.hostLabel.setEnabled(state)
        self.ui.hostList.setEnabled(state)
        self.ui.groupLabel.setEnabled(state)
        self.ui.groupList.setEnabled(state)
        self.ui.removeGroupButton.setEnabled(state)
        self.ui.addGroupButton.setEnabled(state)
        self.ui.groupEdit.setEnabled(state)
        self.ui.tablePermissions.setEnabled(state)

    def resetAcls(self):
        AclTab.resetAcls(self)
        state = (len(self.hosts) != 0)
        self.ui.removeGroupButton.setEnabled(state)

    def getCurrentHost(self):
        return unicode(self.ui.hostList.currentText())

    def hostChanged(self, index):
        if self.ui.applyButton.isEnabled() and \
            QMessageBox.warning(self, tr("Save your changes"),
                                      tr("Do you want to saves current changes about the '%s' group?") % self.group,
                                      QMessageBox.Yes|QMessageBox.No) == QMessageBox.Yes:
            self.applyAcls()
        self.resetAcls()

    def getAcl(self, group='', role=''):
        if self.getCurrentHost() in self.hosts:
            return self.client.call('multisite_master', 'getHostsGroupAcl', group, role, self.getCurrentHost())
        return []

    def setAcl(self, group, name):
        self.client.call('multisite_master', 'setHostsGroupAcl', self.getCurrentHost(), group, name)

    def deleteAcl(self, group, name):
        self.client.call('multisite_master', 'deleteHostsGroupAcl', self.getCurrentHost(), group, name)

    def applyAcls(self):
        AclTab.applyAcls(self)
        group = unicode(self.ui.groupList.currentText())
        if group:
            self.client.call('multisite_master', 'refreshAcls', group, self.getCurrentHost())
        self.setModified(False)

    def listAllRoles(self):
        roles = {}

        RW_ROLES = ['nuauth_command', 'nuconf', 'nuconntrack', 'nuface', 'nulog', 'nupki', 'nurestore']
        RW_MULTISITE_ROLES = ['multisite']

        ALL_ROLES = ['nucentral_admin']
        for role in RW_ROLES:
            for type in ['_write', '_read']:
                ALL_ROLES += [role + type]

        ALL_MULTISITE_ROLES = ALL_ROLES + ['multisite_admin']
        for role in RW_MULTISITE_ROLES:
            for type in ['_write', '_read']:
                ALL_MULTISITE_ROLES += [role + type]

        # Take the full list of services
        for role in ALL_MULTISITE_ROLES:
            roles[role] = []

        # Retrieve the depencies we have informations about:
        for role, deps in self.client.call('acl', 'listAllRoles').iteritems():
            roles[role] = deps

        # Some dependencies are still missing ... see #
        return roles
