# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""
from __future__ import with_statement
from copy import deepcopy
from PyQt4.QtCore import Qt, SIGNAL
from PyQt4.QtGui import QMessageBox

from nucentral.common import tr
from nupki.qt.nupki_embed import PkiEmbedWidget

from certificates_ui import Ui_CertificatesTab

from .base_tab import BaseTab

PKI_MAGIC_ID = 'nucentral_certificate'

class CertificatesTab(BaseTab, Ui_CertificatesTab):
    ICON = ':/icons/certificat.png'

    WARNING_STR = tr("""You are about to force users to authenticate to EAS using a valid certificate.
You will not be able to connect to the configuration service if you have not created a user certificate yet.
Should any problem occur, you can disable this verification through the Edenwall console interface. For more information, see the System and Network Administrator Guide.
Are you sure you want to enable certificate authentication?""")

    def __init__(self, client, window):
        BaseTab.__init__(self, window)
        Ui_CertificatesTab.__init__(self)
        self.setupUi(self)

        self.client = client
        self.window = window

        self.pki_widget = PkiEmbedWidget(self.client, self, 'CORE', PkiEmbedWidget.SHOW_ALL | PkiEmbedWidget.CRL_OPTIONAL, self.setModified)

        self.verticalLayout.addWidget(self.pki_widget)
        self.verticalLayout.addStretch()

        self.onReset()

        self.apply.connect(self.apply, SIGNAL("clicked()"), self.onApply)
        self.reset.connect(self.reset, SIGNAL("clicked()"), self.onReset)

        self.require_cert.connect(self.require_cert, SIGNAL('stateChanged(int)'), self.requireCertConfirmation)

    def default_config(self):
        return deepcopy({
            'check_clients' : False,
            'use_nupki' : True,
            'nupki_pki' : None,
            'nupki_cert' : None,
        })

    def onReset(self):
        self.config = self.default_config()

        server_config = self.client.call('CORE', 'getSSLConfig')
        self.config.update(server_config)

        self.require_cert.setChecked(self.config['check_clients'])

        self.pki_widget.setConfig(self.config)
        self.setModified(False)

    def onApply(self):
        validity = self.pki_widget.validate()
        if validity is not None:
            QMessageBox.critical(self,
                tr('Certificate configuration'),
                tr('The certificate configuration is invalid:\n%s') % validity)
            return

        self.config = self.default_config()
        self.config.update( self.pki_widget.getConfig() )
        self.config['check_clients'] = (self.require_cert.checkState() == Qt.Checked)
        self.client.call('CORE', 'setSSLConfig', self.config)
        self.setModified(False)
        QMessageBox.information(self,
            tr('Configuration saved'),
            tr('To apply the new certificate configuration, you must restart the configuration service. '
               'Use System > Tools > Restart the configuration server: [Restart].'))
        self.pki_widget.feed()

    def setModified(self, modified = True):
        self.apply.setEnabled(modified)
        self.reset.setEnabled(modified)
        self.window.setCentralModified(modified)

    def showError(self, e):
        QMessageBox.critical(None, tr('Configuration error'), e)

    def requireCertConfirmation(self, state):
        if state == Qt.Checked:
            if QMessageBox.warning(None, tr('Warning'), self.WARNING_STR, QMessageBox.Yes | QMessageBox.No,  QMessageBox.No) == QMessageBox.No:
               self.require_cert.setChecked(False)
               return
        self.setModified(True)

