# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from PyQt4.QtCore import SIGNAL
from PyQt4.QtGui import QLineEdit

#from nucentral.client.error import NuCentralError
from nucentral_admin.console_access_ui import Ui_ConsoleAccessTab

from .base_tab import BaseTab

_component = "console_access"

class ConsoleAccessTab(BaseTab):
    ICON = ':/icons/edit.png'

    def __init__(self, client, window):
        BaseTab.__init__(self, window)

        self.ui = Ui_ConsoleAccessTab()
        self.ui.setupUi(self)
        self.ui.passwordLineEdit.setEchoMode(QLineEdit.Password)
        self.ui.confirmPasswordLineEdit.setEchoMode(QLineEdit.Password)

        self.window = window
        self.client = client

        self.reset()

        self.connect(self.ui.passwordLineEdit, SIGNAL('textChanged(QString)'),
                     self.text_changed)
        self.connect(self.ui.confirmPasswordLineEdit,
                     SIGNAL('textChanged(QString)'), self.text_changed)
        self.connect(self.ui.applyButton, SIGNAL('clicked()'), self.apply)
        self.connect(self.ui.cancelButton, SIGNAL('clicked()'), self.reset)

    def text_changed(self, text):
        self.setModified(True)

    def fetchConfig(self):
        self.conf = self.client.call(_component, "getConsoleAccessConfig")

    def sendConfig(self):
        self.conf = {"enabled": True,
                     "password": unicode(self.ui.passwordLineEdit.text()),
                     "confirm password":
                         unicode(self.ui.confirmPasswordLineEdit.text())}
        self.client.call(_component, "setConsoleAccessConfig", self.conf)

    def setModified(self, modified):
        self.ui.applyButton.setEnabled(modified)
        self.ui.cancelButton.setEnabled(modified)
        # This forgets modifications done in other tabs:
        self.window.setCentralModified(modified)

    def apply(self):
        # There is no save button, so we send the config now.
        self.sendConfig()
        self.client.call(_component, "applyConsoleAccessConfig")
        self.setModified(False)

    def reset(self):
        self.fetchConfig()

        self.ui.passwordLineEdit.clear()
        self.ui.confirmPasswordLineEdit.clear()

        self.setModified(False)
