# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from PyQt4.QtGui import QWidget, QHeaderView, QListWidgetItem, QMessageBox, QInputDialog, QLineEdit
from PyQt4.QtCore import Qt, SIGNAL, QVariant, QRegExp

from nucentral_admin.edit_user_ui import Ui_EditUserDialog
from nucentral.common import tr
from nucentral.client import NuCentralError
from nucentral.qt.central_dialog import CentralDialog

ASCII7_REGEXP = QRegExp(ur"^[\x00-\x7f]+$")

class EditUserDialog(CentralDialog):

    def __init__(self, client, parent=None):
        CentralDialog.__init__(self, parent)

        self.ui = Ui_EditUserDialog()
        self.ui.setupUi(self)

        self.client = client

        self.connect(self.ui.groupAddButton, SIGNAL("clicked()"), self._cb_add_group)
        self.connect(self.ui.method, SIGNAL('currentIndexChanged(int)'), self._cb_check_external_method)
        self.connectButtons(self.ui.buttonBox)
        self.setRegExpValidator(self.ui.username, ASCII7_REGEXP)
        self.setRegExpValidator(self.ui.password, QRegExp(ur"^.{6,}$"))

    def save(self):
        return True

    def _cb_add_group(self):
        if not self.ui.groupEdit.text():
            return

        group = unicode(self.ui.groupEdit.text())

        if self.ui.groupsList.findItems(group, Qt.MatchExactly|Qt.MatchCaseSensitive):
            raise Exception(unicode(self.tr('The %s group already exists.')) % group)

        item = QListWidgetItem(group)
        item.setFlags(Qt.ItemIsUserCheckable|Qt.ItemIsEnabled)
        item.setCheckState(Qt.Checked)
        self.ui.groupsList.addItem(item)
        self.client.call('auth', 'addGroup', group)

        self.ui.groupEdit.clear()

    def _cb_change_method(self, i):
        if unicode(self.ui.password.text()) == "******":
            self.ui.password.clear()

    def _cb_check_external_method(self, i):
        text = unicode(self.ui.method.itemText(i))
        if text == "EXTERNAL":
            self.ui.password.setEnabled(False)
        else:
            self.ui.password.setEnabled(True)

    def password_confirmation(self, password):
        while 1:
            confirm, ok = QInputDialog.getText(self, self.tr("Password confirmation"),
                                               self.tr("Please confirm the new password for this user:"),
                                               QLineEdit.Password)
            if not ok:
                return False
            if password == unicode(confirm):
                return True

            QMessageBox.critical(self, self.tr("Wrong password"),
                                       self.tr("Password mismatch."))

    def editUser(self, username):
        try:
            method, groups = self.client.call('auth', 'getUser', username)
        except NuCentralError, err:
            print err
            pass
        else:
            self.setWindowTitle(tr("Edit user %s") % username)
            self.ui.username.setText(username)
            self.ui.username.setEnabled(False)
            self.ui.password.setText("******")
            i = self.ui.method.findText(method, Qt.MatchExactly) # case unsensitive
            if i >= 0:
                self.ui.method.setCurrentIndex(i)
            self.loadGroups(groups)
            self.connect(self.ui.method, SIGNAL('currentIndexChanged(int)'), self._cb_change_method)

            while 1:
                if self.execLoop():
                    method = unicode(self.ui.method.itemText(self.ui.method.currentIndex()))
                    password = unicode(self.ui.password.text())
                    if method == "EXTERNAL":
                        password = '!'
                    elif password == "******":
                        password = ''
                    elif not self.password_confirmation(password):
                        continue

                    groups = self.getSelectedGroups()
                    if username == self.client.login:
                        found = False
                        for group in groups:
                            acls = self.client.call('acl', 'getAcl', group)
                            if 'nucentral_admin' in (r for i,g,r in acls):
                                found = True
                                break
                        if not found and \
                           QMessageBox.warning(self, tr("Warning"),
                                               tr("You are about to deny yourself Administrator rights. Are you sure?"),
                                               QMessageBox.Yes|QMessageBox.No) != QMessageBox.Yes:
                            return False
                    self.client.call('auth', 'editUser', username, method, password, groups)
                    return True
                else:
                    return False

    def addUser(self):
        self.setWindowTitle(tr("Add a User"))
        self.loadGroups()
        while 1:
            if self.execLoop():
                username = unicode(self.ui.username.text())
                password = unicode(self.ui.password.text())
                method = unicode(self.ui.method.itemText(self.ui.method.currentIndex()))
                groups = self.getSelectedGroups()
                if method == "EXTERNAL":
                    password = "!"
                elif not self.password_confirmation(password):
                    continue

                self.client.call('auth', 'addUser', username, method, password, groups)
                return True
            return False

    def loadGroups(self, groups=[]):
        all_groups = self.client.call('auth', 'listGroups')
        for group in all_groups:
            item = QListWidgetItem(group)
            item.setFlags(Qt.ItemIsUserCheckable|Qt.ItemIsEnabled)
            if group in groups:
                item.setCheckState(Qt.Checked)
            else:
                item.setCheckState(Qt.Unchecked)

            self.ui.groupsList.addItem(item)

    def getSelectedGroups(self):
        groups = []
        for row in xrange(self.ui.groupsList.count()):
            item = self.ui.groupsList.item(row)
            if item.checkState() == Qt.Checked:
                groups.append(unicode(item.text()))
        return groups

