# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from PyQt4.QtCore import Qt, SIGNAL, QRegExp
from PyQt4.QtGui import QMessageBox

from nucentral.qt.central_dialog import CentralDialog
from nucentral.common import tr

from nucentral_admin.hosts_edit_ui import Ui_HostEdit

class HostEditDialog(CentralDialog):
    def __init__(self, client, existing_groups=[], group_name='', hosts=''):
        CentralDialog.__init__(self, None)
        self.client = client

        self.ui = Ui_HostEdit()
        self.ui.setupUi(self)

        self.connect(self.ui.addHost, SIGNAL('clicked()'), self.addHost)
        self.connect(self.ui.delHost, SIGNAL('clicked()'), self.delHost)
        self.connect(self.ui.allHosts, SIGNAL('itemDoubleClicked(QListWidgetItem*)'), self.addHost)
        self.connect(self.ui.groupHosts, SIGNAL('itemDoubleClicked(QListWidgetItem*)'), self.delHost)
        self.connect(self.ui.groupName, SIGNAL('textChanged(QString)'), self.setGroupName)
        self.connectButtons(self.ui.buttonBox)
        self.setRegExpValidator(self.ui.groupName, QRegExp(ur"^.+$"))

        self.group_name = group_name
        self.existing_groups = existing_groups

        # All hosts list
        lst = self.client.call('multisite_master', 'listFirewalls')
        self.all_hosts = [item[0] for item in lst]

        # My hosts list
        self.hosts = hosts
        self.display()

    def addHost(self, *arg):
        if self.ui.allHosts.currentItem() is None:
            return
        host = unicode(self.ui.allHosts.currentItem().text())
        self.hosts.append(host)
        #self.ui.groupHosts.addItem(host)
        # Doesn't work ??
        #item = self.ui.groupHosts.findItems(host, Qt.MatchExactly)[0]
        #self.ui.groupHosts.removeItemWidget(item)
        self.display()

    def delHost(self, *arg):
        if self.ui.groupHosts.currentItem() is None:
            return
        host = unicode(self.ui.groupHosts.currentItem().text())
        self.hosts.remove(host)
        # Doesn't work ??
        #item = self.ui.groupHosts.findItems(host, Qt.MatchExactly)[0]
        #self.ui.groupHosts.removeItemWidget(item)
        self.display()

    def display(self):
        self.ui.groupName.setText(self.group_name)

        self.ui.allHosts.clear()
        for host in self.all_hosts:
            if host not in self.hosts:
                self.ui.allHosts.addItem(host)

        self.ui.groupHosts.clear()
        for host in self.hosts:
            self.ui.groupHosts.addItem(host)

    def getGroupName(self):
        return self.group_name

    def getHosts(self):
        return self.hosts

    def setGroupName(self, string):
        self.group_name = unicode(self.ui.groupName.text())

    def save(self):
        if self.getGroupName() in self.existing_groups:
            QMessageBox.critical(self, tr('Invalid group name'), tr('A group with the name %s already exists') % self.getGroupName())
            return False
        return True
