# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from PyQt4.QtCore import Qt, SIGNAL
from PyQt4.QtGui import QTableWidgetItem, QMessageBox

from nucentral.common import tr

from nucentral_admin.hosts_group_ui import Ui_HostsGroupsTab
from nucentral_admin.hosts_edit import HostEditDialog

from .base_tab import BaseTab

class HostsGroupsTab(BaseTab):
    ICON = ':/icons/computeur.png'

    def __init__(self, client, window):
        BaseTab.__init__(self, window)
        self.client = client
        self.window = window

        self.ui = Ui_HostsGroupsTab()
        self.ui.setupUi(self)

        self.connect(self.ui.addGroup, SIGNAL('clicked()'), self.editGroup)
        self.connect(self.ui.delGroup, SIGNAL('clicked()'), self.delGroup)
        self.connect(self.ui.editGroup, SIGNAL('clicked()'), self.editSelectedGroup)
        self.connect(self.ui.hostsGroup, SIGNAL('cellDoubleClicked(int, int)'), self.editCell)
        self.connect(self.ui.reset, SIGNAL('clicked()'), self.reset)
        self.connect(self.ui.apply, SIGNAL('clicked()'), self.apply)

        self.reset()

    def refreshDelButton(self):
        state = (len(self.hosts_groups.keys()) != 0)
        self.ui.delGroup.setEnabled(state)

    def loadData(self):
        self.hosts_groups = self.client.call('multisite_master', 'listHostsGroups')

    def editCell(self, row, col):
        group = unicode(self.ui.hostsGroup.item(row, 0).text())
        self.editGroup(group)

    def editSelectedGroup(self):
        row = self.ui.hostsGroup.currentRow()
        if row != -1:
            self.editCell(row, 0)

    def editGroup(self, group=''):
        hosts_list = []
        if group:
            hosts_list = self.hosts_groups[group]

        existing_groups = self.hosts_groups.keys()
        if group:
            existing_groups.remove(group)
        dialog = HostEditDialog(self.client, existing_groups, group, hosts_list)

        if dialog.execLoop():
            if group and group != dialog.getGroupName():
                # The name changed delete the previouys entry before setting the new one
                del self.hosts_groups[group]

            group = dialog.getGroupName()

            self.hosts_groups[group] = dialog.getHosts()
            self.display()
            self.setModified(True)

    def delGroup(self):
        row = self.ui.hostsGroup.currentRow()
        if row == -1:
            return

        group = unicode(self.ui.hostsGroup.item(row, 0).text())

        if QMessageBox.warning(self, tr('Group deletion'), tr('You are about to delete the %s host group. Are you sure you want to proceed?') % group, QMessageBox.Yes | QMessageBox.No) == QMessageBox.No:
            return

        del self.hosts_groups[group]
        self.setModified(True)
        self.display()

    def display(self):
        self.ui.hostsGroup.clear()
        self.ui.hostsGroup.setColumnCount(2)
        self.ui.hostsGroup.setHorizontalHeaderLabels([tr('Group name'), tr('Firewalls in the group')])
        self.ui.hostsGroup.setRowCount(len(self.hosts_groups.keys()))
        self.refreshDelButton()

        for no, group_name in enumerate(self.hosts_groups.keys()):
            item = QTableWidgetItem(group_name)
            item.setFlags(item.flags() & ~ Qt.ItemIsEditable)
            self.ui.hostsGroup.setItem(no, 0, item)

            host_list = ', '.join(self.hosts_groups[group_name])
            item = QTableWidgetItem(host_list)
            item.setFlags(item.flags() & ~ Qt.ItemIsEditable)
            self.ui.hostsGroup.setItem(no, 1, item)

    def reset(self):
        self.hosts_groups = {}
        self.loadData()
        self.display()
        self.setModified(False)

    def apply(self):
        self.client.call('multisite_master', 'setHostsGroup', self.hosts_groups)
        self.emit(SIGNAL('host_groups_changed'))
        self.setModified(False)

    def setModified(self, modified):
        self.ui.reset.setEnabled(modified)
        self.ui.apply.setEnabled(modified)
        self.window.setCentralModified(modified)
