# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from logging import debug
from PyQt4.QtCore import Qt
from PyQt4.QtCore import SIGNAL
from PyQt4.QtGui import QMessageBox, QTabWidget, QWidget, QIcon

from nucentral_admin.acls import AclTab
from nucentral_admin.acls_multisite import AclMultisiteTab
from nucentral_admin.certificates import CertificatesTab
from nucentral_admin.console_access import ConsoleAccessTab
from nucentral_admin.hosts_group import HostsGroupsTab
from nucentral_admin.sessions import SessionsTab
from nucentral_admin.users import UsersTab
from nucentral.client import CLIENT_SUITE_SHORTNAME
from nucentral.common import tr
from nucentral.common.logger import Logger
from nucentral.common.multisite import MULTISITE_MASTER
from nucentral.common.version import EDENWALL
from nucentral.qt.central_window import CentralWindow
from nucentral.qt.central_window import STANDALONE


class MainWindow(CentralWindow):
    ROLES = set(('nucentral_admin',))
    ICON = ':/icons/users.png'

    TABS_DEF = {
                'session' : {
                            'name' : tr("Sessions"),        # Tab name
                            'role' : None,                  # Required role to enable the tab (the tab is disabled otherwise)
                            'component' : None,             # Required component to show the tab (the tab is hidden otherwise)
                            'class' : SessionsTab,          # Tab class
                            },
                'rights' :  {
                            'name' : tr("%(CLIENT_SUITE_SHORTNAME)s Groups") % {'CLIENT_SUITE_SHORTNAME': CLIENT_SUITE_SHORTNAME},
                            'role' : 'nucentral_admin',
                            'component' : 'auth',
                            'class' : AclTab,
                            },
                'users' :   {
                            'name' : tr("%(CLIENT_SUITE_SHORTNAME)s Users") % {'CLIENT_SUITE_SHORTNAME': CLIENT_SUITE_SHORTNAME},
                            'role' : 'nucentral_admin',
                            'component' : 'auth',
                            'class' : UsersTab,
                            },
                'multisite_rights' : {
                            'name' : tr("Remote rights"),
                            'role' : 'multisite_admin',
                            'component' : 'multisite_master',
                            'class' : AclMultisiteTab,
                            },
                'multisite_hosts' : {
                            'name' : tr("Host groups"),
                            'role' : 'multisite_admin',
                            'component' : 'multisite_master',
                            'class' : HostsGroupsTab
                            },
                'console_access' : {
                            'name' : tr("Console Access"),
                            'role' : 'nucentral_admin',
                            'component' : 'console_access',
                            'class' : ConsoleAccessTab,
                            },
                'certificates' : {
                            'name' : tr("Certificates"),
                            'role' : 'nucentral_admin',
                            'component' : None,
                            'class' : CertificatesTab,
                            }
            }
    TABS_ORDER = ['session', 'rights', 'users', 'multisite_rights', 'multisite_hosts', 'console_access', 'certificates']

    def __init__(self, app, client, standalone=STANDALONE, parent=None, eas_window=None):
        CentralWindow.__init__(self, client, parent, eas_window)
        self.logger = Logger("nucentral-admin")
        self.app = app
        self.client = client
        self.tabs = {}
        self.setupCentralWindow(app, standalone)
        title = tr('%s Users Management') % CLIENT_SUITE_SHORTNAME + u' [*]'
        self.setCentralWindowTitle(title, client)

        self.tabWidget = QTabWidget()
        self.setCentralWidget(self.tabWidget)

        self.EAS_MESSAGES = {
            'multisiteHostDeleted' : self.multisiteHostDeleted,
        }
        self.warn_console_access_default_password()  # May trigger a popup.

    def load(self):
        self.app.setOverrideCursor(Qt.WaitCursor)
        self.setStatus(self.tr("Loading Nucentral information"), 0)

        session = self.client.call('session', 'get')
        roles = session['user']['roles']
        components = self.client.call('CORE', 'getComponentList')

        for tab_id in self.TABS_ORDER:
            self.load_tab(tab_id, roles, components)

        if 'rights'in self.tabs.keys() and 'users' in self.tabs.keys():
            self.connect(self.tabs['users'], SIGNAL('groups_updated'), self.tabs['rights'].updateGroups)
            self.connect(self.tabs['rights'], SIGNAL('groups_updated'), self.tabs['users'].refreshList)

        if 'multisite_rights' in self.tabs.keys() and 'multisite_hosts' in self.tabs.keys():
            hosts = self.tabs['multisite_hosts']
            rights = self.tabs['multisite_rights']
            self.connect(hosts, SIGNAL('host_groups_changed'), rights.resetHosts)

        self.app.restoreOverrideCursor()
        self.setStatus(self.tr("Ready"))

    def load_tab(self, tab_id, roles, components):
        debug("Loading nuacentral_admin tab : %s" % tab_id)
        if self.TABS_DEF[tab_id]['component']:
            if self.TABS_DEF[tab_id]['component'] not in components:
                return

        if EDENWALL:
            enable_tab = (not self.TABS_DEF[tab_id]['role']) or self.TABS_DEF[tab_id]['role'] in roles
        else:
            enable_tab = True
        if enable_tab:
            klass = self.TABS_DEF[tab_id]['class']
            try:
                widget = klass(self.client, self)
                self.tabs[tab_id] = widget
            except Exception, err:
                self.logger.writeError(err, "Error while loading %s" % klass.__name__)
                enable_tab = False
        else:
            enable_tab = False

        if not enable_tab:
            widget = QWidget()

        index = self.tabWidget.addTab(widget, self.TABS_DEF[tab_id]['name'])
        self.tabWidget.setTabEnabled(index, enable_tab)
        if enable_tab and klass.ICON:
            icon = QIcon(klass.ICON)
            self.tabWidget.setTabIcon(index, icon)

    def multisiteHostDeleted(self, from_app):
        if self.multisite_type == MULTISITE_MASTER:
            self.hosts_group.reset()

    def isReadOnly(self):
        return False

    def warn_console_access_default_password(self):
        try:
            if not self.client.call("console_access",
                                    "consoleAccessHasDefaultPassword"):
                return True
        except Exception:
            return False

        title = tr("Please set the console access password")

        double_br = "<br/><br/>"

        message0 = tr("You can now use the console access account through ssh.")
        message1 = tr("The password is still the default password.")
        message2 = tr("Please <strong>change the password</strong> in the "
                      "%s Users Management tab, Console Access subtab.") % CLIENT_SUITE_SHORTNAME

        message = _html_span(
            message0, double_br, message1, double_br, message2
            )

        value = QMessageBox.information(self, title, message)
        return True

def _html_span(*texts):
    return "<span>%s</span>" % " ".join(texts)
