# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from datetime import datetime, timedelta

from PyQt4.QtGui import QTableWidgetItem, QAbstractItemView, QCheckBox, QHeaderView, QShortcut, QKeySequence
from PyQt4.QtCore import Qt, SIGNAL, QVariant

from nucentral.common.human import fuzzyTimedelta, fuzzyDatetime
from nucentral.qt.application_name import APPLICATION_NAME

from nucentral.client.error import NuCentralError
from nucentral.common.transport import parseDatetime
from nucentral_admin.session_ui import Ui_SessionsTab

from .base_tab import BaseTab

class SessionsTab(BaseTab):
    ICON = ':/icons/info.png'

    def __init__(self, client, window):
        BaseTab.__init__(self, window)

        self.ui = Ui_SessionsTab()
        self.ui.setupUi(self)

        self.client = client

        self.ui.sessionsTable.horizontalHeader().setResizeMode(QHeaderView.Stretch)

        self.connect(self.ui.pb_kill_sessions, SIGNAL("clicked()"), self._cb_kill_sessions)
        self.connect(self.ui.pb_refresh_sessions, SIGNAL("clicked()"), self._cb_refresh_sessions)

        self._cb_refresh_sessions()

    def _cb_kill_sessions(self):
        frags = 0
        for i in xrange(self.ui.sessionsTable.rowCount()):
            cb = self.ui.sessionsTable.cellWidget(i,0)
            if not cb.isChecked():
                continue
            item = self.ui.sessionsTable.item(i,1)
            cookie = item.data(Qt.UserRole).toString()
            cookie = unicode(cookie)
            if not cookie:
                continue
            self.client.call('session', 'destroySession', cookie)
            frags += 1
        if frags > 0:
            self._cb_refresh_sessions()

    def _cb_refresh_sessions(self):
        table = self.ui.sessionsTable
        my_cookie = self.client.getCookie()

        sessions = self.client.call('session', 'list')
        table.clearContents()
        table.setSortingEnabled(False)
        columns = [
            self.tr("Kill"),
            self.tr("User"),
            self.tr("Creation"),
            self.tr("Idle"),
            self.tr("Application"),
            ]
        table.setColumnCount(len(columns))
        table.setHorizontalHeaderLabels(columns)
        now = datetime.now()
        table.setRowCount(len(sessions))
        for row, s in enumerate(sessions):
            last_access = parseDatetime(s['last_access'])
            cookie = s.get('cookie', '')
            user = s['user']
            idle = timedelta(seconds=s['idle'])
            idle = fuzzyTimedelta(idle)
            idle = idle.split('.')[0]
            if 'login' in user:
                login = "%s" % user['login']
            else:
                login = "<anonymous user>"

            creation = parseDatetime(s['creation'])
            creation = fuzzyDatetime(creation)

            cb = QCheckBox()
            if not cookie:
                cb.setEnabled(False)
            elif my_cookie == cookie:
                cb.setEnabled(False)
                cb.setText(self.tr('Me'))
            table.setCellWidget(row, 0, cb)
            application = user['client_name']
            application = APPLICATION_NAME.get(application, application)
            if 'client_release' in user:
                application += ' (%s)' % user['client_release']
            columns = (
                u"%s@%s" % (login, user['host']),
                creation,
                idle,
                application,
            )
            for index, text in enumerate(columns):
                item = QTableWidgetItem(text)
                item.setData(Qt.UserRole, QVariant(cookie))
                table.setItem(row, 1 + index, item)

        table.horizontalHeader().setResizeMode(QHeaderView.Stretch)
        table.sortItems(3, Qt.DescendingOrder)
        table.setSortingEnabled(True)
        table.setEditTriggers(QAbstractItemView.NoEditTriggers)

