import psycopg2
from datetime import datetime
from os.path import join as path_join

from nucentral.backend import Component
from nucentral.backend.cron import scheduleOnce
from nucentral.core.audit import AuditEvent
from nucentral.core.context import Context
from nucentral.backend.variables_store import VariablesStore

class DatabaseManager(object):

    def __init__(self):
        self.conn = None
        self.cur = None
        self.conf = None

    def load(self, conf):
        if not self.conf:
            self.conf = conf

    def reset(self):
        self.conf = None

    def connect(self):
        try:
            self.conn = psycopg2.connect(database=self.conf['database'], host=self.conf['hostname'], user=self.conf['username'], password=self.conf['password'])
            self.cur = self.conn.cursor()
            return True
        except:
            print "I am unable to connect the database"
            return False

    def execute(self, query):
        try:
            self.cur.execute(query)
            if self.cur.rowcount > 0:
                return self.cur.fetchall()
            else:
                return []

        except:
            print "You are not connected to the database"

    def disconnect(self):
        self.cur.close()
        self.conn.close()


class Authfail(Component):

    NAME = "authfail"
    API_VERSION = 1
    VERSION = "1.0"
#    ROLES = {
#        'authfail_read' : frozenset((
#            "readAuthFail",
#            ))
#            }
    QUERY =  """SELECT ip_saddr, user_id, username, user_groups, os_sysname, os_release, os_version, reason, count(ip_saddr) FROM authfail
                WHERE time < CURRENT_TIMESTAMP
                AND time > (CURRENT_TIMESTAMP - interval '%s seconds')
                GROUP BY ip_saddr, user_id, username, user_groups, os_sysname, os_release, os_version, reason"""
    COLUMNS = {
            0 : "IP Source: ",
            1 : "User Id: ",
            2 : "Username: ",
            3 : "User Group: ",
            4 : "System: ",
            5 : "System Release: ",
            6 : "System Version: ",
            7 : "Reason: ",
            8 : "abortive attempt: "
              }

    def init(self, core):
        self.core = core
        self.delay = 60 # seconds
        self.database = DatabaseManager()
        self.conf_filename = path_join(core.var_dir, 'nulog.xml')
        self.conf = VariablesStore()

        if self.conf_filename:
            self.conf.load(self.conf_filename)
            self.database.load(self.conf)
        else:
            print "No database conf file...."

        scheduleOnce(self.delay, self.readAuthFail)

    def readAuthFail(self):
        context = Context.fromComponent(self)
        results = []
        if self.database.connect():
            query = self.QUERY % self.delay
            results = self.database.execute(query)
            results = self._clean(results)
            if results or len(results) > 0:
                self._createEvent(context, results)
        else:
            print "You are not connected to ulogd database"
        scheduleOnce(self.delay, self.readAuthFail)

    def _createEvent(self, context, data):
        message = ''
        user = ''
        ip_src = ''
        port_src = -1

        for row in data:
            if row[8] < 3:
                continue
            for index, col in enumerate(row):
                if len(unicode(col)) > 0:
                    message = 'Authentication failure on NuAuth: \nDetails:\n'
                    user = ''
                    message += ' - '
                    message += self.COLUMNS[index]
                    message += unicode(col)
                    message += "\n"
                    if "Username" in self.COLUMNS[index]:
                        user = col
                    if "IP Source" in self.COLUMNS[index]:
                        ip_src = col
                    if "Source Port" in self.COLUMNS[index]:
                        port_src = col

            event = AuditEvent.fromAuthFailure(context, message, eas=False)
            if len(user) > 0:
                event.user = user
            if len(ip_src) > 0:
                event.ip_src = ip_src
            if port_src != -1:
                event.port_src = port_src
            self.core.audit.emit(event)

    def _clean(self, rows):
        results = []
        for row in rows:
            line = []
            for col in row:
                if col is None:
                    col = ""
                if isinstance(col, datetime):
                    col = str(col)
                line.append(col)
            results.append(line)
        return results

