#coding: utf-8
"""
Copyright (C) 2009-2011 EdenWall Technologies
Written by Pierre Chifflier <chifflier AT inl.fr>

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

# apt-get install python-egenix-mxdatetime python-psycopg2
import psycopg2 as psycopg

from logging import Formatter, Handler
from logging import INFO

from nucentral.backend import Component, NuCentralError

class Logger(Component, Handler):
    NAME = "logger"
    VERSION = "1.0"
    API_VERSION = 1

    default_conf = {'host':           'localhost',
                    'port':           5432,
                    'dbname':         'edenlog',
                    'user':           'edenlog',
                    'password':       'edenlog',
                   }

    ROLES = {
        'nucentral_admin' : set(('get_log',)),
        #'multisite_read' : set(('@nucentral_admin',)), # avoid making nucentral_admin a dependancy of multiste_read
        'multisite_read' : set(('get_log',)),
    }

    _isConnected = False

    MAX_FAIL = 3

    def __init__(self):
        Component.__init__(self)
        Handler.__init__(self)

    def init(self, core):
        self.nucentral_core = core

        self.remote_servers = {}
        self.o = None

        self.config = self.default_conf
        self.fail_count = 0

        self._db_connect()

        format = '%(message)s'
        formatter = Formatter(format)
        self.setFormatter(formatter)

        core.addLogHandler(self, INFO)

    def _db_connect(self):
        try:
            connection_string = ""
            if self.config.has_key('host') and len(self.config['host'])>0:
                connection_string += ("host=%(host)s " % self.config)
            connection_string += ("port=%(port)s " % self.config)
            connection_string += ("dbname=%(dbname)s user=%(user)s password=%(password)s" % self.config)
            self.o = psycopg.connect(connection_string)
            self._isConnected = True
            self.fail_count = 0
            print("PostgreSQL Logger: connection to database restored")
        except psycopg.OperationalError, e:
            pass

    def _log_msg(self, severity, message, prefix):
        if not self._isConnected:
            self._db_connect()
            if not self._isConnected:
                # We can't log message, there would be an infinite loop
                print("PostgreSQL Logger: could not connect to database")
                self.fail_count += 1
                if self.fail_count > self.MAX_FAIL:
                    print "PostgreSQL Logger: fail count exceeded, disabling module"
                    self.nucentral_core.removeHandler(self)
                return
        c = self.o.cursor()
        data = {'severity':int(severity),
                'prefix':prefix,
                'message':message[:512]}
        try:
            query = """INSERT INTO edenlog (date,severity,prefix,message)"""\
                    """VALUES (NOW(),%(severity)s,%(prefix)s,%(message)s)"""
            c.execute(query, data)
            self.o.commit()
        except:
            c.connection.rollback ()

    def emit(self, record):
        if not hasattr(record, 'prefix'):
            record.prefix = ''
        self._log_msg(record.levelno, record.getMessage(), record.prefix)
        return True

    def service_log_message(self, context, severity, message, prefix=''):
        self._log_msg(severity, message, prefix)
        return True

    def service_get_log(self, context, severity_threshold, max_items, filter_criteria):
        if not self._isConnected:
            self._db_connect()
            if not self._isConnected:
                # We can't log message, there would be an infinite loop
                print("PostgreSQL Logger: could not connect to database")
                self.fail_count += 1
                if self.fail_count > self.MAX_FAIL:
                    print "PostgreSQL Logger: fail count exceeded, disabling module"
                    self.nucentral_core.removeHandler(self)
                raise NuCentralError("PostgreSQL Logger: could not connect to database")
                return
        c = self.o.cursor()
        data = {'severity':int(severity_threshold),'max_items':int(max_items)}
        if (len(filter_criteria)) > 0:
            data['filter'] = "%%%s%%" % filter_criteria
            query = """SELECT id,date::varchar,severity,message FROM edenlog WHERE severity >= %(severity)s AND message ILIKE %(filter)s ORDER BY date DESC LIMIT %(max_items)s"""
        else:
            query = """SELECT id,date::varchar,severity,message FROM edenlog WHERE severity >= %(severity)s ORDER BY date DESC LIMIT %(max_items)s"""
        c.execute(query, data)
        rows = c.fetchall()
        return rows

