#coding: utf-8
"""
Copyright (C) 2009-2011 EdenWall Technologies
Written by Julien Miotte <jmiotte AT edenwall.com>
$Id$
"""

from __future__ import with_statement
from os import makedirs, symlink, rmdir, remove, getuid
from os.path import isdir, islink, isfile, basename
from ConfigParser import ConfigParser
from shutil import copy
from nucentral.common.unicode_stdout import installUnicodeStdout
from nucentral.client.nucentral_client_base import NuCentralClientBase
from nucentral.common.download import encodeFileContent
from nucentral.client.error import NuCentralError

from nucentral.common.journal_messages import ROLLING_BACK
from nucentral.common.journal_messages import GLOBAL_DONE
from nucentral.common.journal_messages import PHASE_CHANGE
from nucentral.common.journal_messages import ERRORS

from paramiko import SSHClient, AutoAddPolicy
from py import test
import os
import re
import time
import socket
import subprocess

import tempfile
from os import write, close, unlink

class Test(object):
    """
        This class is the parent of every test class. It provides several methods
        that ease setup, tests, and teardown.
    """
    glob_results_path = "/root/tests_results/"
    method = "https"
    login = "admin"
    passwd = "admin"
    host = "127.0.0.1"
    other_fwall = {"host":"172.16.1.17",
                   "method":"https",
                   "login":"admin",
                   "passwd":"admin"
    }
    cluster_ha = {"network":"172.16.0.0/16",
                  "service":"172.16.1.23",
                  "primary":"172.16.1.16",
                  "secondary":"172.16.1.17"
    }
    DNS_server = {"ip":"192.168.33.129", }
    client = None
    ssh_client = None
    remote_client = None
    log = None
    date = None
    REQUIRE_COMPONENTS = tuple()
    license = "/root/license.asc"

    def __init__(self):
        installUnicodeStdout()

# {{{ def setup_class(cls)
    @classmethod
    def setup_class(cls):
        """
            This method is called by every test class to set up the test_directory,
            the dir.last symlink, and checks that nucentral is listenning.
        """
        if cls.date is None:
            uid = getuid()
            if uid != 0:
                error = "You have to run clss using the root user (current=%s)"
                raise Exception(error % uid)

            cls.readConf()

            handle = subprocess.Popen("date +%F-%R:%S",
                                 shell=True,
                                 stdout=subprocess.PIPE
            )
            cls.date = '_'.join(handle.stdout.read().rstrip().split(':'))

        cls.results_path = os.path.join(cls.glob_results_path, cls.date)
        if not isdir(cls.results_path):
            makedirs(cls.results_path)
            if islink(cls.glob_results_path+"dir.last"):
                remove(cls.glob_results_path+"dir.last")
            symlink(cls.results_path, cls.glob_results_path+"dir.last")

        if not cls.check_nucentral():
            raise "!! NuCentral is not responding !!"

        cls.client = cls.createClient()
# }}}
# {{{ def readConf(cls)
    @classmethod
    def readConf(cls):
        """
            Read the parameters of autotest.conf.
        """
        conf = ConfigParser()
        res = conf.read('autotest.conf')
        if res != []:
            Test.method = conf.get('client', 'method')
            Test.client = conf.get('client', 'login')
            Test.passwd = conf.get('client', 'passwd')
            Test.host = conf.get('client', 'host')
            Test.glob_results_path = conf.get('paths', 'results')
            if Test.results_path[-1:] != "/":
                Test.glob_results_path = Test.glob_results_path+"/"
            Test.other_fwall["host"] = conf.get('other_fwall', 'host')
            Test.other_fwall["method"] = conf.get('other_fwall', 'method')
            Test.other_fwall["login"] = conf.get('other_fwall', 'login')
            Test.other_fwall["passwd"] = conf.get('other_fwall', 'passwd')
            Test.cluster_ha["network"] = conf.get('cluster_ha', 'network')
            Test.cluster_ha["service"] = conf.get('cluster_ha', 'service')
            Test.cluster_ha["primary"] = conf.get('cluster_ha', 'primary')
            Test.cluster_ha["secondary"] = conf.get('cluster_ha', 'secondary')
            Test.license = conf.get('license','filename')
            Test.DNS_server["ip"] = conf.get('DNS_server','ip')
# }}}
# {{{ def createClient(cls, ssl_config=None, cust_host=None)
    @classmethod
    def createClient(cls, ssl_config=None, cust_host=None):
        """
            Conveniance method to create a nucentral client. If no license is set,
            it will upload one.
        """
        if cust_host is None:
            cust_host = cls.host

        client = NuCentralClientBase(
                                protocol=cls.method,
                                host=cust_host,
                                ssl_config=ssl_config
        )
        client.authenticate(cls.login, cls.passwd)
        cls._pleaseLeave(client)
        for component in cls.REQUIRE_COMPONENTS:
            has = client.call('CORE', 'hasComponent', component)
            if not has:
                client.logout()
                test.skip('Missing component: %s' % component)

        # Check that the EdenWall has a valid license
        try:
            logout = cls._checkAndUploadLicense(client)
        except:
            client.logout()
            raise

        if logout:
            # New license disabled the minimal mode:
            # reconnect to get the new permissions
            print "restarting client"
            client.logout()
            client = NuCentralClientBase(protocol=cls.method, host=cls.host)
            client.authenticate(cls.login, cls.passwd)
        return client
# }}}
# {{{ def createSSHClient(cls, host="5.0.0.2", login="root", password="fai")
    @classmethod
    def createSSHClient(cls, host="5.0.0.2", login="root", password="fai"):
        """
            Method to create a ssh client !
        """
        client = SSHClient()
        client.set_missing_host_key_policy(AutoAddPolicy())
        client.connect(host, username=login, password=password)
        return client
# }}}
# {{{ def sshOutput(cls, command):
    @classmethod
    def sshOutput(cls, command):
        stdin, stdout, stderr = cls.ssh_client.exec_command(command)
        return stdout.read()
# }}}
# {{{ def _checkAndUploadLicense(cls, client)
    @classmethod
    def _checkAndUploadLicense(cls, client):
        """
            This method checks if the EW has a correct license.
            If it doesn't, it looks for one in the current directory and upoads it.
            The license expected filename is "license.asc".
        """
        if not client.call('CORE', 'hasComponent', 'license'):
            # No license component
            return False

        if client.call('license','checkValidity'):
            return False

        filename = Test.license
        if not isfile(filename):
            raise Exception(
                "Please copy a license into the current directory. "
                "The current expected filename is %s."
                % filename)

        with open(filename, 'rb') as handle:
            content = handle.read()

        minimalmode = client.call('acl', 'getMinimalMode')

        print "Uploaded license file %s" % basename(filename)

        try:
            client.call("session", "acquire", "nuconf_write")
        except NuCentralError:
            pass
        try:
            client.call("nuconf", "takeWriteRole")
        except NuCentralError:
            pass

        res = client.call('license', 'sendLicense', encodeFileContent(content))
        if not res:
            raise Exception("No new license for this appliance in this file.")

        return minimalmode and (not client.call('acl', 'getMinimalMode'))
# }}}
# {{{ def _pleaseLeave(cls, client)
    @classmethod
    def _pleaseLeave(cls, client):
        """
            AKA GetTheF***Out
        """
        _me = client.call('session', 'get')['cookie']
        for item in client.call('session', 'list'):
            cookie = item['cookie']
            if cookie == _me:
                continue
            client.call('session', 'destroySession', cookie)
# }}}
# {{{ def restart_nucentral(cls)
    @classmethod
    def restart_nucentral(cls):
        """
            Conveniance method to restart nucentral.
        """
        handle = subprocess.Popen(
                                    "/etc/init.d/nucentral-server restart",
                                    shell=True,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE
        )
        handle.wait()
        time.sleep(1)
        if not cls.check_nucentral():
            test.skip("!! NuCentral is not responding !!")
# }}}
# {{{ def start_nucentral(cls)
    @classmethod
    def start_nucentral(cls):
        """
            Conveniance method to start nucentral.
        """
        handle = subprocess.Popen(
                                    "/etc/init.d/nucentral-server start",
                                    shell=True,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE
        )
        handle.wait()
        time.sleep(1)
        if not cls.check_nucentral():
            raise "!! NuCentral is not responding !!"
# }}}
# {{{ def stop_nucentral(cls)
    @classmethod
    def stop_nucentral(cls):
        """
            Conveniance method to stop nucentral.
        """
        handle = subprocess.Popen(
                                    "/etc/init.d/nucentral-server stop",
                                    shell=True,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE
        )
        handle.wait()
# }}}


# {{{ def nucentral_pid(cls)
    @classmethod
    def nucentral_pid(cls):
        """
            We fetch nucentral's pid.
        """
        cmd = "netstat -lnpt |grep :8080 | awk '{print $7}' | cut -d / -f1"
        handle = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
        pid = handle.stdout.read()
        return pid
# }}}
# {{{ def check_nucentral(cls, pid, timeout)
    @classmethod
    def check_nucentral(cls, pid=None, timeout=30):
        """
            Since we now reboot nucentral in some testcases, we should wait a bit
             before declaring nucentral is dead. We first check that nucentral
            has really rebooted (pids should differs), and wait for TIMEOUT seconds.
        """
        if pid: # compatibility reasons
            iterations = 0
            while cls.nucentral_pid() == pid:
                if iterations > 5:
                    raise "!! Nucentral has not restarted !!"
                time.sleep(1)
                iterations += 1

        iterations = 0
        while iterations < timeout:
            try:
                handle = socket.socket()
                handle.connect((cls.host, 8443))
                handle.close()
                return True
            except socket.error:
                pass
            finally:
                iterations += 1
                time.sleep(1)
        return False
# }}}
# {{{ def correctly_written(cls, file, expected, should_find, remote)
    @classmethod
    def correctly_written(cls, filename, expected, should_find=True, remote=False):
        """
            Checks that [filename] contains what was [expected].
        """
        assert type(expected) == type(()), \
            "Expected %s, found %s" % (type(()), type(expected))

        if not remote:
            with open(filename) as handle:
                results = handle.read()
        else:
            results = cls.sshOutput("cat %s" % filename)

        if remote:
            error = "[remote] "
        else:
            error = ""
        error += "check %s copied in %s, wrong info" % (filename, cls.results_path)

        try:
            cls._checked(expected, results, should_find, error)
        except:
            copy(filename, cls.results_path)
            path = os.path.join(cls.results_path, os.path.basename(filename))
            cls._autotestsSays(path, expected)
            with open(path) as handle:
                print handle.read()
            raise
# }}}
# {{{ def output_check(cls, command, expected, should_find)
    @classmethod
    def output_check(cls, command, expected, should_find = True):
        """
            Check the output of [command] against what was [expected].
        """
        # Check and process
        assert type(expected) == type(()), \
            "Expected %s, found %s" % (type(()), type(expected))
        filename = "_".join(command.split(" ")).replace('/', '_')

        # Execute command
        handle = subprocess.Popen(command,
                                  shell=True,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE)
        results = handle.stdout.read()

        file_handle, temp_name = tempfile.mkstemp(prefix=filename+"_",
                                        dir=cls.results_path)

        error = "check output of `%s` copied in %s, wrong info" \
                                            % (command, temp_name)

        # Check output
        try:
            cls._checked(expected, results, should_find, error)
        except:
            try:
                write(file_handle, results)
            finally:
                close(file_handle)

            cls._autotestsSays(temp_name, expected, should_find)
            with open(temp_name) as handle:
                print handle.read()

            raise
        else:
            unlink(temp_name)

# }}}
# {{{ def _checked(cls, expected, results, should_find, error)
    @classmethod
    def _checked(cls, expected, results, should_find, error):
        for entry in expected:
            entry = unicode(entry)
            if should_find:
                error += "'%s' not found in '%s'" % (entry, results)
                assert re.search(entry, results), error
            else:
                error += "'%s' found in '%s'" % (entry, results)
                assert not re.search(entry, results), error

# }}}
# {{{ def _autotestsSays(cls, filename, expected)
    @classmethod
    def _autotestsSays(cls, filename, expected, should_find=True):
        """
            Write in [filename] what was [expected].
        """
        with open(filename, "a") as handle:
            handle.write("\n### AUTOTESTS SAYS ###\n")
            if should_find:
                handle.write("# We should have :\n")
            else:
                handle.write("# We should not have :\n")
            for exp in expected:
                handle.write("%s\n"%exp)
            handle.write("\n######################\n\n")
            handle.close()
# }}}
# {{{ def md5sum(cls, filename)
    @classmethod
    def md5sum(cls, filename):
        """
            Returns md5sum of [filename]
        """
        handle = subprocess.Popen("md5sum %s" % filename,
                                  shell=True,
                                  stdout=subprocess.PIPE)
        results = handle.stdout.read()
        return results.split(" ")[0]
# }}}
# {{{ def takeNuconfWriteRole(cls)
    @classmethod
    def takeNuconfWriteRole(cls):
        """
            Takes the nuconf_write role
        """
        try:
            cls.client.call("session", "acquire", "nuconf_write")
        except NuCentralError:
            pass
        try:
            cls.client.call("nuconf", "takeWriteRole")
        except NuCentralError:
            pass
# }}}
# {{{ def apply_nuconf(cls, force=False)
    @ classmethod
    def apply_nuconf(cls, force=False):
        """
            Apply config and raise exception if the application failed
        """
        # No way to do restoration with config.applyStart
        if force:
            timeout = 30
            cls.client.call('config', 'applyStart', 'force=True')

            iterations = 0
            item = 0
            stop = False
            while not stop:
                logs = cls.client.call('config','applyLog',item)
                item += len(logs)

                for log in logs:
                    timestamp, mess_type, content = log
                    if mess_type == PHASE_CHANGE:
                        if log[2] == ROLLING_BACK: # rollback
                            test.skip("!! Apply Rollback !!")
                        elif log[2] == GLOBAL_DONE:
                            stop = True
                    elif mess_type in ERRORS:
                        test.skip("!! Apply Error !!\n%s" % content)

                iterations += 1
                if iterations > timeout:
                    test.skip("!! Waited %s seconds for application !!" % timeout )

                time.sleep(1)
        else:
            errors = cls.client.call('config', 'apply')
            assert not errors, "Apply Error %s" % errors
# }}}
# {{{ def _clean_results_dir(cls)
    @classmethod
    def _clean_results_dir(cls):
        """
            Remove every empty directory in the test_results directory.
        """
        for test_date in os.listdir(cls.glob_results_path):
            if test_date == "dir.last":
                continue
            dir = os.path.join(cls.glob_results_path, test_date)
            for test_name in os.listdir(dir):
                try:
                    rmdir(os.path.join(cls.glob_results_path, test_date, test_name))
                except OSError:
                    pass
            try:
                rmdir(os.path.join(cls.glob_results_path, test_date))
            except OSError:
                pass
# }}}
# {{{ def teardown_class(cls)
    @classmethod
    def teardown_class(cls):
        """
            Let's do some cleaning in the test_results directory, containing all
            the tests directories
        """
        if cls.client:
            cls.client.logout()
        cls._clean_results_dir()
# }}}

