#!/bin/sh

SVN_ROOT="https://edw.inl.fr/edw-svn/src/nucentral"
SVN_URL="$SVN_ROOT/trunk"

set -e

VERSION=$(grep "^nucentral_version" nucentral/version.py | sed 's/.*"\([^"]\+\)"/\1/')
OLD_CWD=$(pwd)

cleanup()
{
  cd $OLD_CWD
  [ ! -d tmp ] || rm -rf tmp
}

check_setup_release()
{
  SETUP_VERS=$(grep 'version=' nucentral-$VERSION/setup.py | sed 's/.*="\([^"]\+\)".*/\1/')

  if [ "${SETUP_VERS}" != "${VERSION}" ]; then
    echo "setup.pu release number is not correct, it should be ${VERSION} (instead of $SETUP_VERS)"
    exit 1
  fi
}

check_debian_release()
{
  DEB_VERS=$(head -1 nucentral-$VERSION/debian/changelog | sed 's/.*(\([^)]\+\)).*/\1/')

  if [ "${DEB_VERS}" != "${VERSION}-1" ]; then
    echo "Debian release number is not correct, it should be ${VERSION}-1 (instead of $DEB_VERS)"
    exit 1
  fi
}


[ ! -d tmp ] || rm -rf tmp

trap cleanup EXIT

mkdir tmp

cd tmp

svn export -q "$SVN_URL" "nucentral-$VERSION"

check_debian_release
check_setup_release

TGZ="nucentral-$VERSION.tar.gz"

tar czf $TGZ nucentral-$VERSION
mv $TGZ $OLD_CWD/

cd $OLD_CWD
echo "Release file $TGZ is ok"
echo -n "$TGZ: "
SIZE=$(stat -c "%s" $TGZ)
echo "$SIZE bytes"

echo -n "MD5 : "
SUM=$(md5sum $TGZ | cut -f 1 -d' ')
echo "$SUM"

echo -n "SHA1: "
SUM=$(sha1sum $TGZ | cut -f 1 -d' ')
echo "$SUM"

echo ""
echo "Don't forget to run:"
echo "svn copy $SVN_ROOT/trunk $SVN_ROOT/tags/$VERSION"

[ ! -d tmp ] || rm -rf tmp
