# -*- coding: utf-8 -*-

# $Id$

"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""


from nucentral.common import tr, EDENWALL
from nucentral.client import NuCentralError
from nuconf.client.main_window import NuConfMainWindow
from nucentral.common.abstract_cfg import DatastructureIncompatible

# NuConf front-end modules:
from nuconf.client.services.status_page import StatusPage
from nuconf.client.services.access import AccessFrontend
from nuconf.client.services.authentication import AuthenticationFrontend
if EDENWALL:
    from nuconf.client.services.dhcp import DhcpFrontend
    from nuconf.client.services.ids_ips import IdsIpsFrontend
    from nuconf.client.services.mail import MailServicesFrontend
    from nuconf.client.services.proxy import ProxyFrontend
    from nuconf.client.services.roadwarrior import RoadWarriorFrontend, QOpenVpnObject
    from nuconf.client.services.site2site import Site2SiteFrontend
    from nuconf.client.services.snmpd import SnmpdFrontend

SERVICES = (
    StatusPage,
    AccessFrontend,
    AuthenticationFrontend,
)
if EDENWALL:
    SERVICES += (
        DhcpFrontend,
        IdsIpsFrontend,
        ProxyFrontend,
        MailServicesFrontend,
        Site2SiteFrontend,
        RoadWarriorFrontend,
        SnmpdFrontend,
        )

class MainWindow(NuConfMainWindow):
    ROLES = set(('nuconf_read', 'nuconf_write'))
    ICON = ':/icons/application.png'

    def __init__(self, application, client, **kw):
        NuConfMainWindow.__init__(self, application, client, tr("Services"), **kw)

    def _load(self):
        if EDENWALL and (not NuConfMainWindow.minimalMode):
            # before creation of frontends, initialize Q

            try:
                serialized_openvpn_conf = self.init_call('openvpn', 'getOpenVpnConfig')
                QOpenVpnObject.getInstance().openvpn = serialized_openvpn_conf
            except (NuCentralError, DatastructureIncompatible):
                # openvpn backend is not available
                pass

    def getTree(self, minimal_mode, model):
        if minimal_mode:
            return [ ]
        else:
            if model == 'EMF':
                for page in (
                    "Authentication server",
                    "DHCP service",
                    "IDS/IPS",
                    "Proxy service"):
                    try:
                        self.remove_page_from_tree(SERVICES, page)
                    except ValueError:
                        pass
            return SERVICES

