#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright(C) 2006 INL
Written by Damien Boucard <damien.boucard AT inl.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
---
auth_sha.py a module for pyctd for authentication.

It manages a password file which is located as defined in the module attribute
PASSWD_FILE.
  * To add a user, run the script itself : python auth_sha.py username password
  * To remove a user, edit the password file and remove the appropriate line.
  * To use this module with pyctd, pass its location (relative or absolute) as
an option --mod-auth (see pyctd.py -h for usage information).
"""
import sys, sha, random, os, os.path

PASSWD_FILE = "/tmp/mypasswd"

def authenticate(username, password):
    file = open(PASSWD_FILE, "r")
    lines = file.read().split(os.linesep)
    file.close()

    for l in lines:
        u,p = l.split(':')
        if u == username:
            algo, salt, hash = p.split('$')
            if algo == 'sha1':
                return hash == sha.new(salt+password).hexdigest()
    return False

def set_password(username, password):
    algo = 'sha1'
    salt = sha.new(str(random.random())).hexdigest()[:5]
    hash = sha.new(salt+password).hexdigest()
    line = '%s:%s$%s$%s' % (username,algo, salt, hash)

    lines = []
    found = False
    if os.path.exists(PASSWD_FILE):
        pwfile = open(PASSWD_FILE, "r")
        lines = pwfile.read().split(os.linesep)
        pwfile.close()
        for i in range(len(lines)):
            u,p = lines[i].split(':')
            if u == username:
                lines[i] = line
                found = True
                break
    if not found:
        lines.append(line)
    pwfile = open(PASSWD_FILE, "w")
    pwfile.write(os.linesep.join(lines))
    pwfile.close()

if __name__ == '__main__':
    if len(sys.argv) != 3:
        print >>sys.stderr, "USAGE: %s <username> <password>" %(sys.argv[0])
        sys.exit(1)

    set_password(sys.argv[1], sys.argv[2])
