#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Copyright(C) 2006, 2007 INL
Written by Damien Boucard <damien.boucard AT inl.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
---
server.py is a part of pyctd which manages XML-RPC service.
"""

from viewer import ConntrackViewer
from nucentral.backend import Component
from conntracking import DataCollector

class NuConntrack(Component):

    NAME = "nuconntrack"
    VERSION = "0.4.1"
    API_VERSION = 1
    #REQUIRES = ('config',)
    #ACLS = {'config': set(('getValue', 'setValue'))}
    ROLES = {'nuconntrack_read': set((
                    'view',
                )),
             'nuconntrack_write': set((
                    '@nuconntrack_read',
                    'kill', 'change'
                )),
             'nuconntrack_anonymous': set((
                    '@nuconntrack_read',
                ))
            }

    def init(self, core):

        uidmask = 0xffff
        filters = None
        # TODO: use the config module!
        #try:
        #    uidmask = parser.get('NuConntrack', 'uidmask')
        #    if uidmask:
        #        uidmask = int(uidmask, 16)
        #except:
        #    uidmask = None

        #try:
        #    filters = parser.get('NuConntrack', 'filter').split(',')
        #except:
        #    filters = None

        data = DataCollector("nuconntrack", uidmask, filter=filters, update_second=5.0)
        data.refresh()
        self.data = data

    def service_view(self, context, filters, order_by, order_dir, start, length):
        """
        ``view`` remote procedure handler.

        Parameters:
         - filters (dict): field name (str): field value (type depends to the field)
         - order_by (str): field name ordered by
         - order_dir (str): order direction, 'A' for ascending and 'D' for descending
         - start (int): first connection index to render (used for pagination)
         - length (int): maximal number of connections to render (used for pagination)

        Available fields: orig_ipv4_src, orig_port_src, mark, etc.

        Return a dict containing a list of connections formatted to be
        XMLRPC compliant.
        """
        print "[%r]" % filters
        viewer = ConntrackViewer(context=context)
        viewer.set_order(order_by, order_dir)
        viewer.set_filters(filters)
        viewer.set_pagination(start, length)
        self.data.refresh()
        connset = self.data.current
        result = viewer.xmlrpc_view(connset)
        return result

    def service_kill(self, context, conn_ids):
        """
        ``kill`` remote procedure handler.

        Parameters:
         - conn_ids (list): connection identifiers to destroy (int)

        Return the number of successfully killed connections.
        """
        return self.data.kill(conn_ids)

    def service_change(self, context, id, field, value):
        """
        ``change`` remote procedure handler.

        Parameters:
         - id (int): connection identifier to modify
         - field (str): field name of the connection to change
         - value (int): new value to set to the field

        Note: the type of value parameter is int because only changeable fields
        are integer-typed for the moment, but it can change in future versions.
        """
        try:
            return (self.data.change(id, field, value) and 1 or 0)
        except (TypeError, ValueError):
            return 2
