from os.path import join as pathjoin, exists
from nucentral.backend.variables_store import VariablesStore

# ****************************** Clasification ****************
class Classification():

    PATH = '/var/lib/nucentral/classification'
    FILENAME = 'classification'

    def __init__(self, path):
        self.user_conf_dir = path
        self.config_managers = {}

    def  __call__(self, name, args):
        if name == 'GetClassification':
            return self.get()
        if name == 'SetClassification':
            self.set(args)
        if name == 'SaveClassification':
            self.save()
        if name == 'CloseClassification':
            self.close()


    def _getXMLName(self):
        return '%s.xml' % pathjoin(self.user_conf_dir, self.FILENAME)

    def _getConfigManager(self):
        if self.FILENAME not in self.config_managers:
            cm = VariablesStore()
            filename = self._getXMLName()
            if exists(filename):
                cm.load(filename)
            self.config_managers[self.FILENAME] = cm
        return self.config_managers[self.FILENAME]

    def get(self):
        kw = {}
        return self._getConfigManager().get(self.FILENAME, **kw)

    def set(self, context, *path):
        return self._getConfigManager().set(*path)

    def delete(self, context, *path):
        """
        Delete specified path.
        Raise an error if the path doesn't exist.
        """
        self._getConfigManager().delete(self.FILENAME)

    def save(self, context):
        """
        Write changes into the XML file.
        """
        cm = self._getConfigManager()
        filename = self._getXMLName()
        cm.save(filename)

    def close(self, context):
        """
        Close the configuration without saving the changes.
        """
        try:
            del self.config_managers[self.FILENAME]
            return True
        except KeyError:
            return False


