
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

import psycopg2

class Singleton(type):
    def __init__(cls, name, bases, dict):
        super(Singleton, cls).__init__(name, bases, dict)
        cls.instance = None

    def __call__(cls, *args, **kw):
        if cls.instance is None:
            cls.instance = super(Singleton, cls).__call__(*args, **kw)

        return cls.instance


class DataBaseManager(object):
    __metaclass__ = Singleton

    def __init__(self):
        self.conn = None
        self.cur = None
        self.conf = None

    def load(self, conf):
        if not self.conf:
            self.conf = conf

    def reset(self):
        self.conf = None

    def connect(self):
        try:
            self.conn = psycopg2.connect(database=self.conf['database'], host=self.conf['hostname'], user=self.conf['username'], password=self.conf['password'])
            self.cur = self.conn.cursor()
            return True
        except:
            print "I am unable to connect the database"
            return False

    def execute(self, query, commit=False):
        self.cur.execute(query)
        self.commit()
        if commit:
            return
        if self.cur.rowcount > 0:
            return self.cur.fetchall()
        else:
            return []

    def commit(self):
        self.conn.commit()

    def disconnect(self):
        self.cur.close()
        self.conn.close()
