from decimal import Decimal
import datetime

from datasource import DataSource

class TableBase(DataSource):
    def __init__(self, database):
        DataSource.__init__(self)
        self.database = database

    def getFilters(self):
        return self.FILTERS

    def __call__(self, args):
        if self.database.connect():
            self._make_query(args)
            data = self.database.execute(self.query)
            data = self._clean(data)
            if 'count' in args and args['count']:
                return str(data[0][0])
            results = self._print_results(self.COLUMNS, data, args)
            self.database.disconnect()
            return results
        else:
            return {}

    def _clean(self, rows):
        if not rows:
            return [['']]

        results = []
        for row in rows:
            line = []
            for col in row:
                if col is None:
                    col = ""
                if isinstance(col, Decimal) or isinstance(col, datetime.datetime):
                    col = str(col)
                line.append(col)
            results.append(line)
        return results

    def _print_results(self, columns, data, args):
        """ @param result [array of array]

            @return [dict]:
                ret['args'] = dict with all arguments used for this request
                ret['filters'] = dict with all filters
                ret['columns'] = list of column titles
                ret['rowcount'] = integer number of lines (it doesn't consider 'start' and 'limit')
                ret['table'] = 2D array with table resulted
        """
        results = {}
        results['args'] = args

        if data == None: data = []

        #if isinstance(self, UserIDTable):
        if self.__class__.__name__ == 'UserIDTable':
            new_data = []
            for dat in data:
                if isinstance(dat, list) or isinstance(dat, tuple):
                    new_data.append(dat[0])
                else:
                    new_data.append(dat)
            return new_data

        results['columns'] = columns
        results['rowcount'] = len(data)
        results['filters'] = ""
        results['table'] = data
        results['args']['limit'] =  10
        if 'start' in args:
            results['args']['start'] = args['start']
        else:
            results['args']['start'] = 0

        return results

#
