# -*- coding: utf-8 -*-

"""
Copyright (C) 2008-2011 EdenWall Technologies
Written by Romain Bignon <romain AT inl.fr>

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>

$Id: argfilter.py 14780 2009-11-25 10:11:50Z carrere $
"""

from nulog.client.qt.args.argfilter import *

class ArgFilterTextCompletion(ArgFilterBase, QComboBox):
    # States
    NOLIST = 0
    LIST = 1
    SELECTED = 2

    def __init__(self, client, arg, value, compatibility=None):

        ArgFilterBase.__init__(self, client, arg, value, compatibility)
        QComboBox.__init__(self)

        self.setEditable(True)
        self.setDuplicatesEnabled(False)
#        self.list = []
        self.state = self.NOLIST

        self.connect(self, SIGNAL('currentIndexChanged(int)'), self.indexChanged)
        self.connect(self.lineEdit(), SIGNAL('textEdited(const QString &)'), self.textChanged)

    def setState(self, s):
        self.state = s
        style = ''
        if self.isEnabled() and len(self.lineEdit().text()) > 0:
            if self.state == self.NOLIST:
                style = 'background: #b52323;' # red
            elif self.state == self.LIST:
                style = 'background: #B5AD1C;' # orange
            elif self.state == self.SELECTED:
                style = 'background: #5FB554;' # green
        self.setStyleSheet(style)

    def indexChanged(self, index):
        if self.state == self.NOLIST:
            return
        self.setState(index >= 0 and self.SELECTED or self.LIST)

    def textChanged(self, text):
        # Do not check list for one letter unexistant accounts
        if len(text) < 2:
            self.setState(self.NOLIST)
            self.clear()
            self.list = []
            self.addItem(tr('-- Please enter at least two letters --'), QVariant(-1))
            self.setCurrentIndex(-1)
            self.lineEdit().setText(text)
            return

        self.clear()

    def getValue(self, message):

        selected = -1
        if self.state == self.SELECTED:
            selected = unicode(self.itemData(self.currentIndex()).toString())
        if not selected:
            raise CheckError(message)

        return selected


class ArgFilterProto(ArgFilterTextCompletion):
    """ Argument is a protocol name AND a filter """

    def __init__(self, client, arg, value, compatibility=None):

        ArgFilterTextCompletion.__init__(self, client, arg, value, compatibility)
        self.classification = self.classification = self.client.call('nudpi', 'getClassification')

        if value:
            protocols = self._protocolsList(value)

            if protocols and len(protocols) > 0:
                self.textChanged(protocols[0])

    def _protocolsList(self, value):
        if not self.classification:
            return

        protocols = []
        for proto, map in self.classification['db_association'].iteritems():
            if proto and value:
                if isinstance(value, (str, unicode)) and proto.startswith(value):
                    protocols.append(unicode(proto))

        return sorted(protocols)

    def textChanged(self, text):
        ArgFilterTextCompletion.textChanged(self, text)
        protocols = self._protocolsList(unicode(text))
        self.list = protocols

        select = -1
        for proto in protocols:
            variant = QVariant(proto)
            self.addItem(proto, variant)
            if unicode(proto) == unicode(text):
                select = self.count() - 1

        self.setCurrentIndex(select)
        if select >= 0:
            self.setState(self.SELECTED)
        else:
            self.lineEdit().setText(text)
            if len(protocols) > 0:
                self.setState(self.LIST)
            else:
                self.setState(self.NOLIST)

    def getValue(self):
        message = tr('Please select a protocol from the list')
        return ArgFilterTextCompletion.getValue(self, message)

class ArgFilterUserID(ArgFilterTextCompletion):

    # States
    NOLIST = 0
    LIST = 1
    SELECTED = 2

    def __init__(self, client, arg, value, compatibility=None):

        ArgFilterTextCompletion.__init__(self, client, arg, value, compatibility)

        if value:
            if self.compatibility.user_id:
                userlist = self.client.call('nulog', 'table', 'UserIDTable', {'user_id': value})
            else:
                userlist = self.client.call('nulog', 'table', 'UserIDTable', {'username': value})

            if userlist:
                if self.compatibility.user_id:
                    self.textChanged(userlist[0][0])
                else:
                    self.textChanged(userlist[0])
            else:
                self.textChanged('') # to initialize state to NOLIST
        else:
            self.textChanged('') # to initialize state to NOLIST

    def textChanged(self, text):

        ArgFilterTextCompletion.textChanged(self, text)

        if not self.list:
            users_nulog = self.client.call('nulog', 'table', 'UserIDTable', {'username': '%s%%' % text[:2]})
            users_nudpi = self.client.call('nudpi', 'table', 'UserIDTable', {'username': '%s%%' % text[:2]})

#            self.users = list(set(users_nudpi + users_nulog))
            self.list = users_nulog
            if len(users_nudpi) > 0:
                self.list = list(set(users_nudpi + self.list))

            if self.compatibility.authfail:
                users2 = self.client.call('nulog', 'table', 'UserIDTableAuth', {'username': '%s%%' % text[:2]})
                if not self.compatibility.user_id:
                    self.list = list(set(self.list + users2))
                else:
                    for user2 in users2:
                        if len(user2) > 1:
                            contains = False
                            for user1 in self.list:
                                if len(user1) > 1:
                                    if user1 == user2:
                                        contains = True
                                        break
                            if not contains:
                                self.list.append(user1)

        select = -1
        for username in self.list:
            if self.compatibility.user_id:
                user_id = username[1]
                username = username[0]
                variant = QVariant(user_id)
            else:
                variant = QVariant(username)

            if username.startswith(text):
#                self.addItem(username, QVariant(user_id))
                self.addItem(username, variant)
                if username == text:
                    select = self.count() - 1

        self.setCurrentIndex(select)
        if select >= 0:
            self.setState(self.SELECTED)
        else:
            # Reset text, because of the adding of lines in list
            self.lineEdit().setText(text)
            if len(self.list) > 0:
                self.setState(self.LIST)
            else:
                self.setState(self.NOLIST)

        self.lock = False

    def getValue(self):
        message = tr('Please select a user from the list')
        return ArgFilterTextCompletion.getValue(self, message)

