from PyQt4.QtGui import QWizard, QPixmap, QGridLayout, \
                        QCheckBox, QItemDelegate, QApplication, \
                        QStyle, QStyleOptionButton, QComboBox, QAbstractItemView, \
                        QStylePainter, QPalette, QStyleOptionComboBox, \
                        QGroupBox, QScrollArea, QVBoxLayout, QWidget, QDialog
from PyQt4.QtCore import SIGNAL, Qt, QEvent, QVariant

from nucentral.common import tr
from nucentral.client.error import NuCentralError

from nudpi.client.qt.ui.dpi_configuration_ui import Ui_DpiWizard

class CheckBoxListDelegate(QItemDelegate):

    def __init__(self, parent):
        QItemDelegate.__init__(self, parent)
        self._parent = parent
        self.proto_checkboxs = []

    def paint(self, painter, option, index):
        value = index.data(Qt.UserRole).toBool()
        text = str(index.data(Qt.DisplayRole).toString())
        style = QApplication.style()

        opt = QStyleOptionButton()

        opt.state |= QStyle.State_On if (value or text in self._parent.selectedItems) else QStyle.State_Off
        opt.state |= QStyle.State_Enabled
        opt.text = text
        opt.rect = option.rect
        style.drawControl(QStyle.CE_CheckBox, opt, painter)

    def createEditor(self, parent, option, index):
        editor = QCheckBox(parent);
        self.connect(editor, SIGNAL("clicked(bool)"), self._parent.stateChanged)
        return editor

    def setEditorData(self, editor, index):
        text = str(index.data(Qt.DisplayRole).toString())
        editor.setText(text)
        editor.setChecked(index.data(Qt.UserRole).toBool())
        if text in self._parent.selectedItems:
            editor.setChecked(True)


    def setModelData(self, editor, model, index):
        value = editor.isChecked()

        data = {}
        data[Qt.DisplayRole] = QVariant(editor.text())
        data[Qt.UserRole] = QVariant(value)
        if str(editor.text()) in self._parent.selectedItems:
            data[Qt.UserRole] = QVariant(True)
        model.setItemData(index, data)

    def updateEditorGeometry(self, editor, option, index):
        editor.setGeometry(option.rect)

class CheckBoxList(QComboBox):

    default_text = tr("Select one or several interfaces")

    def __init__(self, widget=None):
        QComboBox.__init__(self, widget)

        self.delegate = CheckBoxListDelegate(self)
        self.view().setItemDelegate(self.delegate)
        self.view().setEditTriggers(QAbstractItemView.CurrentChanged)

        self.view().viewport().installEventFilter(self)
        self.view().setAlternatingRowColors(True)

        self.displayText = ""

        self.allItems = {}
        self.selectedItems = {}

        self.connect(self, SIGNAL('showPopupSignal()'), self._showPopup)

    def eventFilter(self, object, event):
        if event.type() == QEvent.MouseButtonRelease and object == self.view().viewport():
            return True
        return QComboBox.eventFilter(self, object, event)

    def paintEvent(self, p):
        painter = QStylePainter(self)
        painter.setPen(self.palette().color(QPalette.Text))

        opt = QStyleOptionComboBox()

        self.initStyleOption(opt)
        opt.currentText = self.displayText

        if not self.displayText:
            self.displayText = self.default_text

        painter.drawComplexControl(QStyle.CC_ComboBox, opt)
        painter.drawControl(QStyle.CE_ComboBoxLabel, opt)

    def setData(self, text, key):
        if text not in self.allItems:
            self.allItems[text] = key

    def clear(self):
        QComboBox.clear(self)
        self.allItems.clear()
        self.selectedItems = {}
        self.displayText = ""

    def stateChanged(self, state, text=None):
        if not text:
            text = str(self.sender().text())
        self.displayText = ""

        if state:
            if text not in self.selectedItems:
                for name, item in self.allItems.iteritems():
                    if text == name:
                        self.selectedItems[text] = self.allItems[text]
        else:
            if text in self.selectedItems:
                self.selectedItems.pop(text)

        self.displayText = " ".join(self.selectedItems.keys())

        # Notify the page
        self.parent().emit(SIGNAL("completeChanged()"))
        self.emit(SIGNAL('showPopupSignal()'))

    def setDisplayText(self, text):
        self.displayText = text

    def getDisplayText(self):
        return self.displayText

    def _showPopup(self):
        self.showPopup()


class DpiConfiguration(QWizard):

    pages = {#0 : "capture",
             0 : 'interfaces',
             1 : 'filters',
             }
    INTERNET = "0.0.0.0/0"

    def __init__(self, client, devs, internet_network, parent=None):
        QWizard.__init__(self, parent)

        self.client = client
        self.devs = devs
        self.internet_network = internet_network

        self.cb_vlan = QCheckBox("Vlan")

        self.ui = Ui_DpiWizard()
        self.ui.setupUi(self)
        self.setOption(QWizard.NoBackButtonOnStartPage, True)
        self.setPixmap(QWizard.WatermarkPixmap, QPixmap(":/images/wizard_watermark"))
        self.dpi_configuration = {}
        self.ui.interfaces.isComplete = self.isComplete

        self.cb_to = CheckBoxList()
        self.cb_to.setGeometry(self.ui.cb_from.geometry())
        self.cb_to.setMinimumSize(self.ui.cb_from.minimumSize())
        self.ui.horizontalLayout_2.insertWidget(1, self.cb_to)

        self.ui.horizontalLayout.insertWidget(2, self.cb_vlan)

        dpi_config = self.client.call('nudpi', 'configuration')
        self.classification = self.client.call('nudpi', 'getClassification')

        if dpi_config:
            self.loadConfig(dpi_config)

        self.initFilters()

        self.connect(self, SIGNAL("currentIdChanged(int)"), self.pageChanged)
        self.connect(self.ui.cb_interfaces, SIGNAL("stateChanged(int)"), self.setChecked)

    def loadConfig(self, dpi_config):
        for param in ["filters", "from", "rule"]:
            self.dpi_configuration[param] = dpi_config[param]

    def accept(self):
        self.dpi_configuration.clear()
        # get interfaces
        if self.ui.cb_interfaces.isChecked():
            _from = 'any'
            _to = []
        else:
            _from = str(self.ui.cb_from.itemData(self.ui.cb_from.currentIndex()).toString())
#            _to = self.cb_to.selectedItems.values()

        keys = self.cb_to.selectedItems.keys()
        eths = []
        for eth in keys:
            if ':' in eth:
                eth = eth.split(':')[0]
                if eth not in eths:
                    eths.append(eth.split(':')[0])

        self.dpi_configuration['from'] = _from
        self.dpi_configuration['rule'] = self.cb_to.selectedItems.values()
        self.dpi_configuration['eths_rule'] = eths
        self.dpi_configuration['is_vlan'] = self.cb_vlan.isChecked()

        # get filters
        filters = []

        for child in self.proto_checkboxs:
            if child.isChecked():
                proto = str(child.text())
                if proto in self.classification['db_association']:
                    filters.append(str(self.classification['db_association'][proto]))

        self.dpi_configuration['filters'] = ' '.join(filters)

        self.client.call('nudpi', 'setConfiguration', self.dpi_configuration)
        # SERVICE
        QDialog.accept(self)

    def initFilters(self):
        ## TODO : simplify-me

        try:
            families = self.classification['family'].keys()
        except NuCentralError:
            return

        self.proto_checkboxs = []
        main_layout = QGridLayout()

        current_filters = [int(proto) for proto in self.dpi_configuration['filters'].split()]

        for category in families:

            protos = self.classification['family'][category].split()
            layout = QGridLayout()

            nb_items = 3
            nb_rows = len(protos) % nb_items

            column = row = 0
            for index in xrange(len(protos)):
                row = index / nb_items
                column = index % nb_items

                proto = protos[index]
                checkbox = QCheckBox(proto)
                layout.addWidget(checkbox, row + 1, column)

                if 'filters' in self.dpi_configuration and self.classification['db_association'][proto] in current_filters:
                    checkbox.setChecked(True)
                column += 1
                self.proto_checkboxs.append(checkbox)

            frame = QGroupBox(category)
            frame.setLayout(layout)
            main_layout.addWidget(frame)

        scroll_area = QScrollArea(self.ui.gb_filters)
        scroll_area.setWidgetResizable(True)

        scroll_area_content = QWidget(scroll_area)
        scroll_area.setWidget(scroll_area_content)
        scroll_area_content.setLayout(main_layout)

        top_layout = QVBoxLayout()
        top_layout.addWidget(scroll_area)
        self.ui.gb_filters.setLayout(top_layout)

    def setChecked(self, state):
        sender = self.sender()
        if sender == self.ui.cb_interfaces:
            self._setNetworkEnabled(False) if state == Qt.Checked else self._setNetworkEnabled(True)
#        else:
#            sender.stateChanged(state, sender.setText(sender.text()))
        self.ui.interfaces.emit(SIGNAL("completeChanged()"))

    def _setNetworkEnabled(self, state):
        self.ui.lb_from.setEnabled(state)
        self.ui.lb_to.setEnabled(state)
        self.ui.cb_from.setEnabled(state)
        self.cb_to.setEnabled(state)

    def pageChanged(self, id):
        if id == -1:
            return

        if self.pages[id] == "interfaces":
            if len(self.cb_to.allItems) > 0:
                return

            self.ui.cb_from.clear()
            self.cb_to.clear()

            for key in sorted(list(self.devs.iterkeys())):
                self.ui.cb_from.addItem(key, QVariant(key))
                if 'from' in self.dpi_configuration and  key in self.dpi_configuration['from']:
                    self.ui.cb_from.setCurrentIndex(self.ui.cb_from.findText(key))

            for key, values in self.internet_network.items():
                for value in values:
                    item = key + ': ' + value
                    self.cb_to.addItem(item)
                    self.cb_to.setData(item, value)
                    if 'rule' in self.dpi_configuration and value in self.dpi_configuration['rule']:
                        self.cb_to.stateChanged(True, str(item))

            if 'from' in self.dpi_configuration and self.dpi_configuration['from'] == "any":
                self.ui.cb_interfaces.setChecked(True)
                self._setNetworkEnabled(False)


    # overrides QWizardPage::isComplete() of the interfaces page
    def isComplete(self):
        # Refresh the "Next button" state
        next_enabled = self.ui.cb_interfaces.isChecked()
        next_enabled |= (self.cb_to.selectedItems != {})
        return next_enabled
