# -*- coding: utf-8 -*-

"""
Copyright (C) 2008-2011 EdenWall Technologies
Written by Romain Bignon <romain AT inl.fr>

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>

$Id: fragtypes.py 13277 2009-10-09 10:28:24Z haypo $
"""

from nucentral.common import tr

from nudpi.client.qt.fetchers import NudpiTableFetcher, \
                                     NulogTableFetcher, \
                                     OcsFetcher, \
                                     NulogStreamFetcher, \
                                     NuAuthFetcher, \
                                     NuConntrackFetcher, \
                                     IDSIPSFetcher, \
                                     RealTimeStreamFetcher



class FragType:
    """ Class to give information of a fragment type """

    def __init__(self, title, fetcher, views):
        """
            @param title [string] the default title of this kind of fragment
            @param views [list] list of available views for this fragment type
        """
        assert isinstance(views, tuple)

        self.title = title
        self.views = views
        self.fetcher = fetcher

frag_types = {
              ###################################################### NUDPI ##############################################################
              'MostUsedProtoTable' : FragType(tr('Top Ten Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'MostUsedProtoTable_http' : FragType(tr('Top Ten Protocols on HTTP'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'MostUsedProtoTable_unencrypted' : FragType(tr('Top Ten Unencrypted Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'MostUsedProtoTable_ssl' : FragType(tr('Top Ten Protocols on SSL'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'MostUsedProtoTable_encrypted' : FragType(tr('Top Ten Encrypted Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'MostUsedProtoTable_unknown' : FragType(tr('Top Ten Unknown Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'MostUsedProtoTable_port80' : FragType(tr('Top Ten Protocols on port 80'), NudpiTableFetcher, ('concentric_pie', 'treeview')),

#              'GetClassification' : FragType(tr('Classification'),  NudpiTableFetcher,  ('concentric_pie','treeview')),
              'AllProtoTable' : FragType(tr('Protocols Family'), NudpiTableFetcher, ('concentric_pie', 'treeview')),

              'HostsOnDportTable' : FragType(tr('Hosts on Destination Port'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'HostsOnDportTable_http' : FragType(tr('Hosts on Destination Port 80'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'HostsOnDportTable_ssl' : FragType(tr('Hosts on Destination Port 443'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'HostsOnDportTable_dropped' : FragType(tr('Dropped Host Port Connections'), NudpiTableFetcher, ('table',)),

              'HostProtoTable' : FragType(tr('Used Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'HostProtoTable_http' : FragType(tr('Used HTTP Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'HostProtoTable_ssl' : FragType(tr('Used SSL Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'HostProtoTable_encrypted' : FragType(tr('Used Encrypted Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'HostProtoTable_unencrypted' : FragType(tr('Used Unencrypted Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),

#              'DportTable' : FragType(tr('Most Used Destination Port'),  NudpiTableFetcher, ('table', 'pie', 'histo')),
              'DportTable_most' : FragType(tr('Most Used Destination Port'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'DportTable_less' : FragType(tr('Least Used Destination Port'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'DportTable_most_encrypted' : FragType(tr('Most Used Destination Port'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'DportTable_most_unencrypted' : FragType(tr('Most Used Destination Port'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              #'DportTable_unknown' : FragType(tr('Most Used Destination Port'),  NudpiTableFetcher, ('table', 'pie', 'histo')),

              'MostUsedSourcesTable' : FragType(tr('Top Ten Sources'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedSourcesTable_http' : FragType(tr('Top Ten HTTP Sources'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedSourcesTable_ssl' : FragType(tr('Top Ten SSL Sources'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedSourcesTable_unencrypted' : FragType(tr('Top Ten Unencrypted Protocols Sources'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedSourcesTable_encrypted' : FragType(tr('Top Ten Encrypted Protocols Sources'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedSourcesTable_unknown' : FragType(tr('Top Ten Unknown Protocols Sources'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedSourcesTable_port80' : FragType(tr('Top Ten Protocols Sources on port 80'), NudpiTableFetcher, ('table', 'pie', 'histo')),

              'MostDroppedSourceTable' : FragType(tr('Most Dropped Sources'), NudpiTableFetcher, ('table', 'pie', 'histo')),

              'MostUsedDestinationTable' : FragType(tr('Top Ten Destinations'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedDestinationTable_http' : FragType(tr('Top Ten HTTP Destinations'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedDestinationTable_ssl' : FragType(tr('Top Ten SSL Destinations'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedDestinationTable_unencrypted' : FragType(tr('Top Ten Unencrypted Protocols  Destinations'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedDestinationTable_encrypted' : FragType(tr('Top Ten Encrypted Protocols Destinations'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedDestinationTable_unknown' : FragType(tr('Top Ten Unknown Protocols Destinations'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedDestinationTable_port80' : FragType(tr('Top Ten Protocols Destinations on dport 80'), NudpiTableFetcher, ('table', 'pie', 'histo')),

              'ProtoOnDportTable' : FragType(tr('Protocol Used on Port'), NudpiTableFetcher, ('concentric_pie', 'treeview')),


              'MostUsedUsersTable' : FragType(tr('Top Ten Users'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedUsersTable_http' : FragType(tr('Top Ten HTTP Users'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedUsersTable_ssl' : FragType(tr('Top Ten SSL Users'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedUsersTable_unencrypted' : FragType(tr('Top Ten Unencrypted Protocols Users'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedUsersTable_encrypted' : FragType(tr('Top Ten Encrypted Protocols Users'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostUsedUsersTable_unknown' : FragType(tr('Top Ten Unknown Protocols Users'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'MostDroppedUserTable' : FragType(tr('Most Dropped Users (DPI)'), NudpiTableFetcher, ('table', 'pie', 'histo')),

              'UserProtoTable' : FragType(tr('Used Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'UserProtoTable_http' : FragType(tr('Used HTTP Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'UserProtoTable_ssl' : FragType(tr('Used SSL Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'UserProtoTable_encrypted' : FragType(tr('Used Encrypted Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),
              'UserProtoTable_unencrypted' : FragType(tr('Used Unencrypted Protocols'), NudpiTableFetcher, ('concentric_pie', 'treeview')),

              'UsersOnDportTable' : FragType(tr('Users on Destination Port'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'UsersOnDportTable_http' : FragType(tr('Users on Destination Port 80'), NudpiTableFetcher, ('table', 'pie', 'histo')),
              'UsersOnDportTable_ssl' : FragType(tr('Users on Destination Port 443'), NudpiTableFetcher, ('table', 'pie', 'histo')),

              'TrafficTable' : FragType(tr('Time Chart'), NudpiTableFetcher, ('static_line',)),
              'ProtoTimeTable' : FragType(tr('Time Chart'), NudpiTableFetcher, ('static_line',)),
              'LastDroppedConnections' : FragType(tr('Last Dropped Connections'), NudpiTableFetcher, ('table',)),

              ###################################################### NULOG ##############################################################
              'TCPTable':          FragType(tr('TCP ports'), NulogTableFetcher, ('table', 'pie', 'histo')),
              'UDPTable':          FragType(tr('UDP ports'), NulogTableFetcher, ('table', 'pie', 'histo')),
              'IPsrcTable':        FragType(tr('Source IP'), NulogTableFetcher, ('table', 'pie', 'histo')),
              'IPdstTable':        FragType(tr('Destination IP'), NulogTableFetcher, ('table', 'pie', 'histo')),
              'UserTable':         FragType(tr('Users'), NulogTableFetcher, ('table', 'pie', 'histo')),
              # Server version 3.0-1
              'PacketTable':       FragType(tr('Connections'), NulogTableFetcher, ('table',)),

              # Server version 3.0-2
              'LastPacket':       FragType(tr('Connections'), NulogTableFetcher, ('table',)),
              'InputPacket':       FragType(tr('Connections'), NulogTableFetcher, ('table',)),

              'AppTable':          FragType(tr('Applications'), NulogTableFetcher, ('table', 'pie', 'histo')),
              'UserAppTable':      FragType(tr('User applications'), NulogTableFetcher, ('table',)),
              'HostPortTable':     FragType(tr('Host port connections'), NulogTableFetcher, ('table',)),
              'UsersHistoryTable': FragType(tr('User session history'), NulogTableFetcher, ('table',)),
              'AuthFail':          FragType(tr('User session history'), NulogTableFetcher, ('table',)),
              'PacketInfo':        FragType(tr('Packet information'), NulogTableFetcher, ('packetinfo',)),
              'Stats':             FragType(tr('Statistics'), RealTimeStreamFetcher, ('stats',)),
              'TrafficStream':     FragType(tr('Traffic (stream)'), NulogStreamFetcher, ('line',)),
              'LastPacketsStream': FragType(tr('Last connections (stream)'), NulogStreamFetcher, ('line',)),
              'LoadStream':        FragType(tr('Load (stream)'), NulogStreamFetcher, ('line',)),
              'MemoryStream':      FragType(tr('Memory usage'), NulogStreamFetcher, ('line',)),
              'ConntrackTable':    FragType(tr('Conntrack'), NuConntrackFetcher, ('table',)),
              'ConUserTable':      FragType(tr('Connected users'), NuAuthFetcher, ('table',)),
              'HostInformation':   FragType(tr('Host information'), OcsFetcher, ('table',)),
              'IDSIPSTable':       FragType(tr('IDS-IPS logs'), IDSIPSFetcher, ('table',)),
              'PacketsCountTable': FragType(tr('Number of connections'), NulogTableFetcher, ('histo',)),
              'ProxyRequestTable': FragType(tr('Proxy requests'), NulogTableFetcher, ('table',)),
              'ProxyDomainsTable': FragType(tr('Proxy top domains'), NulogTableFetcher, ('table', 'pie', 'histo')),
              'ProxyUsersTable':   FragType(tr('Proxy top users'), NulogTableFetcher, ('table', 'pie', 'histo')),
              'ProxyHostsTable':   FragType(tr('Proxy top hosts'), NulogTableFetcher, ('table', 'pie', 'histo')),
             }





