#coding: utf-8

"""
Copyright (C) 2008-2011 EdenWall Technologies
Written by Romain Bignon <romain AT inl.fr>

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from copy import copy, deepcopy
from sys import exit
import random

from PyQt4.QtGui import QMessageBox, QAction, \
                        QInputDialog, QLineEdit, \
                        QIcon, QMenuBar, \
                        QMenu, QActionGroup
from PyQt4.QtCore import SIGNAL, QVariant, Qt, QTimer

from nucentral.common.error import exceptionAsUnicode
from nucentral.common import tr
from nucentral.client import NuCentralError
from nucentral.qt.application import create_nucentral_application
from nucentral.qt.central_window import CentralWindow, EMBEDDED, STANDALONE

from nulog.client.qt.info_area import InfoAreaWidget
from nulog.client.qt.fetchers.base import GenericFetcher

from nudpi.client.qt.args import Args, Interval
from nudpi.client.qt.user_settings.user_settings import UserSettings
from nudpi.client.qt.user_settings.page import Page
from nudpi.client.qt.ui import Ui_DPIWindow

from nudpi.client.qt.config_window import ConfigDialog
from nudpi.client.qt.dbinfo import DBInfoDialog
from nudpi.client.qt.addfrag_window import AddFragDialog
from nudpi.client.qt.range_window import ChooseRangeDialog
from nudpi.client.qt.printer import PrintDialog, RemoteReportDialog as ReportDialog

from nudpi.client.qt.frag_splitter import FragSplitter
from nudpi.client.qt.status_dialog import StatusDialog
from nudpi.client.qt.widgets.filters_list import FiltersListWidget
from nudpi.client.qt.widgets.pages_list import PagesListWidget
from nudpi.client.qt.views.graphics_view import GraphicsView


## Activate the "layout creation" mode.
# When it's true, when a page is instancied from a template (for any
# argument link), if you change the layout of this page, it's applied
# to the template page.
LAYOUT_CREATION = True

class DPIWindow(CentralWindow):
    ROLES = set(('nudpi_read', 'nudpi_write'))
    ICON = ':/icons/dpi.png'

    def __init__(self, application, client, standalone=EMBEDDED, parent=None, eas_window=None):

        CentralWindow.__init__(self, client, parent, eas_window)
        self.use_edenwall = False
        self.is_loaded = False
        self.standalone = standalone

        self.setupCentralWindow(application, standalone)
        self.setWindowIcon(QIcon(":icons/appicon.png"))
        self.ui = Ui_DPIWindow()
        self.ui.setupUi(self)
        self.read_only = None
        self.frag_widgets = []
        self.user_settings = UserSettings(self, client)
#        self.checkBackend()
        self.createMenu(standalone)

        self.timer_id = None
        QTimer.singleShot(0, self.getDPIStatus)

        # pages history (used with the prev/next buttons)
        self.history = {'prev': [],
                        'next': []}

        self.client = client
        self.current_page = None
        self.cumulative_mode = True

        self.interval = Interval('hourly')

        self.dbinfo_dialog = None

        self.ui.info_area = InfoAreaWidget(None)
        self.ui.info_area.setReadOnly(True)
        self.ui.info_area.setObjectName("InfoArea")
        self.ui.info_dock.setWidget(self.ui.info_area)

        self.ui.pages_list = PagesListWidget(self.user_settings, self)
        self.ui.pages_dock.setWidget(self.ui.pages_list)

        self.ui.vertical_splitter = FragSplitter(self.client, self)
        self.ui.scrollArea.setWidget(self.ui.vertical_splitter)

        self.ui.filters_list = FiltersListWidget(client, self)
        self.ui.filters_list.setCompatibility(self.user_settings.compatibility)
#        self.ui.titleLayout.addWidget(self.ui.filters_list)
        self.ui.horizontalLayout_2.insertWidget(3, self.ui.filters_list)
        self.ui.actionDPIConfiguration.setIcon(QIcon(":/icons/settings.png"))

        self.interval.setGUI(self.ui.updateinterval)

        index = self.ui.updateinterval.findData(QVariant(self.getInterval().getMode()))
        if index >= 0:
            self.ui.updateinterval.setCurrentIndex(index)

        self.connectSlots()
        self.updated = False
        self.EAS_MESSAGES['network_modification'] = self.network_modification
        self.acl_conf = self.client.call('acl', 'check', 'nudpi', 'configuration')

    def network_modification(self, from_app, *args, **kwargs):
        self.getDPIStatus(show_message=StatusDialog.IS_STARTED)

    def isReadOnly(self):
        if self.read_only is None:
            self.read_only = ('nudpi_write' not in self.getRoles())
        return self.read_only

    def getDPIStatus(self, show_message=False):
        status = self.client.call('nudpi', 'getDPIStatus')

        if 'started' in status:
            StatusDialog.IS_STARTED = status['started']
        if 'isConfExpired' in status:
            StatusDialog.IS_CONFIG_EXPIRED = status['isConfExpired']
        if 'file_exists' in status:
            StatusDialog.FILE_EXISTS = status['file_exists']

        self._dpiMessage(show_message)
        self.setDPIStatusText()

    def _dpiMessage(self, show_message=True):
        title = ''
        message = ''

        if not StatusDialog.FILE_EXISTS:
            title = tr("No configuration file found")
            message = tr("You need to configure Protocol Analysis before running it")
        elif StatusDialog.IS_CONFIG_EXPIRED:
            title = tr("Network configuration has changed")
            message = tr("Network configuration has changed: please reconfigure protocol analysis to take changes into account")

        if show_message and self.acl_conf:
            if (not StatusDialog.FILE_EXISTS or StatusDialog.IS_CONFIG_EXPIRED):
                QMessageBox.warning(self, title, message)

    def setDPIStatusText(self):

        if not StatusDialog.IS_STARTED:
            text = tr("Start DPI Engine")
            icon = ":/icons-20/status_on.png"
        else:
            text = tr("Stop DPI Engine")
            icon = ":/icons-20/status_off.png"

        self.ui.actionDPIEngineStatut.setText(text)
        self.ui.actionDPIEngineStatut.setIcon(QIcon(icon))

    def setDPIStatus(self):
        status = ''
        message = True

        if self.timer_id:
            self.killTimer(self.timer_id)
            self.timer_id = None

        self.getDPIStatus()
        if not StatusDialog.IS_CONFIG_EXPIRED and StatusDialog.FILE_EXISTS or StatusDialog.IS_STARTED:
            status = 'start' if not StatusDialog.IS_STARTED else 'stop'
            self.client.call('nudpi', 'setDPIStatus', status)
            StatusDialog.IS_STARTED = not StatusDialog.IS_STARTED

            if StatusDialog.IS_STARTED:
                self.timer_id = self.startTimer(30 * 1000)

        message = (status != 'stop')
        self.getDPIStatus(show_message=message)

    def connectSlots(self):
        self.nudpi_actions = [
                (self.ui.actionAddFragment, self.showAddFragmentDialog),
                (self.ui.actionCreateCustomPage, self.createCustomPage),
                (self.ui.actionSearch, self.showSearchDialog),
                (self.ui.actionPrint, self.showPrintDialog),
                (self.ui.actionCreateReport, self.showReportDialog),
                (self.ui.actionResetSettings, self.resetSettings),
                (self.ui.actionQuit, self.close),
                (self.ui.actionClose, self.closePage),
                (self.ui.actionPrevious, self.prevPage),
                (self.ui.actionNext, self.nextPage)]

#        if not self.use_edenwall:
#            self.nudpi_actions.insert(0, (self.ui.actionSettings, self.showConfigDialog))

        for action, slot in self.nudpi_actions:
            self.connect(action, SIGNAL("triggered()"), slot)

        self.connect(self.ui.actionUpdate, SIGNAL('triggered()'), self.updateFragments)
        self.connect(self.ui.actionToggleCumulative, SIGNAL("toggled(bool)"), self.toggleCumulative)
        self.connect(self.ui.pages_list, SIGNAL(PagesListWidget.SIG_OPEN_PAGE), self.loadPage)

        title_action = QAction(tr('Change title...'), self.ui.title)
        self.connect(title_action, SIGNAL('triggered()'), self.changeTitle)
        self.ui.title.addAction(title_action)
        self.ui.frame.addAction(title_action)

        self.connect(self.ui.filters_list, SIGNAL('removeFilter'), self.removeFilter)
        self.connect(self.ui.filters_list, SIGNAL('changeFilter'), self.changeFilter)

        self.connect(self.ui.updateinterval, SIGNAL('activated(int)'), self.intervalChanged)
        self.connect(self.ui.actionDisableAnimation, SIGNAL("triggered(bool)"), self.ui.vertical_splitter.disableAnimation)
        if self.isReadOnly():
            self.ui.actionDPIConfiguration.setEnabled(False)
            self.ui.actionSettings.setEnabled(False)
            self.ui.actionDPIEngineStatut.setEnabled(False)
        else:
            self.connect(self.ui.actionDPIConfiguration, SIGNAL("triggered()"), self.dpiConfiguration)
            self.connect(self.ui.actionSettings, SIGNAL("triggered()"), self.dpiConfiguration)
            self.connect(self.ui.actionDPIEngineStatut, SIGNAL("triggered()"), self.setDPIStatus)

    def createMenu(self, standalone):
        menubar = QMenuBar(self)
        menuFile = QMenu(self.tr("&File"), menubar)
        menuView = QMenu(self.tr("&View"), menubar)
        self.setMenuBar(menubar)

        if not self.use_edenwall:
            menuFile.addAction(self.ui.actionSettings)
            menuFile.addSeparator()
        menuFile.addAction(self.ui.actionSearch)
        menuFile.addAction(self.ui.actionPrint)

        ## TEMP
        #menuFile.addAction(self.ui.actionCreateReport)
        self.ui.toolBar.removeAction(self.ui.actionCreateReport)
        ##

        menuFile.addSeparator()
        menuFile.addAction(self.ui.actionResetSettings)
        if standalone == STANDALONE:
            menuFile.addSeparator()
            menuFile.addAction(self.ui.actionQuit)

        menuOptions = QMenu(self.tr("&Options"), menubar)
        menuOptions.addAction(self.ui.actionDPIConfiguration)
        menuOptions.addAction(self.ui.actionDPIEngineStatut)

        menuView.addAction(self.ui.actionPrevious)
        menuView.addAction(self.ui.actionNext)
        menuView.addAction(self.ui.actionClose)
        menuView.addAction(self.ui.actionCreateCustomPage)
        menuView.addSeparator()
        menuView.addAction(self.ui.actionUpdate)
        menuView.addAction(self.ui.actionToggleCumulative)
        menuView.addAction(self.ui.actionAddFragment)

        self.menuRemoveFragment = QMenu(self.tr("&Delete Fragment"), menuView)
        self.menuRemoveFragment.setEnabled(False)
        menuView.addSeparator()
        self.ui.actionDisableAnimation.setCheckable(True)
        menuView.addSeparator()
        menuView.addAction(self.ui.actionDisableAnimation)

        menuView.addAction(self.menuRemoveFragment.menuAction())
        menuView.addSeparator()

        menubar.addAction(menuFile.menuAction())
        menubar.addAction(menuView.menuAction())
        menubar.addAction(menuOptions.menuAction())

    def load(self):
        try:
            self.user_settings.load()

            # Remove ConUserTable fragment if nuauth_command is not loaded.
            have_nuauth_command = self.client.call('CORE', 'hasComponent', 'nuauth_command')
            if not have_nuauth_command:
                for page in self.user_settings['pages'].pages.values():
                    for frame in page.frames:
                        for j, (frag_name, frag) in enumerate(frame.frags):
                            if frag_name == 'ConUserTable':
                                frame.frags.pop(j)

        except Exception:
            self.disable_ui()
            raise
        else:
            self.is_loaded = True
            self.enable_ui()
            self.loadPage(self.getMainPage())

        if self.user_settings.compatibility.timer and StatusDialog.IS_STARTED:
            time = 30 * 1000 # 30 seconds
            self.timer_id = self.startTimer(time)
        self.getDPIStatus(StatusDialog.IS_STARTED)

    def dpiConfiguration(self):
        status_diag = StatusDialog(self, self.client)
        status_diag.exec_()

    def quit(self):
        # Save user settings
        if hasattr(self, "user_settings"):
            self.user_settings.save()
        for fragwidget in self.frag_widgets:
            fragwidget.destructor()
        CentralWindow.quit(self)

    def disable_ui(self):
        self.ui.frame.setEnabled(False)
        for action, slot in self.nudpi_actions:
            action.setEnabled(False)

    def enable_ui(self):
        self.ui.frame.setEnabled(True)
        for action, slot in self.nudpi_actions:
            action.setEnabled(True)
        if self.use_edenwall:
            self.ui.actionSettings.setVisible(False)

#    def checkBackend(self):
#        self.has_nudpi = False
#        try:
#            self.has_nudpi = self.client.call('nulog','getComponentVersion')
#        except NuCentralError, err:
#            self.error(
#                tr("DPI backend not found! Exiting."),
#                dialog=(self.standalone == STANDALONE))
#            exit(1)
#
#        if not self.client.call('acl', 'check', 'nulog', 'table'):
#            self.error(tr('You have no rights to use the DPI tab. Exiting...'), dialog=self.standalone)
#            exit(1)
#
#        self.use_edenwall = self.client.call('CORE', 'useEdenWall')

    def database_error(self, error):
        """ Called when there is a database error, and ask user to reconfigure the NuLog backend """

        reply = QMessageBox.question(self, self.tr("Database connection is not configured correctly"),
                                           self.tr("The following error has occurred:\n") +
                                           exceptionAsUnicode(error) + u"\n" +
                                           self.tr("Do you want to configure it now?"),
                                           QMessageBox.Yes | QMessageBox.No);
        if reply == QMessageBox.Yes:
            return ConfigDialog(self.client, self).run()

        return False

    def loadPagesList(self):
        self.ui.pages_list.loadPagesList()

    def loadPage(self, page):
        self.emit(SIGNAL("closed"))
        if self.current_page:
            self.history['next'] = []
            self.history['prev'].append(self.current_page.name)
        return self._loadPage(page)

    def _loadPage(self, page):
        """
            Switch page on main window to an other.
            It adds fragments and update them.

            @param page a Page object
        """

        self.ui.pages_list.setEnabled(False)
        self.ui.pages_list.repaint()
        if self.current_page:
            self.client.async().flushQueue()
            frag_widgets = copy(self.frag_widgets)
            for fragwidget in frag_widgets:
                self.removeFragment(fragwidget, delete=False)

            if self.current_page.search_page != page.search_page:
                self.setSearchMode(page.search_page)

        self.current_page = page
        self.loadPagesList()
        self.frag_widgets = []
        if page.title:
            title = page.title
            self.ui.title.setText('<b>%s</b>' % title)
        else:
            title = None
            self.ui.title.setText(tr('<b>(No name)<b>'))
        ctitle = '%s [%s]' % (title, self.interval.display())
        self.setCentralWindowTitle(tr('Protocol Analysis'), self.client, ctitle)

        if page.force_cumulative and self.cumulative_mode:
            self.setCumulative(True)
            self.ui.actionToggleCumulative.setEnabled(True)
        else:
            self.setCumulative(False)
#            self.ui.actionToggleCumulative.setEnabled(bool(page.filters))

        index = self.ui.updateinterval.findData(QVariant(self.getInterval()))
        if index >= 0:
            self.ui.updateinterval.setCurrentIndex(index)

        self.ui.pagePrevButton.setEnabled(bool(self.history['prev']))
        self.ui.pageNextButton.setEnabled(bool(self.history['next']))

        self.menuRemoveFragment.clear()
        for frame in self.current_page.frames:
            for fragname, fragment in frame.frags:
                fragwidget = self.addFragment(fragment, frame.pos)
                fragwidget.pos = frame.pos
                # Create Actions
                self.addFragRemoveAction(fragment.title, fragment.name)
                self.addFragPositionAction(frame, len(self.current_page.frames))

        if len(self.menuRemoveFragment.actions()) > 0:
            self.menuRemoveFragment.setEnabled(True)

        if not self.updated:
            self.updateFragments(reconfigure=True)

        self.ui.pages_list.setEnabled(True)

    def addFragRemoveAction(self, title, name):
        action = QAction(title, self)
        action.setObjectName(name)
        action.setVisible(True)
        self.connect(action, SIGNAL("triggered()"), self.removeSelectedFragment)
        self.menuRemoveFragment.addAction(action)

    def addFragPositionAction(self, frame, nb_frags):
        if len(self.frag_widgets) == 0:
            return

        frag_widget = self.frag_widgets[len(self.frag_widgets) - 1]

        pos = frame.pos

        action_group = QActionGroup(frag_widget)
        for index in xrange(nb_frags):
            action = QAction(str(index + 1), frag_widget)
            action.setData(QVariant(frame.pos))
            action_group.addAction(action)
            frag_widget.pos_menu.addAction(action)
            self.connect(action, SIGNAL("triggered(bool)"), self.positionFragment)
            action.setCheckable(True)

            checked = False
            if index == pos:
                checked = True
            action.setChecked(checked)

    def positionFragment(self, checked):
        try:
            action = self.sender()
            pos = int(action.text()) - 1
            old_pos = action.data().toInt()[0]
        except Exception:
            return

        self.changedPositionFragment(pos, old_pos)

    def changedPositionFragment(self, pos, old_pos):
        offset = 1
        if old_pos - pos < 0:
            offset = -1

        child = self.ui.pages_list.currentItem()
        child_index = self.ui.pages_list.indexFromItem(child)

        page_name = child.data(0, Qt.UserRole).toString()
        parent = child.parent()
        section_name = parent.data(0, Qt.UserRole).toString()

        frames_pos = {}

        pos_changed = False
        for section in self.user_settings['pagesindex'].sections:
            if section.name == section_name:
                for page in section.pages:
                    if page[0] == page_name:
                        for frame in page[1].frames:
                            new_frame = deepcopy(frame)
                            if new_frame.pos == old_pos and not pos_changed:
                                new_frame.pos = pos
                                pos_changed = True
                            else:
                                if offset == -1 and (new_frame.pos >= old_pos and new_frame.pos <= pos) \
                                or offset == 1 and (new_frame.pos <= old_pos and new_frame.pos >= pos):
                                    new_frame.pos = new_frame.pos + offset

                            frames_pos[new_frame.pos] = new_frame

        while len(self.frag_widgets) != 0:
            self.frag_widgets[0].emit(SIGNAL('closed'))
            self.removeFragment(self.frag_widgets[0], True)

        self.menuRemoveFragment.clear()
        for pos, frame in sorted(frames_pos.iteritems()):
            for frag in frame.frags:
                self.current_page.addFragment(frag[1], pos)
                frag_widget = self.addFragment(frag[1], pos)
                if frag_widget.pos != pos:
                    frag_widget.pos = pos
                self.addFragRemoveAction(frag[1].title, frag[1].name)

            self.addFragPositionAction(frame, len(frames_pos))

        self.user_settings.save()
        self.user_settings.load()

        child = self.ui.pages_list.itemFromIndex(child_index)
        self.ui.pages_list.emit(SIGNAL("itemPressed(QTreeWidgetItem*,int)"), child, 0)

    def closePage(self):
        self.ui.pages_list.removePage(self.current_page.name)

    def getMainPage(self):
        # try to get the first page in pages list
        try:
            page_name = self.user_settings['pagesindex'].sections[0].pages[0][0]
        except IndexError:
            page_name = 'main'

        return self.ui.pages_list.getPage(page_name)

    def addFragment(self, fragment, pos_hint= -1):
        """
            Add a fragment on window.

            @param fragment [Fragment] object defined in user_settings.py to describe
                                       fragment properties.
            @param pos_hint [int]  if >= 0, ask to put fragment as a special position.
                                   This is a hint because if there isn't enough fragments,
                                   it'll placed at the next available position.
        """

        fragwidget = self.ui.vertical_splitter.addFragment(fragment, self.current_page.args, pos_hint)
        fragwidget.setCumulative(self.cumulative_mode)

        self.connect(fragwidget, SIGNAL('removeFragment()'), lambda: self.removeFragment(fragwidget, delete=True))
        self.connect(fragwidget, SIGNAL('open_page'), self._open_page)
        self.connect(fragwidget, SIGNAL('add_filter'), self.changeFilter)
        self.connect(fragwidget, SIGNAL('EAS_Message'), self.EAS_SendMessage)
        self.frag_widgets.append(fragwidget)
        self.connect(self, SIGNAL("closed"), fragwidget, SIGNAL("closed"))

        return fragwidget


    def removeFragment(self, fragwidget, delete=False):
        """
            Remove fragment from layout.

            @param fragwidget [QWidget]  removed widget
            @param delete [bool]  if true, delete fragment from page.
        """

        if delete:
            self.current_page.removeFragment(fragwidget.getFragment().name)
            self.current_page.setDefault(False)
            if self.user_settings.isOrphan(fragwidget.getFragment()):
                self.user_settings.removeFragment(fragwidget.getFragment())

        self.disconnect(fragwidget, SIGNAL('open_page'), self._open_page)
        self.disconnect(fragwidget, SIGNAL('add_filter'), self.changeFilter)
        #self.disconnect(fragwidget, SIGNAL('removeFragment()'), None)

        self.frag_widgets.remove(fragwidget)
        self.ui.vertical_splitter.removeFragment(fragwidget)

    ###########################
    #     SLOTS               #
    ###########################

    def changeTitle(self):
        if not self.current_page:
            return

        title, ok = QInputDialog.getText(self, tr('Change view title'),
                                               tr('Enter new title:'),
                                               QLineEdit.Normal,
                                               self.current_page.title)

        if ok:
            title = unicode(title)
            self.current_page.title = title
            self.current_page.setDefault(False)
            self.ui.title.setText('<b>%s</b>' % title)
            self.loadPagesList()

    def showConfigDialog(self):
        if ConfigDialog(self.client, self).run():
            self.updateFragments()

    def showAddFragmentDialog(self):
        try:
            if AddFragDialog(self, None, self).run():
                self.updateFragments()
        except Exception, err:
            self.writeError(err, "Error when adding a fragment")

    def removeSelectedFragment(self):
        current_action = self.sender()

        for frag_widget in self.frag_widgets:
            if current_action.objectName() == frag_widget.fragment.name:
                self.user_settings.removeFragment(frag_widget.fragment)
                self.user_settings.save()
                self.user_settings.load()
                self._open_page(self.current_page.name, self.current_page.args)
                break

    def createCustomPage(self):
        # create an empty page
        name = 'custom'
        while self.user_settings['pages'].pages.has_key(name):
            name = '%s_%d' % (name, random.randint(0, 999999))
        self._open_page(name, {})

    def resetSettings(self):
        reply = QMessageBox.question(self, self.tr("Reset layout settings"),
                                           self.tr("Your history, bookmarks and customized views " \
                                                   "will be deleted.\n" \
                                                   "Default views will also be reseted.\n" \
                                                   "\n" \
                                                   "Are you sure?"),
                                           QMessageBox.Yes | QMessageBox.No);
        if reply == QMessageBox.Yes:
            self.user_settings.reset()
            self.loadPage(self.getMainPage())

    def showSearchDialog(self):
        fragment = self.user_settings.createFragment('PacketTable')
        fragment.type = 'PacketTable'
        fragment.view = 'table'

        window = AddFragDialog(self, fragment, self)
        window.ui.headerWidget.hide()
        window.setWindowTitle(tr("Search"))
        if window.run():
            page = Page("searchPage")
            page.title = tr('Search result')
            page.search_page = True
            page.addFragment(fragment)
            self._open_page(page, fragment.args)
        else:
            self.user_settings.removeFragment(fragment)

    def showPrintDialog(self):
        PrintDialog(self.user_settings, self).run()

    def showReportDialog(self):
        ReportDialog(self.user_settings, self.client, self).run()

    def updateFragments(self, reconfigure=False):
        """
            Update data in fragments
            @param reconfigure [bool] when set, if update fails, we ask user to reconfigure nudpi backend
        """
        if not self.is_loaded:
            self.updated = True
            self.load()

        self.updated = False

        for frag in self.frag_widgets:
            frag.setInterval(self.interval)

        if not self.current_page:
            return

        if 'family' in self.current_page.args and 'proto' in self.current_page.args:
            new_args_filter = copy(self.current_page.args)
            new_args_filter.pop('proto')
        else:
            new_args_filter = self.current_page.args

        self.ui.filters_list.update(new_args_filter, self.current_page.filters)

        while True:
            try:
                GraphicsView.UPDATE_ALL = True
                GraphicsView.IN_ANIMATION = False

                for frag in self.frag_widgets:
                    frag.updateData()
                return
            except NuCentralError, err:
                if reconfigure and err.type == 'DatabaseError':
                    ok = self.database_error(err)
                    if not ok:
                        return
                else:
                    self.nucentralError(err)
                    return

    def toggleCumulative(self):
        self.setCumulative(self.ui.actionToggleCumulative.isChecked())

    def setCumulative(self, c):
        if self.cumulative_mode == c:
            return

        self.cumulative_mode = c
        self.ui.actionToggleCumulative.setChecked(c)

        for frag in self.frag_widgets:
            frag.setCumulative(self.cumulative_mode)

    def setInterval(self, value):
        if value == 'custom':
            window = ChooseRangeDialog(self, self)
            if window.run():
                # TODO Improve write new range somewhere on UI
                message = unicode(self.tr("""<b>Range selected:</b>
                    <table><tr><td><b>Start:</b></td><td>%s</td></tr>
                    <tr><td><b>End:</b></td><td>%s</td></tr></table>""")) % (self.interval.getStart().toString(), self.interval.getEnd().toString())
                self.ui.info_area.setText(message)
                self.updateFragments(reconfigure=True)
            else:
                index = self.ui.updateinterval.findData(QVariant(self.getInterval().getMode()))
                if index >= 0:
                    self.ui.updateinterval.setCurrentIndex(index)
                return
        elif value == 'search':
            self.interval = Interval(value)
            index = self.ui.updateinterval.findData(QVariant(self.getInterval().getMode()))
            if index >= 0:
                self.ui.updateinterval.setCurrentIndex(index)
            return
        if self.interval.getMode() != value:
            self.interval = Interval(value)
            self.updateFragments(reconfigure=True)
            index = self.ui.updateinterval.findData(QVariant(self.getInterval().getMode()))
            if index >= 0:
                self.ui.updateinterval.setCurrentIndex(index)
        ctitle = '%s [%s]' % (self.current_page.title, self.interval.display())
        self.setCentralWindowTitle(tr('Protocol Analysis'), self.client, ctitle)

    def getInterval(self):
        return self.interval

    def setSearchMode(self, value):
        if value == True:
            self.ui.updateinterval.addItem(tr('Search mode'), QVariant('search'))
            self.ui.updateinterval.setEnabled(False)
            self.setInterval('search')
        else:
            index = self.ui.updateinterval.findData(QVariant('search'))
            if index > 0:
                self.ui.updateinterval.removeItem(index)
            self.ui.updateinterval.setEnabled(True)
            self.setInterval('daily')

    def set_page_title(self, page):
        args_label = Args(page.args, GenericFetcher(self.client)).labels()
        if args_label:
            if page.title:
                page.title = tr('%s: %s') % (page.title, args_label)
            else:
                page.title = args_label

    def _open_page(self, page, args):
        """
            Creates a new page window.

            @param page [str/Page]  Page object to use in new window, or the page name.
            @param args [dict]  with fragments filter
        """
        if not isinstance(page, Page):
            page = self.ui.pages_list.getPage(page)

        # create a copy of the main
        if 'custom' in page.name:
            newpage = page
        else:
            newpage = self.user_settings.createPage(page.name, 'history')
            newpage.args = args
        if LAYOUT_CREATION:
            # If we are in the layout creation mode, we keep reference to the default page.
            # This is usefull, because when the layout of the page instance is modified, the
            # template instance is too. It is only true if the page is instancied at this runtime.
            newpage.frames = page.frames
        else:
            newpage.frames = [copy(frame) for frame in page.frames] # copy the list object but not the Fragment instances

        newpage.title = page.title
        newpage.pagelinks = page.pagelinks.copy()
        newpage.filters = copy(page.filters)
        newpage.force_cumulative = page.force_cumulative
        newpage.search_page = page.search_page
#        self.set_page_title(newpage)

        # keep only 10 values in history
        for section in self.user_settings['pagesindex'].sections:
            if section.name == 'history':
                if len(section.pages) > 10:
                    section.pages.pop(0)
                    break

        self.loadPage(newpage)

    def removeFilter(self, filter):
        try:
            if filter == 'family' and 'proto' in self.current_page.args:
                self.current_page.args.pop('proto')
            self.current_page.args.pop(filter)
        except KeyError:
            return

        self.current_page.title = self.current_page.title.split(':')[0]
#        self.set_page_title(self.current_page)
        self.ui.title.setText('<b>%s</b>' % self.current_page.title)

        self.updateFragments(reconfigure=True)

    def changeFilter(self, key, value):
        self.current_page.args[key] = value

        self.current_page.title = self.current_page.title.split(':')[0]
#        self.set_page_title(self.current_page)
        self.ui.title.setText('<b>%s</b>' % self.current_page.title)

        self.updateFragments(reconfigure=True)

    def intervalChanged(self, index):
        if (self.interval.getMode() == 'search'):
            return
        new_interval = self.ui.updateinterval.itemData(index).toString()
        if (self.getInterval() != new_interval):
            self.setInterval(new_interval)

    def nextPage(self):
        try:
            while 1:
                try:
                    page = self.user_settings['pages'].pages[self.history['next'].pop(0)]
                except KeyError:
                    continue
                else:
                    break

            self.history['prev'].append(self.current_page.name)
            self._loadPage(page)
        except IndexError:
            self.ui.pageNextButton.setEnabled(False)
            return

    def prevPage(self):
        try:
            while 1:
                try:
                    page = self.user_settings['pages'].pages[self.history['prev'].pop()]
                except KeyError:
                    continue
                else:
                    break

            self.history['next'].insert(0, self.current_page.name)
            self._loadPage(page)
        except IndexError:
            self.ui.pagePrevButton.setEnabled(False)
            return

if __name__ == "__main__":
    app, client = create_nucentral_application(
        name="nudpi",
        resource="",
        locale="")
    window = DPIWindow(app, client, standalone=STANDALONE)
    window.showMaximized()
    exit(app.exec_())
