from PyQt4.QtGui import QDialog
from PyQt4.QtCore import SIGNAL

from nucentral.common import tr

from nuconf.client.system.network import QNetObject
from nuconf.client.services.roadwarrior import QOpenVpnObject
from nuconf.common.openvpn_cfg import INTERFACE_NAME as OPENVPN_INTERFACE

from nudpi.client.qt.ui.status_dialog_ui import Ui_NudpiStatusDialog
from nudpi.client.qt.dpi_configuration import DpiConfiguration

class StatusDialog(QDialog):
    IS_CONFIG_EXPIRED = False
    IS_STARTED = False
    FILE_EXISTS = False

    def __init__(self, parent, client):
        QDialog.__init__(self, parent)
        self.ui = Ui_NudpiStatusDialog()
        self.ui.setupUi(self)
        self.client = client
        self.connect(self.ui.modify_button, SIGNAL("clicked()"), self.modifyConf)
        self.connect(self.ui.ok_button, SIGNAL("clicked()"), self.accept)

        # Retrieve configuration from nuconf
        self.q_netobject = QNetObject.getInstance()
        self.netcfg = self.q_netobject.netcfg
        if self.q_netobject.netcfg is None:
            self.q_netobject.fetch_data(parent.client)
            self.netcfg = self.q_netobject.netcfg

        self.vpn_object = QOpenVpnObject.getInstance()
        self.vpn_object.openvpn = self.client.call('openvpn', 'getOpenVpnConfig')
        self.vpncfg = self.vpn_object.getCfg()

        self.devs = {}
        self.internet_network = {}

        self.refreshLabels()

    def refreshLabels(self):
        """Refresh network informations"""
        self.devs = {}
        self.internet_network = {'Internet' : ["0.0.0.0/0"]}

        for interface in self.netcfg.iterNetworkables():
            if interface.hasHA():
                continue
            for network in interface.iterNets():
                dev = interface.system_name
                if not dev in self.devs:
                    self.devs[dev] = []
                if not dev in self.internet_network:
                    self.internet_network[dev] = []

                network = unicode(network.net)
                self.devs[dev].append(network)
                self.internet_network[dev].append(network)

        # Add VPN interace / network, when it's configured
        if self.vpncfg is not None and self.vpncfg.enabled:
            self.devs[OPENVPN_INTERFACE] = [ self.vpncfg.client_network ]
            self.internet_network[OPENVPN_INTERFACE] = [ self.vpncfg.client_network ]

        # Set labes string:
        dpi_config = self.client.call('nudpi', 'configuration')
        if dpi_config:
            if 'from' in dpi_config and dpi_config['from'] != '':
                _to = tr('Not configured')
                _from = tr('Not configured')

                if dpi_config['from'] == 'any':
                    _from = tr('All interfaces')
                    _to = tr('All interfaces')
                else:
                    to_list = []

                    for key, value in self.devs.items():
                        if key in dpi_config['from']:
                            _from = key
                            break

                    for key, values in self.internet_network.items():
                        for value in values:
                            item = key + ': ' + value
                        if 'rule' in dpi_config and value in dpi_config['rule']:
                            to_list.append(item)

                    _to = ', '.join(dpi_config['rule'])

                self.ui.from_label.setText(_from)
                self.ui.to_label.setText(_to)

            if not dpi_config['start_engine']:
                filter_state = tr('not started')
            elif len(dpi_config['filters'].split()) != 0:
                filter_state = tr('started')
            else:
                filter_state = tr('no filter defined')

            self.ui.filter_label.setText(filter_state)

    def modifyConf(self):
        conf = DpiConfiguration(self.client, self.devs, self.internet_network, self)
        status = conf.exec_()
        if status == QDialog.Accepted:
            self.refreshLabels()

