from sys import exit

from nucentral.client import NuCentralError
from nucentral.common import tr
from nucentral.qt.central_window import STANDALONE

SUPPORTED_VERSIONS = ("3.0-1", "3.0-2", "3.0-3")
NULOG_CLIENT_VERSION = "3.0-3"
NUDPI_CLIENT_VERSION = "1.0"

class Compatibility:

    COMPONENTS = {"nulog" : NULOG_CLIENT_VERSION,
                  "nudpi" : NUDPI_CLIENT_VERSION,
                  }

    def __init__(self, main_window, client):
        self.client = client
        self.main_window = main_window

        self.nulog_server_version = None
        self.nudpi_server_version = None

        self.getVersions()
        self.getFeatures()

    def getVersions(self):
        self.main_window.use_edenwall = self.client.call('CORE', 'useEdenWall')

        for component in self.COMPONENTS.iteritems():
            self._versions(component)
        self._reportingVersion()

    def _versions(self, component):

        client_attr = {
            'version': component[1],
        }
        try:
            server_attr = self.client.call(component[0], "setupClient", client_attr)
        except NuCentralError, err:
            self.main_window.error(
                tr("Log backend not found! Exiting."),
                dialog=(self.main_window._standalone == STANDALONE))
            exit(1)

        if not self.client.call('acl', 'check', component[0], 'table'):
            exit(1)

        version = server_attr['version']
        if component[0] == "nulog":
            self.nulog_server_version = version
        elif component[0] == "nudpi":
            self.nudpi_server_version = version

    def _reportingVersion(self):
        try:
            self.reporting_server_version = self.client.call('reporting', 'getComponentVersion')
        except NuCentralError, err:
                self.main_window.error(
                    tr("Log backend not found! Exiting."),
                    dialog=(self.main_window._standalone == STANDALONE))
                exit(1)

    def getFeatures(self):
        self.input_packet_frag = True
        self.csvexport_all = True
        self.authfail = True
        self.print_enterprise = True
        self.user_id = False
        self.timer = True

        if not self.nudpi_server_version:
            self.timer = False

        if self.nulog_server_version < '3.0-3':
            self.user_id = True

        if self.nulog_server_version == "3.0-1":
            self.input_packet_frag = False
            self.csvexport_all = False
            self.authfail = False

        if self.reporting_server_version == "3.0-1":
            self.print_enterprise = False


