from PyQt4.QtCore import QObject, SIGNAL

from nudpi.client.qt.views.graphics_view import GraphicsView

class AnimationManager(QObject):

    __views = []
    __done = []

    def __init__(self, mainwindow):
        QObject.__init__(self)
        self.mainwindow = mainwindow
        self.animation_on = True
        self.nb_updated = 0

    def setEnableAnimation(self, animation_on):

        self.animation_on = not animation_on

        frags = self.mainwindow.frag_widgets
        GraphicsView.ANIMATION_ON = self.animation_on

    def addView(self, view):
        view.animation_on = self.animation_on
        if view.uuid not in AnimationManager.__views:
            AnimationManager.__views.append(view.uuid)

        self.connect(view, SIGNAL('animation_done(QString)'), self.startNewAnimation)
        self.connect(view, SIGNAL('replayAnimation'), self.replayAnimation)
        self.connect(view, SIGNAL('resized(QString)'), self.resized)

    def resized(self, uuid):
        AnimationManager.__done = AnimationManager.__views

        self.reset()
        AnimationManager.__views.append(uuid)

    def replayAnimation(self, uuid):
        if uuid in self.__done:
            if uuid not in self.__views:
                AnimationManager.__views.append(uuid)
            AnimationManager.__done.remove(uuid)

    def _addDoneAnimation(self, uuid):
        if uuid not in AnimationManager.__done:
            AnimationManager.__done.append(uuid)
        if uuid in AnimationManager.__views:
            AnimationManager.__views.remove(uuid)

    def startNewAnimation(self, uuid):
        self._addDoneAnimation(uuid)

        if GraphicsView.IN_ANIMATION:
            return

        if len(self.__views) == 0 or not GraphicsView.UPDATE_ALL:
            AnimationManager.__views = AnimationManager.__done
            AnimationManager.__done = []
            GraphicsView.UPDATE_ALL = False
            return

        frags = self.mainwindow.frag_widgets

        is_graph_view = False
        cpt = 0
        while not is_graph_view and cpt < len(AnimationManager.__views):
            next_uuid = AnimationManager.__views[cpt]
            cpt += 1

            for frag in frags:
                if frag.view.uuid == next_uuid:
                    if frag.view.is_graphics_view:
                        is_graph_view = True
                        break
                    else:
                        self._addDoneAnimation(uuid)

        for frag in frags:
            if frag.view.is_graphics_view:
                if self.animation_on:
                    if frag.view.uuid == next_uuid and frag.view.result:
                        GraphicsView.IN_ANIMATION = True
                        frag.view.show()
                        break
                else:
                    GraphicsView.IN_ANIMATION = False
                    frag.view.show()
            else:
                self._addDoneAnimation(frag.view.uuid)

    def init(self):
        AnimationManager.__views = list(set(AnimationManager.__views + AnimationManager.__done))
        AnimationManager.__done = []
        GraphicsView.UPDATE_ALL = True

    def reset(self):
        AnimationManager.__views = []
        AnimationManager.__done = []
        self.nb_updated = 0

    def remove(self, uuid):
        if uuid in AnimationManager.__views:
            AnimationManager.__views.remove(uuid)
            self.nb_updated -= 1
        if uuid in AnimationManager.__done:
            AnimationManager.__done.remove(uuid)
            self.nb_updated -= 1
        GraphicsView.IN_ANIMATION = False
