# -*- coding: utf-8 -*-

"""
Copyright (C) 2008-2011 EdenWall Technologies
Written by Romain Bignon <romain AT inl.fr>

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>

$Id$
"""
from copy import copy
from PyQt4.QtCore import SIGNAL, Qt, QRect, QUuid
from PyQt4.QtGui import QMessageBox, QInputDialog, QLineEdit, QLabel, QApplication, QPainter, QPaintEvent, QImage

from nudpi.client.qt.args import arg_types, Interval
from nudpi.client.qt.user_settings import db_association

class BaseFragmentView:
    state_colours = ['#e6a9ab', '#cae1a5']

    @staticmethod
    def name(): raise NotImplementedError()

    def __init__(self, fetcher):
        self.fetcher = fetcher
        self.connect(self.fetcher, SIGNAL("want_update"), self.requestData)
        self.fragment = fetcher.fragment
        self.cumulative_mode = False
        self.interval = Interval('daily')

        self.reset_start = False
        self.saved_start = 0
        self.is_closed = False

        self.uuid = QUuid.createUuid().toString()

        if hasattr(self.parent().window, 'current_page'):
            self.current_page = self.parent().window.current_page
        else:
            self.current_page = None
        self.user_settings = None

        # XXX this is ugly
        # TODO kill the crap who has written this.
        top_window = self.parent()
        while top_window:
            try:
                self.user_settings = top_window.user_settings
            except AttributeError:
                top_window = top_window.parent()
            else:
                break

        self.title = QLabel()
        self.title.setAlignment(Qt.AlignLeft | Qt.AlignTop)
        self.title.setStyleSheet('font: bold 13px')

    def setClosed(self):
        self.is_closed = True

    def destructor(self):
        pass

    def getServerTime(self):
        self.fetcher.getTime(self.requestData)

    def requestData(self, time):

        self.fetcher.my_args = {}
        fetcher_args = {}

        if self.fetcher.type == 'streaming' or self.fetcher.type == 'real-time':
            self.fetcher.fetch(self.updateData)
            return

        if self.interval != None and self.interval.getMode() != 'search':
            self.interval.setServerTime(int(float(time)))

            starttime = self.interval.getStart().toTime_t()
            endtime = self.interval.getEnd().toTime_t()
            time_args = {'start_time': starttime, 'end_time': endtime}
            self.fragment.args.update(time_args)
            starttime = str(self.interval.getStart().toString(Qt.ISODate))
            endtime = str(self.interval.getEnd().toString(Qt.ISODate))
            time_args = {'session_start_time': starttime, 'session_end_time': endtime}
            self.fragment.args.update(time_args)

            arg = []
            args = []
            has_proto = False

            for key in self.fetcher.args.keys():
                if key == 'proto':
                    if 'proto' in self.fetcher.args or len(args) > 0:
                        has_proto = True
                        for my_key, value in db_association.iteritems():

                            if len(args) == 0 and key in self.fetcher.args:
                                fetcher_args[key] = copy(self.fetcher.args[key])
                                args = self.fetcher.args.pop(key).split(' ') if isinstance(fetcher_args[key], str) else [self.fetcher.args.pop(key)]
                            for my_arg in args:
                                if value == my_arg:
                                    arg.append(my_key)
                    elif not 'family' in self.fetcher.args:
                        arg.append(self.fetcher.args[key])

            if has_proto:
                if len(arg) == 1:
                    arg = arg[0]
                elif len(arg) == 0:
                    arg = -1
                self.fetcher.my_args.update({'proto' : arg})

            if 'cpu' in self.fragment.type:
                delta_error = 0
                if self.interval.getMode() == "monthly":
                # 1 hour on one month : RRD does not have last data from current time
                    delta_error = 3200
                self.fetcher.delta = self.interval.getEnd().toTime_t() - delta_error - self.interval.getStart().toTime_t()

        self.fetcher.fetch(self.updateData)
        self.fetcher.args.update(copy(fetcher_args))
#
    def updateData(self, result): raise NotImplementedError()
#
#    def isPrintable(self): return False
    def isReady(self): return True

    def getTitle(self): return self.fragment.title

    def getActions(self): return []

    def ask_to_create_page(self, pagename):
        reply = QMessageBox.question(None, self.tr("Create a view"),
                                           unicode(self.tr(
"""There is no page linked to this kind of data:
    %s
Do you want to create it?""")) % pagename, QMessageBox.Yes | QMessageBox.No)

        if reply != QMessageBox.Yes:
            return None

        return self.createPage(pagename)

    def createPage(self, field):
        ok = False
        try:
            title = arg_types[field].label
        except KeyError:
            title = field

        title, ok = QInputDialog.getText(None, self.tr('Create a page'),
                                           self.tr('Please enter the new page title:'),
                                           QLineEdit.Normal,
                                           title)

        if ok and title:
            page = self.user_settings.createPage(field)
            page.title = unicode(title)
            arg_types[field].add_pagelink(page.name)
            return page

        return None

    def loadPage(self, field, label, value, pagename):
        if not self.current_page:
            return

        arg_data = arg_types[field].data(field, (label, value), transform_label=False)

        if not pagename:
            pagename = self.current_page.get_pagelink_default(field, arg_data)

        if not pagename:
            return

        args = self.current_page.get_pagelink_args(field, arg_data, self.fetcher.args)

        if field == 'proto' and hasattr(self, 'classification'):
            self.checkFamilyProto(args)

        self.emit(SIGNAL("open_page"), pagename, args)
#        if field in args:
#            args.pop(field)

    def checkFamilyProto(self, args):
        if 'family' in self.classification and args['proto'] in self.classification['family']:
            args['family'] = args['proto']
            args['proto'] = self.classification['family'][args['proto']]
        return args

    def setCumulative(self, cumulative):
        self.cumulative_mode = cumulative
#
    def setInterval(self, interval):
        self.interval = interval
#
#
    def getToolspace(self):
        return self.title

    def printMe(self, painter, rect):
        # redirect table's painting on a pixmap
        pixmap = QImage(self.viewport().size(), QImage.Format_ARGB32)
        pixmap.fill(0)

        QPainter.setRedirected(self.viewport(), pixmap)
        event = QPaintEvent(QRect(0, 0, self.viewport().width(), self.viewport().height()))
        QApplication.sendEvent(self.viewport(), event)
        QPainter.restoreRedirected(self.viewport())

        # print scaled pixmap
        pixmap = pixmap.scaled(rect.width(), rect.height(), Qt.KeepAspectRatio, Qt.SmoothTransformation)
        painter.drawImage(rect, pixmap, pixmap.rect())
