
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""
import socket
from math import cos, sin, radians

from PyQt4.QtGui import QGraphicsScene, \
                        QPen, \
                        QGraphicsEllipseItem, \
                        QBrush, \
                        QGraphicsLineItem, \
                        QGraphicsTextItem, \
                        QFont

from PyQt4.QtCore import Qt, \
                         SIGNAL, \
                         QRectF

from nucentral.common import tr
from nucentral.common.human import humanFilesize
from nucentral.qt.html import escapeHTML
from nulog.client.qt.views.nulog_base import C_PIECHART, TREEVIEW

from nudpi.client.qt.args import arg_types
from nudpi.client.qt.views.baritem import BarItem
from nudpi.client.qt.views.item import Item
from nudpi.client.qt.views.pieitem import PieItem
from nudpi.client.qt.widgets.tree_model import TreeItem

class GraphicsScene(QGraphicsScene):
    def __init__(self, parent):
        QGraphicsScene.__init__(self, parent)

        self.pen_selected = QPen()
        self.pen_selected.setWidth(3)

        self.pen_unselected = QPen()
        self.pen_unselected.setWidth(1)

        self.text_font = QFont()
        self.text_font.setPointSizeF(10)

        self.last_selected = None
        self.mousePress = False
        self.parent_view = parent
        self.last_color = None
        self.legend = None

        self.setItemIndexMethod(QGraphicsScene.BspTreeIndex)
        self.setSceneRect(-600, -600, 1200, 1200)
        self.compatibility = parent.user_settings.compatibility
        self._items = []

    def setTextSize(self, size):
        self.text_font.setPointSizeF(size)
    def getTextSize(self):
        return self.text_font.pointSizeF()

    textSize = property(getTextSize, setTextSize)


    def addObject(self, item):
        self.addItem(item)
        self._items.append(item)

    def reset(self):
        self.clear()
        self._items = []

    def _loadPageStandardItem(self, item):
        header = self.parent_view.my_model.horizontalHeaderItem(0).text()
        name = self.parent_view.arg_types[unicode(header)]
        types = arg_types[name]

        if len(types.pagelinks) > 0:
            field = types.pagelinks[0]
            pos = item.pos

            label = unicode(self.parent_view.my_model.item(pos, 0).text())
            try:
                value = int(label)
            except ValueError, err:
                if name == 'tcp_dport':
                    value = socket.getservbyname(label)
                else:
                    value = label

            if label != 'Other':
                if 'username' in self.parent_view.result['columns'] and self.compatibility.user_id:
                    for row in range(len(self.parent_view.data)):
                        if unicode(self.parent_view.data[row][0][0]) == label:
                            value = self.parent_view.data[row][0][1]
                            break
                if self.parent_view.cumulative_mode:
                    arg_data = self.parent_view.my_model.data(self.parent_view.my_model.index(pos, 0), Qt.UserRole).toPyObject()
                    field = unicode(self.parent_view.columns[0])
                    filters = self.parent_view.fetcher.getArgs()

                    if field in filters:
                        self.parent_view.emit(SIGNAL('add_filter'), field, value)
                    else:
                        self.parent_view.loadPage(field, arg_data.label, arg_data.value, pagename=None)
                else:
                    self.parent_view.loadPage(field, label, value, pagename=None)

    def _loadPageTreeItem(self, item):

        header = self.parent_view.my_model.rootItem.data(0)
        name = self.parent_view.arg_types[unicode(header)]
        types = arg_types[name]

        if len(types.pagelinks) > 0:
            field = types.pagelinks[0]
#            pos = item.pos

#            label = unicode(self.parent_view.my_model.item(pos, 0).text())
            treeitem_name = item.data(TreeItem.ITEM_NAME).toString()
            label = unicode(treeitem_name)
            value = label

            if self.parent_view.cumulative_mode:
                field = unicode(name)
                filters = self.parent_view.fetcher.getArgs()

                if field in filters:
                    self.parent_view.emit(SIGNAL('add_filter'), field, value)
                else:
                    self.parent_view.loadPage(field, label, value, pagename=None)
            else:
                self.parent_view.loadPage(field, label, value, pagename=None)

    def mouseDoubleClickEvent(self, mouseEvent):

        selected = self.itemAt(mouseEvent.scenePos())

        if selected and (isinstance(selected, BarItem) or isinstance(selected, PieItem)):
            item = self.selectItem(selected)

            if item:
                if self.parent_view.chart_type != C_PIECHART and \
                        self.parent_view.chart_type != TREEVIEW:
                    self._loadPageStandardItem(item)
                else:
                    self._loadPageTreeItem(item)


    def getTotalCount(self):
        total = 0
        for item in self.items():
            if isinstance(item, PieItem) and item.type == C_PIECHART:
                item = item.data(TreeItem.ITEM)
            if hasattr(item, 'value'):
                total = total + int(item.value)
        return total

    def _setInfoStandardModel(self, selected, results):

        pos = selected.pos
        title = self.parent_view.my_model.horizontalHeaderItem(0).text()

        value = selected.value
        if self.parent_view.my_model.item(pos, 0):
            key = unicode(self.parent_view.my_model.item(pos, 0).text())
        else:
            key = ""
        unit = arg_types[self.parent_view.columns[1]].unit
        percent = value / self.getTotalCount() * 100

        if self.parent_view.columns[1] == "packets" or self.parent_view.columns[1] == 'nb_dropp':
            value = str(int(value)) + " " + unit
        else:
            value = humanFilesize(value)

        if int(percent) == 0:
            percent = escapeHTML('< 1')
        else:
            percent = '%i' % percent

        additional = u''
        try:
            sort_col = self.parent_view.columns.index(self.parent_view.result['args']['sortby'])
        except (KeyError, ValueError):
            pass
        else:
            # XXX 1 is an hardcoded value. For now, only the second column is
            # graphable (labelised by first column).
            # When it'll be possible to graph an other column, it must check
            # if sort_col is different than label column (first) and graphed
            # column.
            if sort_col > 1:
                label = tr(arg_types[self.parent_view.columns[sort_col]].label)
                data = self.parent_view.my_model.data(self.parent_view.my_model.index(selected.pos), sort_col)
                additional = "<b>%s</b>: %s" % (label, unicode(data.toString()))

        results['key'] = key
        results['value'] = value
        results['percent'] = percent
        results['additional'] = additional
        results['title'] = title

    def _setInfoTreeModel(self, selected, results):

        treeitem = selected.data(TreeItem.ITEM).toPyObject()
        rootnode = self.parent_view.my_model.rootItem
        parent = rootnode.childItems[0]
        title = rootnode.data(0)

        value = treeitem.value
        key = selected.data(0).toString()
        percent = float(value) / float(rootnode.value) * 100.0

        value = humanFilesize(value)

        if int(percent) == 0:
            percent = escapeHTML('< 1')
        else:
            percent = '%i' % percent

        results['key'] = key
        results['value'] = value
        results['percent'] = percent
        results['additional'] = ''
        results['title'] = title

    def _setInfoArea(self, results, percent_text):

                self.parent_view.info_area.setText("""
                     <table>
                     <tr style="vertical-align: top;">
                     <td style="padding-left: 10px;" >
                     <b>%s :</b> %s<br>
                     <b>%s :</b>
                     <ul>
                     <li><b>%s</b></li>
                     <li>%s</li>
                     %s
                     </ul>
                     </td>
                     </tr>
                     </table>""" % (results['title'], results['key'], tr("Statistics"), results['value'],
                                    percent_text, '<li>%s</li>' % results['additional']))


    def _setLegend(self, selected, info_area):

        results = {'key': '', 'value' : "", 'percent' : '', 'additional' : '', 'title' : ''}

        if isinstance(selected, PieItem) and selected.type == C_PIECHART:
            self._setInfoTreeModel(selected, results)
        else:
            self._setInfoStandardModel(selected, results)

        try:
            percent_text = tr("<b>%s %%</b> ") % (results['percent'])

            if info_area:
                self._setInfoArea(results, percent_text)

            legend = '<b><u>%s </u> : </b> %s (%s)' % (results['key'], percent_text, results['value'])

            if isinstance(selected, PieItem) and not self.parent_view.legend:
                self.drawLegend(selected, legend)
            else:
                selected.setToolTip(legend)

        except Exception, err:
            print err

    def mousePressEvent(self, mouseEvent):
        if (mouseEvent.button() != Qt.LeftButton):
            return

        selected = self.itemAt(mouseEvent.scenePos())
        if selected and (isinstance(selected, PieItem) or isinstance(selected, BarItem)):
            self.selectItem(selected)
            self.unselected(selected)
        else:
            self.mousePress = True
        QGraphicsScene.mousePressEvent(self, mouseEvent)



    def mouseReleaseEvent(self, mouseEvent):
        if self.mousePress:
            self.mousePress = False


    def mouseMoveEvent(self, mouseEvent):
        selected = self.itemAt(mouseEvent.scenePos())
        last_selected = self.itemAt(mouseEvent.lastScenePos())

        # ************** Last Item ********************
        if last_selected:
            try:
                if hasattr(last_selected, 'color'):
                    last_selected.setBrush(last_selected.color)
                else:
                    if self.legend:
                        self.removeItem(self.legend)
                        self.legend = None
            except Exception, err:
                # TODO
                pass

        # ****************** mouse grab *******************
        if selected is None or (not isinstance(selected, PieItem) and not isinstance(selected, BarItem)):
            self.parent_view.setDragMode(self.parent_view.ScrollHandDrag)
        else:
            self.parent_view.setDragMode(self.parent_view.NoDrag)

        # ****************** Legend Item *******************
        try:
            for item in self.items():
                if hasattr(item, 'temp_item') and item.temp_item:
                    self.removeItem(item)
            self.temp_items = []
        except Exception, err:
            # TODO
            self.temp_items = []

        # ************** Selected Item ********************
        if selected and isinstance(selected, Item):
            try:
                info_area = True
                selected.setBrush(selected.darkColor)
                self._setLegend(selected, info_area)

                info_area = False

                if not isinstance(selected, TreeItem) and not isinstance(selected, PieItem):
                    return

                s_treeitem = selected.data(TreeItem.ITEM).toPyObject()
                parent = s_treeitem.parent()

                for item in self.items():
                    item.visible = True

                intersect_items = selected.collidingItems(Qt.IntersectsItemShape)
                for item in intersect_items:
                    item.visible = False

                spanAngle = selected.spanAngle
                for item in self.items():
                    if isinstance(item, Item):
                        treeitem = item.data(TreeItem.ITEM).toPyObject()
                        if treeitem.parent() != parent or s_treeitem == treeitem:
                            continue

                        if spanAngle + item.spanAngle > 15.0 and \
                                (item.visible or (not item.visible and selected.spanAngle + item.spanAngle > 15.0)):
                            self._setLegend(item, info_area)
                        spanAngle = item.spanAngle
            except Exception, err:
                # TODO
                return

    def selectItem(self, selected):
        if selected in self.items():
            if self.last_selected and self.last_selected == selected:
                selected.setPen(self.pen_unselected)
                selected.setSelected(False)
                self.last_selected = None
            else:
                selected.setPen(self.pen_selected)
                selected.setSelected(True)
                self.last_selected = selected
        return selected

    def unselected(self, selected):
        for item in self.items():
            if item != selected:
                if (isinstance(item, PieItem) or \
                    isinstance(item, BarItem)):

                    item.setPen(self.pen_unselected)
                    item.setSelected(False)

    def drawLegend(self, selected, legend):

        if selected.chart_type == C_PIECHART:
            treeitem = selected.data(TreeItem.ITEM).toPyObject()
            init_line = (selected.rect().height() / 2.0) - (selected.rect().height() / 2.0) / (treeitem.depth * 2.0)
            radius_max = self.parent_view.radius_max

            start_angle = treeitem.start_angle
            span = treeitem.span
        else:
            init_line = selected.boundingRect().width() / 4
            radius_max = selected.boundingRect().width() / 2
            start_angle = selected.startAngle
            span = selected.spanAngle

        # **********Draw ellipse to point selected item **********
        span_ellipse_legend = start_angle + (span / 2.0)
        x_ellipse_legend = cos(radians(span_ellipse_legend)) * init_line
        y_ellipse_legend = -sin(radians(span_ellipse_legend)) * init_line

        rect = QRectF(-1, -1, 2, 2)
        ellipse_legend = QGraphicsEllipseItem(rect)
        ellipse_legend.moveBy(x_ellipse_legend, y_ellipse_legend)
        ellipse_legend.setZValue(26)
        ellipse_legend.setBrush(QBrush(Qt.black, Qt.SolidPattern))
        ellipse_legend.temp_item = True

        self.addItem(ellipse_legend)

        # **********Draw first line **********
        x1_line_legend = ellipse_legend.pos().x()
        y1_line_legend = ellipse_legend.pos().y()

        scale = 1.1
        x2_line_legend = cos(radians(span_ellipse_legend)) * radius_max * scale
        y2_line_legend = -sin(radians(span_ellipse_legend)) * radius_max * scale

        line1 = QGraphicsLineItem(x1_line_legend, y1_line_legend, x2_line_legend, y2_line_legend)
        line1.setZValue(26)
        line1.temp_item = True
        self.addItem(line1)

        # **********Draw second line **********
        x1_line2_legend = x2_line_legend
        y1_line2_legend = y2_line_legend

        scale = 1.5
        x2_line2_legend = x1_line2_legend
        y2_line2_legend = y1_line2_legend

        offset = 30
        if cos(radians(span_ellipse_legend)) < 0:
            offset *= -1
        if span_ellipse_legend > 185.0 or span_ellipse_legend < 175.0:
            x2_line2_legend += offset

        line2 = QGraphicsLineItem(x1_line2_legend, y1_line2_legend, x2_line2_legend, y2_line2_legend)
        line2.setZValue(26)
        line2.temp_item = True
        self.addItem(line2)

        # **************** Draw text legend ***********
        x_offset = 0
        textItem = QGraphicsTextItem()
        textItem.setHtml(legend)
        textItem.setFont(self.text_font)
        self.addItem(textItem)

        if offset < 0:
            x_offset = -textItem.boundingRect().width()

        textItem.moveBy(x2_line2_legend + x_offset, y2_line2_legend - textItem.boundingRect().height() / 2)
        textItem.setZValue(26)
        textItem.temp_item = True
