# -*- coding: utf-8 -*-

"""
Copyright (C) 2008-2011 EdenWall Technologies
Written by Romain Bignon <romain AT inl.fr>

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>

$Id: nudpi_base.py 13992 2009-11-03 08:37:23Z eric $
"""

from __future__ import with_statement

from PyQt4.QtGui import QStandardItemModel, QWidget, QHBoxLayout, QVBoxLayout
from PyQt4.QtGui import QColor, QFileDialog
from PyQt4.QtCore import Qt, QVariant, SIGNAL, QModelIndex

from nulog.client.qt.views.nulog_base import C_PIECHART, TREEVIEW

from nudpi.client.qt.args import arg_types
from nudpi.client.qt.tools import createLink, new_tab
from nudpi.client.qt.user_settings.db_association import db_association
from nudpi.client.qt.views.base import BaseFragmentView
from nudpi.client.qt.widgets.tree_model import TreeModel

from nulog.client.qt.args.argdata import ArgDataBase

class NudpiBaseFragmentView(BaseFragmentView):

    def __init__(self, fetcher):
        BaseFragmentView.__init__(self, fetcher)

        self.my_model = None
        self.data = []
        self.result = None
        self.ready = False
        self.columns = []
        self.filters = None
        self.arg_types = {}
        self.keep_python_ref = []
        self.is_closed = False

        if self.fetcher.fragment.name == 'AllProtoTable' and 'classification' in self.fetcher.args and self.fetcher.args['classification']:
            self.classification = None
            self.fetcher.fetchClassification(self.updateClassificationData)

    def isReady(self): return self.ready
#
    def setClosed(self):
        self.is_closed = True

    def label_changed(self, arg_data, row, col):
        self.setUpdatesEnabled(False)
        # row is the line number, and line is the content (a list)
        for row, line in enumerate(self.data):
            self.add_line(row, line)

        self.setUpdatesEnabled(True)
#
    def get_model_label(self, row, col, arg_data):
        return arg_data.label

    def add_line(self, row, line):
        def make_lambda(func, *args):
            return lambda: func(*args)

        id = ''
        if len(line) < len(self.columns):
            line.extend(['' for i in xrange(len(self.columns) - len(line))])
        for col, cell in enumerate(line):
            if 'ProtoTimeTable' not in self.fragment.name:
                if hasattr(self, "user_settings") and self.user_settings:
                    arg_data = arg_types[self.columns[col]].data(self.columns[col], cell, self.fetcher, self.user_settings.compatibility)
                else:
                    arg_data = arg_types[self.columns[col]].data(self.columns[col], cell, self.fetcher, parent=self)
                if col == 0:
                    id = unicode(arg_data.value)
            else:
                arg = 'proto_traffic'
                proto_args = []
                if isinstance(self.columns[col], list) and isinstance(self.columns[col][0], int):
                    for proto in self.columns[col]:
                        if proto in db_association.keys():
                            proto_args.append(db_association[proto])
                    self.columns[col] = proto_args

                if hasattr(self, "user_settings") and self.user_settings:
                    arg_data = arg_types[arg].data(self.columns[col], cell, self.fetcher, self.user_settings.compatibility)
                else:
                    arg_data = arg_types[arg].data(self.columns[col], cell, self.fetcher, parent=self)

            # We store label in the EditRole which is used by QTableView to display it. The UserRole is used to
            # save the value, to be used when we'll open a new page with this info
            model = {}
            index = self.my_model.index(row, col, QModelIndex())
            label = self.get_model_label(row, col, arg_data)

            if isinstance(label, QWidget) and hasattr(self, 'setIndexWidget'):
                if self.indexWidget(index):
                    self.indexWidget(index).setParent(None)
                self.setIndexWidget(index, label)
                self.connect(label, SIGNAL('want_update'), self.requestData)
                self.connect(arg_data, SIGNAL('EAS_Message'), self.EAS_SendMessage)
            else:
                if hasattr(self, 'setIndexWidget') and self.indexWidget(index):
                    self.indexWidget(index).setParent(None)
                    self.setIndexWidget(index, None)

                if label is None: label = ''
                model[Qt.EditRole] = QVariant(unicode(label))

            self.connect(arg_data, SIGNAL('label_changed'), make_lambda(self.label_changed, arg_data, row, col))

            self.keep_python_ref.append(arg_data)
            model[Qt.UserRole] = QVariant(arg_data)

            if 'states' in self.result and id in self.result['states']:
                model[Qt.BackgroundRole] = QVariant(QColor(self.state_colours[self.result['states'][id] % len(self.state_colours)]))

            self.my_model.setItemData(index, model)

    def EAS_SendMessage(self, *args, **kwargs):
        self.emit(SIGNAL('EAS_Message'), *args, **kwargs)

    def requestData(self, time=None):
        self.ready = False
        try:
            self.emit(SIGNAL('updating'))
        except RuntimeError, e:
            # This object seems to be removed, so I leave.
            return

        if time or self.fetcher.type == 'streaming' or self.fetcher.type == 'real-time':
            BaseFragmentView.requestData(self, time)
        else:
            BaseFragmentView.getServerTime(self)

    def setStandardModel(self, result):

        self.data = self.result['table']
        self.rowcount = self.result['rowcount']

        if not self.my_model or len(self.columns) != len(self.result['columns']):
            self.columns = self.result['columns']
            self.my_model = QStandardItemModel(0, len(self.columns), self)
            self.setModel(self.my_model)

        # Create readable legend for ProtoTimeTable fragment
        if "ProtoTimeTable" in self.fragment.name:
            self.columns = self.result['columns']

            if self.columns and self.columns[0] > 0:
                columns = []
                for list_ in self.columns:
                    new_list = []
                    if not isinstance(list_, list):
                        continue
                    for number in list_:
                        new_list.append(db_association[number])
                    if len(new_list) > 0:
                        new_list = ".".join(new_list)
                        new_list = new_list.split('base.ip.')[1] if 'base.ip.' in new_list else new_list
                        columns.append(new_list)

                columns.insert(0, self.columns[0])
                self.columns = columns
        # ******

        # Set column titles
        for col, name in enumerate(self.columns):
            title = ''
            try:
                if isinstance(name, list):
                    title = str(name)
                else:
                    title += arg_types[name].label
                    self.arg_types[title] = name
            except KeyError:
                title += name
                self.arg_types[title] = name

            self.my_model.setHeaderData(col, Qt.Horizontal, QVariant(title))

        # If this is not an iterable table (with 'start' and 'limit' args), hide useless links.
        if not self.result['args'].has_key('start'): # or not self.result['args'].has_key('limit'):
            self.firstLink.hide()
            self.prevLink.hide()
            self.nextLink.hide()
            self.lastLink.hide()
        else:
            if self.result['args']['start'] == 0:
                self.firstLink.hide()
                self.prevLink.hide()
            else:
                self.firstLink.show()
                self.prevLink.show()

            if self.result['args']['limit'] > len(self.data):
                self.nextLink.hide()
                self.lastLink.hide()
            else:
                self.nextLink.show()
                # XXX TODO As it currently does NOT work, as the total entries numbers is
                # missing, this link is always hidden.
                self.lastLink.hide()

        if self.my_model.rowCount:
            if self.my_model.rowCount(QModelIndex()) != len(self.data):
                self.my_model.removeRows(0, self.my_model.rowCount(QModelIndex()), QModelIndex())
                self.my_model.insertRows(0, len(self.data), QModelIndex())

        # cleanup
        children = self.findChildren(ArgDataBase)
        for child in children:
            child.setParent(None)
            child.deleteLater()

        self.setUpdatesEnabled(False)
        # row is the line number, and line is the content (a list)
        del self.keep_python_ref[:]
        for row, line in enumerate(self.data):
            self.add_line(row, line)

        self.setUpdatesEnabled(True)
        self.ready = True

        if len(self.data) > 0:
            return True
        return False

    def setTreeModel(self, results):

        # Do not need to show those elements for treeview
        self.firstLink.hide()
        self.prevLink.hide()
        self.nextLink.hide()
        self.lastLink.hide()

        if 'table' in results and len(results['table']) > 0 and len(results['table'][0]) > 0 and  len(results['table'][0][0]) == 0:
            self.setModel(self.my_model)
            return

        results['table'] = self._maping(results)
        self.total_depth = self._depth(results['table'])

        for name in results['columns']:
            title = ''
            try:
                title += arg_types[name].label
                self.arg_types[title] = name
            except KeyError:
                title += name
                self.arg_types[title] = name

        self.my_model = TreeModel(results)
        self.setModel(self.my_model)

        if len(results['table']) > 0:
            return True
        return False

    def _checkResults(self, results):
        if not 'table' in results:
            return False

        table = results['table']

        for row in table:
            if len(row) > 0 and len(row[0]) > 0:
                if not isinstance(row[0][0], int) and not isinstance(row[0][0], float):
                    return False
        return True

    def _maping(self, results):
        if not self._checkResults(results):
            return results['table']

        data = results['table']
        my_results = []
        has_proto = False
        start = True
        proto = ''
#
        if ('is_base' not in results['args'] or results['args']['is_base'])  and 'proto' in results['args']:
            has_proto = True
            start = False
            proto = results['args']['proto']

        for line in data:
            map = []
            for number in line[0]:
                if number == 3:
                    continue
                if has_proto and number == proto:
                    map.append(db_association[line[0][0]])
                    start = True
                if start:
                    if isinstance(number, int):
                        map.append(db_association[number])
            if has_proto:
                start = False
            my_results.append(map)
            map.append(line[1])

        return my_results

    def _depth(self, data):
        depth = 0

        for line in data:
            if len(line) > depth:
                depth = len(line)

        return depth

    def _updated(self):
        try:
            self.emit(SIGNAL('updated'))
            self.emit(SIGNAL("showButtons"))
        except RuntimeError, e:
            # This object is removed, probably because user changes tab before
            # receiving data for this fragment.
            # It doesn't matter, just leave this method, and python object
            # will be completly removed.
            return

    def updateData(self, result):
        if self.is_closed or not 'table' in result:
            self._updated()
            return False

        self._updated()

        if 'args' in result:
            if 'proto' in result['args']:
                result['args'].pop('proto')

            if self.fragment.name == "TrafficTable" or "ProtoTimeTable" in self.fragment.name:
                result = self._harmonizeTimeRange(result)

        self.result = result
        if hasattr(self, 'chart_type'):
            if self.chart_type == C_PIECHART or self.chart_type == TREEVIEW:
                if hasattr(self, 'classification') and self.classification:
                    if self.fragment.name == 'AllProtoTable' and 'classification' in self.fetcher.args and self.fetcher.args['classification']:
                        self.result['table'] = self._builClassification()
                        self.setTreeModel(result)
                        self.redraw()
                        return True

                return self.setTreeModel(result)
            else:
                return self.setStandardModel(result)

    def _harmonizeTimeRange(self, result):
        data = result['table']
        if not data or not data[0]:
            return result

        max_range = 0
        intervals = [30, # seconds for quarter
                     60, # seconds for hour
                     1440, # seconds for day
                     10080, # seconds for week
                     89280, # seconds for month
                     1051200, #seconds for year
                     ]

        time_range = [15 * 60,
                      60 * 60,
                      1440 * 60,
                      10080 * 60,
                      89280 * 30,
                      1051200 * 30,
                      ]

        # get maximum time range => allows to harmonize
        size = len(data[0])
        for index in xrange(0, len(data) - 1):
            if index == len(data) - 1:
                break

            item1 = data[index]
            item2 = data[index + 1]

            delta = 30
            for range in intervals:
                if item2[0] - item1[0] < range:
                    break
                else:
                    delta = range

            limit = time_range[intervals.index(delta)]
            cpt = delta

            tab_index = index + 1
            while cpt < limit and item1[0] + cpt < item2[0]:
                array = [item1[0] + cpt]
                for i in xrange(1, size):
                    array.append(0)
                data.insert(tab_index, array)
                tab_index += 1
                cpt += delta

        #######
        # max_range => quarter / hour / ....
        # period => seconds on each iteration
        ######

        # Find period and max_range
        t_start = result['args']['start_time']
        t_end = result['args']['end_time']

        delta = t_end - t_start
        max_range = 0

        for value in time_range:
            if delta < value:
                break
            else:
                max_range = value

        # Add 0 if no data returned
        period = intervals[time_range.index(max_range)]
        period_cpt = 0
        cpt = 0
        first_value = data[0][0]
        while first_value > t_start + period_cpt:
            null_data = new_tab(len(data[0]))
            null_data[0] = t_start + period_cpt
            data.insert(cpt, null_data)
            period_cpt += period
            cpt += 1

        # Compression on max_range
        new_data = []
        cpt = period

        compressed_data = new_tab(len(data[0]))
        t0 = data[0][0]

        for index in xrange(len(data) - 1):
            item = data[index]

            # Initialization of compressed_data with the first item
            for i in xrange(1, len(item)):
                compressed_data[i] += int(item[i])

            if item[0] < t0 + cpt:
                continue
            else:
                cpt += period
                for i in xrange(1, len(compressed_data)):
                    compressed_data[i] = int(float(compressed_data[i]) / float(period))

                compressed_data[0] = t0 + cpt - period / 2

                new_data.append(compressed_data)
                if index + 1 < len(data) - 1:
                    compressed_data = new_tab(len(data[index + 1]))

        result['table'] = new_data
        return result


    def updateClassificationData(self, classification):
        # make family protocols
        self.classification = classification

    def _builClassification(self):

        # 0: parent / 1: child protocols / 3: nb_bytes
        family_protos = {}
        families = self.classification['family']
        association = self.classification['db_association']

        for line in self.result['table']:
            for key in line[0]:
                value = db_association[key]

                for key, values in families.iteritems():
                    if value in values.split(' '):

                        if association[key] not in family_protos:
                            family_protos[association[key]] = [[association[value], str(line[1])]]
                        else:
                            added = False
                            for row in family_protos[association[key]]:
                                if row[0] == association[value]:
                                    row[1] = str(int(row[1]) + int(line[1]))
                                    added = True

                            if not added:
                                family_protos[association[key]].append([association[value], str(line[1])])

        results = []
        for key, values in family_protos.iteritems():

            for value in values:
                numbers = []

                value.insert(0, key)
                value.insert(0, 0)
                bytes = value.pop(len(value) - 1)
                numbers.append(value)
                numbers.append(bytes)
                results.append(numbers)

        return results

    def sortBy(self, column_name):
        if ('sort' in self.fragment.args and self.fragment.args['sort'] == 'ASC') or ('sortby' in self.fragment.args and self.fragment.args['sortby'] != column_name):
            self.fragment.args['sort'] = 'DESC'
        else:
            self.fragment.args['sort'] = 'ASC'

        self.fragment.args['sortby'] = column_name
        self.requestData()

#    ###
#    # Slots.
#
    def firstAction(self, string):
        try:
            if self.result['args']['start'] == 0:
                return

            self.fragment.args['start'] = 0
        except:
            return
        self.requestData()

    def prevAction(self, string):
        try:
            self.fragment.args['start'] = self.result['args']['start'] - self.result['args']['limit']

            if self.fragment.args['start'] < 0:
                self.fragment.args['start'] = 0

            if self.fragment.args['start'] == self.result['args']['start']:
                return
        except:
            return
        self.requestData()

    def nextAction(self, string):
        # FIXME: Currently rowcount is the number of entries got from this query.
#        if self.result['args']['start'] + self.result['args']['limit'] > self.result['rowcount']:
#            return
        # Trying here to avoid problem with not yet loaded page
        try:
            self.fragment.args['start'] = self.result['args']['start'] + self.result['args']['limit']

            if self.fragment.args['start'] == self.result['args']['start']:
                return
        except:
            return
        self.requestData()


    def lastAction(self, string):
        try:
            self.fragment.args['start'] = self.result['rowcount'] - self.result['args']['limit']
            self.fragment.args['start'] += self.result['args']['limit'] - self.result['rowcount'] % self.result['args']['limit']

            if self.fragment.args['start'] < 0:
                self.fragment.args['start'] = 0

            if self.fragment.args['start'] == self.result['args']['start']:
                return
        except:
            return
        self.requestData()

    def pauseAction(self, string):
        self.fetcher.pause(True)
        self.pauseLink.hide()
        self.playLink.show()

    def playAction(self, string):
        self.fetcher.pause(False)
        self.playLink.hide()
        self.pauseLink.show()

    def csvAction(self, string):
        filename = unicode(QFileDialog.getSaveFileName(self, self.tr("Save File"),
                                               "", self.tr("CSV file (*.csv)")))
        if not filename:
            return
        elif not filename.endswith('.csv'):
            filename += '.csv'

        with open(filename, "w+") as f:
            for line in self.data:
                f.write('%s\n' % ','.join(('"%s"' % str(s).replace('"', '""') for s in line)))

    def getToolspace(self):

        toolspace = QWidget()
        vbox = QVBoxLayout(toolspace)
        vbox.setContentsMargins(0, 0, 0, 0)

        # Add the titlebar with arrows
        titlebar = QWidget()

        # The navigation buttons
        self.firstLink = createLink(':/icons-20/go-first.png', self.firstAction, self.tr("First view"))
        self.prevLink = createLink(':/icons-20/go-prev.png', self.prevAction, self.tr("Previous view"))
        self.nextLink = createLink(':/icons-20/go-next.png', self.nextAction, self.tr("Next view"))
        self.lastLink = createLink(':/icons-20/go-last.png', self.lastAction, self.tr("Last view"))
        self.playLink = createLink(':/icons-20/play.png', self.playAction, self.tr("Play"))
        self.pauseLink = createLink(':/icons-20/pause.png', self.pauseAction, self.tr("Pause"))
        self.csv = createLink(':/icons-20/save_disk.png', self.csvAction, self.tr("CSV export"))

        hbox = QHBoxLayout(titlebar)
        hbox.addStretch()
        hbox.addWidget(self.firstLink)
        hbox.addWidget(self.prevLink)
        if not self.__class__.__name__ == 'TreeFragmentView' and (hasattr(self, 'chart_type') and self.chart_type != C_PIECHART):
            hbox.addWidget(self.csv)
        hbox.addWidget(self.title)
        if hasattr(self.fetcher, 'pause'):
            hbox.addWidget(self.playLink)
            self.playLink.hide()
            hbox.addWidget(self.pauseLink)
        hbox.addWidget(self.nextLink)
        hbox.addWidget(self.lastLink)
        hbox.addStretch()

        vbox.addWidget(titlebar)
        toolspace.setFixedHeight(35)

        return toolspace
