
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from copy import copy
from nucentral.qt.tools import QVariant_toPyObject

from PyQt4.QtGui import QItemDelegate, QPen
from PyQt4.QtCore import Qt, QPointF


class GaugeDelegate(QItemDelegate):

    def __init__(self, parent=None):
        QItemDelegate.__init__(self, parent)
        self.max_value = -1.0
        self.pen = QPen()
        self.pen.setWidth(1)
        self.bigpen = QPen()
        self.bigpen.setWidth(3)

    def updateEditorGeometry(self, editor, index, option):
        editor.setGeometry(option.rect)

    def reset(self):
        self.max_value = -1

    def paint(self, painter, option, index):
        column = 2
        my_index = index.model().createIndex(index.row(), column, index.internalPointer())
        list_ = index.model().data(my_index, Qt.UserRole).toList()

        size = QVariant_toPyObject(list_[0])
        color = QVariant_toPyObject(list_[1])

        percent = 0.0
        if self.max_value == -1.0:
            # root item value
            self.max_value = float(size) if size else 0
            percent = 100.0
        else:
            percent = float(size) * 100.0 / self.max_value if size else 0

        container = copy(option.rect)
        container.setHeight(80.0 / 100.0 * option.rect.height())
        container.setWidth(90.0 / 100.0 * option.rect.width())
        container.moveCenter(option.rect.center())

        rect = copy(option.rect)
        rect.setWidth(percent * container.width() / 100 - 1)
        rect.setHeight(95.0 / 100.0 * container.height())
        rect.moveCenter(container.center())
        rect.moveLeft(container.left() + 1)

        if percent < 1:
            text = str('< 1') + '%'
        else:
            text = str(int(percent)) + '%'
        fm = option.fontMetrics

        text_width = fm.boundingRect(text).width()
        text_height = fm.boundingRect(text).height()

        text_coords = QPointF(container.center().x() - text_width / 2, container.center().y() + text_height / 4)

        painter.save()
        painter.setPen(self.bigpen)
        painter.setBrush(Qt.black)
        painter.drawRect(container)
        painter.fillRect(container, Qt.lightGray)
        painter.translate(container.x() + container.x() * 5.0 / 100.0, container.y())
        painter.restore()

        painter.save()
        painter.setPen(self.pen)
        painter.drawRect(rect)

        painter.fillRect(rect, color)
        painter.translate(rect.x(), rect.y())
        painter.restore()

        painter.save()
        painter.drawText(text_coords, text)
        painter.restore()
