
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""
from random import choice
from nucentral.common.human import humanFilesize
from nucentral.qt.tools import CreateQVariantList

from PyQt4.QtGui import QColor, QBrush
from PyQt4.QtCore import QVariant, Qt, QModelIndex, QAbstractItemModel

from nudpi.client.qt.args import arg_types
from nudpi.client.qt.user_settings.db_association import color_association

class TreeItem(object):

    ITEM_NAME = 0
    ITEM = 1

    def __init__(self, data, parent=None):
        self.parentItem = parent
        self.itemData = data
        self.childItems = []
        self.colorLabel = ''
        self.color = None
        self.darkColor = None

        self.value = 0
        self.span = 0
        self.z_value = 0
        self.start_angle = 0
        self.radius = 0

    def appendChild(self, item):
        self.childItems.append(item)

    def removeChild(self, item):
        if item in self.childItems:
            self.childItems.remove(item)

    def child(self, row):
        return self.childItems[row]

    def childCount(self):
        return len(self.childItems)

    def columnCount(self):
        return len(self.itemData)

    def data(self, column=None):
        try:
            if column == None:
                return self.itemData
            return self.itemData[column]
        except IndexError:
            return QVariant()

    def setData(self, data, column=0):
        if column == 0:
            self.itemData = data
        else:
            try:
                self.itemData[column] = data
            except IndexError:
                # TODO
                pass

    def replaceData(self, data, column):
        self.itemData[column] = data

    def parent(self):
        return self.parentItem

    def row(self):
        if self.parentItem:
            return self.parentItem.childItems.index(self)
        return 0

    def hasChild(self, childitem=None):
        if childitem:
            for child in self.childItems:
                if child.data(0) == childitem.data(0):
                    return True

        else:
            return len(self.childItems) > 0

        return False

    def getChild(self, childitem):
        for child in self.childItems:
            if child.data(0) == childitem.data(0):
                return child


class TreeModel(QAbstractItemModel):

    colors = {'black_green' : ['#487118', '#63863a', '#7f9b5d', '#9bb180', '#b6c6a3', '#d1dbc5'],
              'dark_green' : ['#196d38', '#3b8356', '#5e9973', '#81af92', '#a3c5af', '#c5dacd'],
              'green' : ['#7ab41d', '#8ebf3f', '#a2ca60', '#b6d683', '#cae1a5', '#deecc6'],
              'grey' : ['#5f6062'],
              'light_grey' : ['#898989'],
              'dark_blue' : ['#212483', '#424595', '#6365a8', '#8587bb', '#a6a7cd', '#c7c8e0'],
              'light_blue' : ['#528fc8', '#6ca0d0', '#86b0d8', '#a0c2e1', '#bad2e9', '#d4e3f1'],
              'dark_orange' : ['#ea752c', '#ed8a4b', '#f09e6b', '#f3b38b', '#f7c8ab', '#fadcca'],
              'orange' : ['#F29324', '#f4a345', '#f6b365', '#f8c487', '#fad4a7', '#fce4c8'],
              'dark_red' : ['#8C0A14', '#9d2f37', '#ae535a', '#c0787e', '#d19da1', '#e2c2c4'],
              'light_red' : ['#c1292e', '#ca494d', '#d3696c', '#dd8a8c', '#e6a9ab', '#efc9cb'],
              'black_red' : ['#8C0A14', '#9d2f37', '#ae535a', '#c0787e', '#d19da1', '#e2c2c4'],
              'yellow' : ['#f8e930', '#f9ec4f', '#faf06e', '#fbf38d', '#fcf6ac', '#fdf9cb'],
              'grey_brown' : ["#b4a8b9"]
              }

    default_colors = {
              'grey' : ['#5f6062'],
              'light_grey' : ['#898989'],
              'grey_brown' : ["#b4a8b9"]
              }


    def __init__(self, data, parent=None):
        super(TreeModel, self).__init__(parent)

        self.results = data

        columns = []
        for column in data['columns']:
            label = arg_types[column].label
            columns.append(label)

        self.rootItem = TreeItem(tuple(columns))
        if len(data['table']) > 0:
            self.setupModelData()

    def columnCount(self, parent):
        if parent.isValid():
            return parent.internalPointer().columnCount()
        else:
            return self.rootItem.columnCount()

    def item(self, index):
        item = index.internalPointer()

        if item:
            return item
        return None

    def data(self, index, role):
        if not index.isValid():
            return QVariant()

        item = index.internalPointer()

        if role == Qt.BackgroundColorRole and index.column() == 0:
            if item.parent().data(0) == "root":
                return QVariant(item.color)

        if role == Qt.UserRole:
            size = QVariant(float(item.value))
            color = QVariant(item.color)
            list = [size, color]
            return CreateQVariantList(list)

        if role == Qt.DisplayRole:
            return QVariant(item.data(index.column()))

        return QVariant()


    def flags(self, index):
        if not index.isValid():
            return Qt.NoItemFlags

        return Qt.ItemIsEnabled | Qt.ItemIsSelectable

    def headerData(self, section, orientation, role):
        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            return QVariant(self.rootItem.data(section))

        return QVariant()

    def index(self, row, column, parent):
        if not self.hasIndex(row, column, parent):
            return QModelIndex()

        if not parent.isValid():
            parentItem = self.rootItem
        else:
            parentItem = parent.internalPointer()

        childItem = parentItem.child(row)
        if childItem:
            return self.createIndex(row, column, childItem)
        else:
            return QModelIndex()

    def parent(self, index):
        if not index.isValid():
            return QModelIndex()

        childItem = index.internalPointer()
        parentItem = childItem.parent()

        if parentItem == self.rootItem:
            return QModelIndex()

        return self.createIndex(parentItem.row(), 0, parentItem)

    def rowCount(self, parent):
        if parent.column() > 0:
            return 0

        if not parent.isValid():
            parentItem = self.rootItem
        else:
            parentItem = parent.internalPointer()

        return parentItem.childCount()

    def setupModelData(self):

        data = self.results['table']
        rootitem = self.rootItem
        parentitem = [rootitem]

        for line in data:
            parentitem = rootitem
            for item in line:

                treeitem = TreeItem([item, "", ""], parentitem)

                if not parentitem.hasChild(treeitem):
                    parentitem.appendChild(treeitem)
                    parentitem = treeitem
                else:
                    parentitem = parentitem.getChild(treeitem)

        depth = 0
        if rootitem.childCount() > 0:
            self.calculValue(rootitem, depth + 1)

    def calculValue(self, parent, depth):
        children = parent.childItems

        # If true : item is a leaf (sum of bytes)
        if len(children) == 0:
            parent.value = int(parent.data(0))
            parent.parent().removeChild(parent)
            return True

        index = 0
        max = len(children)

        while index < len(children):
            child = children[index]

            # ************** Colors *****************
            # Root
            if self.rootItem == parent:
                child.color = QColor(Qt.white)
            # Predefined colors
            elif child.data(TreeItem.ITEM_NAME) in color_association:
                label = color_association[child.data(TreeItem.ITEM_NAME)]
#                if depth < 2:
#                    child.colorLabel = ''
#                    child.color = QColor(self.colors[label][0])
#                else:
                child.colorLabel = label
                child.color = QColor(self.colors[label][0])
            # Colors from parent
            else:
                if len(parent.colorLabel) > 0:
                    color_index = index % len(self.colors[parent.colorLabel])
                    child.color = QColor(self.colors[parent.colorLabel][color_index])
                else:
                    colorLabel = choice(self.default_colors.keys())
                    child.color = QColor(self.default_colors[colorLabel][0])
            child.darkColor = child.color.darker(120)

            if self.calculValue(child, depth + 1):
                index -= 1
            parent.value += child.value
            if not isinstance(parent.data(), tuple):
                value = humanFilesize(parent.value)
                parent.replaceData(value, 2)

            if len(child.data(0)) == 0:
                child.setData(["unknown"])
            index += 1

        return False

