"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from __future__ import with_statement
from os import umask, chmod
from itertools import chain

from nucentral.common.xml_etree import etree, save as xml_save
from nucentral.common.transaction import Transaction

# readable by everyone
FILE_UMASK = 0011

class TimeRangeFileTransaction(Transaction):
    def __init__(self, filename, acls_ipv4, acls_ipv6):
        self.filename = filename
        self.acls = chain(acls_ipv4, acls_ipv6)
        self.previous_file = None

    def save(self):
        self.previous_file = None
        try:
            with open(self.filename, 'r') as file:
                self.previous_file = file.read()
        except IOError:
            pass

    def rollback(self):
        if self.previous_file:
            umask(FILE_UMASK)
            with open(self.filename, 'w') as file:
                file.write(self.previous_file)

    def apply(self):
        root = etree.Element("periods",
            version="3.0",
            name="timerange")

        timeranges = set()

        for acl in self.acls:
            for timerange in acl.durations | acl.periodicities:
                timeranges.add(timerange)

        for timerange in timeranges:
            period_node = etree.SubElement(root, "period", name=timerange.id, desc="Generated by NuFace")
            perioditem_node = etree.SubElement(period_node, "perioditem")
            timerange.exportTimeRangeXML(perioditem_node)

        umask(FILE_UMASK)
        xml_save(root, self.filename)
        # umask() should be enough, but old version of nuface used umask(0033)
        # and an existing periods.xml keeps it old mode. So force the mode the
        # 0644.
        chmod(self.filename, 0644)

