
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

import socket
import os

from nucentral.backend import Component
from rrdinfo import RrdInfo


class GraphManagerComponent(Component):
    API_VERSION = 1
    NAME = 'nugraph'
    VERSION = '1.0'
    REQUIRES = set()
    ROLES = {
        'nugraph_read' : frozenset(('getDirsName', 'getCoords', 'remove')),
    }

    def init(self, core):
        self.core = core
        self.rrdPath = "/var/lib/collectd/rrd/" + str(socket.getfqdn(socket.gethostname())) + "/"
        self.graphInfo = {}

    def service_getDirsName(self, context):
        """
        return all directories where rrd files are, and children of each directory.
        return {parent : [chld1, child2, ...]}
        """
        parentsDir = os.listdir(self.rrdPath)
        directories = {}
        for dir in parentsDir:
            subDirectories = os.listdir(self.rrdPath + str(dir))
            directories[str(dir)] = subDirectories
            for i in xrange(len(subDirectories)):
                subDirectories[i] = subDirectories[i].replace(".rrd", "")
        return directories

    def service_getCoords(self, context, period, uuid, filename = ""):
        """
        create RrdInfo object first this function is called.
        get coordinates from RrdInfo object with uuid identitity.
        """
        if not self.graphInfo.has_key(uuid):
            if filename:
                self.graphInfo[uuid] = RrdInfo(self.rrdPath + filename)
#            elif request:
#                self.graphInfo[uuid] = BddAccess(request)

        return self.graphInfo[uuid].coords(period)

    def service_remove(self, context, uuid):
        """
        remove coordinates not used anymore (free memory)
        """
        if uuid in self.graphInfo:
            del self.graphInfo[uuid]
