
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

import sys
from PyQt4.QtGui import QApplication, QSizePolicy, QListWidget, QListWidgetItem, QAbstractItemView, QDrag
from PyQt4.QtCore import QSize, Qt, QMimeData, QEvent, QString

from pyrrd import rrd
from rrdgraphqwt import RrdWidget
# from cartesianplot import CartesianPlot


# Test
import PyQt4.Qwt5 as Qwt


# ***************Test
from PyQt4.QtCore import QIODevice, QByteArray, QDataStream


HOUR = 3200
DAY = HOUR * 24
WEEK = DAY * 7
MONTH = DAY * 31

# class GraphWidget(QScrollArea):
class GraphWidget(QListWidget):
    """
    Widget that contains one or more RrdWidget.
    """
    def __init__(self, client, parent = None):
        QListWidget.__init__(self, parent)
        self.setMinimumSize(QSize(500, 500))
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.setSelectionMode(QAbstractItemView.SingleSelection)
        self.setDragEnabled(True)
        self.setAcceptDrops(True)
        self.setDropIndicatorShown(True)
        self.setDragDropMode(QAbstractItemView.InternalMove)
        self.client = client

        # Map QTreeWidgetItem item to QListWidgetItem item
        self.items_dict = {}

        # Initial range is one houre
        self.period = 3200

    def itemClickedSlot(self, treeItem, column):
        if treeItem.childCount() == 0:
            if treeItem.isSelected():
                filename = str(treeItem.parent().text(0)) + "/" + str(treeItem.text(0)) + ".rrd"
                widget = RrdWidget(self.client, filename, self.period)
                widget.setMinimumHeight(300)
                widget.setMaximumHeight(300)
                widget.setMinimumWidth(1000)
                listItem = QListWidgetItem()
                listItem.setSizeHint(widget.size())
                listItem.setToolTip(filename)
                self.insertItem(0, listItem)
                self.setItemWidget(listItem, widget)
                self.items_dict[treeItem.text(0)] = listItem
            else:
                listItem = self.items_dict[treeItem.text(0)]
                widget = self.itemWidget(listItem)
                widget.removeGraph()
                self.removeItemWidget(listItem)
                self.takeItem(self.row(listItem))

    def timeRangeChange(self, label):
        period = 0

        if label == "1 hour range":
            self.period = HOUR
            periodString = "HOUR"
        elif label == "24 hours range":
            self.period = DAY
            periodString = "DAY"
        elif label == "7 days range":
            self.period = WEEK
            periodString = "WEEK"
        elif label == "31 days range":
            self.period = MONTH
            periodString = "MONTH"

        for i in xrange(self.count()):
            item = self.item(i)
            widget = self.itemWidget(item)
            widget.refresh(self.period, periodString)

    # **** Implement QListWidget methods ****
#     def supportedDropActions(self):
#         return Qt.MoveAction | Qt.CopyAction



#     ******** TEMP ******************
#     def go(self, value):
#         """Create and plot a sequence of bars taking into account the controls
#         """
#         n = int(value)
#
#         for i in xrange(self.count()):
#             item = self.item(i)
#             widget = self.itemWidget(item)
#
#             widget.graphWidget.xaxis.detach()
#             widget.graphWidget.xaxis.attach(widget.graphWidget)
#             widget.graphWidget.yaxis.detach()
#             widget.graphWidget.yaxis.attach(widget.graphWidget)
#
#             widget.refresh(3200, "HOUR")
#
#             widget.graphWidget.setAxisAutoScale(Qwt.QwtPlot.xBottom)
#             widget.graphWidget.setAxisAutoScale(Qwt.QwtPlot.yLeft)
#             widget.graphWidget.replot()
#             widget.graphWidget.zoomer.setZoomBase()

    def itemUp(self):
        pass

    def dragEnterEvent(self, event):
        event.accept()

    def dropEvent(self, event):
        event.acceptProposedAction()

    # *********** Test
    def startDrag(self, event):
        row = self.currentRow()
        print "current Row : " + unicode(row)
#         itemData = QByteArray()
#         dataStream = QDataStream(itemData, QIODevice.WriteOnly)

        #We put the current row into datastream
#         dataStream << QString(row)
        mimeData = QMimeData()
        mimeData.setText(unicode(row))
#         mimeData.setData("nugraph/graph", itemData)
        drag = QDrag(self)
        drag.setMimeData(mimeData)
        drag.start(Qt.CopyAction)

    def dropEvent(self, event):
#         if event.mimeData().hasFormat("nugraph/graph"):
#             itemData = event.mimeData().data("nugraph/graph")
            text = unicode(event.mimeData().text())

            previousRow = int(text)
            itemToDrag = self.takeItem(previousRow)
            widgetToDrag = self.itemWidget(itemToDrag)

            currentDropRow = self.row(self.itemAt(event.pos()))
            itemDrop = self.takeItem(currentDropRow)
            widgetDrop = self.itemWidget(itemDrop)

            self.removeItemWidget(itemToDrag)
            self.insertItem(currentDropRow, itemToDrag)
            self.setItemWidget(itemToDrag, widgetToDrag)

            self.removeItemWidget(itemDrop)
            self.insertItem(previousRow, widgetDrop)
            self.setItemWidget(itemDrop, widgetDrop)

            self.setCurrentItem(itemToDrag)

#             event.accept()



if __name__ == "__main__":
    app = QApplication(sys.argv)
    widget = GraphWidget()
    widget.show()
    sys.exit(app.exec_())

