
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from nupki.version import VERSION
from time import mktime

# Old client without setupClient() service
DEFAULT_CLIENT_VERSION = "milestone 4"

class Client:
    def __init__(self, attr):
        if isinstance(attr, (str, unicode)):
            # nupki client 4.0.1 uses setupClient('4.0.1') instead of a dict
            attr = {'version': attr}
        self.version = attr.get('version', DEFAULT_CLIENT_VERSION)
        self.use_named_list = True
        self.datetime_timestamp = True
        self.timezone = True
        if self.version in ('milestone 4', '4.0.1'):
            self.use_named_list = False
            self.datetime_timestamp = False
            self.timezone = False
        elif self.version == '4.0.2':
            self.datetime_timestamp = False
            self.timezone = False
        elif self.version == '4.0.3':
            self.timezone = False

    def exportXMLRPC(self):
        if self.version == '4.0.1':
            return VERSION
        else:
            return {
                'version': VERSION,
                'use_named_list': self.use_named_list,
            }

    def _exportTimestamp(self, timestamp):
        if not self.datetime_timestamp:
            # nupki 4.0.1/4.0.2: '1580389797', UNIX epoch
            epoch = mktime(timestamp.timetuple())
            return str(int(epoch))
        else:
            # nupki 4.0.3+: '2010-03-22 14:49:27', datetime format
            if not self.timezone:
                # strip timezone, not supported by nupki < 4.0.4
                timestamp = timestamp.replace(tzinfo=None)
            return unicode(timestamp)

    def exportCertificate(self, cert):
        if self.use_named_list:
            xmlrpc = cert.exportXMLRPC()
            for key in ('start_date', 'end_date'):
                if key not in xmlrpc:
                    continue
                xmlrpc[key] = self._exportTimestamp(xmlrpc[key])
            return xmlrpc
        else:
            return (
                cert.type, cert.id, cert.cname, cert.email, cert.organization_unit,
                cert.location, cert.state, cert.country, cert.status)

