
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from nupki.version import VERSION

class Compatibility:
    def __init__(self, logger, client):
        self.has_create_certificate = True
        self.newcert_has_expire = True
        self.use_old_regex = False
        self.use_named_list = True
        self.has_undo = True
        self.has_getcerttext = True
        self.has_getcatext = True

        server_version = client.call('nupki', 'getComponentVersion')

        if server_version == "4.0.1":
            client.call('nupki', 'setupClient', VERSION)
            logger.info("Server version: %s" % server_version)
            self.use_named_list = False
            self.has_undo = False
            self.has_getcerttext = False
            self.has_getcatext = False
        elif server_version == "milestone 4":
            # Old server without setupClient() service
            server_version = "milestone 4"
            logger.warning("Connected to an old server: no setupClient() service")
            logger.warning("Enable compatibility mode")

            # old nupki versions doesn't support some characters
            # (especially $ and ")
            self.use_old_regex = True

            # check newCert() API
            self.has_create_certificate = False
            newcert_prototype = client.call('CORE', 'help', 'nupki', 'newCert')
            if newcert_prototype[0] == 'newCert(pkiname, cname, type)':
                self.newcert_has_expire = False
            self.use_named_list = False
            self.has_undo = False
            self.has_getcerttext = False
            self.has_getcatext = False
        else:
            # newest API (>= 4.0.2)
            client_attr = {'version': VERSION}
            attr = client.call('nupki', 'setupClient', client_attr)
            logger.info("Server version: %s" % attr['version'])
            self.use_named_list = attr.get('use_named_list', True)

            if server_version == "4.0.2":
                self.has_getcerttext = False
                self.has_getcatext = False
            elif server_version == "4.0.3":
                self.has_getcatext = False

