
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from PyQt4.QtCore import QVariant

class PKIManager:
    def __init__(self, window):
        self.window = window
        self.ui = window.ui
        self.client = window.client
        self.user_settings = window.user_settings
        self.nupki = window.nupki
        self.menu = self.ui.menuPKI_Manager

        self.actionsPKIManager = {}
        pkis = self.nupki.listPKI()
        self.pkis = dict((pki.name, pki) for pki in pkis)

        self.nb_pkis = 0
        self._current_pki = None
        self._actions = {}

        # Create PKI menu entries
        index = 0
#        for cpt, pki in enumerate(self.pkis.itervalues()):
        for cpt, pki in enumerate(pkis):
            self.addPKIMenu(pki.name)
            if pki.name == self.user_settings['current_pki']:
                index = cpt
        if len(pkis):
            self._current_pki = pkis[index]

    def createPKI(self, pki, numberOfDays):
        self.client.call("nupki", "createPKI",
            pki.name,
            pki.organization_unit,
            pki.organization,
            pki.location,
            pki.state,
            pki.country,
            numberOfDays)
        self.pkis[pki.name] = pki
        self.addPKIMenu(pki.name)
        self.window.loadPKI(pki)

    def addPKIMenu(self, name):
        action = self.menu.addAction(name, self.window.openThisPKI)
        action.setCheckable(True)
        self.actionsPKIManager[name] = action

        self._actions[name] = self.nb_pkis
        self.nb_pkis += 1

    def closePKI(self):
        if not self._current_pki:
            return
        name = self._current_pki.name
        self.actionsPKIManager[name].setChecked(False)

    def getCurrent(self):
        return self._current_pki

    def getCurrentName(self):
        if self._current_pki:
            return self._current_pki.name
        else:
            return None

    def setCurrentPKI(self, name):
        if name:
            pki = self.pkis[name]
            self._current_pki = pki
            self.actionsPKIManager[name].setChecked(True)
            if name in self._actions:
#                self.user_settings['current_pki'] = self._actions[name]
                self.user_settings['current_pki'] = name
        else:
            self._current_pki = None

    def deletePKI(self, name):
        self.nb_pkis -= 1
        action = self.actionsPKIManager[name]
        self.menu.removeAction(action)
        self._actions.pop(name)

