"""
$Id: $
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""


from PyQt4.QtCore import SIGNAL
from PyQt4.QtGui import QTableWidgetItem

from .controler import TableControler

class Tables(object):
    def __init__(self, window):
        self.window = window
        self.client = window.client
        self.manager = window.manager
        self.ui = window.ui

        self._client_controler = TableControler(window, self, "client")
        self._server_controler = TableControler(window, self, "server")
        self.current_controler = self._client_controler
        self.controlers = (self._client_controler, self._server_controler)

        # Connect signals
        self.connectSignals()

    def connectSignals(self):
        window = self.window
        ui = self.ui
        window.connect(ui.actionRevoke_Certificate, SIGNAL("triggered()"), self.revokeCertificate)
        window.connect(ui.actionRenew_Certificate, SIGNAL("triggered()"), self.renewCertificate)
        window.connect(ui.actionExport_Certificate, SIGNAL("triggered()"), self.exportCertificates)
        window.connect(ui.tabWidget, SIGNAL("currentChanged (int)"), self.tabChanged)
        window.connect(ui.actionForce_Pending_Actions, SIGNAL("triggered()"), self.applyActions)
        window.connect(ui.actionRefresh, SIGNAL("triggered()"), self.refresh)
        window.connect(ui.actionUndo, SIGNAL("triggered()"), self.undo)

        if not self.window.compatibility.has_undo:
            ui.actionUndo.setVisible(False)

    def tabChanged(self, tab):
        if tab == 0:
            self.current_controler = self._client_controler
        elif tab == 1:
            self.current_controler = self._server_controler
        else:
            self.current_controler = None
        if self.current_controler:
            self.current_controler.buttons_labelize()
        self.refreshActions()

    def loadPKI(self, pki):
        for controler in self.controlers:
            controler.populate(pki)
        if self.current_controler:
            self.current_controler.buttons_labelize()

    def buttons_flush(self):
        for controler in self.controlers:
            controler.buttons_flush()

    def revokeCertificate(self):
        self.current_controler.setCertNewStatus("to be revoked")

    def renewCertificate(self):
        self.current_controler.setCertNewStatus("to be renewed")

    def refreshActions(self):
        if self.current_controler:
            enabled = self.current_controler.hasSelection()
        else:
            enabled = False
        modify = enabled and (not self.window.isReadOnly())
        self.ui.actionRevoke_Certificate.setEnabled(modify)
        self.ui.actionRenew_Certificate.setEnabled(modify)
        self.ui.actionExport_Certificate.setEnabled(enabled)

    def refresh(self):
        pki = self.manager.getCurrent()
        self.buttons_flush()
        if pki:
            self.loadPKI(pki)
            self.window.refreshCRL(pki.name)

    def newCertCreated(self, attr):
        cert_type = attr['type']
        if cert_type == 'server':
            controler = self._server_controler
            tab = 1
        else:
            controler = self._client_controler
            tab = 0
        self.window.setCurrentTab(tab)
        controler.model.visible_tobecreated += 1
        self.refresh()

    def exportCertificates(self):
        self.current_controler.exportCertificates()

    def applyActions(self):
        revoke = any(
            any(cert.new_status  == 'to be revoked'
                for cert in controler.model.certs)
            for controler in self.controlers)
        pki_name = self.manager.getCurrentName()
        self.client.call("nupki", "forceEachCertAction", pki_name)
        self.refresh()
        if revoke:
            self.window.refreshCRL(pki_name)

    def undo(self):
        name = self.manager.getCurrentName()
        try:
            deleted = self.client.call("nupki", "undo", name)
        except Exception, err:
            self.window.exception(err)
            return
        if not deleted:
            return
        self.refresh()

