
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from PyQt4.QtCore import SIGNAL, QObject, QDate, QRegExp, QLocale
from PyQt4.QtGui import QWizard, QWizardPage, QVBoxLayout, QLabel, QLineEdit, QComboBox

from nucentral.common import tr
from nucentral.qt.validate_widgets import ValidateWidgets
from nucentral.qt.tools import QDateTime_as_datetime

from nupki.common.regex import (PUBLIC_PKI_NAME_REGEX, COUNTRY_REGEX,
    ORGANIZATION_UNIT_REGEX, STATE_REGEX,
    CN_NAME_REGEX, EMAIL_REGEX,
    OLD_ORGANIZATION_UNIT_REGEX, OLD_STATE_REGEX, OLD_EMAIL_REGEX)
from nupki.common.pki import PKI

from .datetimeedit import PyDateEdit

class env:
    def __init__(self):
        pass

def createDateEdit(year_delta):
    today = QDate.currentDate()
    date_format = QLocale().dateFormat()
    dateedit = PyDateEdit()
    dateedit.setDisplayFormat(date_format)
    dateedit.setDate(QDate(today.year() + year_delta, today.month(), today.day()))
    return dateedit
#
# <Create a new PKI>
#
class choosePKINamePage(QWizardPage, ValidateWidgets):
    def __init__(self, parent, window):
        QWizardPage.__init__(self,  parent)
        ValidateWidgets.__init__(self)
        self.env = env
        self.client = window.client
        self.setTitle(tr("Create a new PKI"))

        label = QLabel(tr("Welcome to your first PKI. It will be used to create user and server certificates."))
        label.setWordWrap(True)
        pkilabel = QLabel(tr("PKI name:"))

        self.env.pkiname = QLineEdit()
        regex = QRegExp(PUBLIC_PKI_NAME_REGEX)
        self.setRegExpValidator(self.env.pkiname, regex)
        self.registerField("pkiName*", self.env.pkiname)

        layout = QVBoxLayout()
        layout.addWidget(label)

        layout.addWidget(pkilabel)
        layout.addWidget(self.env.pkiname)

        expire = QLabel(tr("Expiration date:"))
        layout.addWidget(expire)
        self.env.dateedit = createDateEdit(5)
        layout.addWidget(self.env.dateedit)

        self.setLayout(layout)


class setPKISettings(QWizardPage, ValidateWidgets):
    def __init__(self, parent, window):
        QWizardPage.__init__(self,  parent)
        ValidateWidgets.__init__(self)
        self.env = env

        self.window = window
        self.client = window.client
        self.setTitle(tr("Set default PKI parameters"))

        if window.compatibility.use_old_regex:
            regex = QRegExp(OLD_ORGANIZATION_UNIT_REGEX)
        else:
            regex = QRegExp(ORGANIZATION_UNIT_REGEX)

        label = QLabel(tr("Please set the default PKI parameters:"))
        label.setWordWrap(True)
        orgunitlabel = QLabel(tr("Organizational Unit:"))
        self.env.orgunit = QLineEdit()
        self.setRegExpValidator(self.env.orgunit, regex)
        self.registerField("orgUnit", self.env.orgunit)

        if window.compatibility.use_old_regex:
            state_regex = QRegExp(OLD_STATE_REGEX)
        else:
            state_regex = QRegExp(STATE_REGEX)

        orglabel = QLabel(tr("Organization:"))
        self.env.org = QLineEdit()
        self.setRegExpValidator(self.env.org, state_regex)
        self.registerField("*Org", self.env.org)

        locationlabel = QLabel(tr("Location:"))
        self.env.location = QLineEdit()
        self.setRegExpValidator(self.env.location, regex)
        self.registerField("Location", self.env.location)

        statelabel = QLabel(tr("State:"))
        self.env.state = QLineEdit()
        self.setRegExpValidator(self.env.state, state_regex)
        self.registerField("State*", self.env.state)

        countrylabel = QLabel(tr("Country (two characters):"))
        self.env.country = QLineEdit()
        regex = QRegExp(COUNTRY_REGEX)
        self.setRegExpValidator(self.env.country, regex)

        self.registerField("Country*", self.env.country)

        layout = QVBoxLayout()
        layout.addWidget(label)

        layout.addWidget(orgunitlabel)
        layout.addWidget(self.env.orgunit)

        layout.addWidget(orglabel)
        layout.addWidget(self.env.org)

        layout.addWidget(locationlabel)
        layout.addWidget(self.env.location)

        layout.addWidget(statelabel)
        layout.addWidget(self.env.state)

        layout.addWidget(countrylabel)
        layout.addWidget(self.env.country)

        self.setLayout(layout)

    def validatePage(self):
        if not self.isFinalPage():
            return True

        date = self.env.dateedit.date()
        today = QDate.currentDate()
        numberOfDays = today.daysTo(date)

        # We create the PKI
        pki = PKI(
            name=unicode(self.env.pkiname.text()),
            organization_unit=unicode(self.env.orgunit.text()),
            organization=unicode(self.env.org.text()),
            location=unicode(self.env.location.text()),
            state=unicode(self.env.state.text()),
            country=unicode(self.env.country.text()))

        try:
            self.window.manager.createPKI(pki, numberOfDays)
        except Exception, err:
            self.window.exception(err)
            return False

        self.emit(SIGNAL("WizardValidatePage"), pki.name)
        return True

class NewPkiWizard(QObject):
    def __init__(self, window):
        QObject.__init__(self)
        self.env = env()
        self.wizard = QWizard(window)
        # Independent pages: Back button doesn't clear fields
        self.wizard.setOption(QWizard.IndependentPages)
        page = choosePKINamePage(self.wizard, window)
        self.wizard.addPage(page)
        page = setPKISettings(self.wizard, window)
        self.wizard.addPage(page)
        self.wizard.setWindowTitle(tr("Create a new PKI"))
        self.connect(page, SIGNAL("WizardValidatePage"), self.receiveWizardValidatePage)
        self.wizard.show()
        self.validatecb = None

    def registerValidateCallback(self, callback, userdata=None):
        self.validatecb = callback
        self.validatecb_userdata = userdata

    def receiveWizardValidatePage(self, pkiname):
        if self.validatecb:
            self.validatecb(self.validatecb_userdata)

#
# </Create a new PKI>
#

#
# </Create a new Certificate>
#

class setCertificateSettings(QWizardPage, ValidateWidgets):
    def __init__(self, parent, env, window):
        QWizardPage.__init__(self,  parent)
        ValidateWidgets.__init__(self)
        self.env = env

        self.window = window
        self.client = window.client
        self.nupki = window.nupki
        self.setTitle(tr("Set default certificate parameters"))

        pki = self.nupki.getPKI(self.env.workingPKI)

        if window.compatibility.use_old_regex:
            regex = QRegExp(OLD_ORGANIZATION_UNIT_REGEX)
        else:
            regex = QRegExp(ORGANIZATION_UNIT_REGEX)

        label = QLabel(tr("Please set the default certificate parameters:"))
        label.setWordWrap(True)
        orgunitlabel = QLabel(tr("Organizational Unit:"))
        self.env.orgunit = QLineEdit()
        self.setRegExpValidator(self.env.orgunit, regex)
        self.env.orgunit.setText(pki.organization_unit)
        self.registerField("orgUnit", self.env.orgunit)

        orglabel = QLabel(tr("Organization:"))
        self.env.org = QLabel()
        self.env.org.setText(pki.organization)
        self.registerField("Organization", self.env.org)

        locationlabel = QLabel(tr("Location:"))
        self.env.location = QLineEdit()
        self.setRegExpValidator(self.env.location, regex)
        self.env.location.setText(pki.location)
        self.registerField("Location", self.env.location)

        statelabel = QLabel(tr("State:"))
        self.env.state = QLabel()
        self.env.state.setText(pki.state)
        self.registerField("State", self.env.state)

        countrylabel = QLabel(tr("Country (two characters):"))
        self.env.country = QLabel()
        self.env.country.setText(pki.country)
        self.registerField("Country", self.env.country)

        layout = QVBoxLayout()
        layout.addWidget(label)

        layout.addWidget(orgunitlabel)
        layout.addWidget(self.env.orgunit)

        layout.addWidget(orglabel)
        layout.addWidget(self.env.org)

        layout.addWidget(locationlabel)
        layout.addWidget(self.env.location)

        layout.addWidget(statelabel)
        layout.addWidget(self.env.state)

        layout.addWidget(countrylabel)
        layout.addWidget(self.env.country)

        self.setLayout(layout)

    def validatePage(self):
        if not self.isFinalPage():
            return True

        # We create the PKI
        pki = self.env.workingPKI
        cname = unicode(self.env.cnname.text())
        cert_type = unicode(self.env.certtypebox.currentText())
        email = unicode(self.env.email.text())
        organization = unicode(self.env.org.text())
        orgunit = unicode(self.env.orgunit.text())
        location = unicode(self.env.location.text())
        state = unicode(self.env.state.text())
        country = unicode(self.env.country.text())

        end_date = self.env.dateedit.dateTime()
        end_date = QDateTime_as_datetime(end_date)
        end_date = unicode(end_date)

        try:
            self.window.nupki.createCertificate({
                'pki': pki,
                'cname': cname,
                'type': cert_type,
                'email': email,
                'organization': organization,
                'organization_unit': orgunit,
                'location': location,
                'state': state,
                'country': country,
                'end_date': end_date,
                # FIXME: set serial?
            })
        except Exception, err:
            self.window.exception(err)
            return False

        certificate = {
            'pki': self.env.workingPKI,
            'cname': self.env.cnname.text(),
            'type': self.env.certtypebox.currentText(),
            'email': self.env.email.text(),
            'orgunit': self.env.orgunit.text(),
            'location': self.env.location.text(),
            'state': self.env.state.text(),
            'country': self.env.country.text(),
            'status': "to be created",
        }
        self.emit(SIGNAL("WizardValidatePage"), certificate)
        return True

class newCertificateSetCNandExpire(QWizardPage, ValidateWidgets):
    def __init__(self, parent, env, window):
        QWizardPage.__init__(self,  parent)
        ValidateWidgets.__init__(self)
        self.env = env

        label = QLabel(tr("The Common Name (CN) is the unique name used to identify your certificate."))
        label.setWordWrap(True)

        cnlabel = QLabel(tr("Common Name:"))
        self.env.cnname = QLineEdit()
        regex = QRegExp(CN_NAME_REGEX)
        self.setRegExpValidator(self.env.cnname, regex)
        self.registerField("cnName*", self.env.cnname)

        layout = QVBoxLayout()
        layout.addWidget(label)

        layout.addWidget(cnlabel)
        layout.addWidget(self.env.cnname)

        emaillabel = QLabel(tr("Email address:"))
        self.env.email = QLineEdit()
        if window.compatibility.use_old_regex:
            regex = QRegExp(OLD_EMAIL_REGEX)
        else:
            regex = QRegExp(EMAIL_REGEX)
        self.setRegExpValidator(self.env.email, regex)
        self.registerField("email", self.env.email)

        layout.addWidget(emaillabel)
        layout.addWidget(self.env.email)

        certtypelabel = QLabel(tr("Certificate type:"))
        self.env.certtypebox = QComboBox()
        self.env.certtypebox.insertItem(0, "client")
        self.env.certtypebox.insertItem(1, "server")

        if window.getCurrentTab() == window.TAB_SERVER:
            self.env.certtypebox.setCurrentIndex(1)

        layout.addWidget(certtypelabel)
        layout.addWidget(self.env.certtypebox)

        expire = QLabel(tr("Expiration date:"))
        layout.addWidget(expire)
        self.env.dateedit = createDateEdit(1)
        layout.addWidget(self.env.dateedit)

        self.setLayout(layout)



class NewCertificateWizard(QObject):
    def __init__(self, window):
        QObject.__init__(self)
        self.env = env()
        self.wizard = QWizard(window)
        # Independent pages: Back button doesn't clear fields
        self.wizard.setOption(QWizard.IndependentPages)

        self.env.workingPKI = window.manager.getCurrentName()
        page = newCertificateSetCNandExpire(self.wizard, self.env, window)
        self.wizard.addPage(page)
        page = setCertificateSettings(self.wizard, self.env, window)
        self.wizard.addPage(page)
        self.wizard.setWindowTitle(tr("Create a New Certificate"))
        self.connect(page, SIGNAL("WizardValidatePage"), self.receiveWizardValidatePage)
        self.validatecb = None
        self.wizard.show()

    def registerValidateCallback(self, callback):
        self.validatecb = callback

    def receiveWizardValidatePage(self, certificate):
        if self.validatecb:
            self.validatecb(certificate)

