/*
Version 2.3:

 + Add organization, start_date and end_date columns to the certificate table
 + Remove expire and info_in_pki columns of the certificate table

Version 2.2:

 + Add new_status column to the certificate table

Version 2.1:

 + Add expire column to the certificate table

Version 2.0:

 + First schema version
*/

DROP TABLE IF EXISTS _schema;
DROP TABLE IF EXISTS pki;
DROP TABLE IF EXISTS certificate;

/* FIXME: delete unused tables: ca and ldap */
DROP TABLE IF EXISTS ca;
DROP TABLE IF EXISTS ldap;

CREATE TABLE _schema (
 name TEXT NOT NULL,
 version TEXT NOT NULL
);
INSERT INTO _schema (name, version) VALUES('nupki', '2.3');

CREATE TABLE pki (
  id           INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
  name         TEXT NOT NULL UNIQUE,
  orgunit      TEXT,
  organization TEXT,
  location     TEXT,
  state	       TEXT,
  country      TEXT,
  lastserial   INTEGER NOT NULL /* Count the serials set on certs to avoid duplicates */
) ;

-- There are information that are both in the certificate and
-- the database, simply to avoid fetching infos from the file
-- AND from the database: type, serial
CREATE TABLE certificate (
  id        INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
  type      TEXT NOT NULL, /* Should only be client or server */
  pkiid     INTEGER NOT NULL,
  cn        TEXT NOT NULL,

  -- This can differ from the PKI parameters. While it is stored in the certificate,
  -- it is here for convenience (and avoid looking at both the database and the
  -- certificate.
  email        TEXT,
  organization TEXT,
  orgunit      TEXT,
  location     TEXT,
  state	       TEXT,
  country      TEXT,

  serial    INTEGER NOT NULL,

  /* status: 'to be created', 'valid' or 'revoked' */
  status TEXT NOT NULL,

  /* new_status: 'to be renewed', 'to be revoked' or NULL */
  new_status TEXT,

  /* Timestamp in UTC */
  start_date TIMESTAMP,
  end_date TIMESTAMP
) ;

CREATE TABLE ca (
  id              INTEGER NOT NULL PRIMARY KEY, /* Gets the PKI table id */
  privkeypath     TEXT NOT NULL,
  pubcertpath     TEXT NOT NULL,
  has_password    TEXT NOT NULL,
  mail            TEXT NOT NULL,
  cn              TEXT NOT NULL UNIQUE,
  country         TEXT NOT NULL,
  province        TEXT NOT NULL,
  city            TEXT NOT NULL,
  org             TEXT NOT NULL,
  date_valid_from DATETIME NOT NULL,
  date_valid_to   DATETIME NOT NULL
) ;

CREATE TABLE ldap (
  id         INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
  ldap_host  TEXT NOT NULL,
  ldap_port  INTEGER NOT NULL,
  ldap_bind  TEXT NOT NULL,
  ldap_pwd   TEXT NOT NULL,
  ldap_base  TEXT NOT NULL,
  ldap_group TEXT NOT NULL
) ;


