# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""


from OpenSSL import crypto

PKI_NAME = u'__edenwall_multisite'
PKI_ORGUNIT = u'edenwall'
PKI_ORG = u'com'
PKI_LOCATION = u'Paris'
PKI_STATE = u'France'
PKI_COUNTRY = u'FR'
PKI_EMAIL = u'edenwall@edenwall.com'

NUPKI_SERVICES = set(('listPKI', 'createPKI', 'listServerCerts', 'createCert', 'getCert', 'getCertPath',
                        'getPrivateCert', 'getPrivateCertPath', 'getCACert', 'getCACertPath', 'signRequest',
                        'getCertRequest'))

class PKI:
    def __init__(self, client):
        self.client = client
        self.ca_path = {}
        self.key_path = {}
        self.cert_path = {}
        self.ca = {}
        self.key = {}
        self.cert = {}

    def createPki(self, pki_list):
        for pki in pki_list:
            if pki[1] == PKI_NAME:
                break
        else:
            self.client.call('nupki', 'createPKI', PKI_NAME, PKI_ORGUNIT, PKI_ORG, PKI_LOCATION, PKI_STATE, PKI_COUNTRY)

    def loadPki(self):
        d = self.client.call('nupki', 'listPKI')
        self.createPki(d)

    ## Certificate, private key, CA path and creation
    def loadServerCertsPaths(self, cert_name):
        d = self.client.call('nupki', 'listServerCerts', PKI_NAME)
        d = self.getServerCerts_create_cert(d, cert_name)
        d = self.getServerCertsPaths_get_cert(d, cert_name)
        d = self.getServerCertsPaths_get_key(d, cert_name)
        d = self.getServerCertsPaths_get_ca(d, cert_name)
        d = self.getServerCertsPaths_done(d, cert_name)
        return d

    def getServerCertsPaths_get_cert(self, done, cert_name):
        return self.client.call('nupki', 'getCertPath', PKI_NAME, cert_name)

    def getServerCertsPaths_get_key(self, cert_path, cert_name):
        self.cert_path[cert_name] = cert_path
        return self.client.call('nupki', 'getPrivateCertPath', PKI_NAME, cert_name)

    def getServerCertsPaths_get_ca(self, key_path, cert_name):
        self.key_path[cert_name] = key_path
        return self.client.call('nupki', 'getCACertPath', PKI_NAME)

    def getServerCertsPaths_done(self, ca_path, cert_name):
        self.ca_path[cert_name] = ca_path
        return "done"

    def getServerCertsPaths(self, cert_name):
        return self.cert_path[cert_name], self.key_path[cert_name], self.ca_path[cert_name]

    ## Certificate, private key, CA blob and creation
    def loadServerCerts(self, cert_name):
        d = self.client.call('nupki', 'listServerCerts', PKI_NAME)
        d = self.getServerCerts_create_cert(d, cert_name)
        d = self.getServerCerts_get_cert(d, cert_name)
        d = self.getServerCerts_get_key(d, cert_name)
        d = self.getServerCerts_get_ca(d, cert_name)
        d = self.getServerCerts_done(d, cert_name)
        return d

    def getServerCerts_get_cert(self, done, cert_name):
        return self.client.call('nupki', 'getCert', PKI_NAME, cert_name)

    def getServerCerts_get_key(self, cert_path, cert_name):
        self.cert[cert_name] = cert_path
        return self.client.call('nupki', 'getPrivateCert', PKI_NAME, cert_name)

    def getServerCerts_get_ca(self, key_path, cert_name):
        self.key[cert_name] = key_path
        return self.client.call('nupki', 'getCACert', PKI_NAME)

    def getServerCerts_done(self, key_path, cert_name):
        self.ca[cert_name] = key_path
        return "done"

    def getServerCerts(self, cert_name):
        return self.cert[cert_name], self.key[cert_name], self.ca[cert_name]

    ## Cetificate signing
    def signCertRequest(self, req, hostname):
        d = self.client.call('nupki', 'signRequest', PKI_NAME, hostname, req, "client")
        d = self.client.call('nupki', 'getCert', PKI_NAME, hostname)
        self.signRequest_get_ca(d, hostname)
        self.signRequest_done(d, hostname)

    def signRequest_get_ca(self, cert, hostname):
        self.cert[hostname] = cert
        return self.client.call('nupki', 'getCACert', PKI_NAME)

    def signRequest_done(self, ca, hostname):
        self.ca[hostname] = ca
        return "done"

    def getSignedCert(self, hostname):
        return self.cert[hostname], self.ca[hostname]


    def getServerCerts_create_cert(self, cert_list, cert_name):
        for cert in cert_list:
            if cert[2] == cert_name:
                break
        else:
            return self.client.call('nupki', 'createCert', PKI_NAME, cert_name, 'server', '01', PKI_EMAIL, PKI_ORGUNIT, PKI_ORG, PKI_LOCATION, PKI_STATE, PKI_COUNTRY)
        return "done"

    def client_buildCertRequest(self, CN, key=''):
        if not key:
            key = self.client___createKeyPair(crypto.TYPE_RSA, 1024)
        else:
            key = crypto.load_privatekey(crypto.FILETYPE_PEM, key)

        req = self.client___createCertRequest(key, CN=CN)
        return (crypto.dump_privatekey(crypto.FILETYPE_PEM, key),
                crypto.dump_certificate_request(crypto.FILETYPE_PEM, req))

    def client___createKeyPair(self, type, bits):
        """
        Create a public/private key pair.

        Arguments: type - Key type, must be one of TYPE_RSA and TYPE_DSA
                   bits - Number of bits to use in the key
        Returns:   The public/private key pair in a PKey object
        """
        pkey = crypto.PKey()
        pkey.generate_key(type, bits)
        return pkey

    def client___createCertRequest(self, pkey, digest="md5", **name):
        """
        Create a certificate request.

        Arguments: pkey   - The key to associate with the request
                   digest - Digestion method to use for signing, default is md5
                   **name - The name of the subject of the request, possible
                            arguments are:
                              C     - Country name
                              ST    - State or province name
                              L     - Locality name
                              O     - Organization name
                              OU    - Organizational unit name
                              CN    - Common name
                              emailAddress - E-mail address
        Returns:   The certificate request in an X509Req object
        """
        req = crypto.X509Req()
        subj = req.get_subject()

        for (key,value) in name.items():
            setattr(subj, key, value)

        req.set_pubkey(pkey)
        req.sign(pkey, digest)
        return req

