#!/usr/bin/python

"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""


import sys
import shutil # To copy the cert
from PyQt4 import QtCore, QtGui
#sys.path.append('.')
from NupkiGui import UiNupki
from nupki import NupkiDB

key_storage = "./keys"
CNselected = None
db = NupkiDB()

def list_cert_user(data):
    print data

class nupkiTables:
    def __init__(self, ui, tablename):
        self.ui = ui
        if tablename == "overview":
            self.initOverviewTable()
        if tablename == "clients":
            self.initClientsTable()
        if tablename == "servers":
            self.initServersTable()

        self.tablename = tablename

    def __initTable__(self, table):
        headers = ["id", "Common name", "Email address", "Organizational Unit", "Organization", "Location", "State", "Country", "Status"]
        for i in headers:
            table.insertColumn(table.columnCount())
        table.setHorizontalHeaderLabels(headers)
        table.horizontalHeader().setResizeMode(QtGui.QListView.Adjust, QtGui.QHeaderView.Interactive)
        table.verticalHeader().hide()
        table.setShowGrid(True)
        table.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        table.hideColumn(0)
        table.connect(table, QtCore.SIGNAL('itemClicked(QTableWidgetItem *)'),
                      self.itemClicked)

    def initOverviewTable(self):
        self.__initTable__(self.ui.tableOverview)

    def initClientsTable(self):
        self.__initTable__(self.ui.tableClients)

    def initServersTable(self):
        self.__initTable__(self.ui.tableServers)

    def itemClicked(self, item):
        print "item clicked"
        print self.tablename
        print item.row()
        print item.text()
        if self.tablename == "clients":
            print self.ui.tableClients.item(item.row(), 0).text()
            # Ugly! but remember we are in /tests :P
            global CNselected
            CNselected = self.ui.tableClients.item(item.row(), 1).text()
            print "Selected CN:" + CNselected

    def updateRows(self, db, pkiid, row):
        print "==>" + str(row)
        if self.tablename == "clients" and row[0] == "client":
            rownb = self.ui.tableClients.rowCount()

            self.ui.tableClients.insertRow(rownb)
            i = 0
            while i < self.ui.tableClients.columnCount() - 1:
                self.ui.tableClients.setItem(rownb, i, QtGui.QTableWidgetItem(str(row[i+1])))
                i = i + 1

            certStatus = db.getCertStatus(pkiid, row[1])
            if certStatus == "expired":
                item = self.ui.tableClients.item(0, i)
                print str(item)
                #.setBackground(QtGui.QBrush(QtGui.QColor(255, 0, 0)))
            self.ui.tableClients.setItem(rownb, i, QtGui.QTableWidgetItem(str(certStatus)))



def exportcert():

    if CNselected:
        filename = QtGui.QFileDialog.getSaveFileName(None, "Export certificate",
                                                     "%s.pem" % (CNselected),
                                                     "Certificate (*.pem)")
        print "Export cert %s as file %s" % (db.getCert("client", CNselected), filename)
        shutil.copy(db.getCert("client", CNselected), filename)
    else:
        print "Nothing selected!"

if __name__ == "__main__":
    ui = UiNupki.Ui_MainWindow()
    app = QtGui.QApplication(sys.argv)
    window = QtGui.QMainWindow()

    ui.setupUi(window)
    window.setWindowTitle("NuPki GUI")
    window.show()

    pkiid = db.pkiIDGet("test1")

    overview = nupkiTables(ui, "overview")
    clients = nupkiTables(ui, "clients")
    servers = nupkiTables(ui, "servers")

    ui.exportcert.connect(ui.exportcert, QtCore.SIGNAL('clicked()'),
                          exportcert)

    db.certUserList(pkiid, clients.updateRows)


#    db.certUserList("test1", list_cert_user)

#    headers = ["foo", "bar"]
#    ui.tableWidget.clear()
#    ui.tableWidget.setHorizontalHeaderLabels(headers)
#    ui.tableWidget.horizontalHeader().setResizeMode(QtGui.QListView.Adjust, QtGui.QHeaderView.Interactive)
#    ui.tableWidget.verticalHeader().hide()
#    ui.tableWidget.setShowGrid(True)
#
#    row = ui.tableWidget.rowCount()
#    ui.tableWidget.insertRow(row)
#    ui.tableWidget.setItem(row, 0, QtGui.QTableWidgetItem("blah"))
#    ui.tableWidget.setItem(row, 1, QtGui.QTableWidgetItem("tintin"))

    sys.exit(app.exec_())

