#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Copyright (C) 2009-2011 EdenWall Technologies
Written by Sebastien Tricaud <stricaud AT inl.fr>

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>

$Id: nupki.py 5969 2009-03-11 09:37:59Z toady $
"""

import sqlite3
from OpenSSL import crypto

key_storage="./keys"

class NupkiDB:
    def __init__(self):
        self.conn = sqlite3.connect('/tmp/pki/certs.sql')
        self.cursor = self.conn.cursor()

    def close(self):
        self.cursor.close()

    def pkiCreator(self, pkiname):
        self.cursor.execute("INSERT INTO pki VALUES (NULL, \"%s\")" % (pkiname))
        self.conn.commit()

    def pkiIDGet(self, pkiname):
        """
        Print all entries in PKI
        """
        self.cursor.execute("SELECT id FROM pki WHERE name = \"%s\";" % (pkiname))
        return self.cursor.fetchone()[0]


#CREATE TABLE user (
#  id        INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
#  pkiid     INTEGER NOT NULL,
#  type      TEXT NOT NULL, /* Should only be C(client) or S(server) */
#  name      TEXT NOT NULL,
#  firstname TEXT NULL,
#  mail      TEXT NOT NULL,
#  cn        TEXT NOT NULL UNIQUE,
#  country   TEXT NOT NULL,
#  province  TEXT NOT NULL,
#  city      TEXT NOT NULL,
#  org       TEXT NOT NULL
#) ;

    def certUserList(self, pkiid, cb):
#            headers = ["Common name", "Email address", "Organizational Unit", "Organization", "Location", "State", "Country", "Status"]
        self.cursor.execute("SELECT type, id, cn, mail, province, org, city, country, name FROM user WHERE pkiid = %d;" % (pkiid))
#        self.cursor.execute("SELECT * FROM user WHERE pkiid = %d;" % (pkiid))
        for row in self.cursor:
            cb(self, pkiid, row)

    def getCertStatus(self, pkiid, cname):
        return "expired"

    def appendCertificate(self, pkiid, type, cname):
        self.cursor.execute("INSERT INTO user VALUES (NULL, %d, \"%s\", \"\", \"\", \"\", \"%s\", \"\", \"\", \"\", \"\")" % (pkiid, type, cname))
        self.conn.commit()

    def getCert(self, type, cname):
        return "%s/%s-%s.cert" % (key_storage, type, cname)


class NupkiOpenSSL:
    def __init__(self):
        pass

    def createKeyPair(self, type, bits):
        """
        Create a public/private key pair.

        Arguments: type - Key type, must be one of TYPE_RSA and TYPE_DSA
                   bits - Number of bits to use in the key
        Returns:   The public/private key pair in a PKey object
        """
        pkey = crypto.PKey()
        pkey.generate_key(type, bits)
        return pkey

    def createCertRequest(self, pkey, digest="md5", **name):
        """
        Create a certificate request.

        Arguments: pkey   - The key to associate with the request
                   digest - Digestion method to use for signing, default is md5
                   **name - The name of the subject of the request, possible
                            arguments are:
                              C     - Country name
                              ST    - State or province name
                              L     - Locality name
                              O     - Organization name
                              OU    - Organizational unit name
                              CN    - Common name
                              emailAddress - E-mail address
        Returns:   The certificate request in an X509Req object
        """
        req = crypto.X509Req()
        subj = req.get_subject()

        for (key,value) in name.items():
            setattr(subj, key, value)

        req.set_pubkey(pkey)
        req.sign(pkey, digest)
        return req

    def createCertificate(self, req, (issuerCert, issuerKey), serial, (notBefore, notAfter), digest="md5"):
        """
        Generate a certificate given a certificate request.

        Arguments: req        - Certificate request to use
                   issuerCert - The certificate of the issuer
                   issuerKey  - The private key of the issuer
                   serial     - Serial number for the certificate
                   notBefore  - Timestamp (relative to now) when the certificate
                                starts being valid
                   notAfter   - Timestamp (relative to now) when the certificate
                                stops being valid
                   digest     - Digest method to use for signing, default is md5
        Returns:   The signed certificate in an X509 object
        """
        cert = crypto.X509()
        #cert.set_serial_number(serial)
        cert.gmtime_adj_notBefore(notBefore)
        cert.gmtime_adj_notAfter(notAfter)
        cert.set_issuer(issuerCert.get_subject())
        cert.set_subject(req.get_subject())
        cert.set_pubkey(req.get_pubkey())
        cert.sign(issuerKey, digest)
        return cert


class Nupki:
    """
    Manages the PKI itself. There are two kinds of PKI models:
    * simple: where best default choices are choosen for the user. Fonction names like fooSimpleBar(self, ...)
    * advanced: where every detail can be tuned. Fonction names like fooBar(self, ...)
    """
    def __init__(self):
        self.db = NupkiDB()
        self.ssl = NupkiOpenSSL()

    def newSimplePki(self, pkiname):
        self.cakey = self.ssl.createKeyPair(crypto.TYPE_RSA, 1024)
        self.careq = self.ssl.createCertRequest(self.cakey, CN='%s' % (pkiname))
        self.cacert = self.ssl.createCertificate(self.careq, (self.careq, self.cakey), 0, (0, 60*60*24*365*5)) # five years
        open('%s/CA-%s.priv' % (key_storage, pkiname), 'w').write(crypto.dump_privatekey(crypto.FILETYPE_PEM, self.cakey))
        open('%s/CA-%s.cert' % (key_storage, pkiname), 'w').write(crypto.dump_certificate(crypto.FILETYPE_PEM, self.cacert))

        try:
            self.db.pkiCreator(pkiname)
        except sqlite3.IntegrityError:
            print "The PKI already exists"

    def newSimpleCert(self, pkiname, cname, type="client"):
            pkey = self.ssl.createKeyPair(crypto.TYPE_RSA, 1024)
            req = self.ssl.createCertRequest(pkey, CN=cname)
            cert = self.ssl.createCertificate(req, (self.cacert, self.cakey), 1, (0, 60*60*24*365*5)) # five years
            open('%s/%s-%s.pkey' % (key_storage, type, cname), 'w').write(crypto.dump_privatekey(crypto.FILETYPE_PEM, pkey))
            open('%s/%s-%s.cert' % (key_storage, type, cname), 'w').write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))

            pkiid = self.db.pkiIDGet(pkiname)
            self.db.appendCertificate(pkiid, type, cname)



    def close(self):
        self.db.close()

if __name__ == "__main__":
    print "Nupki2 © INL 2009..."
    #pki = NupkiOpenSSL()
    nupki = Nupki()
    nupki.newSimplePki("test1")
    #nupki.newSimpleCert("test1", "user1")
    nupki.newSimpleCert("test1", "user2")
    nupki.close()

