#coding: utf-8
"""
Copyright (C) 2009-2011 EdenWall Technologies
Written by Julien Miotte <jmiotte AT edenwall.com>
$Id$
"""

from os import makedirs
from nucentral.client.error import NuCentralError
from templateTest import Test
from os.path import isdir
import sqlite3
from pprint import pprint

class templateCreatePKI(Test):
    """ Test of the PKI creation :
        - check that the directory is created in the filesystem
        - check that the PKI is created in the database
    """
    pki_test = {}
    @classmethod
    def setup_class(cls):
        """
            This method is called once when the class is first instanciated. It
            sets up the testing conditions, while saving the current configuration.
        """
        Test.setup_class()

        cls.test_path = cls.results_path + cls.date + "/TestPKI/"
        if not isdir(cls.test_path):
            makedirs(cls.test_path)

        # If the pki already exists, try to delete it
        try:
            cls.client.call("nupki", "deletePKI", cls.pki_test["name"])
        except NuCentralError:
            print "PKI %s was present, we deleted it", cls.pki_test["name"]

        cls.client.call('nupki', 'createPKI',
                        cls.pki_test["name"],
                        cls.pki_test["org unit"],
                        cls.pki_test["org"],
                        cls.pki_test["loc"],
                        cls.pki_test["state"],
                        cls.pki_test["country"], 1826)

    def test_directory_has_been_created(self):
        """
            Check that the pki directory has been created
        """
        expected = self.pki_test["name"]
        error = "Directory hasn't been created"
        assert isdir("/var/lib/nucentral/nupki2/pki/"+expected), error

    def test_pki_has_been_created_in_db(self):
        """
            Check that the pki has been created in the database
        """
        filename = "/var/lib/nucentral/nupki2/certs.db"
        conn = sqlite3.connect(filename)
        curs = conn.cursor()

        curs.execute("SELECT * FROM pki")
        results = []
        for row in curs:
            results.append(row)

        found = False
        our_pki = None
        for pki in results:
            if pki[1] == self.pki_test["name"]:
                found = True
                our_pki = pki

        assert found == True, "the pki hasn't been created in the db"

        print "Input :"
        pprint(self.pki_test)
        print "In db : "
        pprint (our_pki)

        error = "%s is not provided in the created pki, in the db"
        for item in self.pki_test:
            assert self.pki_test[item] in our_pki, error % item

    def test_pki_has_been_created_in_nupki(self):
        """
            Check that the pki has been created in nupki
        """
        results = self.client.call('nupki', 'listPKI')

        found = False
        our_pki = None
        for pki in results:
            if pki[1] == self.pki_test["name"]:
                found = True
                our_pki = pki

        assert found == True, "nuPKI doesn't show the pki created"

        print "Input :"
        pprint(self.pki_test)
        print "In nupki : "
        pprint (our_pki)

        error = "%s is no provided in the created pki, in nupki"
        for item in self.pki_test:
            assert self.pki_test[item] in our_pki, error % item

    @classmethod
    def teardown_class(cls):
        """
            This method is called once when all tests have been done.
            It restores the saved configuration.
        """
        cls.client.call("nupki", "deletePKI", cls.pki_test["name"])
        Test.teardown_class()
