#coding: utf-8
"""
Copyright (C) 2009-2011 EdenWall Technologies
Written by Julien Miotte <jmiotte AT edenwall.com>
$Id$
"""

from os import makedirs
from templateTest import Test
from os.path import isdir, isfile
from nucentral.client.error import NuCentralError
import sqlite3
from pprint import pprint

class templatePKICreateCertificate(Test):
    """ Test of the certificate creation :
        - check that the certificate is created in the filesystem
        - check that the certificate is created in the database
        - check that NuPKI displays the right info about the certificate
    """

    pki_test = {}
    cert_test = {}

    @classmethod
    def setup_class(cls):
        """
            This method is called once when the class is first instanciated. It
            sets up the testing conditions, while saving the current configuration.
        """
        Test.setup_class()

        cls.test_path = cls.results_path + cls.date + "/TestPKI/"
        if not isdir(cls.test_path):
            makedirs(cls.test_path)

        # If the pki already exists, try to delete it
        try:
            cls.client.call("nupki", "deletePKI", cls.pki_test["name"])
        except NuCentralError:
            print "PKI %s was present, we deleted it", cls.pki_test["name"]

        cls.client.call('nupki', 'createPKI',
                        cls.pki_test["name"],
                        cls.pki_test["organization_unit"],
                        cls.pki_test["organization"],
                        cls.pki_test["location"],
                        cls.pki_test["state"],
                        cls.pki_test["country"], 1826)

        cert = dict(cls.cert_test)
        cert['pki'] = cls.pki_test["name"]
        cert['force'] = True
        cls.client.call('nupki', 'createCertificate', cert)

    def test_cert_is_created(self):
        pki_name = self.pki_test["name"]
        cert_name = self.cert_test["cname"]

        filename = "/var/lib/nucentral/nupki2/pki/%s/%s" % (pki_name, cert_name)
        error = "%s%s hasn't been created"
        for ext in (".csr", ".cert", ".pkey"):
            assert isfile(filename + ext), error % (cert_name, ext)

    def test_cert_is_created_in_db(self):
        filename = "/var/lib/nucentral/nupki2/certs.db"
        conn = sqlite3.connect(filename)
        curs = conn.cursor()

        curs.execute('SELECT * FROM pki')
        for row in curs:
            if row[1] == self.pki_test["name"]:
                pkiid = row[0]

        curs.execute('SELECT type, id, cn, email, orgunit, location, state,'
                     ' country, status FROM certificate WHERE pkiid = %s and'
                     ' type = "%s"' % (str(pkiid), self.cert_test["type"]))
        results = []
        for row in curs:
            results.append(row)

        found = False
        our_cert = None
        for cert in results:
            if cert[2] == self.cert_test["cname"]:
                found = True
                our_cert = cert

        error = "the certificate hasn't been created in the db"

        assert found == True, error

        print "Input :"
        pprint(self.pki_test)
        print "In db : "
        pprint (our_cert)

        error = "%s is no provided in the created certificate, in the db"
        for item in self.cert_test:
            assert self.cert_test[item] in our_cert, error % item

    def test_cert_is_created_in_nupki(self):
        if self.cert_test["type"] == 'client':
            results = self.client.call('nupki',
                                        'listClientCerts',
                                        self.pki_test["name"]
                                      )
        else:
            results = self.client.call('nupki',
                                        'listServerCerts',
                                        self.pki_test["name"]
                                      )

        found = False
        for cert in results:
            if cert[2] == self.cert_test["cname"]:
                found = True
                our_cert = cert

        error = "nuPKI doesn't show the created certificate"
        assert found == True, error

        print "Input :"
        pprint(self.pki_test)
        print "In nupki : "
        pprint (our_cert)

        error = "%s (%s) is no provided in the created pki, in nupki"
        for item in self.cert_test:
            assert self.cert_test[item] in our_cert, error % (item, self.cert_test[item])

    @classmethod
    def teardown_class(cls):
        """
            This method is called once when all tests have been done.
            It restores the saved configuration.
        """
        cls.client.call("nupki", "deletePKI", cls.pki_test["name"])
        Test.teardown_class()
