#coding: utf-8
"""
Copyright (C) 2009-2011 EdenWall Technologies
Written by Julien Miotte <jmiotte AT edenwall.com>
$Id$
"""

from os import makedirs
from templateTest import Test
from nucentral.client.error import NuCentralError
from os.path import isdir
import sqlite3

class TestDeletePKI(Test):
    """
        Test of the PKI deletion :
        - check that the certificate is deleted in the filesystem
        - check that the certificate is deleted in the database
        - check that NuPKI doesn't display info about the certificate
    """

    pki_test = { "name"      : u"pkitest",
                 "org unit"  : u"testou",
                 "org"       : u"testo",
                 "loc"       : u"testl",
                 "state"     : u"teststate",
                 "country"   : u"fr"}
    cert_test = { "type"      : u"client",
                  "cname"     : u"cornelius",
                  "email"     : u"cornelius@edenwall.com",
                  "org unit"  : u"testou",
                  "loc"       : u"testl",
                  "state"     : u"teststate",
                  "country"   : u"fr"}

    @classmethod
    def setup_class(cls):
        """
            This method is called once when the class is first instanciated. It
            sets up the testing conditions, while saving the current configuration.
        """
        Test.setup_class()

        cls.test_path = cls.results_path + cls.date + "/TestPKI/"
        if not isdir(cls.test_path):
            makedirs(cls.test_path)

        # If the pki already exists, try to delete it
        try:
            cls.client.call("nupki", "deletePKI", cls.pki_test["name"])
        except NuCentralError:
            print "PKI %s was present, we deleted it", cls.pki_test["name"]

        cls.client.call('nupki', 'createPKI',
                        cls.pki_test["name"],
                        cls.pki_test["org unit"],
                        cls.pki_test["org"],
                        cls.pki_test["loc"],
                        cls.pki_test["state"],
                        cls.pki_test["country"], 1826)

        cert = dict(cls.cert_test)
        cert['pki'] = cls.pki_test["name"]
        cert['force'] = True
        cls.client.call('nupki', 'createCertificate', cert)
        cls.client.call("nupki", "deletePKI", cls.pki_test["name"])

    def test_directory_deleted(self):
        """
            Check that the cert directory has been deleted
        """
        expected = self.pki_test["name"]
        error = "directory hasn't been deleted"

        assert not isdir("/var/lib/nucentral/nupki2/pki/"+expected), error

    def test_pki_deleted_in_db(self):
        """
            Check that the pki is no longer in the database
        """
        filename = "/var/lib/nucentral/nupki2/certs.db"
        conn = sqlite3.connect(filename)
        curs = conn.cursor()

        curs.execute("SELECT * FROM pki")
        results = []
        for row in curs:
            results.append(row)

        found = False
        for pki in results:
            if pki[1] == self.pki_test["name"]:
                found = True

        error = "The pki hasn't been deleted in the db"
        assert found == False, error

    def test_pki_deleted_in_nupki(self):
        """
            Check that the pki is no longer shown in nupki
        """
        results = self.client.call('nupki','listPKI')

        found = False
        for pki in results:
            if pki[1] == self.pki_test["name"]:
                found = True

        error = "nuPKI still shows the pki created"
        assert found == False, error

    def test_cert_deleted_in_db(self):
        """
            Check that the test certificate is deleted in the database
        """
        filename = "/var/lib/nucentral/nupki2/certs.db"
        conn = sqlite3.connect(filename)
        curs = conn.cursor()

        request = 'SELECT type, id, cn, email, orgunit, ' \
                         'location, state, country, status  ' \
                  ' FROM certificate WHERE type = "%s"'
        curs.execute(request % self.cert_test["type"])

        results = []
        for row in curs:
            results.append(row)

        found = False
        for cert in results:
            if cert[2] == self.cert_test["cname"]:
                found = True

        error = "The certificate hasn't been deleted in the db"
        assert found == False, error

    def test_cert_deleted_in_nupki(self):
        """
            Check that the test certificate is deleted in nupki
        """
        found = True
        try:
            if self.cert_test["type"] == 'client':
                results = self.client.call('nupki',
                                           'listClientCerts',
                                           self.pki_test["name"])
            else:
                results = self.client.call('nupki',
                                           'listServerCerts',
                                           self.pki_test["name"])

            found = False
            for cert in results:
                if cert[2] == self.cert_test["cname"]:
                    found = True
        except NuCentralError:
            found = False

        error = "nuPKI still shows one certificate"
        assert found == False, error

    @classmethod
    def teardown_class(cls):
        """
            This method is called once when all tests have been done.
            It restores the saved configuration.
        """
        Test.teardown_class()
