#!/bin/sh

# generates certificates for tests
#== Validité des certificats ==
#La nomenclature définie est la suivante :
# * V : Valide
# * O : trop tOt pour utiliser (pas encore valide)
# * A : trop tArd pour utiliser (plus valide)
# * R : Révoqué (non applicable à la CA)
# * N : Non trusted (appliqué au client ou serveur)

#== Arguments ==
# $1 : CA
# $2 : subCA ("_" pour désactiver)
# $3 : user cert
# $4 : user cert signed by CA or subCA ("c" or "s")
# $5 : server cert
# $6 : server cert signed by CA or subCA ("c" or "s")
# $7 : is the crl valid, too old or too young ?

set -e

make distclean-local

# Default values 
val_ca=v
val_subca=v
val_user=v
user_signer=c
val_server=v
server_signer=c
crl_expiration=v

# Custom values
val_ca=$1
val_subca=$2
val_user=$3
user_signer=$4
val_server=$5
server_signer=$6
crl_expiration=$7

ca_name="CA"
subca_name="subCA"
user_name="cornelius"
server_name="ew4.edenwall.com"

future=`date -d "now + 20 years" +%y%m%d%I%M%S`

dir_name="$val_ca$val_subca$val_user$user_signer$val_server$server_signer$crl_expiration"
echo $dir_name

echo "ooooooooooooooooooooooooo"
echo "ooooooo CA ! oooooooooooo"
echo "ooooooooooooooooooooooooo"

# Making of CA 
[ "x$val_ca" = "xv" ] && make CA NAME=$ca_name CUSTOM_OU_NAME="$dir_name"
[ "x$val_ca" = "xo" ] && make CA NAME=$ca_name MAC_FLY=yes CUSTOM_OU_NAME="$dir_name"
[ "x$val_ca" = "xa" ] && make CA NAME=$ca_name CA_DAYS=-20 CUSTOM_OU_NAME="$dir_name"

echo 
echo "ooooooooooooooooooooooooo"
echo "ooooooo subCA ! ooooooooo"
echo "ooooooooooooooooooooooooo"

## Making of subCA
[ "x$val_subca" = "xv" ] && make subca NAME=$subca_name CUSTOM_OU_NAME="$dir_name" 
[ "x$val_subca" = "xo" ] && make subca NAME=$subca_name CUSTOM_OU_NAME="$dir_name" START_DATE=$future
[ "x$val_subca" = "xa" ] && make subca NAME=$subca_name CUSTOM_OU_NAME="$dir_name" CA_DAYS=-20
[ "x$val_subca" = "xr" ] && (   make subca  NAME=$subca_name CUSTOM_OU_NAME="$dir_name"; \
                                make revoke NAME=$subca_name CUSTOM_OU_NAME="$dir_name" CA_NAME=$ca_name   )

echo "ooooooooooooooooooooooooo"
echo "ooooooo user ! oooooooooo"
echo "ooooooooooooooooooooooooo"

# Making of user cert
this_signer=$ca_name
if [ "x$user_signer" = "xs" ] 
then
    this_signer=$subca_name
fi
[ "x$val_user" = "xv" ] && make cert NAME=$user_name CUSTOM_OU_NAME="$dir_name" CA_NAME=$this_signer CERT_TYPE=user
[ "x$val_user" = "xo" ] && make cert NAME=$user_name CUSTOM_OU_NAME="$dir_name" CA_NAME=$this_signer CERT_TYPE=user START_DATE=$future
[ "x$val_user" = "xa" ] && make cert NAME=$user_name CUSTOM_OU_NAME="$dir_name" CA_NAME=$this_signer CERT_TYPE=user CERT_DAYS=-20
[ "x$val_user" = "xr" ] &&( make cert NAME=$user_name CUSTOM_OU_NAME="$dir_name" CA_NAME=$this_signer CERT_TYPE=user; \
                            make revoke NAME=$user_name; )

echo "ooooooooooooooooooooooooo"
echo "ooooooo server ! oooooooo"
echo "ooooooooooooooooooooooooo"

# Making of server cert
this_signer=$ca_name
if [ "x$server_signer" = "xs" ] 
then
    this_signer=$subca_name
fi
[ "x$val_server" = "xv" ] && make cert NAME="$server_name" CUSTOM_OU_NAME="$dir_name" CA_NAME=$this_signer CERT_TYPE=server
[ "x$val_server" = "xo" ] && make cert NAME="$server_name" CUSTOM_OU_NAME="$dir_name" CA_NAME=$this_signer CERT_TYPE=server START_DATE=$future
[ "x$val_server" = "xa" ] && make cert NAME="$server_name" CUSTOM_OU_NAME="$dir_name" CA_NAME=$this_signer CERT_TYPE=server CERT_DAYS=-20
[ "x$val_server" = "xr" ] && ( make cert NAME="$server_name" CUSTOM_OU_NAME="$dir_name" CA_NAME=$this_signer CERT_TYPE=server; \
                               make revoke NAME="$server_name" CA_NAME=$this_signer )

echo "#########################"
echo "####### gencrl ! ########"
echo "#########################"

[ "x$crl_expiration" = "xv" ] && make gencrl CA_NAME=$ca_name
[ "x$crl_expiration" = "xa" ] && make gencrl CRL_DAYS=-20 CA_NAME=$ca_name
[ "x$crl_expiration" = "xo" ] && make gencrl MAC_FLY=yes CA_NAME=$ca_name

[ "x$val_subca" != "x_" ] && [ "x$crl_expiration" = "xv" ] && make gencrl CA_NAME=$subca_name
[ "x$val_subca" != "x_" ] && [ "x$crl_expiration" = "xa" ] && make gencrl CRL_DAYS=-20 CA_NAME=$subca_name
[ "x$val_subca" != "x_" ] && [ "x$crl_expiration" = "xo" ] && make gencrl MAC_FLY=yes CA_NAME=$subca_name

echo "#########################"
echo "####### Last step #######"
echo "#########################"

if [ "x$val_subca" != "x_" ]
then
    cat "$ca_name".crt "$subca_name".crt > allCA.crt
    cat "$ca_name"crl.pem "$subca_name"crl.pem > allcrl.pem
else
    cp "$ca_name".crt allCA.crt
    cp "$ca_name"crl.pem allcrl.pem
fi
echo "Moving *.crt and *.key to certs/"$dir_name
mkdir certs/"$dir_name"
mv *.crt *.key *crl.pem certs/"$dir_name"
