#vim:fdm=marker
from _winreg import OpenKey, CreateKey, CloseKey, SetValueEx, \
                    HKEY_CURRENT_USER, KEY_ALL_ACCESS, REG_SZ
from ConfigParser import ConfigParser

from nucentral.client.nucentral_client_base import NuCentralClientBase
from nucentral.common.download import encodeFileContent

import subprocess
import os
import socket
import time
import re
import sys

FILEPATTERN = "[voa][voar_][voar][cs][voar][cs][voa]"
# {{{1 class TestCase
class TestCase:
    conf = {'CA':{'path':'', 'validity':''},
            'subCA':{'path':'', 'validity':''},
            'user_cert':{'path':'', 'validity':'', 'signer':''},
            'user_key':{'path':'', 'validity':'Irrelevant.'},
            'server_cert':{'path':'', 'validity':'', 'signer':''},
            'server_key':{'path':'', 'validity':'Irrelevant.'},
            'CAcrl':{'path':'', 'validity':''},
            'subCAcrl':{'path':'', 'validity':''}
    }
# {{{2 def __init__
    def __init__(self, test_dir):
        if os.path.abspath(test_dir) != test_dir:
            raise Exception, "Please give the absolute path to the test directory as an argument."
        case = os.path.basename(test_dir)

        if not re.match(FILEPATTERN, case):
            raise Exception, "The given directory name is not valid. It should be a match for : %s"%FILEPATTERN

        self.conf['CA']['path'] = test_dir+"\\"+"allCA.crt"
        self.conf['CA']['validity'] = self.switchValidity(case[0])

        self.conf['subCA']['path'] = test_dir+"\\"+"allCA.crt"
        self.conf['subCA']['validity'] = self.switchValidity(case[1])

        self.conf['user_cert']['path'] = test_dir+"\\"+"cornelius.crt"
        self.conf['user_cert']['validity'] = self.switchValidity(case[2])
        self.conf['user_cert']['signer'] = self.switchSigner(case[3])

        self.conf['user_key']['path'] = test_dir+"\\"+"cornelius.key"

        self.conf['server_cert']['path'] = test_dir+"\\"+"ew4.edenwall.com.crt"
        self.conf['server_cert']['validity'] = self.switchValidity(case[4])
        self.conf['server_cert']['signer'] = self.switchValidity(case[5])

        self.conf['server_key']['path'] = test_dir+"\\"+"ew4.edenwall.com.key"

        self.conf['CAcrl']['path'] = test_dir+"\\"+"CAcrl.pem"
        self.conf['CAcrl']['validity'] = self.switchValidity(case[6])

        self.conf['subCAcrl']['path'] = test_dir+"\\"+"subCAcrl.pem"
        self.conf['subCAcrl']['validity'] = self.switchValidity(case[6])
# 2}}}
# {{{2 def switchValidity
    def switchValidity(self, value):
        if value == "v":
            return "Valid."
        if value == "r":
            return "Revoked."
        if value == "a":
            return "Too old."
        if value == "o":
            return "Not old enough."
# 2}}}
# {{{2 def switchSigner
    def switchSigner(self, value):
        if value == "c":
            return "CA signed."
        if value == "s":
            return "SubCA signed."
# 2}}}
# {{{2 def isUserSubCASigned
    def isUserSubCASigned(self):
        return self.conf['user_cert']['signer'] == 's'
# 2}}}
# {{{2 def isServerSubCASigned
    def isServerSubCASigned(self):
        return self.conf['server_cert']['signer'] == 's'
# 2}}}
# 1}}}
# {{{1 class registry_conf
class RegistryConf:
    """
        This class helps us to set values to the windows registry.
    """
# {{{2 def __init__
    def __init__(self):
        pass
# 2}}}
# {{{2 def set_reg_conf
    def set_reg_conf(self, test_path, ew4_ip, check_fqdn=True, use_crl=True):
        """
            This method reads the self.configuration object and inserts the following
            fields in the Windows registry :
             * hostname
             * port
             * cert
             * key
             * ca
             * crl
             * cert_auth_by_cert
             * cert_use_client_cert
             * cert_verify_remote
             * cert_media
             * use_certificate
             * suppress_fqdn
            The registry directory for the agent is :
            HKEY_CURRENT_USER\SOFTWARE\EdenWall Technologies\EdenWall Agent
        """
        self.clean_reg_conf()
        keyVal = r'SOFTWARE\EdenWall Technologies\EdenWall Agent'
        try:
            key = OpenKey(HKEY_CURRENT_USER, keyVal, 0, KEY_ALL_ACCESS)
        except:
            key = CreateKey(HKEY_CURRENT_USER, keyVal)

        if check_fqdn:
            check_fqdn = "true"
        else:
            check_fqdn = "false"
        SetValueEx(key, "cert_check_fqdn", 0, REG_SZ, check_fqdn)
        SetValueEx(key, "cert_auth_by_cert", 0, REG_SZ, "true")
        SetValueEx(key, "cert_use_client_cert", 0, REG_SZ, "true")
        SetValueEx(key, "cert_verify_remote", 0, REG_SZ, "true")
        SetValueEx(key, "cert_media", 0, REG_SZ, "local")

        SetValueEx(key, "hostname", 0, REG_SZ, ew4_ip)
        SetValueEx(key, "port", 0, REG_SZ, "4129")

        SetValueEx(key, "ca", 0, REG_SZ, "%s\\allCA.crt"%test_path)
        SetValueEx(key, "cert", 0, REG_SZ, "%s\\cornelius.crt"%test_path)
        SetValueEx(key, "key", 0, REG_SZ, "%s\\cornelius.key"%test_path)
        if use_crl:
            SetValueEx(key, "crl", 0, REG_SZ, "%s\\crl.pem"%test_path)

        CloseKey(key)
# 2}}}
# {{{2 def clean_reg_conf
    def clean_reg_conf(self):
        """
            This method will wipe everything that was set in set_reg_conf
        """
        keyVal = r'SOFTWARE\EdenWall Technologies\EdenWall Agent'
        try:
            key = OpenKey(HKEY_CURRENT_USER, keyVal, 0, KEY_ALL_ACCESS)
        except:
            key = CreateKey(HKEY_CURRENT_USER, keyVal)
        SetValueEx(key, "hostname", 0, REG_SZ, "")
        SetValueEx(key, "port", 0, REG_SZ, "")
        SetValueEx(key, "ca", 0, REG_SZ, "")
        SetValueEx(key, "cert", 0, REG_SZ, "")
        SetValueEx(key, "key", 0, REG_SZ, "")
        SetValueEx(key, "crl", 0, REG_SZ, "")
        SetValueEx(key, "cert_auth_by_cert", 0, REG_SZ, "")
        SetValueEx(key, "cert_use_client_cert", 0, REG_SZ, "")
        SetValueEx(key, "cert_verify_remote", 0, REG_SZ, "")
        SetValueEx(key, "cert_media", 0, REG_SZ, "")
        SetValueEx(key, "cert_check_fqdn", 0, REG_SZ, "")
        CloseKey(key)
# 2}}}
# 1}}}
# {{{1 class Ew4Com
class Ew4Comm:
    """
        This class helps us to communicate with the EW.
    """

    _client = None
    _ew4_method = ""
    _ew4_ip = ""
    _ew4_login = ""
    _ew4_passwd = ""

    FORBIDDEN = 0
    ALLOWED = 1
    MANDATORY = 2

# {{{2 def __init__
    def __init__(self, ip, method, login, passwd):
        self._ew4_ip = ip
        self._ew4_method = method
        self._ew4_login = login
        self._ew4_passwd = passwd

        self._client = self._connect_ew4()
        self._pleaseLeave()

        self._check_ew4_license()
        self._client.call('nuconf', 'takeWriteRole')
# 2}}}
# {{{2 def teardown
    def teardown(self):
        """
            Logout.
        """
        self._client.call('session','destroy')
        self._client.logout()
# 2}}}
# {{{2 def _checkAndUploadLicense
    def _checkAndUploadLicense(self, client):
        """
            This method checks if the EW has a correct license.
            If it doesn't, it looks for one in the current directory and upoads it.
            The license expected filename is "license.asc".
        """
        if not client.call('CORE', 'hasComponent', 'license'):
            # No license component
            return False

        if client.call('license', 'checkValidity'):
            return False

        filename = "license.asc"
        if not os.path.isfile(filename):
            raise Exception(
                "Please copy a license into the current directory. "
                "The current expected filename is %s."
                % filename)

        fd = open(filename, 'rb')
        content = fd.read()

        minimalmode = client.call('acl', 'getMinimalMode')

        print "Uploaded license file %s" % os.path.basename(filename)
        ok = client.call('license', 'sendLicense', encodeFileContent(content))
        if not ok:
            raise Exception("No new license for this appliance in this file.")

        return minimalmode and (not client.call('acl', 'getMinimalMode'))
# 2}}}
# {{{2 _check_ew4_license
    def _check_ew4_license(self):
        """
            Check we have a license, and upload one if we need.
        """
        __client = self._connect_ew4()

        try:
            self._checkAndUploadLicense(__client)
        except:
            __client.logout()
            raise

        __client.logout()
# 2}}}
# {{{2 def _pleaseLeave
    def _pleaseLeave(self):
        """
            AKA GetTheF***Out
        """
        me = self._client.call('session', 'get')['cookie']
        for item in self._client.call('session', 'list'):
            cookie = item['cookie']
            if cookie == me:
                continue
            self._client.call('session', 'destroySession', cookie)
# 2}}}
# {{{2 def connect_ew4
    def _connect_ew4(self):
        """
            Conveniance method, we connect to the ew and return the handle.
        """
        _client = NuCentralClientBase(  protocol=self._ew4_method,
                                        host=self._ew4_ip
                                     )
        _client.authenticate(self._ew4_login, self._ew4_passwd)

        return _client
# 2}}}
# {{{2 def _set_ew4_iptables_conf
    def _set_ew4_iptables_conf(self, _client):
        """
            This method connects to the EdenWall 4 to set the correct filtering rules.
        """

        pass
# 2}}}
# {{{2 def _set_ew4_cert_conf
    def _set_ew4_cert_conf(self,
                           test_case,
                           strict_mode=True,
                           auth_by_cert=2,
                           use_crl=True):
        """
            We should make auth_by_cert and strict vary
            This method connects to the EdenWall 4 to set the correct certificates.
            ('session', 'destroy')

            Certs names are :
            * allCA.crt
            * ew4.edenwall.com.crt
            * ew4.edenwall.com.key
            * crl.pem

        """
        certs = {
            'ca':'allCA.crt',
            'certificate':'ew4.edenwall.com.crt',
            'key':'ew4.edenwall.com.key',
        }
        if use_crl:
            certs['crl'] = 'crl.pem'

        big_dir = {
            'DATASTRUCTURE_VERSION': 2,
            '__type__': 'nuconf.common.auth_cert_cfg.AuthCertConf',
            'auth_by_cert': auth_by_cert,
            'portal_enabled': False,
            'portal_nets': {'__type__': 'set'},
            'strict': strict_mode,
            'use_pki': False
        }

        for type in big_dir:
            abs_filename = test_case+'\\'+certs[type]

            fd = open(abs_filename, 'rb')
            content = fd.read()

            self._client.call(  'auth_cert', 'sendFile',
                                type, encodeFileContent(content)
                             )
            big_dir[type] = unicode(abs_filename)

            fd.close()

        self._client.call(  'auth_cert', 'setAuthCertConfig',
                            big_dir, u'testing certs'
                         )
        self._client.call('config', 'apply')
# 2}}}
# 1}}}
# {{{1 class TestCert
class TestCert:
    _remote_ip = "192.168.22.10"
    _remote_port = "80"
    _ew4_method = "https"
    _ew4_ip = "172.17.4.14"
    _ew4_login = "admin"
    _ew4_passwd = "admin"

# {{{2 def __init__
    def __init__(self):
        self._get_test_conf()
# 2}}}
# {{{2 def _get_test_conf
    def _get_test_conf(self):
        """
            This method reads the configuration in <current_directory>/tests_agent.conf
            The configuration fields are :
             * EW4 IP address
             * remote IP
        """
        conf = ConfigParser()
        res = conf.read('autotest.conf')
        if res != []:
            self._ew4_method = conf.get('ew4', 'method')
            self._ew4_ip = conf.get('ew4', 'ip')
            self._ew4_login = conf.get('ew4', 'login')
            self._ew4_passwd = conf.get('ew4', 'passwd')

            self._remote_ip = conf.get('remote','ip')
# 2}}}
# {{{2 def _launch_agent
    def _launch_agent(self):
        """ This method launches the agent.
        """
        time.sleep(2)
        prog = "\"C:\\Program Files\\EdenWall Technologies\\EdenWall Agent\\bin\\nuapplet2.exe\""
        handle = subprocess.Popen(prog)
        return handle.pid
# 2}}}
# {{{2 def _kill_agent
    def _kill_agent(self, pid):
        """ This method terminates the process using the pid given.
        """
        subprocess.Popen(   "taskkill /f /im nuapplet2.exe",
                            stdout=subprocess.PIPE,
                            shell=True
                        )
# 2}}}
# {{{2 def _check_connection
    def _check_connection(self):
        """ Here we check that we have connection.
        """
        try:
            socket.create_connection((self._remote_ip, self._remote_port), 3)
        except:
            return "fail"
        return "succeed"
# 2}}}
# {{{2 def _report_result
    def _report_result(self, logfile, test_dir, result):
        """ Here we report the results in a text file.
        """
        logfile.write("%s \t\t: %s\n"%(test_dir, result))
# 2}}}
# {{{2 def run_test
    def run_test(self,
                 test_dir,
                 manual=False,
                 fqdn_check=True,
                 strict_cert=True,
                 auth_mode=2,
                 user_crl=True,
                 server_crl=True):
        """
            Here we do one test.
        """
        my_test_case = TestCase(test_dir)

        if server_crl and my_test_case.isUserSubCASigned():
            server_crl = "subCA"
        else:
            server_crl = "CA"

        if user_crl and my_test_case.isUserSubCASigned():
            user_crl = "subCA"
        else:
            user_crl = "CA"

        client_ew4 = Ew4Comm(self._ew4_ip,
                             self._ew4_method,
                             self._ew4_login,
                             self._ew4_passwd)

        client_ew4._set_ew4_cert_conf(my_test_case,
                                      strict_mode=strict_cert,
                                      auth_by_cert=auth_mode,
                                      use_crl=server_crl)

        reg_agent = RegistryConf()
        reg_agent.set_reg_conf(my_test_case,
                               self._ew4_ip,
                               check_fqdn=fqdn_check,
                               use_crl=user_crl)

        time.sleep(1)
        pid = self._launch_agent()
        client_ew4.teardown()

        if not manual:
            time.sleep(1)
            result = self._check_connection()
            self._kill_agent(pid)
            return result
        return True
# 2}}}
# 1}}}

if __name__ == "__main__":
    pattern = "[voa][voar_][voar][cs][voar][cs][voa]"
    # MAIN
    this_test = TestCert()

    # Is this a cert directory ?
    this_dir = os.path.basename(os.path.abspath(os.path.curdir))
    if re.match(pattern, this_dir):
        this_test.run_test(os.path.curdir)
        sys.exit(0)

    # If it's not, run tests normally
    else:
        #if single_test:
        #    logfile=open(single_test+"./RESULTS.txt", "a")
        #else:
        #    logfile=open("./RESULTS.txt", "a")

        #if not single_test:
        #    logfile.write("Test Name.Expected \t: Result\n")

        #if not single_test:
        #    for test_dir in os.listdir(os.path.curdir):
        #        if os.path.isdir(test_dir) and re.match(pattern, test_dir):
        #            result=self._one_test(test_dir, kill)

        #for test_dir in os.listdir(os.path.curdir):
        #    if os.path.isdir(test_dir) and re.match(pattern, test_dir):
        #        result=this_test.run_test(test_dir, kill)
        sys.exit(0)
