#!/usr/bin/python

"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""


from __future__ import with_statement
from os.path import basename

#from ldap import initialize, SCOPE_SUBTREE
from nucentral.client.nucentral_client_base import NuCentralClientBase
from nucentral.client.error import NuCentralError
from nucentral.common.error import exceptionAsUnicode
from nuconf.common.nuauthcfg import NuauthCfg
from nuconf.common.nuauthcfg import LDAPConf
from nuconf.common.netcfg_rw import deserialize
from nuconf.common.net_objects_rw import NetRW
from nucentral.common.download import encodeFileContent
from nuconf.common.openvpn_cfg import OpenVpnConf
from nuconf.common.access_cfg import AccessConf

from IPy import IP

from autoconf.config import CONF

from copy import deepcopy


# connection IP
# first network
EW4_IP0 = 'x.x.x.x'
EW4_NET0 = 'x.x.x.x/y'
# second network
EW4_IP1 = 'x.x.x.x'
EW4_NET1 = 'x.x.x.x/y'

class TestClass():

    method = 'https'
    ruleV4 = {
                'name':"acl_test1",

                'description':"  * SSH, avec auth, avec label, single/single",

                'rule': {'applications': [],
                         'auth_quality': 3,
                         'comment': '',
                         'decision': 'ACCEPT',
                         'destinations': ['Test2'],
                         'durations': [],
                         'enabled': True,
                         'id': 10,
                         'log': False,
                         'log_prefix': '',
                         'mandatory': True,
                         'operating_systems': [],
                         'periodicities': [],
                         'protocols': ['SSH'],
                         'sources': ['Test1'],
                         'user_groups': ['tests']},
           }


    def setup(self):
        self.client = NuCentralClientBase(host=CONF['host'],protocol=self.method)
        self.client.authenticate(CONF['user'],CONF['pass'])

    def version(self):
        return self.client.call('CORE','getStatus')

    def setupLicense(self):
        """
            # cleanup:
            rm -f /var/lib/nucentral/license/*.gpg
            sqlite3 /var/lib/nucentral/license/licenses.db "DELETE FROM licenses"
            /etc/init.d/nucentral-server restart
        """
        print "[+] setting up license"
        filename = CONF['license']
        basefilename = basename(filename)
        with open(filename, 'rb') as fd:
            content = fd.read()
        try:
            minimalmode = self.client.call('acl', 'getMinimalMode')
            if self.client.call('license', 'sendLicense',
                                encodeFileContent(content)):
                print "Uploaded license file `%s'" % basefilename
                if minimalmode and not self.client.call('acl',
                                                        'getMinimalMode'):
                    print "restarting client"
                    self.client.logout()
                    self.tearDown()
                    self.setup()
            else:
                print "No new license for this appliance in this file."
        except NuCentralError, err:
            print "Error while uploading license file `%s': %s" % (basefilename, exceptionAsUnicode(err))

    def setupNetwork(self):
        print "[+] setting up network"
        # We save the configuration to restore it later
        serialized = cls.client.call('network','getNetconfig')
        cls.orig=deepcopy(serialized)

        # Let's tune the netconf to match our needs
        netcfg = deserialize(serialized)

        eth0 = netcfg.getInterfaceBySystemName('eth0')
        eth1 = netcfg.getInterfaceBySystemName('eth1')

        netcfg.removeInterface(eth0)
        netcfg.removeInterface(eth1)

        net1 = NetRW(
                    "Test1",
                    IP(EW4_NET0),
                    service_ip_addrs=set(IP(EW4_IP0))
                    )

        net2 = NetRW(
                    "Test2",
                    IP(EW4_NET1),
                    service_ip_addrs=set(IP(EW4_IP1))
                    )

        eth0.addNet(net1)
        eth1.addNet(net2)

        serialized = netcfg.serialize()
        cls.client.call("network","setNetconfig",serialized, "none")

        # set access permissions
        data = self.client.call('access', 'getConfig')
        access_cfg = AccessConf.deserialize(data)
        #print access_cfg.permissions['nucentral_access']
        s = access_cfg.permissions['nucentral_access']
        s.add( ('eth0',IP(EW4_NET0)) )
        s.add( ('eth1',IP(EW4_NET1)) )
        access_cfg.permissions['nucentral_access'] = s
        self.client.call('access','setConfig', 'pas de message', access_cfg.serialize())

        cls.client.call("config","apply")


    def setupLDAP(self):
        print "[+] setting up LDAP"

        userConf=LDAPConf()
        groupConf=LDAPConf()

        groupConf.dn = "ou=Groups,dc=inl,dc=fr"
        userConf.dn = "ou=Users,dc=inl,dc=fr"

        for conf in groupConf, userConf:
            conf.user = "cn=admin,dc=inl,dc=fr"
            conf.password = "INLbabar286"
            conf.setUri("ldap://192.168.33.190")
            conf.custom_or_nupki = "SSL_DISABLED"
            conf.reqcert = "allow"

        nuauthcfg = NuauthCfg(userConf,groupConf)

        serialized = nuauthcfg.serialize()

        self.client.call('nuauth','setNuauthConfig',serialized,'pas de message')
        self.client.call('config','apply')

    def setupRules(self):
        print "[+] setting up Rules"
        # We save the last applied ruleset name
        self.origRuleset = None
        productionRules = self.client.call('nuface','productionRules')
        if productionRules.has_key('ruleset'):
            self.origRuleset=self.client.call('nuface','productionRules')['ruleset']

        # If the ruleset used exists, we delete it
        for ruleset in self.client.call('nuface','rulesetList','ruleset'):
            if self.ruleV4['name'] in ruleset:
                self.client.call('nuface','rulesetDelete','ruleset',self.ruleV4['name'])
                break

        # We create a new ruleset and name it
        self.client.call('nuface','rulesetCreate','ruleset','')

        # We create a user group
        self.client.call('nuface','objectCreate','user_groups', {'group': '9000', 'id': 'tests'}, False)

        # We add the rule
        self.client.call('nuface','ruleCreate','acls-ipv4',self.ruleV4['rule'])

        # Save and apply the rule
        self.client.call('nuface','rulesetSaveAs',self.ruleV4['name'])
        self.client.call('nuface','applyRules','True','True')
        self.client.call('nuface','rulesetClose')

    def setupPKI(self):
        print "[+] setting up PKI"
        ret = self.client.call('nupki','listPKI')
        #print "PKI list ", ret
        for lol in ret:
            if not lol[1].startswith('__'):
                print "deleting PKI %d: %s" % (lol[0],lol[1])
                self.client.call('nupki','deletePKI',lol[1])
        # create PKI
        self.client.call('nupki','createPKI', 'blah', 'org', 'ou', 'loc', 'st', 'fr', 1826)
        # create Certificate
        self.client.call('nupki','newCert','blah','server.inl.fr','server')
        self.client.call('nupki','updateCertSettings','blah','server.inl.fr','1','cert@inl.fr','org', 'ou', 'loc', 'st', 'fr')
        # force creation
        self.client.call('nupki','forceEachCertAction','blah')

    def setupOpenVPN(self):
        print "[+] setting up OpenVPN"
        c = self.client.call('openvpn','copyPKI','blah','server.inl.fr')
        c = self.client.call('openvpn','getOpenVpnConfig')
        print c
        c = OpenVpnConf(''   # ca
            ,'server.inl.fr' # cert
            ,'10.0.0.0/24'   # client_network
            ,''              # crl
            ,True            # enabled
            ,''              # key
            ,'1194'          # port
            ,'udp'           # protocol
            ,False           # redirect
            ,EW4_IP0         # server
            ,'blah'          # use_pki
            )
        serialized = c.serialize()
        c = self.client.call('openvpn','setOpenVpnConfig',serialized,'pas de message')
        self.client.call('config','apply')
        self.client.call('access','applyRules')

    def tearDown(self):
        self.client.logout()

if __name__ == '__main__':
    cls = TestClass()
    cls.setup()
    print cls.version()
    cls.setupLicense()
    cls.setupNetwork()
    cls.setupLDAP()
    cls.setupRules()
    cls.setupPKI()
    cls.setupOpenVPN()
    cls.tearDown()
