#coding: utf-8
"""
Copyright (C) 2009-2011 EdenWall Technologies
Written by Julien Miotte <jmiotte AT edenwall.com>
$Id: $
"""
from __future__ import with_statement
from templateTest import Test
from os.path import isdir, isfile
from os import makedirs
from ConfigParser import ConfigParser
from nucentral.common.download import encodeFileContent
from nucentral.client.error import NuCentralError
from py import test
from shutil import copy

class templateAuthCertConf(Test):
    """
        Test de la configuration du mécanisme de nucentral d'authentification
        par certificat.
        * La configuration se fait par appels de service. Ce test est
          complémentaire avec TestAuthCertLogin qui vérifie que l'on peut se
          logger en utilisant un certificat.
        * Le test consiste à se vérifier la conformité du fichier de
          configuration.
    """
    config = {}
    paths = {}
    disable_crl = False
    apply_fail = False

    @classmethod
    def setup_class(cls):
        """
            This method is called once when the class is first instanciated. It
            sets up the testing conditions, while saving the current configuration.
        """
        Test.setup_class()

        # Set default value for Exception on apply
        cls.oops = False

        cls.test_path = cls.results_path+cls.date+"/TestAuthCertConf/"
        if not isdir(cls.test_path):
            makedirs(cls.test_path)

        # Check we have all certificates we need
        for cert_file in cls.paths:
            if cls.paths[cert_file] and not isfile(cls.paths[cert_file]):
                test.skip('Missing certificate: %s' % cls.paths[cert_file])

        # Check that nucentral.conf is not empty
        with open("/etc/nucentral/nucentral.conf") as handle:
            if handle.read() == "":
                test.skip("OOPS ! /etc/nucentral/nucentral.conf is empty")

        # Save our nucentral.conf
        src = "/etc/nucentral/nucentral.conf"
        dst = "./"
        copy(src, dst)

        for cert_type in ('ca', 'crl', 'key', 'cert'):
            cert_path = cls.paths[cert_type]
            if cert_path:
                cls.config[cert_type] = cls._path_to_base64(cert_path)
        cls.config['disable_crl'] = cls.disable_crl

    @classmethod
    def _path_to_base64(cls, path):
        with open(path, 'rb') as handle:
            content = handle.read()

        return encodeFileContent(content)

    def _test_apply_conf(self):
        # Set SSL config
        self.oops = False
        try:
            self.client = self.createClient()
            self.client.call("CORE", "setSSLConfig", self.config)
            self.client.logout()
        except NuCentralError, err:
            print str(err)
            self.oops = True
        else:
            self.oops = False
        finally:
            err_mess = "should" if self.apply_fail else "shouldn't"
            error = "This configuration %s fail" % err_mess
            assert self.oops == self.apply_fail, error

    def _test_nucentral_conf(self):
        if not self.oops:
            current_conf = ConfigParser()
            current_conf.read('/etc/nucentral/nucentral.conf')

            # Check that values are correct
            assert current_conf.get("CORE","enable_ssl") == "yes"

            check_clients = "yes" if self.config['check_clients'] else "no"
            assert current_conf.get("ssl", "check_clients") == check_clients, \
                    "Wrong value for check_clients"

            use_nupki = "yes" if self.config['use_nupki'] else "no"
            assert current_conf.get("ssl", "use_nupki") == use_nupki, \
                    "Wrong value for use_nupki"

            # Check that the certificates are the good ones
            for cert_type in ('ca', 'key', 'cert'):

                cert_path = self.paths[cert_type]
                current_cert_path = current_conf.get("ssl", cert_type)
                error = "The %s cert hasn't been copied correctly." % cert_type

                if cert_path:
                    assert self._path_to_base64(cert_path) == \
                        self._path_to_base64(current_cert_path), error

            # Check the crl
            if self.config['disable_crl'] or not self.config['crl']:
                running_crl = current_conf.get("ssl", "crl")
                error = "The crl is set to \"%s\" but it shouldn't be set."
                assert running_crl == "", error % running_crl

            elif not self.config['disable_crl']:
                assert self._path_to_base64(self.paths["crl"]) == \
                    self._path_to_base64(current_conf.get("ssl", "crl")), \
                    "The crl cert hasn't been copied correctly."

    def _test_toggle_CRL(self):
        if not self.oops:
            if self.config['disable_crl']:
                self.config['disable_crl'] = False
            else:
                self.config['disable_crl'] = True

            # Set SSL config
            self.client = self.createClient()
            self.client.call("CORE", "setSSLConfig", self.config)
            self.client.logout()

            # Re use our previous test
            self._test_nucentral_conf()

    def test_generator(self):
        yield self._test_apply_conf
        yield self._test_toggle_CRL
        yield self._test_nucentral_conf

    @classmethod
    def teardown_class(cls):
        """
            This method is called once when all tests have been done.
            It restores the saved configuration.
        """
        # Restore nucentral.conf
        src = "./nucentral.conf"
        dst = "/etc/nucentral/nucentral.conf"
        copy(src, dst)

        cls.restart_nucentral()

        # Teardown
        Test.teardown_class()
