#coding: utf-8
"""
Copyright (C) 2010-2011 EdenWall Technologies
Written by Julien Miotte <jmiotte AT edenwall.com>
$Id: $
"""
from __future__ import with_statement
from templateTest import Test
from os.path import isdir, isfile
from os import makedirs
from ConfigParser import ConfigParser
from nucentral.client.ssl_config import ClientSSLConfig
from nucentral.client.error import NuCentralError
import time
from py import test

CERT = 'cert', "/root/certs/%s/ew4.edenwall.com.crt",
KEY = 'key', "/root/certs/%s/ew4.edenwall.com.key",
USER_CERT = 'user_cert', "/root/certs/%s/cornelius.crt",
USER_KEY  = 'user_key', "/root/certs/%s/cornelius.key",
CRL = 'crl', "/root/certs/%s/allcrl.pem",
CA = 'ca', "/root/certs/%s/allCA.crt"

FILES = CERT, KEY, USER_CERT, USER_KEY, CRL, CA

def genTests(_locals):
    """
        This magic command takes a dictionnary containing test_cases and shortlogs
        to build test classes.
    """
    dic = _locals['DIC']

    for case in dic.keys():
        should_fail = ("[valid]" not in dic[case])

        file_paths = dict((file[0], file[1] % case) for file in FILES)

        _locals['TestAuthCertLogin_%s' % case] = \
            type(
                'TestAuthCertLogin_%s' % case,
                (templateAuthCertLogin,),
                dict(
                    shortlog = dic[case],
                    our_case = (case, dic[case]),
                    should_fail = should_fail,
                    paths = file_paths
                )
             )

class templateAuthCertLogin(Test):
    """
        Test du mécanisme de nucentral d'authentification par certificat.
        * La configuration se fait en écrivant directemment dans le fichier de
          configuration. Ce test est complémentaire avec TestAuthCertConf qui
          vérifie que les appels de service écrivent une configuration correcte.
        * Le test consiste à se logger en utilisant un certificat

    """
#    REQUIRE_COMPONENTS = ('resolv', 'bind')
    paths = {}
    should_fail = False
    our_case = ("", "")
    shortlog = ""

    @classmethod
    def setup_class(cls):
        """
            This method is called once when the class is first instanciated. It
            sets up the testing conditions, while saving the current configuration.
        """
        # Store the orignial // this must come before createClient, so that
        # teardown can do his work
        cls.orig = ConfigParser()
        res = cls.orig.read('/etc/nucentral/nucentral.conf')
        assert res, "OOPS ! /etc/nucentral/nucentral.conf is missing"

        Test.setup_class()

        cls.test_path = cls.results_path+cls.date+"/TestAuthCertLogin/"
        if not isdir(cls.test_path):
            makedirs(cls.test_path)

        # Check we have all certificates we need
        for filename in cls.paths:
            if not isfile(cls.paths[filename]):
                error = 'Missing certificate: %s' % cls.paths[filename]
                test.skip(error)

        # Read nucentral.conf and modify the configuration
        current_conf = ConfigParser()
        current_conf.read('/etc/nucentral/nucentral.conf')
        current_conf.set("CORE", "enable_ssl","yes")
        current_conf.set("ssl", "check_clients","yes")
        current_conf.set("ssl", "use_nupki","no")
        for cert_type in ('ca', 'crl', 'key', 'cert'):
            current_conf.set("ssl", cert_type, cls.paths[cert_type])

        # Write test configuration
        with open("/etc/nucentral/nucentral.conf","w") as handle:
            current_conf.write(handle)

        # Check nothing went wrong
        with open("/etc/nucentral/nucentral.conf") as handle:
            if handle.read() == "":
                test.skip("OOPS ! /etc/nucentral/nucentral.conf is empty")

        # Restart nucentral
        Test.restart_nucentral()

    def test_auth(self):
        """
            Check we can authenticate
        """
        # Configure SSL
        ssl_config = ClientSSLConfig()
        ssl_config.setConfig(
                            self.paths["ca"],
                            self.paths["user_cert"],
                            self.paths["user_key"],
                            self.paths["crl"]
        )
        ssl_config.send_cert = True
        ssl_config.check = True

        # Try to connect
        iterations = 0

        while True:
            try:
                self.createClient(ssl_config)
            except NuCentralError, err:
                oops = True, err
            else:
                oops = False, None

            # If we achieve connection or if the error was SSL related
            if not oops[0] or "Connection refused" not in str(oops[1]):
                break
            time.sleep(1)
            iterations += 1
            assert iterations != 10, "!! Waited 10 seconds for nucentral !!"

        err_mess = "should" if self.should_fail else "shouldn't"
        print "Test case :", self.our_case[1]
        print "Error", str(oops[1])
        error = "[%s] This configuration %s fail"% \
                                     (self.our_case[0], err_mess)
        assert oops[0] == self.should_fail, error

    @classmethod
    def teardown_class(cls):
        """
            This method is called once when all tests have been done.
            It restores the saved configuration.
        """
        # Restore nucentral.conf
        if cls.orig and cls.orig != ConfigParser():
            with open("/etc/nucentral/nucentral.conf", "w") as handle:
                cls.orig.write(handle)

        # Restart nucentral
        Test.restart_nucentral()

        Test.teardown_class()
