#coding: utf-8
"""
Copyright (C) 2010-2011 EdenWall Technologies
Written by Julien Miotte <jmiotte AT edenwall.com>
"""

from os import listdir
from templateTest import Test
from py import test
from os.path import isdir, isfile, join
from shutil import copy
from pprint import pprint
import re

#def pytest_generate_tests(metafunc):
#    ressources = join('ressources', 'testcomp')
#    for entry in listdir(ressources):
#        entry_path = join(ressources, entry)
#        match = re.match("[a-zA-Z0-9_]*\.xml", basename(entry))
#
#        # It's a file :
#        if isfile(entry_path) and match:
#            print ressources, "creating test", entry_path
#            metafunc.addcall(id=entry_path, funcargs={"filename" : entry_path})
#
#        # It's a directory :
#        elif isdir(entry_path) and entry_path != ".svn":
#            print "Found test directory : ", entry_path
#            for sub_entry in listdir(entry_path):
#                sub_entry_path = join(entry_path, sub_entry)
#                match = re.match("[a-zA-Z0-9_]*\.xml", basename(sub_entry))
#                if isfile(sub_entry_path) and match:
#                    print entry_path, "Creating test", entry_path
#                    metafunc.addcall(id=sub_entry_path, funcargs={"filename" : sub_entry_path})

class TestComp(Test):
    COMPONENTS_ENABLED = None

    @classmethod
    def setup_class(cls):
        Test.setup_class()

        # backup configuration.xml
        src = "/var/lib/nucentral/versionned/configuration/configuration.xml"
        dst = "./"
        copy(src, dst)

        cls.COMPONENTS_ENABLED = cls.get_components_enabled()

    @classmethod
    def get_components_enabled(cls):
        """
            Returns the list of components that are actually enabled.
        """
        enabled = set()
        for entry in listdir('/var/lib/nucentral/mods-available/'):
            if isdir(join('/var/lib/nucentral/mods-available/', entry)):
                enabled.add(entry)
        return enabled

    def apply_and_test_conf(self, filename):
        """
            Replaces the running configuration with the test configuration
            and launchs several tests.
        """
        print "test", filename

        # Copy filename to /var/lib/nucentral/versionned/configuration/
        src = filename
        dst = "/var/lib/nucentral/versionned/configuration/"

        # Restart nucentral and wait
        nucentral_pid=self.nucentral_pid()
        Test.stop_nucentral()

        # Setup the configuration
        copy(src, dst)
        Test.start_nucentral()
        if not self.check_nucentral(pid=nucentral_pid, timeout=20):
            test.skip("!! Nucentral was not here after 10 seconds !!")

        myclient = self.createClient()

        # Let's do our tests :
        try:
            self.check_get_conf(myclient, filename)
        finally:
            myclient.logout()

    def check_get_conf(self, myclient, filename):
        """
            We compare the expected configuration to the configuration we read
            from the same call to the running NuCentral.
        """
        ext = '.xml'
        assert filename[-len(ext):] == ext
        filename = filename[:-len(ext)]
        module_name = filename.replace('/', '.')
        # Import test call and expected result
        mod = __import__('%s' % module_name, globals(), locals(), [''], -1)
        conf = myclient.call(*mod.CALL)

        # Print debug
        print "What we got :"
        pprint(conf)
        print "What we expected :"
        pprint(mod.EXPECTED)

        error = "We did not get the expected configuration."
        assert conf == mod.EXPECTED, error

    def test_generative(self, path=None):
        """
            Generates the tests we need
        """
        if not path:
            ressources = join('ressources', 'testcomp')
        else:
            ressources = path

        for entry in listdir(ressources):
            entry_path = join(ressources, entry)
            match = re.match("[a-zA-Z0-9_]*\.xml", entry)

            # It's a file :
            if isfile(entry_path) and match:
                expected_script_present = isfile("%s.py" % entry.split(".")[0])
                if expected_script_present:
                    print ressources, "creating test", entry_path
                yield self.apply_and_test_conf, entry_path

            # It's a directory :
            elif isdir(entry_path) and not entry_path.endswith('.svn'):
                print "Found test directory : ", entry_path
                for test in self.test_generative(path=entry_path):
                    yield test

    @classmethod
    def teardown_class(cls):
        Test.teardown_class()

        # restore configuration.xml
        src = "./configuration.xml"
        dst = "/var/lib/nucentral/versionned/configuration/"
        Test.stop_nucentral()
        copy(src, dst)
        Test.start_nucentral()
