#coding: utf-8
"""
Copyright (C) 2009-2011 EdenWall Technologies
Written by Julien Miotte <jmiotte AT edenwall.com>
$Id$
"""

from templateTest import Test
from nurestore import constants
from os import makedirs
from os.path import isfile, isdir
import shutil

#def postMortem(method):
#    def new_method(self,*args,**kwargs):
#        try:
#            return method(self,*args,**kwargs)
#        except:
#            if method.__name__ == "test_conf_file":
#                shutil.copy(self.FILENAME+"_dont_read", self.test_path)
#            raise
#    return new_method

class TestRestoreList(Test):
    """ Test that we have no regression with the Nurestore white list
        - check that the values readed from the conf file contain the hardcoded
          values
    """
    REQUIRE_COMPONENTS = ('resolv', 'bind')

    @classmethod
    def setup_class(cls):
        """
            This method is called once when the class is first instanciated. It
            sets up the testing conditions, while saving the current configuration.
        """
        Test.setup_class()

        cls.test_path = cls.results_path+cls.date+"/TestRestoreList/"
        if not isdir(cls.test_path):
            makedirs(cls.test_path)

        cls.FILENAME = constants._FILENAME

        # Check that the file is present
        file_present = isfile(cls.FILENAME)
        assert file_present, "File not found"

        cls.first_read = constants.readAuthorizedStarts()

    def test_conf_file(self):
        """
             Check that the configuration file is coherent with the hardcoded
             values
        """
        shutil.move(self.FILENAME, self.FILENAME+"_dont_read")
        # Check that the file has moved
        file_present = isfile(self.FILENAME)

        second_read = constants.readAuthorizedStarts()

        print "In the flat file:\n", self.first_read
        print "Hardcoded values:\n", second_read

        assert not file_present, "File hasn't move "

        error = "We have a difference between the hardcoded values and the file"
        assert self.first_read == second_read, error

    @classmethod
    def teardown_class(cls):
        """
            This method is called once when all tests have been done.
            It restores the saved configuration.
        """
        shutil.move(cls.FILENAME+"_dont_read", cls.FILENAME)
        # Check that the file is present
        file_present = isfile(cls.FILENAME)
        assert file_present, "File not found"

        Test.teardown_class()
