#!/usr/bin/env python2.5
#coding: utf-8

"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""


from __future__ import with_statement

from shutil import rmtree
from tempfile import mkdtemp
import os

from nucentral.core.config.states import NotInitializedState, IdleState
from nucentral.core.config.manager import ConfigurationManager
from nucentral.backend.versionning import VersionningComponent
from nucentral.common.abstract_cfg import AbstractConf

from mock import Core, FakeComponent

class TestAbstractConf:
    def setup_method(self, method):
        self.environ = dict(os.environ)
        os.environ['SVN_I_LOVE_CORRUPTED_WORKING_COPIES_SO_DISABLE_SLEEP_FOR_TIMESTAMPS'] = 'yes'

        self.vardir = mkdtemp(prefix='vardir')

        core = Core(self.vardir)
        self.fake_component = FakeComponent(core)
        self.versionning_component = VersionningComponent()
        self.versionning_component.init(core)
        self.scm = ConfigurationManager(self.fake_component, self.vardir)
        assert self.scm.state is NotInitializedState, "found %s" % self.scm.state
        self.scm.initializeConfig(self.versionning_component, "configuration/configuration.xml")

        assert self.scm.state is IdleState, "found %s" % self.scm.state
        self.scm.reset()
        assert self.scm.state is IdleState, "found %s" % self.scm.state

    def testSerialization(self):
        NAME = 'abstractconf_serialization'
        inputs = []
        inputs.append({})
        inputs.append([])
        inputs.append(tuple())
        inputs.append((1, 2, 3))
        inputs.append({'key00':'value00', 'key01':'value01'})
        inputs.append({ 'key1': {} })
        inputs.append({ 'key2': [] })
        inputs.append({ 'key3': { 'subkey31':'value31', 'subkey32':'value32'} })
        inputs.append({ 'key4': ['value41'] })
        inputs.append({ 'key5': ['value51', 'value52'] })
        inputs.append({ 'key6': { 'subkey61': {} } })
        inputs.append({ 'key7': ['value71', {} ] })
        inputs.append({ 'key8': ['value81', { 'subkey83':'value82' } ] })
        inputs.append({ 'key9': ['value91', { 'subkey93':{} } ] })
        inputs.append({ 'keyA': ['valueA1', { 'subkeyA3':{'subkeyA4':'valueA2'} } ] })

        for obj in inputs:
            obj_s = AbstractConf.serializeElement(obj)
            obj_d = AbstractConf.deserializeElement(obj_s)
            assert id(obj) != id(obj_s)
            assert id(obj_d) != id(obj_s)
            assert obj == obj_d

            self.scm.set(NAME, obj_s)
            self.scm.commit(NAME)
            assert self.scm.get(NAME) == obj_s, "before apply : not equal '%s' '%s' (%s)" % (self.scm.get(NAME), obj_s, obj)
            self.scm.apply()
            assert self.scm.get(NAME) == obj_s, "after apply : not equal '%s' '%s' (%s)" % (self.scm.get(NAME), obj_s, obj)

            self.scm.delete(NAME)

    def teardown_method(self, method):
        rmtree(self.vardir)
        os.environ.clear()
        os.environ.update(self.environ)

