#!/usr/bin/env python2.5
#coding: utf-8

"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from __future__ import with_statement

from copy import deepcopy
from os.path import join#, dirname
import sys

from nucentral.common.abstract_cfg import DatastructureIncompatible
from nuconf.common.auth_cert_cfg import AuthCertConf
#import nuconf.backend

from test_abstract_cfg import TestAbstractConf

CERT_DIR = join('ressources','certs')

class TestAuthCertConf(TestAbstractConf):
    def setup_class(cls):
        old_path = list(sys.path)
        try:
            if '' not in sys.path:
                sys.path.insert(0, '')
            #mod = __import__( '%s/components/auth_cert/auth_cert' % dirname(nuconf.backend.__file__))
            mod = __import__( '/var/lib/nucentral/mods-enabled/auth_cert')
        finally:
            sys.path = old_path

        class FakeAuthCertComponent(mod.AuthCertComponent):
            NAME = "fake_auth_cert_fake"
            MASTER_KEY = NAME

            ACLS = {}

            CERT_PATH = join(CERT_DIR, 'cert.pem')
            KEY_PATH = join(CERT_DIR, 'key.pem')
            CRL_PATH = join(CERT_DIR, 'crl.pem')
            CA_PATH = join(CERT_DIR, 'ca.pem')

        cls.FakeAuthCertComponent = FakeAuthCertComponent

        cls.v2_not_configured = {
            'DATASTRUCTURE_VERSION': 2,
            '__type__': 'nuconf.common.auth_cert_cfg.AuthCertConf',
            'auth_by_cert': 0,
            'ca': '',
            'certificate': '',
            'crl': '',
            'key': '',
            'portal_enabled': False,
            'portal_nets': {'__type__': 'set'},
            'strict': False,
            'use_pki': ''
        }

        cls.v2_without_certs = {
            'DATASTRUCTURE_VERSION': 2,
            '__type__': 'nuconf.common.auth_cert_cfg.AuthCertConf',
            'auth_by_cert': 1,
            'ca': u'',
            'certificate': u'',
            'crl': u'',
            'key': u'',
            'portal_enabled': False,
            'portal_nets': {'__type__': 'set'},
            'strict': False,
            'use_pki': False
        }

        cls.v2_nupki = {
            'DATASTRUCTURE_VERSION': 2,
            '__type__': 'nuconf.common.auth_cert_cfg.AuthCertConf',
            'auth_by_cert': 1,
            'ca': '',
            'certificate': 'SSL nucentral',
            'crl': '',
            'key': '',
            'portal_enabled': False,
            'portal_nets': {'__type__': 'set'},
            'strict': False,
            'use_pki': 'TestDev'
        }

        """
        certs are here:
        CERT_BASE = '/etc/nufw/certs'
        CERT_PATH = join(CERT_BASE, 'nuauth-cert.pem')
        KEY_PATH = join(CERT_BASE, 'nuauth-key.pem')
        CRL_PATH = join(CERT_BASE, 'nuauth-crl.pem')
        CA_PATH = join(CERT_BASE, 'nuauth-cacert.pem')
        """
        cls.v2_pki_ext = {
            'DATASTRUCTURE_VERSION': 2,
            '__type__': 'nuconf.common.auth_cert_cfg.AuthCertConf',
            'auth_by_cert': 1,                  # OK
            'ca': '',                           # -> read file
            'certificate': '',                  # -> renamed into cert
            'crl': '',                          # -> read file & disable_crl
            'key': '',                          # -> read file
            'portal_enabled': False,            # OK
            'portal_nets': {'__type__': 'set'}, # OK
            'strict': False,                    # OK
            'use_pki': False                    # deleted
                                                # Add : use_nupki, nupki_pki, nupki_cert
        }


    def testSerialization_v2tov3_1(self):
        # test default conf
        fake_component = self.FakeAuthCertComponent()
        conf_v3(fake_component, self.v2_not_configured)

    def testSerialization_v2tov3_2(self):
        # test conf with no certs
        fake_component = self.FakeAuthCertComponent()
        conf_v3(fake_component, self.v2_without_certs)

    def testSerialization_v2tov3_3(self):
        # test conf with pki from nupki
        fake_component = self.FakeAuthCertComponent()
        conf_v3(fake_component, self.v2_nupki)

    def testSerialization_v2tov3_4(self):
        # test conf with an external pki
        fake_component = self.FakeAuthCertComponent()
        conf_v3(fake_component, self.v2_pki_ext)

    def testSerialization_v2tov3_5(self):
        # test without CRL
        fake_component = self.FakeAuthCertComponent()
        fake_component.CRL_PATH = join(CERT_DIR, 'inexistant_dir', 'inexistant_file')
        conf_v3(fake_component, self.v2_pki_ext)

    def testSerialization_v2tov3_6(self):
        # test without CA
        fake_component = self.FakeAuthCertComponent()
        fake_component.CA_PATH = join(CERT_DIR, 'inexistant_dir', 'inexistant_file')
        conf_v3(fake_component, self.v2_pki_ext)

def conf_v3(fake_component, conf):
    # sample conf
    confv3 = {
        'DATASTRUCTURE_VERSION': 3,
        '__type__': 'nuconf.common.auth_cert_cfg.AuthCertConf',
        'auth_by_cert': 0,
        'ca': 'XXX',
        'cert': 'XXX',
        'crl': 'XXX',
        'disable_crl': False,
        'key': 'XXX',
        'nupki_cert': '',
        'nupki_pki': '',
        'portal_enabled': False,
        'portal_nets': {'__type__': 'set'},
        'strict': False,
        'use_nupki': False
    }

    conf = deepcopy(conf)
    try:
        obj_d = AuthCertConf.deserialize(conf)
    except DatastructureIncompatible:
        # Simulate behaviour of read_config
        fake_component.upgradeFields(conf)
        obj_d = AuthCertConf.deserialize(conf)
    obj_s = obj_d.serialize()
    assert(obj_s['DATASTRUCTURE_VERSION'] == 3)
    assert(obj_d.DATASTRUCTURE_VERSION == 3)

    assert(len(obj_s) == len(confv3))
    for attr in confv3.iterkeys():
        assert(attr in obj_s)

