#!/usr/bin/env python2.5
#coding: utf-8

"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from __future__ import with_statement

from copy import deepcopy
from os.path import join#, dirname
import sys

from nucentral.common.abstract_cfg import DatastructureIncompatible
from nuconf.common.openvpn_cfg import OpenVpnConf
#import nuconf.backend

from test_abstract_cfg import TestAbstractConf

CERT_DIR = join('ressources', 'certs')

class TestOpenVpnConf(TestAbstractConf):
    def setup_class(cls):
        old_path = list(sys.path)
        try:
            if '' not in sys.path:
                sys.path.insert(0, '')
            #mod = __import__( '%s/components/openvpn/openvpn' % dirname(nuconf.backend.__file__))
            mod = __import__('/var/lib/nucentral/mods-enabled/openvpn')
            print dir(mod)
        finally:
            sys.path = old_path

        class FakeOpenVpnComponent(mod.OpenvpnComponent):
            NAME = "fake_auth_cert_fake"
            MASTER_KEY = NAME

            ACLS = {}

            CERT_PATH = join(CERT_DIR, 'cert.pem')
            KEY_PATH = join(CERT_DIR, 'key.pem')
            CRL_PATH = join(CERT_DIR, 'crl.pem')
            CA_PATH = join(CERT_DIR, 'ca.pem')

        cls.FakeAuthCertComponent = FakeOpenVpnComponent

        cls.v2_not_configured = {
            'DATASTRUCTURE_VERSION': 2,
            '__type__': 'nuconf.common.openvpn_cfg.OpenVpnConf',
            'ca': '',
            'certificate': '',
            'client_network': '',
            'crl': '',
            'enabled': '___UNSET___',
            'key': '',
            'manual_pushed_routes': {'__type__': 'tuple'},
            'port': '1194',
            'protocol': 'udp',
            'redirect': False,
            'server': '',
            'use_pki': ''
        }

        cls.v2_without_certs = {
            'DATASTRUCTURE_VERSION': 2,
            '__type__': 'nuconf.common.openvpn_cfg.OpenVpnConf',
            'ca': u'',
            'certificate': u'',
            'client_network': '',
            'crl': u'',
            'enabled': '___UNSET___',
            'key': u'',
            'manual_pushed_routes': {'__type__': 'tuple'},
            'port': '1194',
            'protocol': 'udp',
            'redirect': False,
            'server': '',
            'use_pki': False
        }

        cls.v2_nupki = {
            'DATASTRUCTURE_VERSION': 2,
            '__type__': 'nuconf.common.openvpn_cfg.OpenVpnConf',
            'ca': '',
            'certificate': 'SSL nucentral',
            'client_network': '',
            'crl': '',
            'enabled': '___UNSET___',
            'key': '',
            'manual_pushed_routes': {'__type__': 'tuple'},
            'port': '1194',
            'protocol': 'udp',
            'redirect': False,
            'server': '',
            'use_pki': 'TestDev'
        }

        """
        certs are here:
        CERT_BASE = '/etc/nufw/certs'
        CERT_PATH = join(CERT_BASE, 'nuauth-cert.pem')
        KEY_PATH = join(CERT_BASE, 'nuauth-key.pem')
        CRL_PATH = join(CERT_BASE, 'nuauth-crl.pem')
        CA_PATH = join(CERT_BASE, 'nuauth-cacert.pem')
        """
        cls.v2_pki_ext = {
            'DATASTRUCTURE_VERSION': 2,
            '__type__': 'nuconf.common.openvpn_cfg.OpenVpnConf',
            'ca': '',
            'certificate': '',
            'client_network': '192.168.99.0/24',
            'crl': '',
            'enabled': False,
            'key': '',
            'manual_pushed_routes': {'__type__': 'tuple'},
            'port': '1194',
            'protocol': 'udp',
            'redirect': True,
            'server': '1.1.1.1',
            'use_pki': False
        }



    def testSerialization_v2tov3_1(self):
        # test default conf
        fake_component = self.FakeAuthCertComponent()
        conf_v3(fake_component, self.v2_not_configured)

    def testSerialization_v2tov3_2(self):
        # test conf with no certs
        fake_component = self.FakeAuthCertComponent()
        conf_v3(fake_component, self.v2_without_certs)

    def testSerialization_v2tov3_3(self):
        # test conf with pki from nupki
        fake_component = self.FakeAuthCertComponent()
        conf_v3(fake_component, self.v2_nupki)

    def testSerialization_v2tov3_4(self):
        # test conf with an external pki
        fake_component = self.FakeAuthCertComponent()
        conf_v3(fake_component, self.v2_pki_ext)

    def testSerialization_v2tov3_5(self):
        # test without CRL
        fake_component = self.FakeAuthCertComponent()
        fake_component.CRL_PATH = join(CERT_DIR, 'inexistant_dir', 'inexistant_file')
        conf_v3(fake_component, self.v2_pki_ext)

    def testSerialization_v2tov3_6(self):
        # test without CA
        fake_component = self.FakeAuthCertComponent()
        fake_component.CA_PATH = join(CERT_DIR, 'inexistant_dir', 'inexistant_file')
        conf_v3(fake_component, self.v2_pki_ext)

def conf_v3(fake_component, conf):
    # sample conf
    confv3 = {
        'DATASTRUCTURE_VERSION': 3,
        '__type__': 'nuconf.common.openvpn_cfg.OpenVpnConf',
        'ca': 'XXX',
        'cert': 'XXX',
        'crl': 'XXX',
        'client_network': '192.168.99.0/24',
        'disable_crl': False,
        'enabled': False,
        'key': 'XXX',
        'manual_pushed_routes': {'__type__': 'tuple'},
        'port': '1194',
        'protocol': 'udp',
        'redirect': True,
        'server': '1.1.1.1',
        'nupki_cert': '',
        'nupki_pki': '',
        'use_nupki': False
    }

    conf = deepcopy(conf)
    try:
        obj_d = OpenVpnConf.deserialize(conf)
    except DatastructureIncompatible:
        # Simulate behaviour of read_config
        fake_component.upgradeFields(conf)
        obj_d = OpenVpnConf.deserialize(conf)
    obj_s = obj_d.serialize()
    assert(obj_s['DATASTRUCTURE_VERSION'] == 3)
    assert(obj_d.DATASTRUCTURE_VERSION == 3)

    assert(len(obj_s) == len(confv3))
    for attr in confv3.iterkeys():
        assert(attr in obj_s)

