#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import with_statement

from nucentral.client import NuCentralClientBase
import re
import sys
import time

CLIENT_VERSION = '3.0-3'

revision_and_rc_re = re.compile(r"^\[([^]]+)\] (?:\[([^]]+)\])?")

def getBackendVersion(client):
    highest_applied = client.call("update", "getHighestApplied")
    if highest_applied < 5000:
        return "màj %d" % highest_applied
    history = client.call("update", "history")
    for update in history:
        if update[1] == highest_applied:
            m = revision_and_rc_re.search(update[3])
            if m:
                rc = m.group(2)
                if rc:
                    rc = " %s" % rc
                else:
                    rc = ""
                return "màj %d %s%s" % (highest_applied, m.group(1), rc)
    return ""  # Error: update not recognized.

class NulogBenchmark():
    def __init__(self, host):
        self.host = host
        self.connected = False
        self.period = 3600 * 24 * 31
        self.max_length = 80
        self.benchmark_version = "2"
        self.table_sequence_all = (
            "LastPacket",
            "InputPacket",
            "InputPacket",
            "ProxyDomainsTable",
            "ProxyHostsTable",
            "UserAppTable",
            "UserTable",
            "UserAppTable",
            "UsersHistoryTable",
            "AuthFail",
            "AppTable",
            "AppTable",
            "AppTable",
            "AppTable",
            "AppTable",
            "AppTable",
            "UserAppTable",
            "AppTable",
            "AppTable",
            "UserAppTable",
            "IPsrcTable",
            "IPsrcTable",
            "HostPortTable",
            "IPsrcTable",
            "IPsrcTable",
            "HostPortTable",
            "UserTable",
            "UserTable",
            "UserAppTable",
            "AppTable",
            "UserAppTable",
            "AppTable",
            "UserTable",
            "PacketsCountTable",
            "UserAppTable",
            "IPsrcTable",
            "PacketsCountTable",
            "IPdstTable",
            "UserTable",
            "IPdstTable",
            )
        self.packet_filters = (
            {},
            {'username': 'bob'},
            {'username': 'bob', u'ip_daddr': '10.0.12.64'},
            {u'ip_daddr': '10.0.12.64'},
            {},
            {u'ip_daddr': '10.0.12.98'},
            {u'ip_daddr': '10.66.66.66'},
            {},
            {u'ip_saddr': '10.66.6.66'},
            {u'dport': 80},
            {u'dport': 666},
            {'username': 'rumney', u'dport': 80},
            {'username': 'rumney'},
            {'username': 'rumney', u'dport': 443},
            {'username': 'rumney', u'dport': 666},
            )
        self.user_ids = {"bob": 1045, "rumney": 1020}

    method = 'https'

    def setup(self):
        for i in range(60):
            try:
                self.client = NuCentralClientBase(host=self.host,
                                                  protocol=self.method)
                self.client.authenticate("admin", "admin")
                time.sleep(1)
                self.connected = True
                break
            except Exception, err:
                print "  !! Error in setup (%s)." % err
                time.sleep(2)

    def getTable(self, name, extra_params=None):
        if not self.connected:
            self.setup()
        start_time = time.time()
        params = {
            "limit": 30,
            "start_time": int(start_time) - self.period,
            "end_time": int(start_time),
            }
        if extra_params:
            if self.highest_applied <= 43:
                # Compatibility with versions < 4.2.
                username = extra_params.get("username", "")
                if username:
                    extra_params["username"] = self.user_ids[username]
            params.update(extra_params)
        table = self.client.call("nulog", "table", name, params)
        end_time = time.time()
        return (end_time - start_time, table)

    def _addedLength(self, name, extra_params):
        return len(name) + len(unicode(extra_params)) + 10

    def getTableAndPrint(self, name, extra_params=None):
        if extra_params is None:
            extra_params = {}
        try:
            elapsed, table = self.getTable(name, extra_params)
            if table["table"]:
                line = unicode(table["table"][0])
                added_length = self._addedLength(name, extra_params)
                if len(line) + added_length > self.max_length:
                    line = line[:self.max_length-3-added_length] + "..."
            else:
                line = "(Empty list.)"
        except Exception, err:
            elapsed = 0.0
            line = "Error: \"%s\"" % err
        print "%7.3f %s %s %s" % (elapsed, name, extra_params, line)
        sys.stdout.flush()
        return elapsed

    def runTest(self):
        self.highest_applied = self.client.call("update", "getHighestApplied")
        backend_version = getBackendVersion(self.client)
        if not backend_version:
            backend_version = "non reconnu"
        print "Version du benchmark : %s." % self.benchmark_version
        print "Niveau de mise à jour : %s." % backend_version
        self.client.call("nulog", "setupClient", {'version': CLIENT_VERSION})
        total = 0.0
        for params in self.packet_filters:
            for name in ("LastPacket", "InputPacket"):
                total += self.getTableAndPrint(name, params)
        subtotal = total
        print "Sous-total 1 : %8.3f" % total
        seen = set()
        for name in self.table_sequence_all:
            if name in seen:
                continue
            seen.add(name)
            total += self.getTableAndPrint(name)
        print "Sous-total 2 : %8.3f" % (total - subtotal)
        print "Total : %8.3f" % total

    def tearDown(self):
        self.client.logout()
        self.connected = False

if __name__ == '__main__':
    if len(sys.argv) < 2:
        sys.stderr.write("Usage: %s <host>\n" % sys.argv[0])
        sys.exit(2)
    cls = NulogBenchmark(sys.argv[1])
    cls.setup()
    cls.runTest()
    cls.tearDown()
