#!/usr/bin/env python
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""


from __future__ import with_statement
from nucentral.common.xml_etree import etree
from sys import argv, exit
import re

# Match %s, %1, but not %%s
ARGUMENT_REGEX = re.compile('(?<!%)%[s1-9]')
INVALID_ARGUMENT_REGEX = re.compile(r'%\([^)]+\)[^s]')

class CheckTS:
    def __init__(self):
        self.exitcode = 0

    def getTag(self,  parent, name):
        node = parent.find(name)
        if node is None:
            raise Exception("ERROR: Missing tag <%s>" % name)
        return node

    def _getText(self, node, default=None):
        text = node.text
        if text is None:
            if default is None:
                raise Exception("ERROR: tag <%s> is empty (no content)" % node.tag)
            else:
                return default
        return text

    def getText(self, node, name):
        text = self.getTag(node, name)
        return self._getText(text)

    def _testMessage(self, context, message, source, translation):
        a = len(ARGUMENT_REGEX.findall(source))
        b = len(ARGUMENT_REGEX.findall(translation))
        if a != b:
            print "ERROR: Different argument count (%%s, %%1, %%2, ...) in context %s" % context
            print 'source (%s): "%s"' % (a, source)
            print 'translation (%s): "%s"' % (b, translation)
            print
            self.exitcode = 1

        if INVALID_ARGUMENT_REGEX.search(translation):
            print "ERROR: Invalid format character in context %s" % context
            print 'translation: "%s"' % (translation,)
            print
            self.exitcode = 1

    def testMessage(self, context, message):
        if self.getTag(message, "translation").attrib.get('type') == 'unfinished':
            return
        source = self.getText(message, "source")
        if message.attrib.get('numerus') == 'yes':
            for numerusform in self.getTag(message, 'translation').findall('numerusform'):
                translation = self._getText(numerusform, '')
                if not translation:
                    continue
                self._testMessage(context, message, source, translation)
        else:
            translation = self.getText(message, "translation")
            self._testMessage(context, message, source, translation)

    def testFile(self, filename):
        with open(filename) as fp:
            document = etree.parse(fp).getroot()
        nb_context = 0
        nb_message = 0
        for context in document.findall('context'):
            name = self.getText(context, "name")
            nb_context += 1
            for message in context.findall('message'):
                self.testMessage(name, message)
                nb_message += 1
        print "Tested: %s context, %s message" % (nb_context, nb_message)

    def main(self):
        if len(argv) != 2:
            print "usage: %s file.ts" % argv[0]
            exit(1)
        filename = argv[1]
        try:
            self.testFile(filename)
        except Exception, err:
            print "Parsing error: %s" % err
            exit(1)
        exit(self.exitcode)

if __name__ == "__main__":
    CheckTS().main()

