#!/usr/bin/env python
"""
Small script to create symlinks to simplify EdenWall hacker's life.


Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

# TODO: create directories
# /usr/share/nucentral/resources/
# /usr/share/nucentral/i18n/
# /var/lib/nucentral/mods-enabled/
# /etc/nucentral/
# /var/lib/nuface3/
# /var/lib/nucentral/nupki2/templates/
# /usr/share/nucentral/templates/
# /var/lib/nuface3/templates/
# /usr/share/nucentral/scripts/

from os import lstat, unlink, symlink
from os.path import (join as path_join,
    realpath, exists, basename, dirname)
from sys import exit, argv
from stat import S_ISLNK
from errno import ENOENT, EACCES
from glob import glob

COMPONENT_BLACKLIST = set((
    'openswan',        # work in progress
))

DEV = {
    # Dest dir  (relative to dev dir) => Source dir (relative to src dir)
    "nuface3": "nuface3",
    "eas": "edenwall-administration-suite",
    "nucentral": "nucentral",
    "nucentral-multisite": "nucentral-multisite",
    "nulog3": "nulog3",
    "nucentral-admin": "nucentral-admin",
    "nuconf": "nuconf",
    "nupki": "nupki",
    "tools": "tools",
    "nucentral-edenwall": "nucentral-edenwall",
    "nuconntrack": "nuconntrack",
    "nurestore": "nurestore",
    "nudpi": "nudpi",
}

PYTHON_NUCENTRAL = {
    # NuCentral submodle => Source dir (relative to src dir)
    "qt": "nucentral-edenwall/nucentral/qt",
    "util": "nucentral-edenwall/nucentral/util",
}

class CreateLink:
    def __init__(self):
        if len(argv) == 2:
            devdir = argv[1]
            self.mode = 'install'
        elif len(argv) == 3 and argv[2] == 'uninstall':
            devdir = argv[1]
            self.mode = 'uninstall'
        else:
            print "usage: %s dev-directory [uninstall]" % argv[0]
            exit(1)
        src = path_join(dirname(__file__), '..')
        self.src_root_dir = realpath(src)
        self.dev_root_dir = realpath(devdir)

        self.links = tuple(self.createLinks())
        print "Source directory: %s" % self.src_root_dir
        print "Dev directory: %s" % self.dev_root_dir
        print

    def createLinks(self):
        python25_dir = '/usr/lib/python2.5/site-packages'
        python26_dir = '/usr/lib/python2.6/dist-packages'
        yield (DEV, self.dev_root_dir)
        yield self.createResources()
        yield self.createQM()
        yield self.createPython(python25_dir)
        yield (PYTHON_NUCENTRAL, path_join(python25_dir, 'nucentral'))
        if exists(python26_dir):
            yield self.createPython(python26_dir)
            yield (PYTHON_NUCENTRAL, path_join(python26_dir, 'nucentral'))
        yield self.createComponents()
        yield ({'default.nucentral.conf': 'nucentral/default.nucentral.conf'}, '/etc/nucentral')
        yield ({'library.xml': 'nuface3/example/library.xml'}, '/var/lib/nuface3')
        yield self.createNuPKI()
        yield self.createTemplates()
        yield ({'files_to_export.list': 'nurestore/config/files_to_export.list'}, '/usr/share/nucentral')
        yield self.createBin()
        yield self.createSbin()
        yield self.createNuFace()
        yield self.createScripts()

    def createScripts(self):
        scripts = (
            'nuconf/nuconf/backend/scripts/diagnostic',
        )
        return (self._createDict(scripts), '/usr/share/nucentral/scripts')

    def createNuFace(self):
        templates = (
            'nuface3/example/local_ipv4_firewall.xml',
            'nuface3/example/local_ipv6_firewall.xml')
        return (self._createDict(templates), '/var/lib/nuface3/templates')

    def createBin(self):
        programs = ('nucentral/tools/nucentral_client',)
        return (self._createDict(programs), '/usr/bin')

    def createSbin(self):
        return (self._createDict(('nuface3/scripts/nuface',)), '/usr/sbin')

    def createNuPKI(self):
        pattern = path_join(self.src_root_dir, 'nupki/nupki/templates/*.tmpl')
        files = glob(pattern)
        files = self._createDict(files)
        return (files, '/var/lib/nucentral/nupki2/templates')

    def _createDict(self, filenames):
        return dict((basename(filename), filename) for filename in filenames)

    def createResources(self):
        resources = self._createDict((
            "nuface3/nufaceqt/nuface3.rcc",
            "nulog3/nulog-qt/nulog3.rcc",
            "nudpi/nudpi-qt/nudpi.rcc",
            "edenwall-administration-suite/console_edenwall.rcc",
            "nupki/nupki/qt/nupki.rcc",
            "nucentral-admin/nucentral_admin/nucentral_admin.rcc",
            "nuconf/nuconf/client/system/system.rcc",
            "nucentral-edenwall/edenwall.rcc",
            "nucentral-multisite/qt/ew4_multisite/multisite.rcc",
        ))
        return (resources, '/usr/share/nucentral/resources')

    def createQM(self):
        qm = self._createDict((
            "nucentral/nucentral.fr.qm",
            "nucentral-edenwall/nucentral_edenwall.fr.qm",
        ))
        return (qm, '/usr/share/nucentral/i18n')

    def getNuConfComponents(self):
        """
        Get nuconf components as a dict: componant name => directory absolute name
        """
        nuconf = {}
        pattern = path_join(self.src_root_dir, 'nuconf/nuconf/backend/components', '*')
        for fullname in glob(pattern):
            name = basename(fullname)
            if name in COMPONENT_BLACKLIST:
                continue
            nuconf[name] = fullname
        return nuconf

    def createPython(self, directory):
        links = {
            # Python module name (relative to python dir) => Source dir (relative to src dir)
            "nuface": "nuface3/nuface",
            "nufaceqt": "nuface3/nufaceqt",
            "nucentral": "nucentral/nucentral",
            "nucentral_admin": "nucentral-admin/nucentral_admin",
            "nulog": "nulog3/nulog",
            "nuconf": "nuconf/nuconf",
            "nupki": "nupki/nupki",
            "nudpi": "nudpi/nudpi",
        }
        return (links, directory)

    def createComponents(self):
        components = {
            # Dest dir (relative to vardir) => Source dir (relative to src dir)
            "nuface_localfw": "nuface3/nuface/localfw",
            "nuface_forwad": "nuface3/nuface/forward",
            "nulog": "nulog3/nulog/core",
            "nupki": "nupki/nupki/module",
            "nuconntrack": "nuconntrack/nuconntrack",
            "nurestore": "nurestore/nurestore",
            "multisite_transport": "nucentral-edenwall/nucentral/multisite/transport",
            "nudpi": "nudpi/nudpi/backend",
    #        "multisite_slave": "nucentral-multisite/nucentral/multisite/slave",
    #        "multisite_master": "nucentral-multisite/nucentral/multisite/master",
        }
        nuconf = self.getNuConfComponents()
        components.update(nuconf)
        return (components, '/var/lib/nucentral/mods-enabled')

    def createTemplates(self):
        pattern = path_join(self.src_root_dir, 'nuconf/nuconf/backend/templates/*')
        files = glob(pattern)
        files = self._createDict(files)
        return (files, '/usr/share/nucentral/templates')

    def main(self):
        if self.mode == 'uninstall':
            # check
            for links, dest_dir in self.links:
                self.run(links, dest_dir, self.unlink)
            print "EdenWall uninstalled."
        else:
            # check
            for links, dest_dir in self.links:
                self.run(links, dest_dir, self.check_link)
            # replace links
            for links, dest_dir in self.links:
                self.run(links, dest_dir, self.create_link)
            print "EdenWall installed using symlinks."

    def check_link(self, src, dest):
        try:
            mode = lstat(dest).st_mode
        except OSError, err:
            if err.errno == ENOENT:
                # file not exist, nothing to do
                return
            else:
                raise
        if not S_ISLNK(mode):
            print "Error: %s is not a link! Remove it" % dest
            exit(1)
#        link = readlink(dest)
#        if not exists(link):
#            print "Remove broken link: %s (-> %s)" % (dest, link)
#        else:
#            print "Warning: %s (-> %s) will be replaced" % (dest, link)

    def unlink(self, src, dest):
        try:
            mode = lstat(dest).st_mode
        except OSError, err:
            if err.errno == ENOENT:
                # file not exist, nothing to do
                return
            else:
                raise
        if not S_ISLNK(mode):
            print "Error: %s is not a link! Remove it manually" % dest
            exit(1)
        unlink(dest)

    def create_link(self, src, dest):
        try:
            try:
                unlink(dest)
            except OSError, err:
                if err.errno == ENOENT:
                    pass
                else:
                    raise
#            print "Create %s (-> %s)" % (dest, src)
            symlink(src, dest)
        except OSError, err:
            if err.errno == EACCES:
                print "Error: you are not allowed to replace %s. Retry as root" % dest
                exit(1)
            else:
                raise

    def run(self, links, dest_dir, func):
        for dest, src in links.iteritems():
            src = path_join(self.src_root_dir, src)
            dest = path_join(dest_dir, dest)
            func(src, dest)

def main():
    app = CreateLink()
    app.main()

if __name__ == "__main__":
    main()

