#!/usr/bin/env python2.5
"""
Remove (disable) broken components:
 - nuconf components without the init script
 - component raising an ImportError on load (eg. nuauth
   without the nuauth_command Python module)


Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""


from os.path import join as path_join, exists, basename, isabs
from os import listdir, unlink
from inspect import isclass
import sys
from nucentral.backend import Component
from nuconf.backend.unix_service import UnixServiceComponent

MODULES_DIR = '/var/lib/nucentral/mods-enabled'

def disableComponent(name, filename, reason):
    print "Disable component %s: %s" % (name, reason)
    unlink(filename)

def getComponents():
    old_path = sys.path
    try:
        if '' not in sys.path:
            # If '' is present sys.path, __import__ accept a module
            # filename with an absolute directory
            sys.path.insert(0, '')

        for name in listdir(MODULES_DIR):
            fullname = path_join(MODULES_DIR, name)
            try:
                module = __import__(fullname)
            except BaseException, err:
                disableComponent(basename(fullname), fullname,
                    "ImportError: %s" % err)
                continue
            for attrname in dir(module):
                attr = getattr(module, attrname)
                if not isclass(attr) \
                or not issubclass(attr, Component) \
                or attr == Component:
                    continue
                yield attr, fullname
    finally:
        sys.path = old_path

def disableMissing():
    for component, filename in getComponents():
        if not issubclass(component, UnixServiceComponent):
            continue
        init_script = component.INIT_SCRIPT
        if not isabs(init_script):
            init_script = path_join("/etc/init.d", init_script)
        if exists(init_script):
            continue
        disableComponent(component.NAME, filename,
            "missing init script (%s)" % init_script)
    print "done"

if __name__ == "__main__":
    disableMissing()

