#!/bin/bash
if [ "$#" != 2 ]
then
	echo "Syntax: $0 [config file] [test directory]" >&2
	exit 1
fi

set -x
source "$1"

export TEST_IP
export TEST_MASTER
export TEST_NETWORK
export TEST_NETWORK2
export TEST_IP2A
export TEST_IP2B
export TEST_IP2C
export TEST_IP2GW
export TEST_PARENT_PROXY
export TEST_URL
export TEST_DST_NETWORK
export TEST_DST_HOST
export TEST_IP_PRIMARY
export TEST_IP_SECONDARY
export TEST_SRC_HOST_VIA_SSH

export PYTHONPATH=$PYTHONPATH:../../../nucentral:../../replay:..

TEST_SECONDARY_HOSTNAME=$TEST_PRIMARY_HOSTNAME-secondary
export TEST_NUCENTRAL_IP=$TEST_IP_PRIMARY

TEST_DIR=$(cd $(dirname $0); pwd)

wait_nucentral()
{
	# $1 : ip address
	date
	for((i=0; i<60; i++))
	do
		echo | socat STDIN TCP4:$1:8443 && return 0
		sleep 10
	done
	date
	return 1
}

wait_ssh()
{
	# $1 : ip address
	date
	for((i=0; i<60; i++))
	do
		echo | socat STDIN TCP4:$1:22 && return 0
		sleep 10
	done
	date
	return 1
}

dump_nucentral_log()
{
    /home/virt/opt/fab \
        -H "$1" -u root -p fai \
       -f $TEST_DIR/fabfile.py \
       cat_nucentral_log
}

wait_hostname()
{
	# $1 : ip address
	# $2 : hostname
	date
	for((i=0; i<30; i++))
	do
		if [ "$(snmpwalk $1 -v 2c -c public SNMPv2-MIB::sysName.0)" == "SNMPv2-MIB::sysName.0 = STRING: $2" ]
		then
			break
		fi
		sleep 10
	done
	date
	if [ "$(snmpwalk $1 -v 2c -c public SNMPv2-MIB::sysName.0)" == "SNMPv2-MIB::sysName.0 = STRING: $2" ]
	then
		return 0
	fi
	echo "Host $2 didn't take over"
	return 1
}

res=0
cd $2
for file in *.check
do
	echo "Starting ${file//.check/}"

	# Configuration commands
	if [ -e ./${file//.check/.config} ]
	then
		# Use custom configuration command if it exists
		# otherwise just start edw xml-rpc commands
		./${file//.check/.config}
	else
		../replay_and_apply.py $TEST_IP_PRIMARY -v ${file//.check/.edw}
	fi
	res=$(($res|$?))
        if [ $res -ne 0 ]; then
            dump_nucentral_log $TEST_IP_PRIMARY
            break
        fi

	# Wait for the primary to return to its active state
	# (useful during network modifications, to avoid #2333
	wait_hostname $TEST_IP $TEST_PRIMARY_HOSTNAME || exit 1

	# Perform the check on primary
	./${file}
	res=$(($res|$?))
        if [ $res -ne 0 ]; then
            dump_nucentral_log $TEST_IP_PRIMARY
            break
        fi

	# Wait for the conf to be replicated from primary to secondary
    echo "Wait for replication"
    ../wait_end_of_replication.py $TEST_IP_PRIMARY $TEST_IP_SECONDARY
	res=$(($res|$?))
        if [ $res -ne 0 ]; then
            echo "Replication failed"
            echo "DUMP PRIMARY AND SECONDARY LOGS"
            echo "** PRIMARY LOG **"
            dump_nucentral_log $TEST_IP_PRIMARY
            echo "** SECONDARY LOG **"
            dump_nucentral_log $TEST_IP_SECONDARY
            break
        fi
    echo "Replication done"

	echo "Killing primary"
	eval $TEST_SUSPEND_PRIMARY_SCRIPT

	wait_hostname $TEST_IP $TEST_SECONDARY_HOSTNAME || exit 1

	# Perform the check on secondary
	./${file}
	res=$(($res|$?))
        if [ $res -ne 0 ]; then
            dump_nucentral_log $TEST_IP_SECONDARY
            break
        fi

	echo "Resuming primary"
	eval $TEST_RESUME_PRIMARY_SCRIPT

	# Wait for the primary to be up
	wait_nucentral $TEST_IP_PRIMARY
	wait_ssh $TEST_IP_PRIMARY

	# Wait for startup to be finnished
	sleep 10s

	# Call takeover command on primary
	for((i=0; i < 3; i++))
	do
		../ha_takeover.py
		wait_hostname $TEST_IP $TEST_PRIMARY_HOSTNAME && break
                if [ $i -eq 2 ]; then
                    dump_nucentral_log $TEST_IP_PRIMARY
                    exit 1
                fi
	done
done

exit $res
