# Script to listen on a port range: [PORT_INIT; PORT_INIT+NTHREADS(

import socket, time, datetime, threading

HOST = '0.0.0.0'
PORT_INIT = 4000
NTHREADS = 20

class Wait(threading.Thread):
  def __init__(self, port):
    threading.Thread.__init__(self)
    self.port = port

  def run(self):
    self.stop = True
    self.c = 0
    self.t = 0
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.bind((HOST, self.port))
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    s.listen(5)
    self.stop = False
    while not self.stop:
      client = s.accept()
      self.c += 1
      self.t += 1
    s.close()
    self.stop = True

class Manager:
  def __init__(self):
    self.start = time.time()
    self.last = self.start

  def stats(self):
    now = time.time()
    diff = now - self.last
    total_diff = now - self.start
    c = 0
    t = 0
    for thread in self.threads:
      c += thread.c
      t += thread.t
      thread.c = 0
    if diff:
      cur = '%.1f / sec' % (c / diff)
    else:
      cur = '---'
    if total_diff:
      avg = '%.1f / sec' % (t / total_diff)
    else:
      avg = '---'
    print("cur=%s, avg=%s, total=%s in %s" % (
          cur,
          avg,
          t, datetime.timedelta(seconds=total_diff)))
    self.last = time.time()

  def main(self):
    print("Start %s threads" % NTHREADS)
    self.threads = [Wait(PORT_INIT + n) for n in xrange(NTHREADS)]
    for thread in self.threads:
      thread.start()
    print("Listen on ports %s..%s (host %s)" % (
        PORT_INIT, PORT_INIT+NTHREADS-1, HOST))

    try:
        while 1:
          time.sleep(3.0)
          self.stats()
          if any(thread.stop for thread in self.threads):
            print("-error-")
            break
    except KeyboardInterrupt:
        print("stop")
    print("Stop %s threads" % NTHREADS)
    for thread in self.threads:
      thread.stop = True
    for thread in self.threads:
      thread.join()
    self.stats()

Manager().main()

