#!/usr/bin/python

"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""


from __future__ import with_statement
import sys

from nucentral.client.nucentral_client_base import NuCentralClientBase
from nucentral.client.error import NuCentralError
from nucentral.common.error import exceptionAsUnicode

def format_result(result):
    if isinstance(result, (str, unicode)):
        return u'"%s"' % result
    return unicode(result)

def replay(host, recorded_calls, verbose=False):
    client = NuCentralClientBase(host=host)
    for call in recorded_calls:
        call = call[1]
        if verbose:
            sys.stdout.write(u'    call:   "%s"\n' % unicode(call))
        try:
            result = client.call(*call)
            if verbose:
                sys.stdout.write(u'    result: %s\n' % format_result(result))
        except NuCentralError, err:
            sys.stderr.write(u"!! %s !!\n" % exceptionAsUnicode(err))
            sys.exit(3)
        except Exception, err:
            sys.stderr.write(u"!!! %s !!!\n" % exceptionAsUnicode(err))
            sys.exit(3)

def replay_record_files(record_filenames, host, options, env={}):
    for record_filename in record_filenames:
        record_file_contents = open(record_filename).read()
        exec record_file_contents in env
        if 'recorded_calls' not in env:
            sys.stderr.write("!!!! %s is not a record file. !!!!\n" %
                             record_filename)
            sys.exit(2)
        recorded_calls = env['recorded_calls']
        replay(host, recorded_calls, verbose=options.verbose)

if __name__ == '__main__':
    from optparse import OptionParser
    usage = "Usage: %prog [options] host record_file [record_file...]"
    parser = OptionParser(usage=usage)
    parser.add_option("-v", "--verbose", action="store_true", default=False,
                      help="Print calls and results")
    (options, args) = parser.parse_args()
    if len(args) < 2:
        parser.error("Please specify host and record file(s).")
    replay_record_files(args[1:], args[0], options)

