#!/bin/bash

set -e

EDW_TO_SYNC="$@"

DESTDIR=/
TMPDIR=~/tmp/edw4-tmp

function build_tree()
{
	# $1 : source dir
	# $2 : destination dir

	# Create the directory if necessary
	if [  -d $1 ]
	then
		mkdir -p "$2" 2> /dev/null
	else
		# Create parent dir
		mkdir -p "$(dirname "$2")" 2> /dev/null
	fi
	
	rsync -a --exclude=.svn --exclude=*.pyc $1 $2
}


# Missing sync:
# - conf files
# - binaries
rm -rf $TMPDIR

#build_tree edenwall-snmp/edenwall_snmp/			$TMPDIR/python-support/edenwall-snmp/

build_tree nucentral/nucentral/backend/			$TMPDIR/usr/share/python-support/nucentral-server/nucentral/backend/
build_tree nucentral/nucentral/core/			$TMPDIR/usr/share/python-support/nucentral-server/nucentral/core/
build_tree nucentral/nucentral/client/			$TMPDIR/usr/share/python-support/nucentral-client/nucentral/client/
build_tree nucentral/nucentral/common/			$TMPDIR/usr/share/python-support/nucentral-common/nucentral/common/
build_tree nucentral/nucentral/python/			$TMPDIR/usr/share/python-support/nucentral-common/nucentral/python/

build_tree nucentral-edenwall/nucentral/logger/		$TMPDIR/usr/share/python-support/nucentral-edenwall/nucentral/logger/
build_tree nucentral-edenwall/nucentral/multisite/	$TMPDIR/usr/share/python-support/nucentral-edenwall/nucentral/multisite/
build_tree nucentral-edenwall/nucentral/util/		$TMPDIR/usr/share/python-support/nucentral-edenwall/nucentral/util/

build_tree nucentral-multisite/nucentral/multisite/	$TMPDIR/usr/share/python-support/nucentral-multisite/nucentral/multisite/

build_tree nuconf/nuconf/backend/templates/		$TMPDIR/usr/share/nucentral/templates/
build_tree nuconf/nuconf/backend/			$TMPDIR/usr/share/python-support/nuconf-backend/nuconf/backend/
build_tree nuconf/nuconf/common/			$TMPDIR/usr/share/python-support/nuconf-common/nuconf/common/

build_tree nuface3/nuface/config.py			$TMPDIR/usr/share/python-support/nuface-common/nuface/config.py
build_tree nuface3/nuface/error.py			$TMPDIR/usr/share/python-support/nuface-common/nuface/error.py
build_tree nuface3/nuface/__init__.py			$TMPDIR/usr/share/python-support/nuface-common/nuface/__init__.py
build_tree nuface3/nuface/version.py			$TMPDIR/usr/share/python-support/nuface-common/nuface/version.py
build_tree nuface3/nuface/common/			$TMPDIR/usr/share/python-support/nuface-common/nuface/common/

build_tree nuface3/nuface/forward/			$TMPDIR/usr/share/python-support/nuface-forward/nuface/forward/
build_tree nuface3/nuface/iptables/			$TMPDIR/usr/share/python-support/nuface-forward/nuface/iptables/

build_tree nuface3/nuface/localfw/			$TMPDIR/usr/share/python-support/nuface-localfw/nuface/localfw/
build_tree nuface3/scripts/nuface			$TMPDIR/usr/sbin/

build_tree nulog3/nulog/core/				$TMPDIR/usr/share/python-support/nulog/nulog/core/
build_tree nudpi/nudpi/backend/				$TMPDIR/usr/share/python-support/nudpi/nudpi/backend/

build_tree nupki/nupki/version.py			$TMPDIR/usr/share/python-support/nupki-backend/nupki/version.py
build_tree nupki/nupki/core/				$TMPDIR/usr/share/python-support/nupki-backend/nupki/core/
build_tree nupki/nupki/module/				$TMPDIR/usr/share/python-support/nupki-backend/nupki/module/

build_tree nurestore/nurestore/				$TMPDIR/usr/share/python-support/nurestore/nurestore/
build_tree nurestore/config/				$TMPDIR/usr/share/nucentral/

#build_tree nuconntrack/nuconntrack/			$TMPDIR/python-support/nuconntrack/nuconntrack/


# Set permissions
find $TMPDIR -type f -exec chmod 644 {} \;
find $TMPDIR -type d -exec chmod 755 {} \;

function sync_edw()
{
	# $1 : hostname to sync
	rsync --no-owner -avz --rsh=ssh $TMPDIR/ $1:$DESTDIR/
	ssh $1 "chown root:root / ; chmod 755 /usr/sbin/nuface; chown -R root:root /usr ; /etc/init.d/nucentral-server restart"
}

for edw in $EDW_TO_SYNC
do
	sync_edw $edw &
done
wait


