from nucentral.qt.application import create_application, setup_logging, parse_options
from nucentral.qt.auth_window import AuthWindow
from PyQt4.QtTest import QTest
from PyQt4.QtCore import Qt
from nucentral.client import CLIENT_SUITE_SHORTNAME
from console_edenwall import MainWindow
from PyQt4.QtGui import QSplashScreen, QPixmap, QRegion


EW4_IP = "172.16.1.20"
EW4_LOGIN = "admin"
EW4_PASSWORD = "admin"

# This tells us if we want every widget to be shown
NOISY = False

class Test():
# {{{ def setup_class(cls)
    @classmethod
    def setup_class(cls):
        cls.app, cls.client = cls.fake_auth(EW4_IP, EW4_LOGIN, EW4_PASSWORD)
#        cls.client = cls._check_and_upload_license()
        cls.add_splash_screen()
        cls.app.processEvents()
        cls.window = MainWindow(cls.app, cls.client)
        if NOISY:
            cls.window.show()
# }}}
# {{{ def embed_shell(self)
    def embed_shell(self):
        """
            Useful method that invokes an interactive python interface in the
            middle of the script.
        """
        try:
            __IPYTHON__
        except NameError:
            nested = 0
            args = ['']
        else:
            print "Running nested copies of IPython."
            print "The prompts for the nested copy have been modified"
            nested = 1
            # what the embedded instance will see as sys.argv:
            args = ['-pi1','In <\\#>: ','-pi2','   .\\D.: ',
                    '-po','Out<\\#>: ','-nosep']

        from IPython.Shell import IPShellEmbed
        ipshell = IPShellEmbed(args,
                               banner = '___ Entering IPython ___',
                               exit_msg = '___ Leaving IPython ___')
        ipshell()
# }}}
# {{{ def text_input(self, widget, text)
    @classmethod
    def text_input(cls, widget, text):
        """
            This uses QTest to simulate the input of a user, to fill a widget.
        """
        QTest.mouseMove(widget)
        QTest.mouseClick(widget, Qt.LeftButton)
        nomodifier = Qt.KeyboardModifiers()
        for key in text:
            QTest.keyClick(widget, key)
# }}}
# {{{ def text_clear(self, widget)
    @classmethod
    def text_clear(cls, widget):
        """
            This uses QTest to simulate an user deleting all text in a widget.
        """
        length = 12
        try:
            length = len(widget.text())
        except:
            pass
        try:
            length = len(widget.lineEdit().text())
        except:
            pass
        for i in range(length):
            QTest.keyClick(widget, Qt.Key_Backspace)
# }}}
# {{{ def my_create_application(self, create_options=None, resource=None, locale=None)
    @classmethod
    def my_create_application(cls, create_options=None, resource=None, locale=None):
        """
            This creates the EAS application.
        """
        # Parse command line options and setup the logging module
#        options = parse_options(create_options)
        class options():
            username = None
            secure = False
            cleartext = False
            host = None
            debug = True
            password = False
            streaming_port = None
            port = None
            verbose = False

        setup_logging(options())

        # Create the application
        app = create_application(resource, locale)
        app.options = options()

        return app
# }}}
# {{{ def fake_auth(self, host, login, password)
    @classmethod
    def fake_auth(cls, host, login, password):
        """
            This asks for the creation of the EAS application, than
            then authenticates to HOST with LOGIN and PASSWORD.
        """
        app = cls.my_create_application(None, None)

        login_window = AuthWindow()
        if NOISY:
            login_window.show()
        cls.text_clear(login_window.host_edit)
        cls.text_input(login_window.host_edit, EW4_IP)

        cls.text_clear(login_window.login_edit)
        cls.text_input(login_window.login_edit, EW4_LOGIN)

        cls.text_clear(login_window.password_edit)
        cls.text_input(login_window.password_edit, EW4_PASSWORD)

        assert str(login_window.host_edit.lineEdit().text()) == EW4_IP
        assert str(login_window.login_edit.text()) == EW4_LOGIN
        assert str(login_window.password_edit.text()) == EW4_PASSWORD

        QTest.mouseClick(login_window.ok_button, Qt.LeftButton)

        client = login_window.get_client(CLIENT_SUITE_SHORTNAME)

        return app, client
# }}}
# {{{ def add_splash_screen(self, app)
    @classmethod
    def add_splash_screen(cls):
        """
            We need this splash screen to launch EAS.
        """
        from nucentral.common.multisite import NO_MULTISITE, MULTISITE_SLAVE, MULTISITE_MASTER
        SPLASH_SCREENS = {
            NO_MULTISITE: ':/images/splash_start.png',
            MULTISITE_SLAVE: ':/images/splash_start.png',
            MULTISITE_MASTER: ':/images/splash_emf.png',
        }

        multisite_type = cls.client.call('CORE', 'getMultisiteType')
        pixmap = QPixmap(SPLASH_SCREENS[multisite_type])
        emf = (multisite_type == 'master')

        splash = QSplashScreen(pixmap)
        splash.setMask(QRegion(pixmap.mask()));
        splash.setPixmap(pixmap);
        if NOISY:
            splash.show()
        cls.app.splash = splash
# }}}
# {{{ def teardown_class(self)
    @classmethod
    def teardown_class(cls):
        ## Quit EAS :
        # Try to click :
        #QTest.mouseClick(window.action_Quit, Qt.LeftButton) # This doesn't work !

        # Try Ctrl+Q :
        #QTest.keyClick(window, Qt.Key_Q, Qt.ControlModifier) # This doesn't work !

        # Emit the correct signal to the action. If anyone knows how to to it better ...
        from PyQt4.QtCore import SIGNAL
        cls.window.action_Quit.emit(SIGNAL("triggered()"))
# }}}

