
"""
Copyright (C) 2009-2011 EdenWall Technologies

This file is part of NuFirewall. 
 
 NuFirewall is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, version 3 of the License. 
 
 NuFirewall is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with NuFirewall.  If not, see <http://www.gnu.org/licenses/>
"""

from random import randint
from .create_arg import createArgument

# maximum number of arguments generated for such service:
#    def func(*args): ...
# minimum number is zero
MAX_ARG = 4

class Service:
    def __init__(self, component, service, min_arg, max_arg):
        self.component = component
        self.service = service
        self.min_arg = min_arg
        self.max_arg = max_arg

    def createArguments(self):
        nb_arg = randint(self.min_arg, self.max_arg)
        return tuple(createArgument() for index in xrange(nb_arg))

def parsePrototype(prototype):
    """
    >>> parsePrototype('func(a, b)')
    (2, 2)
    >>> parsePrototype('func(a, *args)')
    (1, 4)
    >>> parsePrototype('func(a, b=3)')
    (1, 2)
    >>> parsePrototype('func(a, **kw)')
    (1, 1)
    >>> parsePrototype('func(a, *args, **kw)')
    (1, 4)
    """
    args = prototype[prototype.index('(')+1:-1]
    if not args:
        return None
    args = tuple(arg.strip() for arg in args.split(','))
    min_arg = None
    max_arg = None
    for arg_index, arg in enumerate(args):
        if arg.startswith('**'):
            if min_arg is None:
                min_arg = arg_index
            max_arg = arg_index
            break
        elif arg.startswith('*'):
            if min_arg is None:
                min_arg = arg_index
            max_arg = MAX_ARG
            break
        elif ('=' in arg) and (min_arg is None):
            min_arg = arg_index
    if min_arg is None:
        min_arg = len(args)
    if max_arg is None:
        max_arg = len(args)
    return min_arg, max_arg

